Package: BMS
Type: Package
Title: Bayesian Model Averaging Library
Version: 0.3.5
Date: 2022-08-05
Author: Martin Feldkircher and Stefan Zeugner and Paul Hofmarcher
Maintainer: Stefan Zeugner <stefan.zeugner@ec.europa.eu>
Depends: methods, stats, graphics, R (>= 2.10)
Description: Bayesian Model Averaging for linear models with a wide choice of (customizable) priors. Built-in priors include coefficient priors (fixed, hyper-g and empirical priors), 5 kinds of model priors, moreover model sampling by enumeration or various MCMC approaches. Post-processing functions allow for inferring posterior inclusion and model probabilities, various moments, coefficient and predictive densities. Plotting functions available for posterior model size, MCMC convergence, predictive and coefficient densities, best models representation, BMA comparison. Also includes Bayesian normal-conjugate linear model with Zellner's g prior, and assorted methods.
License: BSD_3_clause + file LICENSE
URL: http://bms.zeugner.eu
RoxygenNote: 7.2.1
LazyData: true
NeedsCompilation: no
Packaged: 2022-08-05 10:42:16 UTC; admin-b1
Repository: CRAN
Date/Publication: 2022-08-09 11:10:02 UTC
Built: R 4.3.3; ; 2025-01-24 04:54:38 UTC; unix

Package: BMS
Type: Package
Title: Bayesian Model Averaging Library
Version: 0.3.5
Date: 2022-08-05
Author: Martin Feldkircher and Stefan Zeugner and Paul Hofmarcher
Maintainer: Stefan Zeugner <stefan.zeugner@ec.europa.eu>
Depends: methods, stats, graphics, R (>= 2.10)
Description: Bayesian Model Averaging for linear models with a wide choice of (customizable) priors. Built-in priors include coefficient priors (fixed, hyper-g and empirical priors), 5 kinds of model priors, moreover model sampling by enumeration or various MCMC approaches. Post-processing functions allow for inferring posterior inclusion and model probabilities, various moments, coefficient and predictive densities. Plotting functions available for posterior model size, MCMC convergence, predictive and coefficient densities, best models representation, BMA comparison. Also includes Bayesian normal-conjugate linear model with Zellner's g prior, and assorted methods.
License: BSD_3_clause + file LICENSE
URL: http://bms.zeugner.eu
RoxygenNote: 7.2.1
LazyData: true
NeedsCompilation: no
Packaged: 2022-08-05 10:42:16 UTC; admin-b1
Repository: CRAN
Date/Publication: 2022-08-09 11:10:02 UTC

# Generated by roxygen2: do not edit by hand

importFrom(stats,density)
importFrom(stats,quantile)
importFrom(stats,variable.names)
importFrom(stats,vcov)
importFrom(stats,predict)
importFrom(stats,deviance)
importFrom(stats,coef)
importFrom(stats,model.frame)
importFrom("methods", "is", "new")
importFrom("stats", "as.formula", "cor", "na.omit", "var")

S3method("[",bma)
S3method("[",topmod)
S3method(c,bma)
S3method(coef,bma)
S3method(density,bma)
S3method(density,zlm)
S3method(deviance,bma)
S3method(deviance,zlm)
S3method(image,bma)
S3method(logLik,zlm)
S3method(model.frame,bma)
S3method(plot,bma)
S3method(plot,pred.density)
S3method(predict,bma)
S3method(predict,zlm)
S3method(print,bma)
S3method(print,pred.density)
S3method(print,topmod)
S3method(quantile,coef.density)
S3method(quantile,density)
S3method(quantile,pred.density)
S3method(summary,bma)
S3method(summary,zlm)
S3method(variable.names,bma)
S3method(variable.names,zlm)
S3method(vcov,zlm)
export(as.zlm)
export(beta.draws.bma)
export(bin2hex)
export(bms)
export(estimates.bma)
export(f21hyper)
export(fullmodel.ssq)
export(gdensity)
export(hex2bin)
export(info.bma)
export(is.bma)
export(is.topmod)
export(lps.bma)
export(plotComp)
export(plotConv)
export(plotModelsize)
export(pmp.bma)
export(pmpmodel)
export(post.pr2)
export(post.var)
export(pred.density)
export(topmod)
export(topmodels.bma)
export(zlm)
importFrom(stats,density)

###########################################
# This version: adjusted on 2011-05-05    #
###########################################
# it includes all the auxiliary functions that should only be called INSIDE the bms function



  #START: SUBFUNCTIONS ################



.ols.terms2<-function(positions,yty,k=NULL,N=N,K=K,XtX.big=XtX.big,Xty.big=Xty.big,...){
# function calculates ols terms uhat'uhat ("ymy"), beta, (X'X)^-1,...
# it's child function child.ymy and mutate are most used


   syminv <- function(symmat, ndim=ncol(symmat)) {
     #this does the same as chol2inv(chol.default(x)), but is stripped-down for speed purposes
     # Caution: symmat must always have length(symmat)>0!!!
     if (!is.matrix(symmat)) {symmat=as.matrix(symmat)}
     return( chol2inv(chol(symmat), size=ndim) )
   }

      if(is.null(k)) k=length(positions)
      XtXinv.return=numeric(0)
      # if nullmodel
      if(sum(k)==0){
        Xty=numeric(0);XtXinv=matrix(0,0,0);bhat=numeric(0);ymy=yty;positions=0;
      } else {
        XtX<-XtX.big[positions,positions,drop=FALSE]
        Xty<-Xty.big[positions]
        #do cholesky split: A=XtX=LL'
        #get lower triangular matrix from cholesky split
        XtXinv<-syminv(XtX,ndim=k)
        bhat<-crossprod(XtXinv,Xty); ymy<-yty-crossprod(Xty,bhat)[[1]]
      }

      return(list(
      full.results = function() {

        return(list(ymy=ymy, bhat=bhat, diag.inverse=XtXinv[1:k+0:(k-1)*k]))
        },

      child.ymy = function(addix=0,dropix=0,...) {
          if (!any(as.logical(c(addix,dropix)))) {return(ymy)}
          if (all(as.logical(c(addix,dropix)))) {  #swap
              jhere={1:k}[positions==dropix]; poshere=positions[-jhere];Xj=XtXinv[,jhere];Xtxi=XtX.big[poshere,addix]
              bxlessj=crossprod(XtXinv,XtX.big[positions,addix])-Xj*XtX.big[addix,dropix]; bhatx=bxlessj[-jhere]-Xj[-jhere]*bxlessj[jhere]/Xj[jhere]
              child.ymy = ymy+bhat[jhere]^2/Xj[jhere]-{Xty.big[addix]-crossprod(Xty.big[poshere],bhatx)[[1]]}^2/{XtX.big[addix,addix]-crossprod(bhatx,Xtxi)[[1]]}
              return(child.ymy)
          } else {
            if (addix==0) { #drop
              jhere={1:k}[positions==dropix]
              child.ymy=ymy+bhat[jhere]^2/XtXinv[jhere,jhere]
              return(child.ymy)
            } else { #add
              Xtxi=XtX.big[positions,addix]
              bhatx=crossprod(XtXinv,Xtxi)[,1]
              child.ymy = ymy - {Xty.big[addix]-crossprod(bhatx,Xty)[[1]]}^2 /{XtX.big[addix,addix]-crossprod(bhatx,Xtxi)[[1]]}
              return(child.ymy)
            }

          }

      },
      mutate= function(addix=0,dropix=0,newpos=numeric(0),newk=0,...) {
            #return(ols.terms2(newpos,yty,length(k),N,K=K,XtX.big=XtX.big,Xty.big=Xty.big,...,return.inverse=F))
         if (newk==0) {
             XtXinv<<-matrix(0,0,0); Xty<<-numeric(0)
         } else {
#         if (newk<7|dropix[[1]]*addix[[1]]!=0|length(c(dropix,addix))>2) { #
         if (newk<7|addix[[1]]!=0|length(c(dropix,addix))>2) { #
             Xty<<-Xty.big[newpos]
             XtXinv<<- syminv(XtX.big[newpos,newpos,drop=FALSE],ndim=newk)
         } else {

           if (dropix[1]>0) {
               jhere=sum(positions<=dropix)
               Xty<<-Xty[-jhere]
               Xj=XtXinv[,jhere];
               XtXinv<<- {XtXinv-tcrossprod(Xj/Xj[jhere],Xj)}[-jhere,-jhere]

           } else {
               jhere=sum(positions<addix)+1
               Xtxx=XtX.big[addix,newpos];Xtx=Xtxx[-jhere];Xty<<-Xty.big[newpos]
               bhatx=crossprod(XtXinv,Xtx)[,1]; bhatxadj=c(bhatx[0:(jhere-1)],-1,bhatx[jhere:k]); if (jhere==newk) bhatxadj=bhatxadj[-(jhere+1:2)]
               newinv=tcrossprod(bhatxadj,bhatxadj/(Xtxx[jhere]-crossprod(Xtx,bhatx)[[1]]))
               newinv[-jhere,-jhere]=newinv[-jhere,-jhere]+XtXinv
               XtXinv<<- newinv;
           }
         }}
#         if (any(diag(XtXinv)<0)) browser()
         positions<<-newpos; k<<-newk; bhat<<-crossprod(XtXinv,Xty)[,1]; ymy<<-yty-crossprod(Xty,bhat)[[1]]
         return(list(ymy=ymy, bhat=bhat, diag.inverse=XtXinv[1:k+0:{k-1}*k]))
      },
      return.inverse= function() XtXinv,
      ymy=ymy, bhat=bhat, diag.inverse=XtXinv[1:k+0:{k-1}*k]
      ))
 }





  




## MODEL PRIOR DEFINITIONS #####################
#the following functions define an mprior.info object
# out of the provided parameters
# mpparam: corresponds to argument 'mprior.size' in function bms   
# K: the number of covariates
# arguments passed from bms to .mprior.*.init:
# mpmode=mprior,mpparam=mprior.size,K=K,X.data=X.data,fixed.pos=fixed.pos
# Example:
# bms(attitude,mprior=.mprior.uniform.init)
# bms(attitude,mprior=.mprior.pip.init(K=ncol(attitude)-1,mpparam=seq(.1,.6,.1)))

.mprior.uniform.init = function(K,...) {
  #defines uniform model prior
    return(list(
      mp.mode="uniform", #name of the model prior
      mp.msize=K/2, #prior expected model size
      pmp=function(...) return(0), #the actual model prior function
      mp.Kdist=exp(lchoose(K,0:K)-K*log(2)) #a K+1 vector of prior model probailtiy for each model size - optional, but necessary for some processing functions
    ))
}


.mprior.fixedt.init = function(K, mpparam, ...) {
  #Defines binomial model prior ("fixed theta")
  #user checks:
  if (is.na(mpparam[1])) mpparam<-K/2
  if((mpparam[[1]]>=K)&(length(mpparam)==1)){
    warning("Submitted prior model size is >= than the nr. of   regressors\n, used K/2 instead\n\n")
    mpparam<-K/2
  }

  # actual model prior
  m=mpparam[[1]]
  return(list(
    mp.mode="fixed",
    mp.msize=m,
    pmp=function(ki,...) {
         post.odds1=ki*log(m/K)+{K-ki}*log(1-m/K)
         return(post.odds1)
    },
    mp.Kdist=stats::dbinom(x=0:K,size=K,prob=m/K,log=FALSE)
  ))
}



.mprior.randomt.init = function(K, mpparam, ...) {
  #model prior definition for beta-binmomial (random theta) model prior
  #user checks:
  if (is.na(mpparam[1])) mpparam<-K/2
  if((mpparam[[1]]>=K)&(length(mpparam)==1)){
    warning("Submitted prior model size is >= than the nr. of   regressors\n, used K/2 instead\n\n")
    mpparam<-K/2
  }
  
  # necessary initialization variables for speeding up sampling
  m=mpparam[[1]]
  vecofpriors=lgamma(1+0:K)+lgamma({K-m}/m+K-0:K)
  beta.bin=function(a=1,b=(K-m)/m,K=K,w=0:K){lgamma(a+b)-{lgamma(a)+lgamma(b)+lgamma(a+b+K)}+log(choose(K,w))+lgamma(a+w)+lgamma(b+K-w)}
  
  #actual mpinfo object
  return(list(
    mp.mode="random",
    mp.msize=m,
    pmp=function(ki,...) {
       return(vecofpriors[[ki+1]])
    },     
    mp.Kdist=exp(beta.bin(a=1,b={K-m}/m,K=K,w=0:K))
  ))

}

.getpolycoefs <- function(polyroots) {
   # helper function for mprior.pip.init
   #given the roots of a homogenous polynomial, 
   # this function finds the polynomial coefficients (ordered from highest exponent term to constant)
  if (length(polyroots)==1) return(c(1,polyroots))
  restterms=.getpolycoefs(polyroots[-1])
  c(restterms,0)+c(0,polyroots[1]*restterms)
}


.mprior.pip.init = function(K,mpparam,...) {
  #defines model prior for custom prior inclusion probabilities
  if (any(is.na(mpparam))) mpparam=rep(.5,K);
  if (!is.numeric(mpparam)) stop("For prior inclusion probabilites, you need to provide a K vector with elements between 0 and 1 for argument 'mprior.size'.")
  mpparam=as.vector(mpparam)
  if (!((length(mpparam)==K)&all(mpparam>0)&all(mpparam<=1))) stop("For prior inclusion probabilites, you need to provide a K vector with elements between 0 and 1 for argument 'mprior.size'.")
  if (any(mpparam==1L)) warning("Prior Inclsuion Prob. = 1 are impractical. Try using the argument fixed.reg")
  inclfacts=log(mpparam/(1-mpparam))

  return(list(
    mp.mode="pip",
    mp.msize=sum(mpparam),
    pmp=function(mdraw,...) {
      return(sum(inclfacts[as.logical(mdraw)]))
    },
    mp.Kdist=.getpolycoefs(mpparam/{1-mpparam})*prod(1-mpparam)
    ))
}

.mprior.customk.init = function(K, mpparam, ...) {
  # defines model prior for custom size-based model priors
  if (any(is.na(mpparam))) mpparam=rep(.5,K);
  if (!is.numeric(mpparam)) stop("For custom model size priors, you need to provide a K+1 vector with positive elements for argument 'mprior.size'.")
  mpparam=as.vector(mpparam)
  if (!((length(mpparam)==(K+1))&all(mpparam>0))) {     stop("For custom model size priors, you need to provide a K+1 vector with positive elements for argument 'mprior.size'.")     }
  mpkvec=log(mpparam)

  return(list(
    mp.mode="custom",
    mp.msize=sum(choose(K,0:K)*mpparam*{0:K})/sum(choose(K,0:K)*mpparam),      
    pmp=function(ki,...) {
      return(mpkvec[[ki+1]])
    },
    mp.Kdist=choose(K,0:K)*mpparam/sum(choose(K,0:K)*mpparam)
  ))
}





.fixedset.mprior = function(mprior.function,fullK,fixed.pos=numeric(0),K=NA,...) {
  #CAUTION: NO adjustment for mpparam yet!
  if (length(fixed.pos)==0) return(mprior.function(K=fullK,...))
  fixed.pos={1:fullK}[fixed.pos] #to convert from possible binary vector
  flexpos={1:fullK}[-fixed.pos]; flexk=length(flexpos)
  mprior=mprior.function(K=flexk,...)
  fixk=length(fixed.pos)
  mpl=list(
    mp.mode=mprior$mp.mode,
    mp.msize=mprior$mp.msize+fixk,
    pmp=function(ki,mdraw,...) {
      return(mprior$pmp(ki=ki-fixk,mdraw=mdraw[flexpos]))
    },
    mp.Kdist = c(numeric(fixk),mprior$mp.Kdist)
   )
  return(mpl)
}


.choose.mprior <- function(mpmode,mpparam,K,...,fixed.pos=numeric(0)) {
   #this function fetches an mprior.info object
  # mpmode: corresponds to argument 'mprior' in function bms
  # mpmode determines one of 5 generic model prior classes:
  # uniform, fixed equal inclusion probs, random 'theta' equal inclusion probs, 
  # custom model size priors and custom inclusion probs
  # but can also be a custom function or list such as the indivdual model prior definition functions
  # mpparam: corresponds to argument 'mprior.size' in function bms   
  # K: the number of covariates
  # fixed.pos: the positions of the variables in X the need to be kept in any model
   
   origargs=list(mpmode=mpmode,mpparam=mpparam) #read input arguments
   fixed.pos={1:K}[fixed.pos]; fixed.exist=as.logical(length(fixed.pos)); fixk=length(fixed.pos)
 
 
   
  if (!( is.character(mpmode)|| is.function(mpmode) || is.list(mpmode))) stop("'mprior' parameter must be character! (or function/list)")
  
  #switch list: choose model prior - Caution: mpparam is adjusted for fixed.set variables 
   if (is.function(mpmode) || is.list(mpmode)) { # custom model prior provided as function
     mpinfo=mpmode; 
   } else if (any(grep("fix",mpmode,ignore.case=TRUE))) { #fixed theta prior
     mpinfo=.mprior.fixedt.init     
     if (is.numeric(mpparam)) mpparam=mpparam[[1]]-fixk
  } else if (any(grep("unif",mpmode,ignore.case=TRUE))) { #uniform
     mpinfo=.mprior.uniform.init
  } else if (any(grep("custom",mpmode,ignore.case=TRUE))) { #custom model size prior 
     mpinfo=.mprior.customk.init
     if (fixed.exist && is.numeric(mpparam)) if (length(mpparam)==K+1) mpparam=mpparam[(fixk+1):length(mpparam)]
  } else if (any(grep("pip",mpmode,ignore.case=TRUE))) { #prior inclusion probabilities
     mpinfo=.mprior.pip.init  
     if (fixed.exist && is.numeric(mpparam)) if (length(mpparam)==K) mpparam=mpparam[-fixed.pos]
  } else { #random theta prior
     mpinfo=.mprior.randomt.init 
     if (is.numeric(mpparam)) mpparam=mpparam[[1]]-fixk
  } 
  
  if (is.function(mpinfo)) mpinfo=.fixedset.mprior(mpinfo,fullK=K,fixed.pos=fixed.pos,K=NA,mpparam=mpparam, mpmode=mpmode, ...)
  if (!all( c("mp.mode","mp.msize", "pmp") %in% names(mpinfo))) stop("The provided custom-built model prior is deficient.")
  if (!("origargs" %in% names(mpinfo))) mpinfo$origargs=origargs; 
  if (length(fixed.pos)>0) mpinfo$fixed.pos=fixed.pos
  class(mpinfo) <- c("mprior", class(mpinfo))
  return(mpinfo) 
}


## end: model priors




.starter=function(K,start.value,y,N=N,XtX.big=XtX.big,Xty.big=Xty.big,X=X,fixed.pos=numeric(0)){
#"starter" draws random start vector of size start.value, and keeps those regressors with a t-stat>0.2
    # in case user submitted a single number as start.value
    # we randomly draw a model with start.value regressors
    # finally only regressors with t-stats>0.2 are kept and build   the starting model
    
    #some input checks
    if (is.na(start.value[1])) {start.value=min((N-3),K)}
    if (any(start.value<0)|!(is.numeric(start.value)|is.logical(start.value))) { 
      start.value=min((N-3),K)
      warning("Argument 'start.value' did not conform to required format. start.value has been changed to default - min(N-3,K)")
    }
    if (length(start.value)==0) {start.value=numeric(K)}
    if (length(start.value)==1) {if (start.value==0) {start.value=numeric(K)}}      
    if (length(start.value)>1 && any(start.value>1)) {sv=numeric(K); sv[start.value]=1; start.value=sv; rm(sv) } 
    
    if(length(start.value)==1){
        if (start.value>min((N-3),K)){
          cat("Submitted Start value is too large, used\n min(N-3,K) as starting model size instead\n\n")
          start.value=min((N-3),K)
        }
        # draw randomly
        sorter=stats::runif(K)
        start.position=order(sorter,seq(1:K))[1:start.value]


        # calculate bhats and t-stats
        XtX.start<-XtX.big[start.position,start.position]
        XtXinv.start<-chol2inv(chol(XtX.start))
        bhat=XtXinv.start%*%Xty.big[start.position]
        e=y-X[,start.position]%*%bhat
        sse=crossprod(e)
        s2=as.numeric(sse/(N-length(start.position)))
        bcov=s2*XtXinv.start
        bt=bhat/sqrt(diag(bcov))
        # choose only regressors with t-stat>0.2
        molddraw=rep(0,K)
        goodguy=as.numeric(abs(bt)>.2)
        molddraw[start.position]=goodguy
        start.position = (1:K)[as.logical(molddraw)]
    outstart=list(molddraw=molddraw,bhat=bhat,start.position=start.position)
  }
    # else we start with the user specified starting model
    # in this case we do not test whether the t-stat's are greater than 0.2
    # but start with exactly the selected model no matter whether it is a
    # good starting model or not

    # in case you want to start with the null model
    if(length(start.value)>1 && sum(start.value)==0){
      outstart=list(molddraw=rep(0,K),bhat=rep(0,K),start.position=integer(0))
    }

    if(length(start.value)>1 && sum(start.value)>0){
          if(length(start.value)!=K){
            # in case user specified a too big model
            stop("Starting Model contains unequal to K regressors,please respecify")
          }

        start.position=which(as.logical(start.value))
        XtX.start<-XtX.big[start.position,start.position]
        XtXinv.start<-chol2inv(chol(XtX.start))
        bhat=XtXinv.start%*%Xty.big[start.position]
        molddraw=rep(0,K); molddraw[start.position]=1

      outstart=list(molddraw=molddraw,bhat=bhat,start.position=start.position)
    }
    fixed.pos=(1:K)[fixed.pos]
    if (length(fixed.pos)>0) { 
      outstart$molddraw[fixed.pos]=1 
      outstart$start.position = (1:K)[as.logical(outstart$molddraw)]
    }
    
    return(outstart)
}


###########################################################################################################################
#Sample Functions
############################################################################################################################
#First, we have implemented the original FLS Sample Function; here a variable is drawn from the set of
# K regressors, then
#conditional on whether it is included in the current model it can be discarded or added.


 .fls.samp=function(molddraw=molddraw,K=K,...,maxk=Inf,oldk=0){
 # the original FLS Sample Function; here a variable is drawn from the set of K regressors, then
 #conditional on whether it is included in the current model it can be discarded or added.


    indch<-ceiling(stats::runif(1,0,K)) #rounding to the smallest integer part by floor, uniform distr. [0,1]  
    bdropit<-as.logical(molddraw[[indch]]); 
    if (oldk==maxk) if (!bdropit) {indch=(1:K)[molddraw==1][[ceiling(stats::runif(1,0,sum(molddraw)))]]; bdropit=molddraw[[indch]]}
     if (bdropit){  #dropping
         addvar<-0;dropvar<-indch;
         molddraw[[indch]]<-0
     }
     else {                    #adding
          addvar<-indch;dropvar<-0;
          molddraw[[indch]]<-1
     }
#    addvar<-(!bdropit)*indch; dropvar<-bdropit*indch;
#    molddraw[[indch]]<- !bdropit
  
    positionnew <- {1:K}[molddraw==1]
  return(list(mnewdraw=molddraw,positionnew=positionnew,addi=addvar,dropi=dropvar))
 }
#############################################################################################################################
#Second, a reversible jump algorithm, where we
#have added a move step. See below
##############################################################################################################################
 #Reversible Jump Algorithm
 .rev.jump=function(molddraw=molddraw,K=K,...,maxk=Inf,oldk=0){
  #Reversible Jump Algorithm: with equal prob decides between swap step or add/drop step (.fls.samp)
    rev.idx=ceiling(stats::runif(1,0,2))  #rev.idx is a flag that indicates  the three possible steps of
                                   #the reversible jump algorithm, 1=birth or death and 2=move.
   # Perform Death, Birth or Move Step
   # if rev.idx is 1, do the same as in fls sampler (i.e. increase or  decrease depending
   # on variables already included in the model
   if(rev.idx==1){
     birth.death=.fls.samp(molddraw=molddraw,K=K,maxk=maxk,oldk=oldk)
     mnewdraw=birth.death[["mnewdraw"]]
     positionnew=birth.death[["positionnew"]]
     addvar=birth.death[["addi"]]; dropvar=birth.death[["dropi"]]
   }
   #move step
   if(rev.idx==2){
      var.in=(1:K)[as.logical(molddraw)]  #positions of the variables that are currently in the model
      var.out=(1:K)[!as.logical(molddraw)] #positions of the variables that are currently out of the model
      var.in.rand=ceiling(length(var.in)*stats::runif(1,0,1)); 
      addvar=var.out[ceiling(length(var.out)*stats::runif(1,0,1))]
      dropvar=var.in[var.in.rand]
      mnewdraw=molddraw; mnewdraw[addvar]=1; mnewdraw[dropvar]=0;
      positionnew=(1:K)[as.logical(mnewdraw)]
      dropvar=max(dropvar,0); addvar=max(addvar,0) # in case one of the indexes is integer(0) (borderline case)
   }
   return(list(mnewdraw=mnewdraw,positionnew=positionnew,addi=addvar,dropi=dropvar))
 }
#############################################################################################################################
#Third, there is a 'contiguity enumeration' sampler, that enumerates all possible combinations of models. 
#In particular it moves between continguous models (without repeating itself). i.e. there is always only an "add" or "drop" opration in the move to the next model.
##############################################################################################################################

.iterenum <- function(molddraw=numeric(0),K=length(molddraw),...) { 
 #Contiguity Enumeration Sampler
 # takes a binary vector (like molddraw) and iterates it such that it performs 'contiguity enumeration', 
 # i.e. enumerating all possible combinations by always changing only one entry in molddraw
         even.lead1={1:K}[!{cumsum(molddraw)%%2}]; i=even.lead1[length(even.lead1)]
        # i ist the last entry where the number of 1's up to entry i is even:
        molddraw[i]=!molddraw[i]; # then change entry i (either T->F or F->T)
        addi=molddraw[i]*i;dropi={!molddraw[i]}*i; #indch=i    
          return(list(mnewdraw=molddraw,positionnew={1:K}[as.logical(molddraw)],addi=addi,dropi=dropi))
} 

##### Enumeration in case K>N-2 ######################

.iterenum.bone = function(molddraw=numeric(0),maxk=Inf) {
# an auxiliary function for iterenum.KgtN
        even.lead1=((1:length(molddraw))[!(cumsum(molddraw)%%2)]); i=even.lead1[length(even.lead1)]
        # i ist the last entry where the number of 1's up to entry i is even:
        molddraw[i]=!molddraw[i]; # then change entry i (either T->F or F->T)
        if (sum(molddraw)>maxk) return(.iterenum.bone(molddraw,maxk)) else return(molddraw)
}

.iterenum.KgtN <- function(molddraw=numeric(0),maxk=Inf,oldk=0,...) { 
#iterenum.KgtN is slightly slower than iterenum and itererates only through models with k<maxk; else similar to iterenum
         mnewdraw=.iterenum.bone(molddraw=molddraw,maxk)
         addi=(1:length(mnewdraw))[molddraw<mnewdraw]; if (length(addi)==0) addi=0
         dropi=(1:length(mnewdraw))[molddraw>mnewdraw]; if (length(dropi)==0) dropi=0
           return(list(mnewdraw=mnewdraw,positionnew=(1:length(mnewdraw))[as.logical(mnewdraw)],addi=addi,dropi=dropi))
} 

#####  ############
#get the enumeration sampler vector for a certain iteration index
.enum_fromindex <- function(lindex){
   #lindex: an integer index; this function returns a logical vector that corresponds to the enumeration sampler draw for this index (without leading zeros)
   lindex=lindex[[1]]
   if (lindex==0) return(FALSE)
   log2=ceiling(log(lindex+1,2))
   return( as.logical((lindex+2^((log2-1):0))%/%(2^(log2:1))%% 2))
}


.enum_startend = function(iter=NA,start.value=0,K=1,maxk=K, fixed.pos=numeric(0)) {
  #does user checks and returns the starting model for enumeration and the number of iterations
  # in its most basic version, it returns list(numeric(K), 2^K-1)
  # however, it also adjusts for N-3<K and fixed variables
  # moreover, it converts to a subspace of the enumeration space if iter and start.value say so
  
  fixed.pos={1:K}[fixed.pos]
  effk=K-length(fixed.pos)
  flexpos={1:K}; if (length(fixed.pos)>0) flexpos={1:K}[-fixed.pos]
   
  start.value2=0
   
   
  if (length(start.value)==1) { 
    start.value2=suppressWarnings(as.integer(start.value)) 
    if (any(is.na(start.value2))|start.value2[[1]]<0|start.value2[[1]]>=(2^effk-1)) {
       start.value=0;start.value2=0
    } 
  } else { 
    start.value=0 
  }  
  if (length(start.value)==1) { 
     #if startvalue is an integer index satysfying above conditions then convert it into a 'draw' to start from
     start.value_cut=.enum_fromindex(start.value2)
     start.value=rep(1,K)
     start.value[flexpos]=c(numeric(effk-length(start.value_cut)),start.value_cut)   
    
  }
 
  #   do all the other enumeration stuff
  if (K>maxk) {lastindex=2^effk-1-sum(choose(effk,(maxk+1):effk))} else lastindex=2^effk-1
  if (is.na(iter)) {iter=lastindex-start.value2} #default iter is the rest to complete the enumeration from start.value
  iter=min(iter,2^effk-1-start.value2); # don't do too much!
  
  return(list(start.value=start.value, iter=iter))

} 

.fixedset.sampler = function(sampler.function, fullK, fixed.pos=numeric(0),...) {
  #converts any model sampler function into one that retains fixed variables defined in fixed.pos
  if (length(fixed.pos)==0) return(sampler.function)
  
  #initialize
  fixed.pos={1:fullK}[fixed.pos] #to convert from possible binary vector
  flexpos={1:fullK}[-fixed.pos]; flexk=length(flexpos)
  outdraw=rep(1,fullK)
  
  #now create a 'stoepsler' function that plugs in the draw of flexible varaibles into the entire set
  outfun = function(molddraw=molddraw,K=flexk,...) {
    flexdraw=sampler.function(molddraw=molddraw[flexpos],K=flexk,...)
    outdraw[flexpos]=flexdraw[["mnewdraw"]]
    addi=flexdraw[["addi"]]; dropi=flexdraw[["dropi"]];
    #indch= flexpos[flexdraw[["indch"]]]; #is this really necessary?
    if (is.numeric(addi)||is.numeric(dropi)) {
    if (addi>0) addi= flexpos[addi] else addi=0
    if (dropi>0) dropi= flexpos[dropi] else dropi=0
    }
    return(list(mnewdraw=outdraw,positionnew={1:fullK}[as.logical(outdraw)],addi=addi,dropi=dropi))
  }
  
  return(outfun)
  
}


###########################################################################################################################
#SAMPLERS WITH INTERACTION TERMS
############################################################################################################################
#initialization
.constr.intmat = function(X,K) {
    #this function identifies the columns of X named with multiple terms (sep'd by "?") as interactions
    #and constructs a matrix whose rows identify the base terms corresponding to each interaction term
   intix=grep("#",colnames(X),fixed=TRUE) # indices of columsn with interaction terms
   mPlus=diag(K)
   colnames(mPlus)<-colnames(X)
   for (jj in 1:length(intix)) {
     cix= intix[jj]
     mPlus[cix,unlist(strsplit(colnames(mPlus)[cix],"#",fixed=TRUE))]=1 # put a one in row (interaction term) and columns (all base terms)
   }
   return(mPlus)      
}


#First, we have implemented the original FLS Sample Function; here a variable is drawn from the set of
# K regressors, then
#conditional on whether it is included in the current model it can be discarded or added.


 .fls.samp.int=function(molddraw=molddraw,K=K,mPlus=mPlus,maxk=Inf,oldk=0){
      #interactions sampler for .fls.samp
      indch=ceiling(stats::runif(1,0,1)*K) #rounding to the smallest integer  part by floor, uniform distr. [0,1]
       # have to make sure that we delete all the regressors                                            
      
      if (molddraw[indch]==1){  #dropping
          mnewdraw=as.numeric(molddraw>mPlus[,indch])
          dropvar = (1:K)[xor(molddraw,mnewdraw)]; addvar=0;
      }
       else{                    #adding
          mnewdraw=as.numeric(molddraw|mPlus[indch,])
          addvar = (1:K)[xor(molddraw,mnewdraw)]; dropvar=0;
      }
      positionnew=which(mnewdraw==1)
      if (length(positionnew)>maxk) {
        return(.fls.samp.int(molddraw=molddraw,K=K,mPlus=mPlus,maxk,oldk))
      } else {
        return(list(mnewdraw=mnewdraw,positionnew=positionnew,addi=addvar,dropi=dropvar))
      }
 }
#############################################################################################################################
#Second, we have implemented a reversible jump algorithm, where we
#have added a move step. See below
##############################################################################################################################
 #Reversible Jump Algorithm
 .rev.jump.int=function(molddraw=molddraw,K=K,mPlus=mPlus,maxk=Inf,oldk=0){
       #interactions sampler for .rev.jump
    rev.idx=floor(stats::runif(1,0,1)*2)  #rev.idx is a flag that indicates  the three possible steps of
                                   #the reversible jump algorithm, 1=birth or death and 2=move.
   # Perform Death, Birth or Move Step
   # if rev.idx is 1, do the same as in fls sampler (i.e. increase or  decrease depending
   # on variables already included in the model
   if((rev.idx)|oldk==0){
     birth.death=.fls.samp.int(molddraw=molddraw,K=K,mPlus=mPlus,maxk,oldk)
     mnewdraw=birth.death$mnewdraw
     positionnew=birth.death$positionnew
     addvar=birth.death$addi; dropvar=birth.death$dropi
   } else {
      var.in=(1:K)[as.logical(molddraw)]  #positions of the variables that are currently in the model
      var.out=(1:K)[!as.logical(molddraw)] #positions of the variables that are currently out of the model
      mnewdraw=(molddraw>mPlus[,var.in[ceiling(length(var.in)*stats::runif(1,0,1))]])
      mnewdraw=mnewdraw|mPlus[var.out[ceiling(length(var.out)*stats::runif(1,0,1))],]
      positionnew=(1:K)[mnewdraw]
      addvar = (1:K)[molddraw<mnewdraw]; dropvar = (1:K)[molddraw>mnewdraw];
      if (length(dropvar)==0) dropvar=0;  if (length(addvar)==0) addvar=0
      
   }
#return(list(mnewdraw=as.numeric(mnewdraw),positionnew=positionnew,addi=addvar,dropi=dropvar,indch=rev.idx))   
   if (length(positionnew)>maxk) {
        return(.rev.jump.int(molddraw=molddraw,K=K,mPlus=mPlus,maxk,oldk))
   } else {
        return(list(mnewdraw=as.numeric(mnewdraw),positionnew=positionnew,addi=addvar,dropi=dropvar))
   }


   
 }





.post.calc <- function(gprior.info,add.otherstats,k.vec,null.count,X.data,topmods,b1mo,b2mo,iter,burn,inccount,models.visited,K,N,msize,timed,cumsumweights=NA,mcmc="bd",possign=NA) {
    #customized function for posterior results from bms
    postad.k.vec <- function(k.vec, null.count) c(null.count,k.vec) #concatenates the vector of freqencies for 1:K model sizes with freq for null model
    
    postad.gprior.info <- function(gprior.info,add.otherstats=numeric(0),cumsumweights=1) {
      # adjusts the gprior.info object resulting from choose.gprior():
      # add.otherstats: vector, cumsumweights: scalar by which otherstats are to be divided (typically nb of draws 'iter')
      if (gprior.info$return.g.stats) {
        if (length(add.otherstats)>0) {
          gprior.info$shrinkage.moments=add.otherstats/cumsumweights
        } else {
          gprior.info$shrinkage.moments=1/(1+1/gprior.info$g)
        }
      }
      return(gprior.info)
    }
    
    
    
    postad.reg.names <- function(X.data) {
      # extracts the column names of covariates or constructs ones: X.data is data.frame
      xdcn=colnames(X.data)
      if(is.null(xdcn)) {xdcn<- rep("",NCOL(X.data))}
      if(anyNA(xdcn)) {xdcn[is.na(xdcn)]<- rep("",NCOL(X.data))[is.na(xdcn)]}
      if(any(trimws(xdcn)=='')) {xdcn[trimws(xdcn)=='']<-paste0("Vbl",0:K)[trimws(xdcn)==''] }
      reg.names=xdcn[-1]
      return(reg.names)
    }


      gprior.info=postad.gprior.info(gprior.info,add.otherstats,cumsumweights)
      k.vec = postad.k.vec(k.vec,null.count)
      #if (is.na(cumsumweights)) cumsumweights=iter
      cons=.post.constant(X.data,b1mo/cumsumweights)
      pmp.10=pmp.bma(topmods,oldstyle=TRUE)
      
      if (nrow(pmp.10)==1|suppressWarnings(length(grep("error",class(try(cor(pmp.10[,1],pmp.10[,2]),silent=TRUE)))))) {
         corr.pmp=NA
      } else {
        if (var(pmp.10[,2])==0) corr.pmp=NA else corr.pmp=cor(pmp.10[,1],pmp.10[,2])
      }
      
      if (is.na(possign[[1]])) possign=numeric(K)
      
      
      info.object=list(iter=iter,burn=burn,inccount=inccount,models.visited=models.visited,b1mo=b1mo,b2mo=b2mo,
        add.otherstats=add.otherstats,cumsumweights=cumsumweights,K=K,N=N,corr.pmp=corr.pmp,msize=msize,timed=timed,k.vec=k.vec,cons=cons,pos.sign=possign)

      reg.names=postad.reg.names(X.data)
      return(list(info=info.object,k.vec=k.vec,cons=cons,gprior.info=gprior.info,pmp.10=pmp.10,reg.names=reg.names))
}
  




 
         





 ################ g prior functions #########################################

# a gprior object is a list, or a function that returns that list
# such a gprior object can be passed to bms (cf example below)
#
# EXAMPLE: a simple constant g prior list is returned by the following function:
# 
# simpleg <- function(g=NA,N=100,K=2,yty=1,...,myg=NULL) {
#  # bms/zlm calls this function with the following arguments: g, N (nb obs), K (nb of total variables), yty (TSS), null.lik (usually NA) and return.g.stats (same as bms argument g.stats), y and X
#  # additional arguments may be used by providing bms with the output of this function, rather than the function itself
#  if (is.null(myg)) myg=10 #stuff to be valid for all evaluations of the sub-functions below
#  
#  gprior.info=list( #general information for ppost-processing
#     gtype = "wappler-g", # a name for the g prior sub-type
#     is.constant = TRUE, # whether g is a predifined scalar, or changes with models (important for post-processing)
#     g=myg, #a scalar - only necessary if is.constant==TRUE
#     return.g.stats = FALSE # whether g-statistics should be collected; TRUE possible in principle, but mor bug-prone
#     )
#     
#   #  the actual gprior-computation routines are in a sub-element of the gprior object - note that myg was defined upfront   
#  gprior.info$lprobcalc = list(
#     just.loglik = function(ymy, k, ...) { 
#       #for speed reasons, this function is used when evaluationg the liklihood of a candiade model in bms
#       return(.5*{ (N-1-k)*log(1+myg)-(N-1)*log(myg*ymy + yty) } ) #the (scalar) log-likelihood of the model
#     },
#     lprob.all=function(ymy,k,bhat,diag.inverse,...) {
#       return(list(
#         lprob=.5*{ (N-1-k)*log(1+myg) - (N-1)*log(myg*ymy + yty)}, # log-likelihood
#         b1new = myg/(1+myg)*bhat, # the posterior expected coefficents
#         b2new ={(yty/myg+ymy) * (myg/(1+myg))^2 /(N-3)}*diag.inverse + (myg/(1+myg)*bhat)^2, #second posteriro moment of the coefficients
#         otherstats=numeric(0) #addtional stats to be counted when return.g.stats==TRUE
#         ))
#     }    
#   )
#   
#   return(gprior.info)
# }
# bms(attitude,g=simpleg)    
# zlm(attitude,g=simpleg)
# bms(attitude,g=simpleg(myg=20,N=nrow(attitude),K=ncol(attitude)-1,yty=var(attitude[,1])*(nrow(attitude)-1)))

.gprior.constg.init <- function(g=NA,return.g.stats=TRUE,N=N,K=K,yty=1,null.lik=NA,...) {
  gg=NULL
  if (!(is.character(g)||is.numeric(g))) g="UIP"
  
  if (any(grep("BRIC",g,ignore.case=TRUE))|any(grep("FLS",g,ignore.case=TRUE))) {
    if (N<=(K^2)){gg=(K^2)} else { gg=N }
    gtype="BRIC"
  }

  if (any(grep("RIC",g,ignore.case=TRUE))&& (!any(grep("BRIC",g,ignore.case=TRUE)))) {
    gg=(K^2)
    gtype="RIC"
  }

  if (any(grep("HQ",g,ignore.case=TRUE))|any(grep("Hannan",g,ignore.case=TRUE))) {
    gg=(log(N))^3
    gtype="Hannan-Quinn"
  }

  if(is.numeric(g)){
    gg=g
    gtype="numeric"
  }
  
  if (is.null(gg)) {
    if (!(any(grep("UIP",g,ignore.case=TRUE))|any(grep("BIC",g,ignore.case=TRUE)))) warning("The provided g prior could not be identified. Therefore the default g prior (UIP) has been selected.")
    gg=N
    gtype="UIP"
  }
 
  gprior.info=list(gtype=gtype, is.constant=TRUE, return.g.stats=return.g.stats, shrinkage.moments=gg/(gg+1), g=gg)
  g=gg
  if (!is.numeric(null.lik))  { null.lik={1-N}/2*log(yty) }
  g2=g/{g+1}; l1g=log(1+g) ; g2sq=g2^2; n1=N-1
  
  gprior.info$lprobcalc <-  list(
     #estimates the standard posterior stats under a cfixed g-prior
     #lprob.constg.init is called only once: it calculates constant terms to reduce redundancy
     #each lprob..init function has the two subfunctions:
     #   * just.loglik: this just calculates the log likelihood from given terms - as the sampler mostly needs just that
     #   * lprob.all: this calculates the log lik, as well as b1new (E(beta|Y) the expected value (normal-gamma) of coefficients), and b2new (the Expected value of coefficents squared E(beta^2|Y)= Var(beta|Y)+E(beta|Y)^2)
   
    #function(ymy, k) { gg=N; l1g=log(1+gg); n1=N-1; g2=g0/(1+g0); .5*{-k*l1g - n1*log(1-g2*(1-ymy/yty))} } -n1
    
     just.loglik=function(ymy,k,...) {
         return(.5*{{n1-k}*l1g-n1*log(g*ymy + yty)})
     },
     lprob.all=function(ymy,k,bhat,diag.inverse,...) {
         b1new = g2*bhat
        return(list(lprob=.5*{{n1-k}*l1g-n1*log(g*ymy + yty)},b1new = b1new,b2new ={{yty/g+ymy}*g2sq/{N-3}}*diag.inverse+b1new^2,otherstats=numeric(0)))
     }
     
     )
  class(gprior.info) <- c("gprior",class(gprior.info))
  return(gprior.info)
 }

 
 
.gprior.eblocal.init <- function(g=NA,return.g.stats=TRUE,N=N,K=K,yty=1,null.lik=NA,...) {
  gprior.info=list(gtype="EBL", is.constant=FALSE, return.g.stats=return.g.stats, shrinkage.moments=numeric(1), g=NA)
  
  if (!is.numeric(null.lik))  { null.lik=(1-N)/2*log(yty) }
  ymy.current=-1; k.current=-1; loglik=null.lik; Fstat=numeric(0); g2=0
  if (return.g.stats) { otherstats=numeric(1) } else { otherstats=numeric(0) }
  
  gprior.info$lprobcalc <- list(
    # estimates the local Empirical Bayes g prior as given in Liang et al (2008): "Mixtures of g Priors for Bayesian Variable Selection"; JASA
    #lprob.eblocal.init is called only once: it initializes an object that links just.loglik and lprob.all
    #each lprob..init function has the two subfunctions:
    #   * just.loglik: this just calculates the log likelihood from given terms - as the sampler mostly needs just that
    #   * lprob.all: this calculates the log lik, as well as b1new (E(beta|Y) the expected value (normal-gamma) of coefficients), and b2new (the Expected value of coefficents squared E(beta^2|Y)= Var(beta|Y)+E(beta|Y)^2)
    #           if at initalization return.g.stats=TRUE, then it additionally returns the estimated value of the shrinkage factor g2
  just.loglik=function(ymy,k,...) {
      ymy.current<<-ymy; k.current<<- k; 
      if (k==0) { return(null.lik) }
      Fstat<<-(N-k-1)/k*(yty-ymy)/ymy
      if (Fstat>1) { #g0=1/max(Fstat-1,0)
        g0<-1/(Fstat-1); g2<<-1/(g0+1)  # g0 corresponds to 1/g
      } else {
        g2<<-0; return(null.lik)
      }
      lFstat=log(Fstat); lg02=-lFstat; #Note that: g2=1-1/F, lg02=log(g0*g2)=log(1/Fstat), loglik=.5* (-k*log(Fstat)-(N-1)*log(ymy+(1/Fstat-1)*yty)-(N-1)*(log(F-1)-log(Fstat))        
      loglik<<-.5*{k*lg02-{N-1}*{log(ymy + g0*yty) + {log(Fstat-1)-lFstat }}}
      return(loglik)
  },
  
  lprob.all=function(ymy,k,bhat,diag.inverse,...) {
      if (k==0) { return(list(lprob=null.lik, b1new=numeric(0), b2new=numeric(0),otherstats=otherstats)) }    
      if ((ymy!=ymy.current)|(k!=k.current)) { #if just.loglik was already called just before with the same parameters, no need to calculate the F-stat stuff again
        Fstat<<- {N-k-1}/k*(yty-ymy)/ymy
        if (Fstat>1) { 
          g0=1/{Fstat-1}; g2<<-1/(g0+1)
          lFstat=log(Fstat); lg02=-lFstat; 
          loglik<<-.5*{k*lg02-{N-1}*{log(ymy + g0*yty) + {log(Fstat-1)-lFstat }}}          
        } else { 
          g0=0; g2<<-0;
          loglik<<-null.lik
        }
      }       
      if (return.g.stats) { otherstats=g2 }
      b1new = g2*bhat
      if (g2>0) { b2new ={{(1/g2-1)*yty+ymy}*{g2^2}/{N-3}}*diag.inverse+b1new^2 } else {b2new = numeric(k)}
      return(list(lprob=loglik,b1new = b1new,b2new=b2new,otherstats=otherstats)) 
  }
  
  )
  class(gprior.info) <- c("gprior",class(gprior.info))
  return(gprior.info)
}
    
    
.gprior.hyperg.init <- function(g=NA,return.g.stats=TRUE,N=N,K=K,yty=1,null.lik=NA,...) {
  #user checks
  if (!is.character(g)) g="hyper"
  
  if (any(grep("=",g))) { 
    f21a=suppressWarnings(as.numeric(unlist(strsplit(g,"="))[2]))
    if (!is.numeric(f21a)|is.na(f21a)) {
      f21a.char=suppressWarnings(as.character(unlist(strsplit(g,"="))[2]))
      if (any(grep("bric",f21a.char,ignore.case=TRUE))) {
          f21a=2+2/max(N,K^2)
      } else if (any(grep("uip",f21a.char,ignore.case=TRUE))) {
          f21a=2+2/N
      } else {
          warning("You did not supply a proper 'a' parameter for the hyper g prior (like e.g. the format g='hyperg=3.1' or g='hyper=UIP') - thus set to default value 'hyper=UIP' instead.")
          f21a=2+2/N
      }
    } else {
      if (f21a<=2|f21a>4) {
        f21a=2+2/N
        warning("You provided an 'a' parameter for the hyper g prior that is not element of (2,4]. I chose the default value 'hyper=UIP' instead.")
      }
    }
  } else {
    f21a=2+2/N
  }
   gprior.info=list(gtype="hyper", is.constant=FALSE, return.g.stats=return.g.stats, shrinkage.moments=numeric(2),g=NA, hyper.parameter=f21a)
  
  
  
  #estimates the hyper g prior as given in Liang et al (2008): "Mixtures of g Priors for Bayesian Variable Selection"; JASA
  #initializing an object that links just.loglik and lprob.all
  #   argument f21a corresponds to the hyper-parameter "a" in Liang et al. (2008): any value in ]2,4], default 3 
  #each lprob..init function has the two subfunctions:
  #   * just.loglik: this just calculates the log likelihood from given terms - as the sampler mostly needs just that
  #   * lprob.all: this calculates the log lik, as well as b1new (E(beta|Y) the expected value (normal-gamma) of coefficients), and b2new (the Expected value of coefficents squared E(beta^2|Y)= Var(beta|Y)+E(beta|Y)^2)
  
    
  #initalizing global values
  if (!is.numeric(null.lik))  { null.lik={1-N}/2*log(yty) }
  gmoments=numeric(2)
  N12={N-1}/2; la2=log(f21a-2)
  log.lik = null.lik; ymy.current=-1; k.current=-1; intconstinv=f21a-2; #intconstinv is the inverse of 1/(a-2) times the integration constant, i.e. (k+a-2)/ 2F1( (N-1)/2,1, (k+a)/2, R^2 )
  
  #initalizing the 2F1 hypergeometric function object
  f21o=.f21_4hyperg(N,K,f21a)
  
  gprior.info$lprobcalc <- list(
  
  just.loglik=function(ymy,k,...) {
      if (k==0) {return(null.lik)}
      ymy.current <<- ymy; k.current <<- k
      intconstinv <<- {k+f21a-2}/f21o[["calcit"]](1-ymy/yty,k)
      if (intconstinv<0) {intconstinv <<- k+f21a-2} #this may happen because of numerical inaccuracies: e.g. ymy marginally greater than yty
      log.lik <<- null.lik + la2 - log(intconstinv)
      return(log.lik)
  },
  
  lprob.all=function(ymy,k,bhat,diag.inverse,...) {
      if (k==0) {return(list(lprob=null.lik,b1new=numeric(0), b2new=numeric(0),otherstats=c(2/f21a,8/f21a/(f21a+2))))}
      N3=N-3; ka2=k+f21a-2; R2=1-ymy/yty; #collect terms
      
      if ((ymy!=ymy.current)|(k!=k.current)) { #if just.loglik was already called just before with the same parameters, no need to calculate the F-stat stuff again
        intconstinv <<- ka2/f21o[["calcit"]](R2,k)
        log.lik <<- null.lik + la2 - log(intconstinv)
      }
      g2hyper= {intconstinv-ka2+N3*R2}/{R2*{N3-ka2}} # E(g/(1+g)|Y)
      gbetavar = {{1+2/N3*R2/{1-R2}}*intconstinv+{N3-2}*R2-ka2} *N3*{1-R2}/{N3-ka2}/{N3-ka2-2}/R2 * yty/N3*diag.inverse #Cov(beta|Y)
      
      if (return.g.stats) {
          ka=ka2+2;
          Eg22= { {{N3-2}*R2-ka}*intconstinv + {N3*R2-ka2}^2-2*{N3*R2^2-ka2} }/R2^2/{N3-ka2}/{N3-ka}
          gmoments=c(g2hyper,Eg22)
      }
                
      return(list(lprob=log.lik, b1new = g2hyper*bhat ,b2new =gbetavar+g2hyper^2*bhat^2, otherstats=gmoments)) 
  }
  
  )
  
  class(gprior.info) <- c("gprior",class(gprior.info))
  return(gprior.info)
  
}



### used for gprior.hyperg.init #####
.f21_4hyperg=function(N,K,f21a,ltermbounds=c(200,600,1400,3000)) {
  # this function calculates the value of a Gaussian hypergeometric function 2F1((N-1)/2,1,(f21a+k)/2,z)
  # as given in Liang et al (2008): "Mixtures of g Priors for Bayesian Variable Selection"; JASA, formula (18)
  # first initialize the object by calling myobject = .f21_4hyoperg() : this calcs recycable terms to reduce redundancy
  # 
  # then evaluate by myobject$calcit(z,k)

  #Initialization:
  create.lterms=function(cc) lapply(as.list(ltermbounds),function(x) (a+0:x)/(cc+0:x)) # a subfunction
  a=(N-1)/2; cv=(f21a+0:K)/2 #create the a and c terms for the (K+1) different model sizes
  lterms=(lapply(cv,create.lterms)) #create the Pochhammer terms in vectors of different lengths, lterms is a list: each element has a list of which each element contains Pochhammer terms up to the index specified in ltermbounds (see function argument)
  ltermbounds=c(0,ltermbounds[-length(ltermbounds)]) #this is just used for deciding which vector length to take
  
 return(list(
  calcit=function(z,k) {
    # this function actually calculates 2F1((N-1)/2,1,(f21a+k)/2,z) in its power series formulation
    nbterms=sum(ceiling(abs({a-cv[k+1]}/{1-z})*1.5)>=ltermbounds) # l=(a-c)/(1-z) is the index from which on the summands decline 
                                                                 # this times 1.5 decides which lvector length (lbounds) to take
    return(sum(cumprod(z*lterms[[k+1]][[nbterms]]))+1) #calculate it
  }                   
 ))
}

.f21simple=function(a,c,z) {
# this function calculates the value of the Gaussian hypergeometric function 2F1(a,1,c,z), where 0=<z=<1
# this function is a simple wrapper destined for user application
  f21o=.f21_4hyperg(2*a+1,0,c*2)
  f21o$calcit(z,0)
}






.choose.gprior <- function(g,N,K,return.g.stats=FALSE,yty=N,...) {
  #chooses the g-prior subject to the two parameters given from bms() user input
  #returns a list with gtype, is.constant, g, as well as prior-specific elements
  # use benchmark uip criterion as standard setting
  # g corresponds to argument "g" in function bms, return.g.stats to "g.stats"
  # N is number of obs, K max number of regressors
  #
  # returns a list conforming to the gprior.info in function 'bms'

  # first check for user-defined g-priors
  if (is.list(g)) { 
    if (!all(c("gtype","is.constant","return.g.stats", "lprobcalc") %in% names(g))) stop("The provided g-prior list (in argument 'g') does not conform to the standards of a g-prior object.")  
    if (!("g" %in% names(g))) {g$is.constant = FALSE; g$g=NA}
    if (!("shrinkage.moments" %in% names(g))) {g$shrinkage.moments=ifelse((g$is.constant&&is.numeric(g)), g/(1+g), 0)}
    if (!all(sapply(g$lprobcalc,is.function))) stop("The slot 'lprobcalc' in the provided g-prior list (in argument 'g') does not conform to the standards of a g-prior object.")
    return(g)
  }
  
  
  if (is.function(g)) {
    return(g(g=g,return.g.stats=return.g.stats,N=N,K=K,yty=yty,...))
  }


  # now guess a standard g-prior from user's arguments
  
  if(is.numeric(g)){
      return(.gprior.constg.init(g=g,return.g.stats=return.g.stats,N=N,K=K,yty=yty))
  }

  # if user specifies empirical bayes estimation (local)
  if (any(grep("EBL",g,ignore.case=TRUE))) {
     return(.gprior.eblocal.init(g=g,return.g.stats=return.g.stats,N=N,K=K,yty=yty))
  }

  # if user specifies hyper-prior (local)
  if (any(grep("hyper",g,ignore.case=TRUE))) {   
       return(.gprior.hyperg.init(g=g,return.g.stats=return.g.stats,N=N,K=K,yty=yty))
  }

  return(.gprior.constg.init(g=g,return.g.stats=return.g.stats,N=N,K=K,yty=yty))
  
}

# end g-Stuff ##########################################




# FUNCTIONS DESIGNED FOR BEING CALLED IN bms() #######################################
# these functions are only subfunction to be called inside bms()
# [therefore they necessitate the statement environemnt(SUBFUNCTION) <- environment() inside bms() ]
# the are defined outside of bms() for readability and modularity purposes


# compatibility with package version 0.2.5
.lprob.constg.init = function(...) {
 gpo=.gprior.constg.init(...)
 return(gpo$lprobcalc)
}

.lprob.eblocal.init = function(...) {
 gpo=.gprior.eblocal.init(...)
 return(gpo$lprobcalc)
}


.lprob.hyperg.init = function(...) {
 gpo=.gprior.hyperg.init(...)
 return(gpo$lprobcalc)
}

#' @details
#' The key function you need is  \code{\link{bms}}.  
#' @author Martin Feldkircher, Paul Hofmarcher, and Stefan Zeugner
#' @seealso \code{\link{coef.bma}}, \code{\link{plotModelsize}} and
#' \code{\link{density.bma}} for some operations on the resulting 'bma' object, 
#' as well as 
#' \code{\link{predict.bma}} or \code{\link{gdensity}}, or
#' \code{\link{zlm}} for individual Zellner regression models.
#' 
#' Check \url{http://bms.zeugner.eu} for additional help.
#' @references 
#' \url{http://bms.zeugner.eu}: BMS package homepage with help and tutorials
#' 
#' Feldkircher, M. and S. Zeugner (2015): Bayesian Model Averaging Employing 
#' Fixed and Flexible Priors: The BMS Package for R, Journal of Statistical Software 68(4).
#' 
#' Feldkircher, M. and S. Zeugner (2009): Benchmark Priors
#' Revisited: On Adaptive Shrinkage and the Supermodel Effect in Bayesian Model
#' Averaging, IMF Working Paper 09/202.
#' 
#' @keywords internal
"_PACKAGE"

#' Class "topmod"
#' 
#' An updateable list keeping the best x models it encounters in any kind of
#' model iteration
#' 
#' 
#' @name topmod-class
#' @docType class
#' @section Objects from the Class: Objects can be created by calls to
#' \code{\link{topmod}}, or indirectly by calls to \code{\link{bms}}.\cr
#' 
#' A 'topmod' object (as created by \code{topmod}) holds three basic vectors:
#' \code{lik} (for the (log) likelihood of models or similar), \code{bool()}
#' for a hexcode presentation of the model binaries (cf. \code{\link{bin2hex}})
#' and ncount() for the times the models have been drawn.\cr All these vectors
#' are sorted descendantly by \code{lik}, and are of the same length. The
#' maximum length is limited by the argument \code{nbmodels}.
#' 
#' If \code{tmo} is a topmod object, then a call to \code{tmo$addmodel} (e.g.
#' \code{tmo$addmodel(mylik=4,vec01=c(T,F,F,T))} updates the object \code{tmo}
#' by a model represented by \code{vec01} (here the one including the first and
#' fourth regressor) and the marginal (log) likelihood \code{lik} (here: 4).\cr
#' If this model is already part of \code{tmo}, then its respective
#' \code{ncount} entry is incremented by one; else it is inserted into a
#' position according to the ranking of \code{lik}.\cr In addition, there is
#' the possibility to save (the first moments of) coefficients of a model
#' (\code{betas}) and their second moments (\code{betas2}), as well as an
#' arbitrary vector of statistics per model (\code{fixed_vector}).\cr
#' @author Martin Feldkircher and Stefan Zeugner
#' @seealso \code{\link{topmod}} to create \code{topmod} objects and a more
#' detailed description,
#' \code{\link{is.topmod}} to test for this class
#' @references \url{http://bms.zeugner.eu}
#' @keywords classes
#' @examples
#' 
#'   tm= topmod(2,4,TRUE,0) #should keep a  maximum two models
#'   tm$addmodel(-2.3,c(1,1,1,1),1:4,5:8) #update with some model
#'   tm$addmodel(-2.2,c(0,1,1,1),1:3,5:7) #add another model
#'   tm$addmodel(-2.2,c(0,1,1,1),1:3,5:7) #add it again -> adjust ncount
#'   tm$addmodel(-2.5,c(1,0,0,1),1:2,5:6) #add another model
#'   
#'   #read out
#'   tm$lik()
#'   tm$ncount()
#'   tm$bool_binary()
#'   tm$betas()
#' 
setClass("topmod",representation(addmodel="function", lik="function", bool="function", ncount="function", nbmodels="function", nregs="function", betas_raw="function", betas2_raw="function", kvec_raw="function", bool_binary="function", betas="function", betas2="function", fixed_vector="function"))


#' Class "bma"
#' 
#' A list holding results from a BMA iteration chain
#' 
#' 
#' @name bma-class
#' @docType class
#' @section Objects from the Class: Objects can be created via calls to
#' \code{\link{bms}}, but indirectly also via \code{\link{c.bma}}\cr A
#' \code{bma} object is a list whose elements hold information on input and
#' output for a Bayesian Model Averaging iteration chain, such as from a call
#' to \code{\link{bms}}:
#' @author Martin Feldkircher and Stefan Zeugner
#' @seealso \code{\link{bms}} for creating \code{bma} objects,\cr or
#' \code{\linkS4class{topmod}} for the topmod object
#' @references \url{http://bms.zeugner.eu}
#' @keywords classes
#' @examples
#' 
#'  data(datafls)
#'  mm=bms(datafls)
#'  #show posterior model size
#'  print(mm$info$msize/mm$info$cumsumweights)
#'  #is the same number as in
#'  summary(mm)
#'  
#' 
setClass("bma",representation(info="list",arguments="list",topmod="topmod",start.pos="integer",gprior.info="list",mprior.info="list",X.data="data.frame",reg.names="character",bms.call="call"))



#' Class "zlm"
#' 
#' A list holding output from the Bayesian Linar Model under Zellner's g prior
#' akin to class 'lm'
#' 
#' 
#' @name zlm-class
#' @docType class
#' @section Objects from the Class: Objects can be created via calls to
#' \code{\link{zlm}}, but indirectly also via \code{\link{as.zlm}}.\cr
#' \code{\link{zlm}} estimates a Bayesian Linear Model under Zellner's g prior
#' - its output is very similar to objects of class \code{\link{lm}} (cf.
#' section 'Value')
#' @author Martin Feldkircher and Stefan Zeugner
#' @seealso \code{\link{zlm}} and \code{\link{as.zlm}} for creating \code{zlm}
#' objects,\cr \code{\link{density.zlm}}, \code{\link{predict.zlm}} and
#' \code{\link{summary.zlm}} for other posterior results
#' @references \url{http://bms.zeugner.eu}
#' @keywords classes
setClass("zlm",representation(coefficients="numeric",residuals="numeric",rank="numeric",fitted.values="numeric",df.residual="numeric",call="call",terms="formula",model="data.frame",coef2moments="numeric",marg.lik="numeric",gprior.info="list"))


#' Class "gprior"
#' 
#' An object pertaining to a coefficient prior
#' 
#' 
#' @name gprior-class
#' @docType class
#' @section Objects from the Class: A \code{gprior} object holds descriptions
#' and subfunctions pertaining to coefficient priors. Functions such as
#' \code{\link{bms}} or \code{\link{zlm}} rely on this class to 'convert' the
#' output of OLS results into posterior expressions for a Bayesian Linear
#' Model. Post-processing functions such as \code{\link{density.bma}} also
#' resort to gprior objects.\cr There are currently three coefficient prior
#' structures built into the BMS package, generated by the following functions
#' (cf. Feldkircher and Zeugner, 2009) : \cr \code{gprior.constg.init}: creates
#' a Zellner's g-prior object with constant \code{g}.\cr
#' \code{gprior.eblocal.init}: creates an Empricial Bayes Zellner's g-prior.\cr
#' \code{gprior.hyperg.init}: creates a hyper g-prior with a Beta-prior on the
#' shrinkage parameter.\cr The following describes the necessary slots
#' @author Martin Feldkircher and Stefan Zeugner
#' @seealso \code{\link{bms}} and \code{\link{zlm}} for creating \code{bma} or
#' \code{zlm} objects. \cr Check the appendix of \code{vignette(BMS)} for a
#' more detailed description of built-in priors.\cr Check
#' \url{http://bms.zeugner.eu/custompriors.php} for examples.
#' @references Feldkircher, M. and S. Zeugner (2009): Benchmark Priors
#' Revisited: On Adaptive Shrinkage and the Supermodel Effect in Bayesian Model
#' Averaging, IMF Working Paper 09/202.
#' @keywords classes
#' @examples
#' 
#' 
#' data(datafls)
#' mm1=bms(datafls[,1:10], g="EBL")
#' gg=mm1$gprior.info # is the g-prior object, augmented with some posterior statistics
#' 
#' mm2=bms(datafls[,1:10], g=gg) #produces the same result
#' 
#' mm3=bms(datafls[,1:10], g=BMS:::.gprior.eblocal.init) 
#' #this passes BMS's internal Empirical Bayes g-prior object as the coefficient prior 
#' # - any other obejct might be used as well
#' 
#' 
#' 
setClass("gprior")


#' Class "mprior"
#' 
#' An object pertaining to a BMA model prior
#' 
#' 
#' @name mprior-class
#' @docType class
#' @section Objects from the Class: An \code{mprior} object holds descriptions
#' and subfunctions pertaining to model priors. The BMA functions
#' \code{\link{bms}} and post-processing functions rely on this class. \cr
#' There are currently five model prior structures built into the BMS package,
#' generated by the following functions (cf. the appendix of
#' \code{vignette(BMS)}): \cr \code{mprior.uniform.init}: creates a uniform
#' model prior object.\cr \code{mprior.fixedt.init}: creates the popular
#' binomial model prior object with common inclusion probabilities.\cr
#' \code{mprior.randomt.init}: creates a beta-binomial model prior object.\cr
#' \code{mprior.pip.init}: creates a binomial model prior object that allows
#' for defining individual prior inclusion probabilities.\cr
#' \code{mprior.customk.init}: creates a model prior object that allows for
#' defining a custom prior for each model parameter size.\cr The following
#' describes the necessary slots:
#' @author Martin Feldkircher and Stefan Zeugner
#' @seealso \code{\link{bms}} for creating \code{bma} objects. \cr Check the
#' appendix of \code{vignette(BMS)} for a more detailed description of built-in
#' priors.\cr Check \url{http://bms.zeugner.eu/custompriors.php} for examples.
#' @keywords classes
setClass('mprior')






#' @importFrom stats density


.flsresultlist =function(item=NULL) {
  if (is.null(item)) return(.flsresults)
  return(.flsresults[[item]])
}

######################################
# Bayesian Model Averaging Program   #
######################################
# This version: adjusted on 2011-05-05
# Martin Feldkircher
# martin.feldkircher@gzpace.net, http://feldkircher.gzpace.net
# Stefan Zeugner
# stefan.zeugner@ulb.ac.be, http://www.zeugner.eu
#####################
# The main code starts at line 169 with the function "bms=function(....)" and is written
# by Martin Feldkircher and Stefan Zeugner as part of their work at the Institute for Advanced Studies (IHS),
# Vienna in 2006/2007. Descriptions of the algorithms and priors used can be found in
# Gary Koop ("Bayesian Econometrics", Wiley & Sons), Fernandez, C., E. Ley and M.F.J. Steel (2001b) 
# ?Model Uncertainty in CrossCountry Growth Regressions,? Journal of Applied Econometrics,
# Fernandez, C., E. Ley and M.F.J. Steel (2001a) ?Benchmark Priors for Bayesian Model Averaging,?
# Journal of Econometrics, 100: 381?427. and 
# Liang, F., R. Paulo, G. Molina, M.A. Clyde, and J.O. Berger (2008): "Mixtures of g Priors for Bayesian Variable Selection",
# Journal of the American Statistical Association, 103: 410-423

####################
# USAGE            # 
###################################################################################################
# bms <-function(X.data,burn=1000,iter=NA,nmodel=100,mcmc="bd",g="UIP",mprior="random",mprior.size=NA,user.int=TRUE,
#                start.value=NA,g.stats=TRUE,logfile=FALSE,logstep=10000,force.full.ols=FALSE)
#
######################
# FUNCTION ARGUMENTS #
###################################################################################################
#X.data       data.frame or matrix: submit a data frame or a matrix, where the first column corresponds to the dependent variable
#             followed by the covariates, including a constant term is not necessary since y and X is 
#             demeaned automatically.
#burn         integer >=0: is the number of burn-in draws, default 1000    (not taken into account if mcmc="enumerate")
#iter         integer >=0: is the number of posterior draws, default 3000  (not taken into account if mcmc="enumerate"); 
#             if mcmc="enumerate", then iter is the number of models to be sampled, starting from 0 (default 2^K-1) - cf. start.value
#nmodel       integer >=0: is the number of best models for which information is stored. Convergence analysis of
#             the sampler by means of the correlation between analytic posterior  model probabilities
#             (pmp's) and hat of the MCMC sampler is based on the number you  have set in nmodels. Also
#             if you want to save the regression coefficients (beta.save=T), they are taken from the
#             nmodel best models. Setting nmodel500 slows down the MCMC sampler. Note that posterior
#             inclusion probabilites, and mean calculations are based on the MCMC frequencies as opposed
#             to exact analytical calculations (as in Fernandez, Ley and Steel).
#             Set nmodel=0 to speed up sampling (if topmodel information is not needed)
#mcmc         character: * MC3 samplers: default is "bd" which corresponds to a birth / death MCMC alogrithm. You can choose
#             also "rev.jump" where we have implemented a true reversible jump algorithm adding a "move" step to the birth / death steps from "bd".
#             * enumeraton sampler: moreover there is mcmc="enumerate"/"enum" which will enumerate all possible regressor combinations (not advisable for K>23) 
#             * interaction sampler: adding an ".int" to an MC3 sampler (e.g. "mcmc="bd.int") does an interaction sampler
#             interaction terms will only be sampled along with their component variables, in the colnumn names of X.data interaction terms need to be 
#             denominated by names consisting of the base terms separated by "#" (e.g. an interaction term of base variables "A", "B" and "C" needs column name "A#B#C")
#g            character or integer: is the hyperparameter on Zellner's g-prior for the regression coefficients. You can specify g="UIP", corresponding to g=N (default)
#             g="bric" corresponding to the benchmark prior suggestion from FLS (2001), i.e   g=max(N, K^2)
#             with K denoting the total number of regressors and N the number of observations, g="EBL" estimates a 
#             local empirical Bayes g-parameter (as in Liang et a. (2008), JASA). g="hyper", takes the 'hyper-g' 
#             prior distribution (As in Liang et al.) with the default hyper-parameter a=3; This hyperparameter can 
#             be adjusted (between 2<a<=4) by setting g="hyper=2.9", for instance.
#mprior       regards the prior on model size. It can be either "random" (Default), fixed", "uniform", "customk", or "pip" and is based on the 
#             working paper "On the Effect of Prior Assumptions in Bayesian Model Averaging with Applications
#             to Growth Regression", by  Ley and Steel (2008). Mprior denotes the a priori inclusion probability
#             of a regressor. Their suggestion is to use a binomial-beta hyperprior on mprior (i.e. mprior="random")
#             in order to be noninformative on model size. You can use mprior="fixed" if you have strong prior
#             information on model size. mprior="uniform" employs the uniform model prior; 
#             mprior="customk" allows for custom model size priors (cf. mprior.size)
#             mprior="pip" allows for custom prior inclusion probabilities (cf mprior.size)
#             Note that the prior on models with more than N-3 regressors is automatically zero; these models will not be sampled.
#mprior.size  corresponds to the expected value of the model size prior (default K/2). For mprior=random there is little
#             impact on results by varying mprior.size. For fixed mprior (i.e. informative) prior this is a
#             sensible choice. 
#             if mprior="customk" then a custom model size prior can be provided as K+1 vector detailing the priors from model size 0 to K 
#             (e.g. rep(1,K+1) for the uniform model prior)
#             if mprior="pip" then custom prior inclusion probabilities can be provided as a vector of size K, with elements in the interval (0,1] 
#user.int     'interactive mode' print out results to command line after ending the routine and do two charts
#start.value  specifies the starting model. You can choose either a specific model by the corresponding
#             column indices (e.g. starting.model=numeric(K) starts from the null model including 
#             solely a constant term) or you set a number (e.g. starting.model=20). In the latter case
#             randomly 20 covariates are chosen and the starting model is identified by those regressors
#             with a t-statistics>0.2. 
#             The default value start.value=NA corresponds to start.value=min(nrow(X)-2,ncol(X),nrow(X)-3)
#             start.value=0 or start.value=NULL starts from the null model
#             * if mcmc="enumerate" then start.value is the index to start the iteration (default=0) . Any number between 0 and K^2-1 is admissible
#g.stats      TRUE or FALSE whether statistics on the shrinkage factor should be collected. default=TRUE
#             set g.stats=FALSE for faster iteration
#logfile      setting logfile=TRUE produces a logfile named "test.log" in your current working directory,
#             in order to keep track of the sampling procedure. 
#             setting logfile equal to some filepath (like logfile="subfolder/bla.txt") puts the logfile 
#             into that specified position. (default: logfile=FALSE)
#             Note that logfile="" implies log printouts on the console
#logstep      specifies at which number of posterior draws information is written to the log file; default: 100 000 iterations
#force.full.ols  default FALSE. If force.full.ols=TRUE, the ols estimation part of the sampling procedure relies on slower matrix inversion, 
#             instead of streamlined routines. force.full.ols=TRUE can slow down sampling but may deal better with highly collinear data
#beta.save    obsolete
#int          obsolete, see "mcmc"
#exact        obsolete: see estimates.bma()
#ask.set      obsolete, with no replacement
#printRes     obsolete; see user.int
#return.g.stats renamed intop 'g.stats'
#theta        renamed into 'mprior'
#prior.msize  renamed into 'mprior.size'
#
################
# OUTPUT       #
###################################################################################################
# A call to bms returns a list/"bma" object with the following elements:
#info  a list containing some posterior statistics. it has the following elements
#   iter      (numeric(1)) The number of iteration runs in the sampler (without burn-ins)
#   burn      (numeric(1)) The number of burn-in   iterations
#   inccount  (numeric(K)) The (weighted) cumulative sum of the binary vector with regressor inclusions
#   models.visited (numeric(1)) the number of model candidates that have been accepted (including burn-ins)
#   b1mo      ((numeric(K)) the (weighted) cumulative sum of first posterior moment of beta: Sum p(M)E(beta|X,y,M)
#   b2mo      ((numeric(K)) the (weighted) cumulative sum of second posterior moment of beta: Sum p(M)E(beta^2|X,y,M)
#   add.otherstats (numeric(0 or some integer)) cumulative sum of some additional statistics (such as posterior moments of g)
#   cumsumweights (numeric(1)) the denominator to turn cumulative sums into (weighted) averages
#   K          (numeric(1)) number of covariates in X.data
#   N          (numeric(1)) number of observations in X.data
#   corr.pmp   (numeric(1)) correlation between analytical and MCMC frequencies for the best nmodel models; if mcmc="enum", then this is NA
#   msize      (numeric(1)) the cumulative sum of model sizes
#   timed      (difftime) time taken for the main sampling routine in seconds
#   k.vec      (numeric(K+1)) cumulative sum of the different model siyes, from zero to K
#   cons       (numeric(1)) scalar value of E(constant|X,y), same as BMAOBJECT$cons
#   pos.sign   (numeric(K)) cumulative sum of the coefficients>0, from 1 to K
# arguments    named list with the all the function arguments to the function bms, after being adjusted for inconsistencies
# topmod       object/list containing the best nmodel models; for mor information see help on function .top10
# start.pos    (numeric(<=K)) the indexes of covariates in the actual starting model, that started the MCMC sampling chain
# gprior.info  (list) A list containing information on the gprior: its type ("BRIC", "EBL", "hyper" or "numeric"), whether it is constant, and its 
#                     first moment (in case of the hyper-prior, also the second moment)
# X.data       (data.frame) the data in the bms function argument X.data; the same as BMAOBJECT$arguments$X.data, retained for backward compatibility
# reg.names    (character(K)) the covariates' column names (generic ones, if no names were provided with X.data)
# bms.call     (call) the function call to bms() as it was entered into the command line (regularized by match.call())
#
###############################
# Deprecated output elements  #
###################################################################################################
# in order to convert your bma object into a bma object of versions before this version: 20090612,
# use the function as.oldbma(BMAOBJECT)
#     ----- The following are deprecated elements of BMAOBJECT =bms(...) -------
# estimates   use estimates.bma(BMAOBJECT)
# estimates when exact=TRUE: use estimates.bma(BMAOBJECT, exact=TRUE)
# info        use info.bma(BMAOBJECT)
# topmodels   use topmodels.bma(BMAOBJECT)
# beta.draws  use beta.draws.bma(BMAOBJECT)
# pmp.10      use pmp.bma(BMAOBJECT)
#
#
#

###### BMS MAIN FUNCTION #########################









#' Bayesian Model Sampling and Averaging
#' 
#' Given data and prior information, this function samples all possible model
#' combinations via MC3 or enumeration and returns aggregate results.
#' 
#' Ad \code{mcmc}: \cr Interaction sampler: adding an ".int" to an MC3 sampler
#' (e.g. "mcmc="bd.int") provides for special treatment of interaction terms.
#' Interaction terms will only be sampled along with their component variables:
#' In the colnumn names of X.data, interaction terms need to be denominated by
#' names consisting of the base terms separated by \code{#} (e.g. an
#' interaction term of base variables \code{"A"}, \code{"B"} and \code{"C"}
#' needs column name \code{"A#B#C"}). Then variable \code{"A#B#C"} will only be
#' included in a model if all of the component variables ("A", "B", and "C")
#' are included.
#' 
#' The MC3 samplers "\code{bd}", "\code{rev.jump}", "\code{bd.int}" and
#' "\code{rev.jump.int}", iterate away from a starting model by adding,
#' dropping or swapping (only in the case of rev.jump) covariates.
#' 
#' In an MCMC fashion, they thus randomly draw a candidate model and then move
#' to it in case its marginal likelihood (marg.lik.) is superior to the
#' marg.lik. of the current model.
#' 
#' In case the candidate's marg.lik is inferior, it is randomly accepted or
#' rejected according to a probability formed by the ratio of candidate
#' marg.lik over current marg.lik.  Over time, the sampler should thus converge
#' to a sensible distribution. For aggregate results based on these MC3
#' frequencies, the first few iterations are typically disregarded (the
#' 'burn-ins').
#' 
#' Ad \code{g} and the hyper-g prior: The hyper-g prior introduced by Liang et
#' al. (2008) puts a prior distribution on the shrinkage factor \eqn{g/(1+g)},
#' namely a Beta distribution \eqn{ Beta(1, 1/2-1)} that is governed by the
#' parameter \eqn{a}. \eqn{a=4} means a uniform prior distribution of the
#' shrinkage factor, while \eqn{a>2} close to 2 concentrates the prior
#' shrinkage factor close to one. \cr The prior expected value is
#' \eqn{E(g/1+g)) = 2/a}. In this sense \code{g="hyper=UIP"} and
#' \code{g="hyper=BRIC"} set the prior expected shrinkage such that it conforms
#' to a fixed UIP-g (eqng=N) or BRIC-g (\eqn{g=max(K^2,N)} ).
#' 
#' 
#' @param X.data a data frame or a matrix, with the dependent variable in the
#' first column, followed by the covariates (alternatively, \code{X.data} can
#' also be provided as a \code{\link{formula}}).  Note that \code{bms}
#' automatically estimates a constant, therefore including constant terms is
#' not necessary.
#' @param burn The (positive integer) number of burn-in draws for the MC3
#' sampler, defaults to 1000. (Not taken into account if mcmc="enumerate")
#' @param iter If mcmc is set to an MC3 sampler, then this is the number of
#' iteration draws to be sampled (ex burn-ins), default 3000 draws. \cr If
#' \code{mcmc="enumerate"}, then iter is the number of models to be sampled,
#' starting from 0 (defaults to \eqn{2^K-1}) - cf. \code{start.value}.
#' @param nmodel the number of best models for which information is stored
#' (default 500). Best models are used for convergence analysis between
#' likelihoods and MCMC frequencies, as well as likelihood-based inference.\cr
#' Note that a very high value for \code{nmodel} slows down the sampler
#' significantly. Set nmodel=0 to speed up sampling (if best model information
#' is not needed).
#' @param mcmc a character denoting the model sampler to be used.\cr The MC3
#' sampler \code{mcmc="bd"} corresponds to a birth/death MCMC algogrithm.
#' \code{mcmc="rev.jump"} enacts a reversible jump algorithm adding a "swap"
#' step to the birth / death steps from "bd".\cr Alternatively, the entire
#' model space may be fully enumerated by setting \code{mcmc="enumerate"} which
#' will iterate all possible regressor combinations (Note: consider that this
#' means \eqn{2^K} iterations, where K is the number of covariates.)\cr Default
#' is full enumeration (\code{mcmc="enumerate"}) with less then 15 covariates,
#' and the birth-death MC3 sampler (\code{mcmc="bd"}) with 15 covariates or
#' more. Cf. section 'Details' for more options.
#' @param g the hyperparameter on Zellner's g-prior for the regression
#' coefficients.\cr \code{g="UIP"} corresponds to \eqn{g=N}, the number of
#' observations (default);\cr \code{g="BRIC"} corresponds to the benchmark
#' prior suggested by Fernandez, Ley and Steel (2001), i.e \eqn{g=max(N, K^2)},
#' where K is the total number of covariates;\cr \code{g="RIC"} sets
#' \eqn{g=K^2} and conforms to the risk inflation criterion by George and
#' Foster (1994)\cr \code{g="HQ"} sets \eqn{g=log(N)^3} and asymptotically
#' mimics the Hannan-Quinn criterion with \eqn{C_{HQ}=3} (cf. Fernandez, Ley
#' and Steel, 2001, p.395)\cr \code{g="EBL"} estimates a local empirical Bayes
#' g-parameter (as in Liang et al. (2008));\cr \code{g="hyper"} takes the
#' 'hyper-g' prior distribution (as in Liang et al., 2008) with the default
#' hyper-parameter \eqn{a} set such that the prior expected shrinkage factor
#' conforms to 'UIP';\cr This hyperparameter \eqn{a} can be adjusted (between
#' \eqn{2<a<=4}) by setting \code{g="hyper=2.9"}, for instance.\cr
#' Alternatively, \code{g="hyper=UIP"} sets the prior expected value of the
#' shrinkage factor equal to that of UIP (default), \code{g="hyper=BRIC"} sets
#' it according to BRIC \cr cf section 'Details' fro more on the hyper-g prior
#' @param mprior a character denoting the model prior choice, defaulting to
#' "random":\cr \code{mprior="fixed"} denotes fixed common prior inclusion
#' probabilities for each regressor as e.g. in Sala-i-Martin, Doppelhofer, and
#' Miller(2004) - for their fine-tuning, cf. \code{mprior.size}. Preferable to
#' \code{mcmc="random"} if strong prior information on model size exists;\cr
#' \code{mprior="random"} (default) triggers the 'random theta' prior by Ley
#' and Steel (2008), who suggest a binomial-beta hyperprior on the a priori
#' inclusion probability;\cr \code{mprior="uniform"} employs the uniform model
#' prior;\cr \code{mprior="customk"} allows for custom model size priors (cf.
#' \code{mprior.size});\cr \code{mprior="pip"} allows for custom prior
#' inclusion probabilities (cf. \code{mprior.size});\cr Note that the prior on
#' models with more than N-3 regressors is automatically zero: these models
#' will not be sampled.
#' @param mprior.size if \code{mprior} is "fixed" or "random",
#' \code{mprior.size} is a scalar that denotes the prior expected value of the
#' model size prior (default K/2).\cr If \code{mprior="customk"} then a custom
#' model size prior can be provided as a K+1 vector detailing the priors from
#' model size 0 to K (e.g. rep(1,K+1) for the uniform model prior);\cr if
#' \code{mprior="pip"}, then custom prior inclusion probabilities can be
#' provided as a vector of size K, with elements in the interval (0,1)
#' @param user.int 'interactive mode': print out results to console after
#' ending the routine and plots a chart (default TRUE).
#' @param start.value specifies the starting model of the iteration chain. For
#' instance a specific model by the corresponding column indices (e.g.
#' starting.model=numeric(K) starts from the null model including solely a
#' constant term) or \code{start.value=c(3,6)} for a starting model only
#' including covariates 3 and 6.\cr If \code{start.model} is set to an integer
#' (e.g. \code{start.model=15}) then that number of covariates (here: 15
#' covariates) is randomly chosen and the starting model is identified by those
#' regressors with an OLS t-statistic>0.2.\cr The default value
#' \code{start.value=NA} corresponds to
#' \code{start.value=min(ncol(X.data),nrow(X.data)-3)}. Note that
#' \code{start.value=0} or \code{start.value=NULL} starts from the null
#' model.\cr If \code{mcmc="enumerate"} then \code{start.value} is the index to
#' start the iteration (default: 0, the null model) . Any number between 0 and
#' \eqn{K^2-1} is admissible.
#' @param g.stats \code{TRUE} if statistics on the shrinkage factor g/(1+g)
#' should be collected, defaulting to TRUE (Note: set \code{g.stats=FALSE} for
#' faster iteration.)
#' @param logfile setting \code{logfile=TRUE} produces a logfile named
#' \code{"test.log"} in your current working directory, in order to keep track
#' of the sampling procedure. \code{logfile} equal to some filepath (like
#' \code{logfile="subfolder/log.txt"}) puts the logfile into that specified
#' position. (default: \code{logfile=FALSE}). Note that \code{logfile=""}
#' implies log printouts on the console.
#' @param logstep specifies at which number of posterior draws information is
#' written to the log file; default: 10 000 iterations
#' @param force.full.ols default FALSE. If \code{force.full.ols=TRUE}, the OLS
#' estimation part of the sampling procedure relies on slower matrix inversion,
#' instead of streamlined routines. \code{force.full.ols=TRUE} can slow down
#' sampling but may deal better with highly collinear data
#' @param fixed.reg indices or variable names of \code{X.data} that are fixed
#' regressors to be always included in every sampled model. Note: the parameter
#' \code{mprior.size} refers to prior model size including these fixed
#' regressors.
#' @return A list of class \code{bma}, that may be displayed using e.g.
#' \code{\link{summary.bma}} or \code{\link{coef.bma}}. The list contains the
#' following elements: \item{info}{a list of aggregate statistics: \code{iter}
#' is the number of iterations, \code{burn} the number of burn-ins.\cr The
#' following have to be divided by \code{cumsumweights} to get posterior
#' expected values: \code{inccount} are the posterior inclusion probabilities,
#' \code{b1mo} and \code{b2mo} the first and second moment of coefficients,
#' \code{add.otherstats} other statistics of interest (typically the moments of
#' the shrinkage factor), \code{msize} is the post. expected model size,
#' \code{k.vec} the posterior model size distribution, \code{pos.sign} the
#' unconditional post. probability of positive coefficients, \code{corr.pmp} is
#' the correlation between the best models' MCMC frequencies and their marg.
#' likelihoods.\cr \code{timed} is the time that was needed for MCMC sampling,
#' \code{cons} is the posterior expected value of the constant. \code{K} and
#' \code{N} are the maximum number of covariates and the sample size,
#' respectively.} \item{arguments}{a list of the evaluated function arguments
#' provided to \code{bms} (see above)} \item{topmod}{a 'topmod' object
#' containing the best drawn models. see \code{\link{topmod}} for more details}
#' \item{start.pos}{the positions of the starting model. If bmao is a'bma'
#' object this corresponds to covariates bmao$reg.names[bmao$start.pos]. If
#' bmao is a chain that resulted from several starting models (cf.
#' \code{\link{c.bma}}, then \code{start.pos} is a list detailing all of them.}
#' \item{gprior.info}{a list of class \code{\link{gprior-class}}, detailing
#' information on the g-prior: \code{gtype} corresponds to argument \code{g}
#' above, \code{is.constant} is FALSE if \code{gtype} is either "hyper" or
#' "EBL", \code{return.g.stats} corresponds to argument \code{g.stats} above,
#' \code{shrinkage.moments} contains the first and second moments of the
#' shrinkage factor (only if \code{return.g.stats==TRUE}), \code{g} details the
#' fixed g (if \code{is.constant==TRUE}), \code{hyper.parameter} corresponds to
#' the hyper-g parameter \eqn{a} as in Liang et al. (2008) }
#' \item{mprior.info}{a list of class \code{\link{mprior-class}}, detailing
#' information on the model prior: \code{origargs} lists the original arguments
#' to \code{mprior} and \code{mprior.size} above; \code{mp.msize} denotes the
#' prior mode size; \code{mp.Kdist} is a (K+1) vector with the prior model size
#' distribution from 0 to K} \item{X.data}{data.frame or matrix: corresponds to
#' argument \code{X.data} above, possibly cleaned for NAs}
#' \item{reg.names}{character vector: the covariate names to be used for X.data
#' (corresponds to \code{\link{variable.names.bma}} } \item{bms.call}{the
#' original call to the \code{bms} function}
#' @note There are several ways to speed-up sampling: \code{nmodel=10} saves
#' only the ten best models, at most a marginal improvement. \code{nmodels=0}
#' does not save the best (500) models, however then posterior convergence and
#' likelihood-based inference are not possible.  %\code{beta.save=FALSE} saves
#' the best models, but not their coefficients, which renders the use of
#' \code{image.bma} and the paramer \code{exact=TRUE} in functions such as
#' \code{coef.bma} infeasible.  \code{g.stats=FALSE} saves some time by not
#' retaining the shrinkage factors for the MC3 chain (and the best models).
#' \code{force.fullobject=TRUE} in contrast, slows sampling down significantly
#' if \code{mcmc="enumerate"}.
#' @section Theoretical background: The models analyzed are Bayesian
#' normal-gamma conjugate models with improper constant and variance priors
#' akin to Fernandez, Ley and Steel (2001): A model \eqn{M} can be described as
#' follows, with \eqn{\epsilon} ~ \eqn{N(0,\sigma^2 I)}: \deqn{latex}{ y=
#' \alpha + X \beta + \epsilon} \deqn{f(\beta | \sigma, M, g) ~ N(0, g \sigma^2
#' (X'X)^-1) }
#' 
#' Moreover, the (improper) prior on the constant \eqn{f(\alpha)} is put
#' proportional to 1. Similarly, the variance prior \eqn{f(\sigma)} is
#' proportional to \eqn{1/\sigma}.
#' @author Martin Feldkircher, Paul Hofmarcher, and Stefan Zeugner
#' @seealso \code{\link{coef.bma}}, \code{\link{plotModelsize}} and
#' \code{\link{density.bma}} for some operations on the resulting 'bma' object,
#' \code{\link{c.bma}} for integrating separate MC3 chains and splitting of
#' sampling over several runs.
#' 
#' Check \url{http://bms.zeugner.eu} for additional help.
#' @references 
#' \url{http://bms.zeugner.eu}: BMS package homepage with help and tutorials
#' 
#' Feldkircher, M. and S. Zeugner (2015): Bayesian Model Averaging Employing 
#' Fixed and Flexible Priors: The BMS Package for R, Journal of Statistical Software 68(4).
#' 
#' Feldkircher, M. and S. Zeugner (2009): Benchmark Priors
#' Revisited: On Adaptive Shrinkage and the Supermodel Effect in Bayesian Model
#' Averaging, IMF Working Paper 09/202.
#' 
#' Fernandez, C. E. Ley and M. Steel (2001): Benchmark priors for Bayesian
#' model averaging. Journal of Econometrics 100(2), 381--427
#' 
#' Ley, E. and M. Steel (2008): On the Effect of Prior Assumptions in Bayesian
#' Model Averaging with Applications to Growth Regressions. working paper
#' 
#' Liang, F., Paulo, R., Molina, G., Clyde, M. A., and Berger, J. O. (2008).
#' Mixtures of g Priors for Bayesian Variable Selection. Journal of the
#' American Statistical Association 103, 410-423.
#' 
#' Sala-i-Martin, X. and G. Doppelhofer and R.I. Miller (2004): Determinants of
#' long-term growth: a Bayesian averaging of classical estimates (BACE)
#' approach. American Economic Review 94(4), 813--835
#' @keywords models
#' @examples
#' 
#'   data(datafls)
#'   #estimating a standard MC3 chain with 1000 burn-ins and 2000 iterations and uniform model priors
#'   bma1 = bms(datafls,burn=1000, iter=2000, mprior="uniform")
#' 
#'   ##standard coefficients based on exact likelihoods of the 100 best models:
#'   coef(bma1,exact=TRUE, std.coefs=TRUE) 
#'   
#'   #suppressing user-interactive output, using a customized starting value, and not saving the best 
#'   #  ...models for only 19 observations (but 41 covariates)
#'   bma2 = bms(datafls[20:39,],burn=1000, iter=2000, nmodel=0, start.value=c(1,4,7,30),
#'      user.int=FALSE)
#'   coef(bma2)
#'   
#'   #MC3 chain with a hyper-g prior (custom coefficient a=2.1), saving only the 20 best models, 
#'   # ...and an alternative sampling procedure; putting a log entry to console every 1000th step
#'   bma3 = bms(datafls,burn=1000, iter=5000, nmodel=20, g="hyper=2.1", mcmc="rev.jump",
#'       logfile="",logstep=1000)
#'   image(bma3) #showing the coefficient signs of the 20 best models
#'   
#'   #enumerating with 10 covariates (= 1024 models), keeping the shrinkage factors 
#'   #  ...of the best 200 models
#'   bma4 = bms(datafls[,1:11],mcmc="enumerate",nmodel=200,g.stats=TRUE)
#' 
#'   #using an interaction sampler for two interaction terms
#'   dataint=datafls
#'   dataint=cbind(datafls,datafls$LifeExp*datafls$Abslat/1000,
#'         datafls$Protestants*datafls$Brit-datafls$Muslim)
#'   names(dataint)[ncol(dataint)-1]="LifeExp#Abslat"
#'   names(dataint)[ncol(dataint)]="Protestants#Brit#Muslim"
#'   bma5 = bms(X.data=dataint,burn=1000,iter=9000,start.value=0,mcmc="bd.int") 
#'   
#'   density(bma5,reg="English") # plot posterior density for covariate "English"
#'   
#'   # a matrix as X.data argument
#'   bms(matrix(rnorm(1000),100,10))
#'   
#'   # keeping a set of fixed regressors:
#'   bms(datafls, mprior.size=7, fixed.reg = c("PrScEnroll", "LifeExp", "GDP60"))
#'   # Note that mprior.size=7 means prior model size of 3 fixed to 4 'uncertain' regressors
#'   
#' @export
bms <-function(X.data,burn=1000,iter=NA,nmodel=500,mcmc="bd",g="UIP",mprior="random",mprior.size=NA,user.int=TRUE,
                start.value=NA,g.stats=TRUE,logfile=FALSE,logstep=10000,force.full.ols=FALSE, fixed.reg=numeric(0)) {
#                beta.save=TRUE,exact=NA,int=NA,printRes=NA,ask.set=NA,return.g.stats=NA,theta=NULL,prior.msize=NULL #deprecated function arguments, retained for compatibility with older versions
                 


### getting data dimensions ####################
  if (class(X.data)[[1]]=="formula") { X.data=stats::model.frame(X.data); if (!is.null(ncol(X.data[[2]]))) X.data=cbind(X.data[[1]],X.data[[2]][,-1]) }

  if (any(is.na(X.data))) {
     X.data=na.omit(X.data)
     if (nrow(X.data)<3) {stop("Too few data observations. Please provide at least three data rows without NA entries.") }
     warning("Argument 'X.data' contains NAs. The corresponding rows have not been taken into account.")
  }                                                  

  
  N<-nrow(X.data)
  K=ncol(X.data)-1
  maxk=N-3  #maximum number of admissible k per model

  
  
############################################################################################################################################
#### User Checks: ##########################################################################################################################
############################################################################################################################################

                
  # check for deprecated arguments
#  if (!is.na(exact)) { warning("Function argument 'exact' has been deprecated, please refer to function 'estimates.bma' instead.") }
#  if (!is.na(int)) { mcmc=paste(mcmc,".int",sep=""); warning("Function argument 'int' has been deprecated, please add an 'int' to the argument 'mcmc' instead.") }
#  if (!is.na(printRes)) { user.int=printRes; warning("Function argument 'printRes' has been deprecated, please refer to the argument 'user.int' instead.") }  
#  if (!is.na(ask.set)) { warning("Function argument 'ask.set' has been deprecated, with no replacement.") }    
#  if (!is.na(return.g.stats)) { g.stats=return.g.stats; warning("Function argument 'return.g.stats' has been renamed into 'g.stats'.") }      
#  if (!is.null(theta)) { mprior=theta; warning("Function argument 'theta' has been renamed into 'mprior'.") }  
#  if (!is.null(prior.msize)) { mprior.size=prior.msize; warning("Function argument 'prior.msize' has been renamed into 'mprior.size'.") }      
#  return.g.stats=g.stats; #theta=mprior; prior.msize=mprior.size
  
  if (is.null(nmodel[1])||is.na(nmodel[1])||nmodel[1]<=0) {dotop=FALSE;nmodel=0} else {dotop=TRUE}
###########################################################################

    nameix=1:K; names(nameix)=colnames(X.data[,-1,drop=FALSE]); fixed.pos=nameix[fixed.reg]; rm(nameix)

######################################################################################################################################      
    #assign the sampling procedure                                                                                              
   if (missing(mcmc)&&((K-length(fixed.pos))<15)) {mcmc="enum"} 

   int=FALSE; is.enum=FALSE #int: whether interaction sampler is wanted; is.enum: whether the sampler is enumeration
   if (is.function(mcmc)) { 
      samplingfun=mcmc
      mcmc="custom"
   } else {
    if (length(grep("int",mcmc,ignore.case=TRUE))) {int=TRUE}   
   
    if (length(grep("enum",mcmc,ignore.case=TRUE)))  {
      is.enum=TRUE; samplingfun=.iterenum
      if (K>maxk) samplingfun=.iterenum.KgtN
    } else if(length(grep("bd",mcmc,ignore.case=TRUE))){
      samplingfun=switch(int+1,.fls.samp,.fls.samp.int)
    } else {
      samplingfun=switch(int+1,.rev.jump,.rev.jump.int)
    }
   }
   if (int&&(length(fixed.pos)>0L)) { warning("interaction sampler does not allow for non-zero argument fixed.pos; consequently it was set fixed.pos=0"); fixed.pos=numeric(0); }
    sampling=.fixedset.sampler(samplingfun,fullK=K,fixed.pos=fixed.pos, X.data=X.data); 
######################################################################################################################################    
  # specific enumeration user checks & init
  if (is.enum) {      
     #check for a start.value index to start enumeration from seomewhere in between (and not do all possible models)  
     burn=0;  int=FALSE; mcmc="enum"; is.enum=TRUE
     tmp=.enum_startend(iter=iter, start.value=start.value, K=K, maxk=maxk, fixed.pos=fixed.pos); iter=tmp$iter; start.value=tmp$start.value
  } else {
     if (is.na(iter)) {iter=3000}; #if no enumeration and iter not given, set to default value 3000
  }

######################################################################################################################################
   # generate logfile if desired
    if(logfile!=FALSE){
        if (is.character(logfile)) {
          sfilename=logfile}
        else {
          sfilename="test.log"
        }
        if (nchar(sfilename)>0) file.create(sfilename)
        logfile=TRUE
      cat(as.character(Sys.time()),": starting loop ... \n",append=TRUE, file=sfilename)  #write one line
      if (logstep!=10000) fact=logstep else fact=max(floor((burn+iter)/100),logstep)
    }  

   


######################################################################################################################################  
######################################################################################################################################  
 # subtract mean from all regressors as in FLS
  y<-as.matrix(X.data[,1])
  X<-as.matrix(X.data[,2:ncol(X.data)])
  y<-y-mean(y)
  X<-X-matrix(colMeans(X),N,K,byrow=TRUE)


 # multiply the whole matrix stuff out before going into the simulation loops
  XtX.big=crossprod(X)
  Xty.big=crossprod(X,y)
  yty = as.vector(crossprod(y))

 # user check: whether we have to use force.full.ols
 coreig=eigen(cor(X),symmetric=TRUE,only.values=TRUE)$values
 if (!force.full.ols) { #this line was added due to feature requests
    if (sum(coreig>1e-7)<min(K,(N-1))) { force.full.ols=TRUE }
 }
 if (any(coreig[1:min(K,(N-1))] <1e-16))  { warning(paste("data seems to be rank-deficient: its rank seems to be only ", sum(coreig>1e-13))) }

######################################################################################################################################
 # for the case that X contains interaction terms
  if(int){
      if(length(grep("#",colnames(X.data),fixed=TRUE))==0) stop("Please separate column names of interaction terms by # (e.g. A#B)")
      mPlus=.constr.intmat(X,K)
  }
  else{ mPlus<-NA }                                
  
######################################################################################################################################
    #Prior Initialization
    
    #model prior: outsourced to stupid function for cleanliness
    pmplist=.choose.mprior(mprior,mprior.size,K=K,X.data=X.data,fixed.pos=fixed.pos)
    mprior=pmplist$mp.mode;
    
    #g-prior
    gprior.info = .choose.gprior(g,N,K,return.g.stats=g.stats,yty=yty,X.data=X.data) # gprior.info is a list that summarizes info about the choice of the g-prior
    lprobcalc = gprior.info$lprobcalc

    
######################################################################################################################################
#The function Starter selects randomly a start matrix and runs a
#regression. From this regression, the
#start Design matrix is that for which the t-stats are >0.2. So we
#can circumvent starting from a very bad start point.
  start.list=.starter(K,start.value,y,N=N,XtX.big=XtX.big,Xty.big=Xty.big,X=X,fixed.pos=fixed.pos)
  molddraw=start.list$molddraw; start.position=start.list$start.position
  kold=sum(molddraw)
  position=(1:K)[molddraw==1]

########################################################################################################################################
 


################################################################################################
#    Initializing                                                                              #
################################################################################################


  # initialize sampler-specific variables    ########################################
  # these are to select additional statistics (such as g)
  collect.otherstats=FALSE
  otherstats=numeric(0)
  add.otherstats=numeric(0)
  # initialize the vector for collecting the empirical shrinkage factor moments
  if (gprior.info$return.g.stats & !(gprior.info$is.constant)) { add.otherstats=gprior.info$shrinkage.moments; collect.otherstats=TRUE } 
  cumsumweights=iter
  null.lik=lprobcalc$just.loglik(ymy=yty,k=0) # calculate Likelihood for NullModel
  if (K < N-3) {
  
    mid.lik=lprobcalc$just.loglik(ymy=yty*(1-as.vector(crossprod(crossprod(chol2inv(chol(XtX.big)),Xty.big),Xty.big)/yty)),k=ceiling(K/2)) # calculate Likelihood for max model
  } else {
    mid.lik=lprobcalc$just.loglik(ymy=yty*.001,k=ceiling(K/2)) # calculate Likelihood for max model
  }
  if (!is.finite(mid.lik)) { mid.lik=sapply(as.list(seq(.1,.9,.1)),function(x) lprobcalc$just.loglik(ymy=yty*x,k=ceiling(K/2)));  mid.lik=max(mid.lik[is.finite(mid.lik)]) }
  #adding up posterior stats has been outsourced to sub-functions for speed reasons
  if (collect.otherstats) {
    addup<-  function() {

      inccount <<- inccount + molddraw #PIPs
      msize<<-msize + kold   # average size of models

      #for speed reasons, iterative adding with indexing should be done in one stacked vector      
      if (kold!=0) {
        bm[c(position,K+position,2*K+kold,3*K+position)]=c(b1,b2,1,b1>0); bmo <<- bmo+bm
        #bmo is partitioned: first K entries have cum. b1 ("b1mo"), second K entries have cum. b2 ("b2mo"), third K entries have model size dist ("k.vec"), and fourth K entries are like inccount for positive betas (add up pos. sign covariate selections)
      } else {
        null.count<<-null.count+1
      }
      
      # collect e.g. estimated g-priors, etc   
      otherstats<<-lik.list[["otherstats"]]; add.otherstats<<-add.otherstats + otherstats        
    }


  } else {

    addup <- function() {

      inccount <<- inccount + molddraw #PIPs
      msize<<-msize + kold   # average size of models

      #for speed reasons, iterative adding with indexing should be done in one stacked vector      
      if (kold!=0) {
        bm[c(position,K+position,2*K+kold,3*K+position)]=c(b1,b2,1,b1>0); bmo <<- bmo+bm
        #bmo is partitioned: first K entries have cum. b1 ("b1mo"), second K entries have cum. b2 ("b2mo"), third K entries have model size dist ("k.vec"), and fourth K entries are like inccount for positive betas (add up pos. sign covariate selections)
      } else {
        null.count<<-null.count+1
      }
    }
  }
  if (is.enum) {
    cumsumweights=0
    
    if (collect.otherstats) {
      addup<- function() {
      
      weight=  exp(pmpold+lprobold-mid.lik)
      inccount <<- inccount + weight*molddraw #PIPs
      msize<<-msize + weight*kold   # average size of models
      cumsumweights<<-cumsumweights+weight #denominator to get at sum of PMPs=1     
      #browser()  
      #for speed reasons, iterative adding with indexing should be done in one stacked vector      
      if (kold!=0) {
        bm[c(position,K+position,2*K+kold,3*K+position)]=weight*c(b1,b2,1,b1>0); bmo <<- bmo+bm
        #bmo is partitioned: first K entries have cum. b1 ("b1mo"), second K entries have cum. b2 ("b2mo"), third K entries have model size dist ("k.vec"), and fourth K entries are like inccount for positive betas (add up pos. sign covariate selections)
      } else {
        null.count<<-null.count+weight
      }
      otherstats<<-lik.list[["otherstats"]]; add.otherstats<<-add.otherstats + weight*otherstats  
      }
    } else {
      addup <- function() {
      weight=  exp(pmpold+lprobold-mid.lik)
      #browser()  
      inccount <<- inccount + weight*molddraw #PIPs
      msize<<-msize + weight*kold   # average size of models
      cumsumweights<<-cumsumweights+weight #denominator to get at sum of PMPs=1     

      #for speed reasons, iterative adding with indexing should be done in one stacked vector      
      if (kold!=0) {
        bm[c(position,K+position,2*K+kold,3*K+position)]=weight*c(b1,b2,1,b1>0); bmo <<- bmo+bm
        #bmo is partitioned: first K entries have cum. b1 ("b1mo"), second K entries have cum. b2 ("b2mo"), third K entries have model size dist ("k.vec"), and fourth K entries are like inccount for positive betas (add up pos. sign covariate selections)
      } else {
        null.count<<-null.count+weight
      }
      }

    }
  }
  environment(addup) <- environment() 
  ##################################################################################

 
 
  ##initialize model varibales with starter model ###################################
  ols.object=.ols.terms2(positions=(1:K)[molddraw==1],yty=yty,k=kold,N,K=K,XtX.big=XtX.big,Xty.big=Xty.big) #OLS results from starter model

  lik.list=lprobcalc$lprob.all(ymy=ols.object$ymy, k=kold, bhat=ols.object$bhat, diag.inverse=ols.object$diag.inverse) #likelihood and expected values for starter model
  lprobold=lik.list$lprob
  b1=lik.list$b1new
  b2=lik.list$b2new
  
  ## calculate the posterior model probability for the first model
  pmpold=pmplist$pmp(ki=kold,mdraw=molddraw)
  ##################################################################################
  
  
  
  
  ## initialize top 10 function ####################################################
  #topmods=.top10(nmaxregressors=K,nbmodel=nmodel,bbeta=FALSE,bbeta2=FALSE,lengthfixedvec=length(add.otherstats))
  topmods=topmod(nbmodels=nmodel,nmaxregressors=K,bbeta=FALSE,lengthfixedvec=length(add.otherstats))
  if (mcmc=="enum") { try(topmods$duplicates_possible(FALSE), silent=TRUE) }
  if (dotop && (burn==0L)) topmods$addmodel(mylik=pmpold+lprobold,vec01=molddraw,fixedvec=lik.list$otherstats)
  ##################################################################################




  ## Initialize the rest  ###########################################################
  null.count=0             #number the null model has been drawn
  models.visited=0         #how often a model has been accepted (in burn-ins and iterations)
  inccount=numeric(K)      #how often the respective covariate has been included
  msize=0                  #average model size
  k.vec=numeric(K)         #how often the respective model size has been accepted
  b1mo=numeric(K)          #holds aggregate first moment of all coefficients
  ab=numeric(K)            #Initialize them here
  b2mo=numeric(K)          #holds aggregate second moment of all coefficients
  bb=numeric(K)              
  possign=inccount         # the number of times the respective coefficent has been positive
  mnewdraw=numeric(K)      #holds the binary vector denoting the proposed model
  if (force.full.ols) {candi.is.full.object=TRUE} else {candi.is.full.object=FALSE} #candi.is.full: if TRUE, standard OLS, else OLS via Frisch-Waugh tricks
  bmo=numeric(4*K); bm=bmo #common placeholder for b1mo, b2mo, k.vec and possign
  if (is.enum) { addup() } # in case the sampler is enumeration then count the starting value as well (no burn-ins)
  if (!is.finite(pmpold)) pmpold = -1e90 # this is if in case of an MCMC sampler the starting model got 0 PMP

###############################################################################################################
###############################################################################################################





#############################################################################################
    set.seed(as.numeric(Sys.time()))              #Set Seed randomly for number generator
                                                 
    t1<-Sys.time()                                #Save time before going into the loop 
###########################################################################################
#START MAIN LOOP
###########################################################################################
nrep=burn+iter; i=0;
while(i<nrep) {
      i=i+1;
      if(logfile){ if (i %% fact==0) { cat(as.character(Sys.time()),":",i,"current draw \n",append=TRUE, file=sfilename)} } #write one line  
    
##########################################################################################
#Start sampling program
###########################################################################################

      #sample a model                                                                                           
      a=sampling(molddraw=molddraw,K=K,mPlus=mPlus,maxk=maxk,oldk=kold)
      mnewdraw=a[["mnewdraw"]]; positionnew=a[["positionnew"]]; knew=length(positionnew)

      #calculate prior model prob
      pmpnew=pmplist[["pmp"]](ki=knew,mdraw=mnewdraw) # get the (log) model prior prob
      

      if (!is.enum) {
        if (int) {if (length(c(a$dropi,a$addi))>2|i<3|force.full.ols) {candi.is.full.object=TRUE} else {candi.is.full.object=FALSE}}
        #candi.is.full.object = TRUE if there were multiple regs dropped or added due to interaction terms

        if (candi.is.full.object) {
            ols.candidate = .ols.terms2(positions=positionnew,yty=yty,k=knew,N,K=K,XtX.big=XtX.big,Xty.big=Xty.big) #in case of changing interaction terms, draw the big OLS stuff
            ymy.candi =ols.candidate[["ymy"]]
        } else {
            ymy.candi=ols.object[["child.ymy"]](a$addi,a$dropi,k=knew) #if standard sampling, use Frisch-Waugh to get the new ResidSS (faster)
        }
    
        if ( (ymy.candi<0) | is.na(ymy.candi) ) stop(paste("stumbled on rank-deficient model" ))
        lprobnew = lprobcalc[["just.loglik"]](ymy=ymy.candi,k=knew) # get the log-likelihood out of the ResidSS
        
        #Now decide whether to accept candidate draw
        
        accept.candi = as.logical(log(stats::runif(1,0,1))< lprobnew-lprobold + pmpnew-pmpold)

      } else {
        accept.candi=TRUE
        candi.is.full.object=FALSE
      }



      if(accept.candi){
          if (!candi.is.full.object) {
               #in case one has used Frisch-Waugh and the new model got accepted,
               #calculate the 'real' inverse in order not to make copying mistakes
              ols.res = ols.object[["mutate"]](addix=a$addi, dropix=a$dropi, newpos=positionnew, newk=knew)
          } else {
              ols.object = ols.candidate
              ols.res = ols.candidate[["full.results"]]()
          }

          lik.list = lprobcalc[["lprob.all"]](max(0,ols.res$ymy), knew, ols.res$bhat, ols.res$diag.inverse)


          lprobold=lik.list[["lprob"]]
          position = positionnew
          pmpold=pmpnew  #get posterior odds for new model  if accepted
          molddraw=mnewdraw
          kold=knew
          models.visited=models.visited+1 #does not account for revisiting models
      }


# Collect Posterior Draws
########################################################################
    if (i>burn){   
          b1=lik.list[["b1new"]]; b2=lik.list[["b2new"]];
          addup() #addup does iterative, cumulative sums of quantities of interest (betas, model size, etc.)
          
          # add log(lik)*p(M) to topmodels
          if (dotop) topmods[["addmodel"]](mylik=pmpold+lprobold,vec01=molddraw,fixedvec=otherstats)
    }
}
###########################################################################################
#END MAIN LOOP
###########################################################################################


###########################################################################################
   #adjust the topmod object and calculate all the betas after sampling
   #similar to havving set bbeta=T, and bbeta2=T in the call to .top10 above
   if (dotop) topmods=.topmod.as.bbetaT(topmods,gprior.info,X.data)

###########################################################################################

###########################################################################################
  
  timed<-difftime(Sys.time(),t1)

  # do aggregating calculations
  if (is.enum) {iter=iter+1; models.visited=models.visited+1}
  bmo=matrix(bmo,4,byrow=TRUE); b1mo=bmo[1,]; b2mo=bmo[2,]; k.vec=bmo[3,]; possign=bmo[4,]; rm(bmo)
  post.inf=.post.calc(gprior.info,add.otherstats,k.vec,null.count,X.data,topmods,b1mo,b2mo,iter,burn,inccount,models.visited,K,N,msize,timed,cumsumweights,mcmc,possign)
  
  result=list(info=post.inf$info,arguments=.construct.arglist(bms, environment()),topmod=topmods,start.pos=sort(start.position),gprior.info=post.inf$gprior.info,mprior.info=pmplist,reg.names=post.inf$reg.names,bms.call=try(match.call(bms,sys.call(0)),silent=TRUE))
  if (!is.null(result$X.data)) { result$X.data<-NULL }
  class(result)=c("bma")
  
  
###########################################################################################  

  # print results to console
  if(user.int){
    print(result)
    print(timed)
    plot.bma(result) # do modelsize plot
  }

  return(invisible(result))
}

#' FLS (2001) growth data
#'
#' The economic growth data set from Fernandez, Ley and Steel, Journal of Applied Econometrics 2001
#'
#' @format A data frame with 53940 rows and 10 variables:
#' A data frame with 72 observations on the following 42 variables.
#'  \describe{
#'    \item{\code{y}}{numeric: Economic growth 1960-1992 as from the Penn World Tables Rev 6.0}
#'    \item{\code{Abslat}}{numeric: Absolute latitude}
#'    \item{\code{Spanish}}{numeric: Spanish colony dummy}
#'    \item{\code{French}}{numeric: French colony dummy}
#'    \item{\code{Brit}}{numeric: British colony dummy}
#'    \item{\code{WarDummy}}{numeric: War dummy}
#'    \item{\code{LatAmerica}}{numeric: Latin America dummy}
#'    \item{\code{SubSahara}}{numeric; Sub-Sahara dummy}
#'    \item{\code{OutwarOr}}{numeric: Outward Orientation}
#'    \item{\code{Area}}{numeric: Area surface}
#'    \item{\code{PrScEnroll}}{numeric: Primary school enrolment}
#'    \item{\code{LifeExp}}{numeric: Life expectancy}
#'    \item{\code{GDP60}}{numeric: Initial GDP in 1960}
#'    \item{\code{Mining}}{numeric: Fraction of GDP in mining}
#'    \item{\code{EcoOrg}}{numeric: Degree of capitalism}
#'    \item{\code{YrsOpen}}{numeric: Number of years having an open economy}
#'    \item{\code{Age}}{numeric: Age}
#'    \item{\code{Buddha}}{numeric: Fraction Buddhist}
#'    \item{\code{Catholic}}{numeric: Fraction Catholic}
#'    \item{\code{Confucian}}{numeric: Fraction Confucian}
#'    \item{\code{EthnoL}}{numeric: Ethnolinguistic fractionalization}
#'    \item{\code{Hindu}}{numeric: Fraction Hindu}
#'    \item{\code{Jewish}}{numeric: Fraction Jewish}
#'    \item{\code{Muslim}}{numeric: Fraction Muslim}
#'    \item{\code{PrExports}}{numeric: Primary exports 1970}
#'    \item{\code{Protestants}}{numeric: Fraction Protestants}
#'    \item{\code{RuleofLaw}}{numeric: Rule of law}
#'    \item{\code{Popg}}{numeric: Population growth}
#'    \item{\code{WorkPop}}{numeric: workers per inhabitant}
#'    \item{\code{LabForce}}{numeric: Size of labor force}
#'    \item{\code{HighEnroll}}{numeric: Higher education enrolment}
#'    \item{\code{PublEdupct}}{numeric: Public education share}
#'    \item{\code{RevnCoup}}{numeric: Revolutions and coups}
#'    \item{\code{PolRights}}{numeric: Political rights}
#'    \item{\code{CivlLib}}{numeric: Civil liberties}
#'    \item{\code{English}}{numeric: Fraction speaking English}
#'    \item{\code{Foreign}}{numeric: Fraction speaking foreign language}
#'    \item{\code{RFEXDist}}{numeric: Exchange rate distortions}
#'    \item{\code{EquipInv}}{numeric: Equipment investment}
#'    \item{\code{NequipInv}}{numeric: Non-equipment investment}
#'    \item{\code{stdBMP}}{numeric: stand. dev. of black market premium}
#'    \item{\code{BlMktPm}}{numeric: black market premium}
#' }
#' @source 
#'   Fernandez, C., Ley, E., and Steel, M. F. (2001b). Model Uncertainty in Cross-Country Growth Regressions. Journal of Applied Econometrics, 16:563-576.
#'  Data set from \url{https://warwick.ac.uk/fac/sci/statistics/staff/academic-research/steel/steel_homepage/software}.
#'
#'  A working paper version of Fernandez, Ley and Steel (2001) is available via \url{https://econpapers.repec.org/article/jaejapmet/v_3a16_3ay_3a2001_3ai_3a5_3ap_3a563-576.htm}.
"datafls"

post.var <-
function (object, exact = FALSE) 
{
    if (!(is.bma(object) | is(object, "lm"))) 
        stop("Required input is an object of class 'bma' or 'lm'/'zlm'.")
    od = deviance(object, exact = exact)
    oy = model.frame(object)[, 1, drop = TRUE]
    ret = od/length(oy)
    attr(ret, "nobs") = length(oy)
    return(ret)
}

pmp.bma <-
function (bmao, oldstyle = FALSE) 
{
    if (!(is.bma(bmao) || is.topmod(bmao))) 
        stop("bmao needs to be a 'bma' object!")
    if (is.topmod(bmao)) {
        topmods = bmao
        was.enum = FALSE
        cumsumweights = sum(topmods$ncount())
        log.null.lik = 0
    }
    else {
        topmods = bmao$topmod
        log.null.lik = (1 - bmao$info$N)/2 * log(as.vector(crossprod(bmao$X.data[, 
            1] - mean(bmao$X.data[, 1]))))
        cumsumweights = bmao$info$cumsumweights
        was.enum = (bmao$arguments$mcmc == "enum")
    }
    lt1 = suppressWarnings(topmods$lik() - max(topmods$lik()))
    lt1 = exp(lt1)/sum(exp(lt1))
    if (was.enum) {
        lt2 = exp(topmods$lik() - log.null.lik)/cumsumweights
    }
    else {
        lt2 = topmods$ncount()/cumsumweights
    }
    cpoint = min(length(lt1), length(lt2))
    lt1 = lt1[1:cpoint]
    lt2 = lt2[1:cpoint]
    if (!oldstyle) 
        lt1 <- lt1 * sum(lt2)
    topmodout = rbind(lt1, lt2)
    rownames(topmodout) = c("PMP (Exact)", "PMP (MCMC)")
    colnames(topmodout) = topmods$bool()
    return(t(topmodout))
}

density.bma <-
function (x, reg = NULL, addons = "lemsz", std.coefs = FALSE, 
    n = 300, plot = TRUE, hnbsteps = 30, addons.lwd = 1.5, ...) 
{
    dtcm = function(x, df, ncp, varp) {
        sqvarp = sqrt(varp)
        dt((x - ncp)/sqvarp, df = df)/sqvarp
    }
    dsgivenykernel <- function(sf, kpa, N, z) {
        (kpa - 2)/2 * (1 - sf)^((kpa - 4)/2) * (1 - sf * z)^(-(N - 
            1)/2)
    }
    dotargs = match.call(expand.dots = FALSE)$...
    bmao = x
    if (!is.bma(bmao)) 
        stop("Argument bmao needs to be a bma object")
    if (hnbsteps%%2) 
        stop("Argument nbsteps needs to be an even integer")
    nbsteps = max(hnbsteps, 2)
    n = max(ceiling(n), 1)
    N = bmao$info$N
    K = bmao$info$K
    if (is.null(reg)) 
        reg = 1:K
    nameix = 1:K
    names(nameix) = bmao$reg.names
    reg = nameix[reg]
    ishyper = (bmao$gprior$gtype == "hyper")
    tm = bmao$topmod
    bools = (tm$bool_binary())
    betas = tm$betas()
    betas2 = tm$betas2()
    if (std.coefs) {
        sddata = apply(as.matrix(bmao$X.data), 2, stats::sd)
        betas = diag(sddata[-1]) %*% betas/sddata[1]
        betas2 = diag(sddata[-1]^2) %*% betas2/sddata[1]^2
    }
    sigmadiag = (betas2 - betas^2) * (N - 3)/(N - 1)
    pmps = pmp.bma(bmao$topmod, oldstyle = TRUE)[, 1]
    pips = c(tcrossprod(bools, t(pmps)))
    Eb1 = c(tcrossprod(betas, t(pmps)))/pips
    Ebsd = sqrt(c(tcrossprod(betas2, t(pmps)))/pips - Eb1^2)
    Ebsd[is.nan(Ebsd)] = 0
    Eb1[is.nan(Eb1)] = 0
    Eball = cbind(Eb1, Ebsd)
    if ((any(grep("E", addons, ignore.case = FALSE))) | (any(grep("S", 
        addons, ignore.case = FALSE)))) {
        Eb1.mcmc = bmao$info$b1mo/bmao$info$inccount
        Ebsd.mcmc = sqrt(bmao$info$b2mo/bmao$info$inccount - 
            Eb1.mcmc^2)
        if (std.coefs) {
            sddata = apply(as.matrix(bmao$X.data), 2, stats::sd)
            Eb1.mcmc = Eb1.mcmc * sddata[-1]/sddata[1]
            Ebsd.mcmc = Ebsd.mcmc * sddata[-1]/sddata[1]
        }
    }
    if (ishyper) {
        yXdata = as.matrix(bmao$X.data)
        yXdata = yXdata - matrix(colMeans(yXdata), N, K + 1, 
            byrow = TRUE)
        if (std.coefs) 
            yXdata = yXdata %*% diag(1/sddata)
        yty = c(crossprod(yXdata[, 1]))
        positions = lapply(lapply(as.list(as.data.frame(bools)), 
            as.logical), which)
        olsmodels = lapply(lapply(positions, .ols.terms2, yty = yty, 
            N = N, K = K, XtX.big = crossprod(yXdata[, -1]), 
            Xty.big = c(crossprod(yXdata[, -1], yXdata[, 1]))), 
            function(x) x$full.results())
        f21a = bmao$gprior.info$hyper.parameter
    }
    plotndens <- function(ix, doplot = FALSE) {
        sss = function(lbound, uboundp1, nbsteps) {
            s.seq = seq(lbound, uboundp1, (uboundp1 - lbound)/nbsteps)[-nbsteps]
            tmat = sapply(as.list(s.seq), function(ss) {
                dtcm(seqs, N - 1, ss * bhati, invdiagi * ss * 
                  (1 - ss * z)/(N - 1) * yty)
            })
            smat = sapply(as.list(s.seq), dsgivenykernel, kpa = k + 
                f21a, N = N, z = z)
            if (any(is.infinite(smat))) 
                smat[is.infinite(smat)] = 0
            intconst = (4 * sum(smat[c(FALSE, TRUE)]) + 2 * sum(smat[c(TRUE, 
                FALSE)]) - 3 * smat[nbsteps] - smat[1]) * (s.seq[nbsteps] - 
                s.seq[1])/nbsteps/3
            return(list(dv = c(4 * tmat[, c(FALSE, TRUE)] %*% 
                smat[c(FALSE, TRUE)] + 2 * tmat[, c(TRUE, FALSE)] %*% 
                smat[c(TRUE, FALSE)] - 3 * tmat[, nbsteps] * 
                smat[nbsteps] - tmat[, 1] * smat[1]) * (s.seq[nbsteps] - 
                s.seq[1])/nbsteps/3, ic = intconst))
        }
        if (pips[ix] == 0) {
            reslist = list(x = numeric(n), y = numeric(n), n = n, 
                call = sys.call(), data.name = names(nameix)[ix], 
                has.na = FALSE)
            class(reslist) = c("density", "coef.density")
            return(reslist)
        }
        lbound = min(betas[ix, as.logical(bools[ix, ])]) - 3 * 
            Eball[ix, 2]
        ubound = max(betas[ix, as.logical(bools[ix, ])]) + 3 * 
            Eball[ix, 2]
        seqs = seq(lbound, ubound, (ubound - lbound)/(n - 1))
        densvec = numeric(length(seqs))
        for (m in 1:length(pmps)) {
            if (bools[ix, m]) {
                if (ishyper) {
                  ixadj = sum(bools[1:ix, m])
                  bhati = olsmodels[[m]]$bhat[[ixadj]]
                  invdiagi = olsmodels[[m]]$diag.inverse[[ixadj]]
                  k = sum(bools[, m])
                  Esf = betas[ix, m]/bhati
                  z = 1 - olsmodels[[m]]$ymy/yty
                  midpoint = 1 - (1 - Esf) * 4
                  if (midpoint < 0.5) {
                    dvl = sss(1e-04, 0.9999999, nbsteps * 2)
                    addvec = dvl$dv/dvl$ic
                  }
                  else {
                    dvl1 = sss(1e-04, midpoint, nbsteps)
                    dvl2 = sss(midpoint, 1, nbsteps)
                    addvec = (dvl1$dv + dvl2$dv)/(dvl1$ic + dvl2$ic)
                  }
                }
                else {
                  addvec = dtcm(seqs, N - 1, betas[ix, m], sigmadiag[ix, 
                    m])
                }
                densvec = densvec + pmps[m] * addvec
            }
        }
        reslist = list(x = seqs, y = densvec, bw = NULL, n = n, 
            call = sys.call(), data.name = names(nameix)[ix], 
            has.na = FALSE)
        class(reslist) = "density"
        if (!doplot) {
            return(reslist)
        }
        main_default = paste("Marginal Density:", names(nameix)[ix], 
            "(PIP", round(c(crossprod(pmps, bools[ix, ])) * 100, 
                2), "%)")
        if (any(grep("p", addons, ignore.case = TRUE))) {
            decr = 0.12
            parplt = par()$plt
            parplt_temp = parplt
            parplt_temp[4] = (1 - decr) * parplt[4] + decr * 
                parplt[3]
            par(plt = parplt_temp)
            main_temp = main_default
            main_default = NULL
        }
        dotargs = .adjustdots(dotargs, type = "l", col = "steelblue4", 
            main = main_default, xlab = if (std.coefs) 
                "Standardized Coefficient"
            else "Coefficient", ylab = "Density")
        eval(as.call(c(list(as.name("plot"), x = as.name("seqs"), 
            y = as.name("densvec")), as.list(dotargs))))
        leg.col = numeric(0)
        leg.lty = numeric(0)
        leg.legend = character(0)
        if (any(grep("g", addons, ignore.case = TRUE))) {
            grid()
        }
        if (any(grep("b", addons, ignore.case = TRUE))) {
            for (m in 1:length(pmps)) {
                Ebm = betas[ix, m]
                if (as.logical(Ebm)) {
                  Ebheight = min(densvec[max(sum(seqs < Ebm), 
                    1)], densvec[sum(seqs < Ebm) + 1])
                  lines(x = rep(Ebm, 2), y = c(0, Ebheight), 
                    col = 8)
                }
            }
            leg.col = c(leg.col, 8)
            leg.lty = c(leg.lty, 1)
            leg.legend = c(leg.legend, "EV Models")
        }
        if (any(grep("e", addons, ignore.case = FALSE))) {
            abline(v = Eball[ix, 1], col = 2, lwd = addons.lwd)
            leg.col = c(leg.col, 2)
            leg.lty = c(leg.lty, 1)
            leg.legend = c(leg.legend, "Cond. EV")
        }
        if (any(grep("s", addons, ignore.case = FALSE))) {
            abline(v = Eball[ix, 1] - 2 * Eball[ix, 2], col = 2, 
                lty = 2, lwd = addons.lwd)
            abline(v = Eball[ix, 1] + 2 * Eball[ix, 2], col = 2, 
                lty = 2, lwd = addons.lwd)
            leg.col = c(leg.col, 2)
            leg.lty = c(leg.lty, 2)
            leg.legend = c(leg.legend, "2x Cond. SD")
        }
        if (any(grep("m", addons, ignore.case = TRUE))) {
            median_index = sum(cumsum(densvec) < sum(densvec)/2)
            abline(v = (seqs[median_index] + seqs[median_index + 
                1])/2, col = 3, lwd = addons.lwd)
            leg.col = c(leg.col, 3)
            leg.lty = c(leg.lty, 1)
            leg.legend = c(leg.legend, "Median")
        }
        if (any(grep("z", addons, ignore.case = TRUE))) {
            abline(h = 0, col = "gray", lwd = addons.lwd)
        }
        if (any(grep("E", addons, ignore.case = FALSE))) {
            abline(v = Eb1.mcmc[ix], col = 4, lwd = addons.lwd)
            leg.col = c(leg.col, 4)
            leg.lty = c(leg.lty, 1)
            leg.legend = c(leg.legend, "Cond. EV (MCMC)")
        }
        if (any(grep("S", addons, ignore.case = FALSE))) {
            abline(v = Eb1.mcmc[ix] - 2 * Ebsd.mcmc[ix], col = 4, 
                lty = 2, lwd = addons.lwd)
            abline(v = Eb1.mcmc[ix] + 2 * Ebsd.mcmc[ix], col = 4, 
                lty = 2, lwd = addons.lwd)
            leg.col = c(leg.col, 4)
            leg.lty = c(leg.lty, 2)
            leg.legend = c(leg.legend, "2x SD (MCMC)")
        }
        if (any(grep("l", addons, ignore.case = TRUE)) & (length(leg.col) > 
            0)) {
            leg.pos = "topright"
            if (Eball[ix, 1] > seqs[floor(n/2)]) 
                leg.pos = "topleft"
            legend(x = leg.pos, lty = leg.lty, col = leg.col, 
                legend = leg.legend, box.lwd = 0, bty = "n", 
                lwd = addons.lwd)
        }
        if (any(grep("p", addons, ignore.case = TRUE))) {
            pusr = par()$usr
            rect(pusr[1], pusr[4] * (1 + decr * 0.2), pusr[2], 
                pusr[4] * (1 + decr), xpd = TRUE, col = 8)
            rect(pusr[1], pusr[4] * (1 + decr * 0.2), pips[ix] * 
                pusr[2] + (1 - pips[ix]) * pusr[1], pusr[4] * 
                (1 + decr), xpd = TRUE, col = 9)
            mtext("PIP:", side = 2, las = 2, line = 1, at = pusr[4] * 
                (1 + decr * 0.6))
            par(plt = parplt)
            title(main_temp)
        }
        return(reslist)
    }
    densres = list()
    oldask = par()$ask
    plots = 0
    for (vbl in 1:length(reg)) {
        doplot = (if (as.logical(pips[reg[vbl]])) 
            plot
        else FALSE)
        plots = plots + doplot
        if (plots == 2) {
            par(ask = TRUE)
        }
        densres[[nameix[vbl]]] = plotndens(reg[vbl], doplot)
        densres[[nameix[vbl]]]$call = sys.call()
    }
    par(ask = oldask)
    if (length(densres) == 1) 
        densres = densres[[1]]
    else class(densres) = c("coef.density", class(densres))
    if (!plot) 
        return(densres)
    if (plot & (plots == 0)) {
        warning("No plot produced as PIPs of provided variables are zero under 'exact' estimation.")
    }
    return(invisible(densres))
}

image.bma <-
function (x, yprop2pip = FALSE, order.by.pip = TRUE, do.par = TRUE, 
    do.grid = TRUE, do.axis = TRUE, cex.axis = 1, ...) 
{
    dotargs = match.call(expand.dots = FALSE)$...
    ests = estimates.bma(x, exact = TRUE, order.by.pip = order.by.pip, 
        include.constant = FALSE)
    ests = ests[nrow(ests):1, ]
    pips = ests[, "PIP"]
    idx = ests[, "Idx"]
    pmp.res = pmp.bma(x, oldstyle = TRUE)
    pmps = pmp.res[, 1]
    normali_factor = sum(pmp.res[, 2])
    betasigns = beta.draws.bma(x)[idx, , drop = FALSE]
    betasigns = betasigns[as.logical(pips), ]
    betasigns = sign(betasigns)/2 + 0.5
    betasigns[betasigns == 0.5] = NA
    pips = pips[as.logical(pips)]
    if (yprop2pip) {
        pipbounds = (c(0, cumsum(pips)))
    }
    else {
        pipbounds = 0:length(pips)
        names(pipbounds) = c("", names(pips))
    }
    pmpbounds = (c(0, cumsum(pmps)))
    if (do.par) {
        oldmar = par()$mar
        spaceforyaxis = strwidth(names(pipbounds)[which.max(nchar(names(pipbounds)))], 
            units = "inches") * (par("mar")/par("mai"))[[2]]
        tempmar = oldmar
        tempmar[2] = min(spaceforyaxis + oldmar[2]/2, 0.5 * par("fin")[[1]] * 
            (par("mar")/par("mai"))[[2]])
        par(mar = tempmar)
    }
    dotargs = .adjustdots(dotargs, ylab = "", xlab = "Cumulative Model Probabilities", 
        col = c("tomato", "blue"), main = paste("Model Inclusion Based on Best ", 
            length(pmps), " Models"))
    dotargs$axes <- FALSE
    tbetasigns = t(betasigns)
    eval(as.call(c(list(as.name("image.default"), as.name("pmpbounds"), 
        as.name("pipbounds"), as.name("tbetasigns")), as.list(dotargs))))
    if (do.axis) {
        axis(1, at = pmpbounds, labels = round(normali_factor * 
            pmpbounds, 2), cex.axis = cex.axis)
        axis(2, at = pipbounds, labels = FALSE, line = FALSE)
        axis(2, at = pipbounds[-1] - diff(pipbounds)/2, labels = names(pipbounds[-1]), 
            tick = FALSE, las = 1, cex.axis = cex.axis)
    }
    if (do.grid) {
        abline(v = round(pmpbounds, 2), lty = "dotted", col = "grey")
        abline(h = round(pipbounds, 2), lty = "dotted", col = "grey")
    }
    if (do.par) {
        par(mar = oldmar)
    }
}

plot.pred.density <-
function (x, predict_index = NULL, addons = "eslz", realized.y = NULL, 
    addons.lwd = 1.5, ...) 
{
    if (!is(x, "pred.density")) 
        stop("x must be of class 'pred.density'!")
    x$plot(predict_index, realized.y = realized.y, addons = addons, 
        addons.lwd = addons.lwd, ...)
}

plotComp <-
function (..., varNr = NULL, comp = "PIP", exact = FALSE, include.legend = TRUE, 
    add.grid = TRUE, do.par = TRUE, cex.xaxis = 0.8) 
{
    col_default = c("#1B9E77", "#D95F02", "#7570B3", "#E7298A", 
        "#66A61E", "#E6AB02", "#A6761D", "#666666")
    bmaList = list(...)
    bmaix = sapply(bmaList, is.bma)
    bmaList = bmaList[bmaix]
    bmaNr = length(bmaList)
    if (!all(sapply(bmaList, is.bma))) {
        stop("Submit only bma objects to compare results (no other objects)")
    }
    dotargs = match.call(expand.dots = FALSE)$...
    dotargs = .adjustdots(dotargs, ylab = paste(comp), pch = 1:bmaNr, 
        col = col_default, type = "p", lty = 1:5, lwd = 1.5, 
        xlab = "", xaxt = "n")
    dotargs = dotargs[!c(bmaix, logical(length(dotargs) - length(bmaix)))]
    xMat = lapply(bmaList, function(x) rownames(estimates.bma(x, 
        exact = exact)))
    xNames = xMat[[1]]
    ind = as.numeric(unlist(lapply(xMat, function(x) length(x))))
    if (length(unique(ind) > 1)) {
        smallestSet = which.min(ind)
        indMat = array(0:0, dim = c(length(xMat[[smallestSet]]), 
            length(ind)))
        for (i in 1:length(ind)) {
            indMat[, i] = as.numeric(xMat[[smallestSet]] %in% 
                xMat[[i]])
        }
        xNamesInd = which(rowSums(indMat) == bmaNr)
        xNames = xMat[[smallestSet]][xNamesInd]
    }
    compNames = c(colnames(estimates.bma(bmaList[[1]])), "Std Mean", 
        "Std Coef")
    if (is.null(xNames)) {
        stop("the bma objects have to have (the same) rownames attached to them")
    }
    if (!(comp %in% compNames)) {
        stop("Please specify comp as one of PIP, Post Mean, Post SD, Std Mean, or Std Coef")
    }
    if (comp == "Std Mean") {
        compMatrix = sapply(bmaList, function(x) estimates.bma(x, 
            std.coefs = TRUE, exact = exact)[xNames, "Post Mean"])
        comp = "Standardized Coefficients"
    }
    else if (comp == "Std SD") {
        compMatrix = sapply(bmaList, function(x) estimates.bma(x, 
            std.coefs = TRUE, exact = exact)[xNames, "Post SD"])
        comp = "Standardized SD"
    }
    else {
        compMatrix = sapply(bmaList, function(x) estimates.bma(x, 
            exact = exact)[xNames, comp])
    }
    bmaNames = names(list(...))[bmaix]
    colnames(compMatrix) = paste("Model", 1:bmaNr)
    if (!is.null(bmaNames) && (length(bmaNames) == ncol(compMatrix))) {
        for (bix in 1:bmaNr) {
            colnames(compMatrix)[[bix]] <- ifelse(bmaNames[[bix]] == 
                "", paste("Model", bix), bmaNames[[bix]])
        }
    }
    if (!is.null(varNr)) {
        compMatrix = compMatrix[varNr, , drop = FALSE]
    }
    if (as.logical(do.par)) {
        oldmar = par()$mar
        spaceforxaxis = strwidth(rownames(compMatrix)[which.max(nchar(rownames(compMatrix)))], 
            units = "inches", cex = cex.xaxis) * (par("mar")/par("mai"))[[2]]
        tempmar = oldmar
        tempmar[1] = min(max(oldmar[1], spaceforxaxis + oldmar[1]/3), 
            0.5 * par("fin")[[2]] * (par("mar")/par("mai"))[[1]])
        par(mar = tempmar)
    }
    eval(as.call(c(list(as.name("matplot"), as.name("compMatrix")), 
        as.list(dotargs))))
    if (as.logical(include.legend)) {
        extractfromdotargs = function(...) {
            dal = list(...)
            return(list(col = dal$col, pch = dal$pch))
        }
        myargs = eval(as.call(c(list(as.name("extractfromdotargs")), 
            as.list(dotargs))))
        legend("topright", colnames(compMatrix), pch = myargs$pch, 
            col = myargs$col, bty = "n")
    }
    if (as.logical(add.grid)) 
        grid()
    axis(1, las = 2, at = 1:nrow(compMatrix), labels = rownames(compMatrix), 
        cex.axis = cex.xaxis)
    if (as.logical(do.par)) 
        par(mar = oldmar)
}

lps.bma <-
function (object, realized.y, newdata = NULL) 
{
    if (!any(class(object) %in% c("pred.density", "bma", "zlm"))) 
        stop("object must be of class 'pred.density', 'bma' or 'zlm'!")
    if (any(class(object) %in% c("bma", "zlm"))) {
        if (is.null(newdata)) 
            stop("newdata must be provided if object is of class 'bma' or 'zlm'.")
        object = pred.density(object, newdata = newdata)
    }
    return(object$lps(realized.y))
}

plotModelsize <-
function (bmao, exact = FALSE, ksubset = NULL, include.legend = TRUE, 
    do.grid = TRUE, ...) 
{
    dotargs = match.call(expand.dots = FALSE)$...
    if (length(exact) > 1) {
        topmodidx = exact
        exact = TRUE
    }
    else {
        topmodidx = NA
    }
    K = bmao$info$K
    if (is.element("mprior.info", names(bmao))) 
        m = bmao$mprior.info$mp.msize
    else m = bmao$arguments$prior.msize
    pmp.10 = pmp.bma(bmao$topmod[topmodidx], oldstyle = TRUE)
    if (exact) {
        modelSmean = sum(apply(.post.topmod.bma(bmao$topmod[topmodidx]), 
            2, function(x) length(which(x == 1))) * pmp.10[, 
            1])
        modelS.var = sum(apply(.post.topmod.bma(bmao$topmod[topmodidx]), 
            2, function(x) length(which(x == 1)))^2 * pmp.10[, 
            1]) - modelSmean^2
        x = apply(.post.topmod.bma(bmao$topmod[topmodidx]), 2, 
            function(x) length(which(x == 1)))
        y = pmp.10[, 1]
        result = c()
        for (i in sort(unique(x))) result = c(result, sum(y[which(x == 
            i)]))
        names(result) = sort(unique(x))
        kvec = rep(0, (K + 1))
        kvec[(as.numeric(names(result)) + 1)] = result
    }
    else {
        k.vec = bmao$info$k.vec
        summi = sum(k.vec)
        modelSmean = sum((1:length(k.vec)) * (k.vec/summi)) - 
            1
        kvec = k.vec/sum(k.vec)
        modelSmean.sq = sum(((1:length(k.vec))^2) * (k.vec/summi))
        modelS.var = modelSmean.sq - modelSmean^2
    }
    upper = min(ceiling(modelSmean + 5 * modelS.var), K)
    lower = max(floor(modelSmean - 5 * modelS.var), 0)
    if (is.element("mp.Kdist", names(bmao$mprior.info))) {
        prior = bmao$mprior.info$mp.Kdist
    }
    else if (is.element("theta", names(bmao$arguments))) {
        theta = bmao$arguments$theta
        if (theta == "random") {
            beta.bin = function(a = 1, b = (K - m)/m, K = K, 
                w = 0:K) {
                return(lgamma(a + b) - (lgamma(a) + lgamma(b) + 
                  lgamma(a + b + K)) + log(choose(K, w)) + lgamma(a + 
                  w) + lgamma(b + K - w))
            }
            prior = exp(beta.bin(a = 1, b = (K - m)/m, K = K, 
                w = 0:K))
        }
        if (theta != "random") {
            prior = dbinom(x = 0:K, size = K, prob = m/K, log = FALSE)
        }
    }
    else {
        prior = rep(NA, length(kvec))
    }
    mat = cbind(kvec, prior)
    upper.ylim = max(kvec, prior, na.rm = TRUE)
    if (is.null(ksubset)) {
        ksubset = (lower:upper)
    }
    dotargs = .adjustdots(dotargs, type = "l", ylim = c(0, 1.1 * 
        upper.ylim), lwd = 1.5, xaxt = "n", col = c("steelblue3", 
        "tomato"), main = paste("Posterior Model Size Distribution", 
        "\n", "Mean:", round(modelSmean, 4)), cex.main = 0.8, 
        xlab = "Model Size", ylab = "", lty = 1:2, pch = 4, cex.axis = 0.9)
    matsubset = mat[ksubset + 1, ]
    eval(as.call(c(list(as.name("matplot"), as.name("matsubset")), 
        as.list(dotargs))))
    if (as.logical(do.grid)) 
        grid()
    points(kvec[ksubset + 1], cex = 0.8, pch = eval(dotargs$pch))
    axis(1, las = 1, at = 1:length(ksubset), labels = ksubset, 
        cex.axis = eval(dotargs$cex.axis))
    if (include.legend) {
        if (is.null(prior) || all(is.na(prior))) {
            legend(x = "topright", lty = eval(dotargs$lty), legend = c("Posterior"), 
                col = eval(dotargs$col), ncol = 1, bty = "n", 
                lwd = eval(dotargs$lwd))
        }
        else {
            legend(x = "topright", lty = eval(dotargs$lty), legend = c("Posterior", 
                "Prior"), col = eval(dotargs$col), ncol = 2, 
                bty = "n", lwd = eval(dotargs$lwd))
        }
    }
    return(invisible(list(mean = modelSmean, var = modelS.var, 
        dens = kvec)))
}

print.pred.density <-
function (x, digits = NULL, ...) 
{
    outmat = matrix(numeric(0), length(x$fit), 2)
    colnames(outmat) = c("Exp.Val.", "Std.Err.")
    rownames(outmat) = names(x$fit)
    outmat[, 1] = x$fit
    outmat[, 2] = x$std.err
    cat("Call:\n")
    print(x$call)
    cat(paste("\nDensities for conditional forecast(s)\n", x$n, 
        " data points, based on ", x$nmodel, " models;\n", sep = ""))
    print(outmat, digits = digits, ...)
}

info.bma <-
function (object, ...) 
{
    bmao = object
    rm(object)
    foo = bmao$info
    iter = foo$iter
    burn = foo$burn
    timed = foo$timed
    models.visited = foo$models.visited
    corr.pmp = foo$corr.pmp
    K = foo$K
    N = foo$N
    msize = foo$msize
    cumsumweights = foo$cumsumweights
    if (is.element("mprior.info", names(bmao))) {
        prior = paste(bmao$mprior.info$mp.mode, "/", bmao$mprior.info$mp.msize)
    }
    else {
        if (is.element("theta", names(bmao$arguments)) && is.element("prior.msize", 
            names(bmao$arguments))) {
            if (!is.null(bmao$arguments$theta) & !is.null(bmao$arguments$prior.msize)) 
                prior = paste(bmao$arguments$theta, "/", bmao$arguments$prior.msize)
            else prior = NA
        }
        else {
            prior = paste(bmao$arguments$mprior, "/", bmao$arguments$mprior.size)
        }
    }
    gprior.info = bmao$gprior.info
    gprior.choice = gprior.info$gtype
    model.space = 2^K
    fraction.model = models.visited/model.space * 100
    fraction.topmodel = sum(bmao$topmod$ncount())/iter * 100
    if (gprior.info$gtype == "hyper") {
        gprior.choice = paste(gprior.choice, " (a=", 2 + signif(gprior.info$hyper.parameter - 
            2, digits = 4), ")", sep = "")
    }
    nr.reg = msize/cumsumweights
    info <- as.character(c(format(round(nr.reg, 4), nsmall = 4), 
        format(iter, nsmall = 0), format(burn, nsmall = 0), format(timed, 
            nsmall = 4), models.visited, format(model.space, 
            digits = 2), format(fraction.model, digits = 2), 
        format(fraction.topmodel, digits = 2), format(round(.cor.topmod(bmao$topmod), 
            4), nsmall = 4), format(N, nsmall = 4), prior, gprior.choice))
    names(info) <- c("Mean no. regressors", "Draws", "Burnins", 
        "Time", "No. models visited", "Modelspace 2^K", "% visited", 
        "% Topmodels", "Corr PMP", "No. Obs.", "Model Prior", 
        "g-Prior")
    if (gprior.info$return.g.stats) {
        gpriorav = gprior.info$shrinkage.moments[1]
        gstatsprint = paste("Av=", format(gpriorav, digits = 4), 
            sep = "")
        if (length(gprior.info$shrinkage.moments) > 1) {
            gpriorstdev = sqrt(gprior.info$shrinkage.moments[2] - 
                gprior.info$shrinkage.moments[1]^2)
            gstatsprint = paste(gstatsprint, ", Stdev=", format(gpriorstdev, 
                digits = 2), sep = "")
        }
        info <- c(info, gstatsprint)
        names(info)[13] <- "Shrinkage-Stats"
    }
    return(info)
}

summary.zlm <-
function (object, printout = TRUE, ...) 
{
    betas = object$coefficients
    betas2 = object$coef2moments
    sds = sqrt(betas2 - betas^2)
    ests = cbind(betas, sds)
    gi = object$gprior.info
    gi.choice = gi$gtype
    if (gi$gtype == "hyper") {
        gi.choice = paste(gi$gtype, " (a=", 2 + signif(gi$hyper.parameter - 
            2, digits = 4), ")", sep = "")
    }
    gi.sd = -1
    gi.sdtext = ""
    if (length(gi$shrinkage.moments) > 1) {
        gi.sd = sqrt(gi$shrinkage.moments[[2]] - gi$shrinkage.moments[[1]]^2)
        gi.sdtext = paste(" St.Dev.:", round(gi.sd, 3))
    }
    rownames(ests) = c("(Intercept)", attr(object$terms, "term.labels"))
    colnames(ests) = c("Exp.Val.", "St.Dev.")
    cat("Coefficients\n")
    print(ests)
    cat("\n Log Marginal Likelihood:\n")
    cat(object$marg.lik)
    cat(paste("\n g-Prior:", gi.choice, "\n"))
    cat(paste("Shrinkage Factor", ifelse(gi$is.constant, ": ", 
        " Exp.Val: "), round(gi$shrinkage.moments[[1]], 3), gi.sdtext, 
        "\n", sep = ""))
    res = list()
    res$residuals <- object$residuals
    res$coefficients <- object$coefficients
    res$coef.sd <- sds
    res$gprior <- gi.choice
    res$E.shrinkage <- gi$shrinkage.moments[[1]]
    if (gi.sd > -1) {
        res$SD.shrinkage <- gi.sd
    }
    res$log.lik <- object$marg.lik
    return(invisible(res))
}

print.topmod <-
function (x, ...) 
{
    tm = x
    if (length(tm$lik()) == 1) {
        infomat = c(tm$bool(), tm$lik(), tm$ncount())
        names(infomat) = c("Model Index", "Marg.Log.Lik.", "Sampled Freq.")
        print(infomat)
        betamat = cbind(as.vector(tm$betas_raw()), sqrt(as.vector(tm$betas2_raw()) - 
            as.vector(tm$betas_raw())^2))
        if (nrow(betamat) != 0) {
            if (ncol(betamat) == 1) {
                colnames(betamat) = "Coef."
            }
            else {
                colnames(betamat) = c("Coef.", "Std.Dev.")
            }
            rownames(betamat) = which(as.logical(as.vector(tm$bool_binary())))
            cat("\nEstimates:\n")
            print(betamat)
        }
        bin = as.vector(tm$bool_binary())
        names(bin) = 1:length(bin)
        cat("\nIncluded Covariates:\n")
        print(bin)
        cat("\nAdditional Statistics:\n")
        print(as.vector(tm$fixed_vector()))
    }
    else {
        mout = cbind(tm$lik(), tm$ncount())
        colnames(mout) = c("Marg.Log.Lik", "MCMC Freq")
        rownames(mout) = tm$bool()
        print(mout, ...)
    }
}

.adjustdots <-
function (dotargs, ...) 
{
    defargs = list(...)
    defargnames = names(defargs)
    dotargs = as.list(dotargs)
    if (is.null(dotargs)) {
        dotargs = list()
    }
    for (di in seq_len(length(defargs))) {
        if (!is.element(defargnames[[di]], names(dotargs))) {
            dotargs[[defargnames[[di]]]] <- defargs[[di]]
        }
    }
    return(dotargs)
}
.choose.gprior <-
function (g, N, K, return.g.stats = FALSE, yty = N, ...) 
{
    if (is.list(g)) {
        if (!all(c("gtype", "is.constant", "return.g.stats", 
            "lprobcalc") %in% names(g))) 
            stop("The provided g-prior list (in argument 'g') does not conform to the standards of a g-prior object.")
        if (!("g" %in% names(g))) {
            g$is.constant = FALSE
            g$g = NA
        }
        if (!("shrinkage.moments" %in% names(g))) {
            g$shrinkage.moments = ifelse((g$is.constant && is.numeric(g)), 
                g/(1 + g), 0)
        }
        if (!all(sapply(g$lprobcalc, is.function))) 
            stop("The slot 'lprobcalc' in the provided g-prior list (in argument 'g') does not conform to the standards of a g-prior object.")
        return(g)
    }
    if (is.function(g)) {
        return(g(g = g, return.g.stats = return.g.stats, N = N, 
            K = K, yty = yty, ...))
    }
    if (is.numeric(g)) {
        return(.gprior.constg.init(g = g, return.g.stats = return.g.stats, 
            N = N, K = K, yty = yty))
    }
    if (any(grep("EBL", g, ignore.case = TRUE))) {
        return(.gprior.eblocal.init(g = g, return.g.stats = return.g.stats, 
            N = N, K = K, yty = yty))
    }
    if (any(grep("hyper", g, ignore.case = TRUE))) {
        return(.gprior.hyperg.init(g = g, return.g.stats = return.g.stats, 
            N = N, K = K, yty = yty))
    }
    return(.gprior.constg.init(g = g, return.g.stats = return.g.stats, 
        N = N, K = K, yty = yty))
}
.choose.mprior <-
function (mpmode, mpparam, K, ..., fixed.pos = numeric(0)) 
{
    origargs = list(mpmode = mpmode, mpparam = mpparam)
    fixed.pos = {
        1:K
    }[fixed.pos]
    fixed.exist = as.logical(length(fixed.pos))
    fixk = length(fixed.pos)
    if (!(is.character(mpmode) || is.function(mpmode) || is.list(mpmode))) 
        stop("'mprior' parameter must be character! (or function/list)")
    if (is.function(mpmode) || is.list(mpmode)) {
        mpinfo = mpmode
    }
    else if (any(grep("fix", mpmode, ignore.case = TRUE))) {
        mpinfo = .mprior.fixedt.init
        if (is.numeric(mpparam)) 
            mpparam = mpparam[[1]] - fixk
    }
    else if (any(grep("unif", mpmode, ignore.case = TRUE))) {
        mpinfo = .mprior.uniform.init
    }
    else if (any(grep("custom", mpmode, ignore.case = TRUE))) {
        mpinfo = .mprior.customk.init
        if (fixed.exist && is.numeric(mpparam)) 
            if (length(mpparam) == K + 1) 
                mpparam = mpparam[(fixk + 1):length(mpparam)]
    }
    else if (any(grep("pip", mpmode, ignore.case = TRUE))) {
        mpinfo = .mprior.pip.init
        if (fixed.exist && is.numeric(mpparam)) 
            if (length(mpparam) == K) 
                mpparam = mpparam[-fixed.pos]
    }
    else {
        mpinfo = .mprior.randomt.init
        if (is.numeric(mpparam)) 
            mpparam = mpparam[[1]] - fixk
    }
    if (is.function(mpinfo)) 
        mpinfo = .fixedset.mprior(mpinfo, fullK = K, fixed.pos = fixed.pos, 
            K = NA, mpparam = mpparam, mpmode = mpmode, ...)
    if (!all(c("mp.mode", "mp.msize", "pmp") %in% names(mpinfo))) 
        stop("The provided custom-built model prior is deficient.")
    if (!("origargs" %in% names(mpinfo))) 
        mpinfo$origargs = origargs
    if (length(fixed.pos) > 0) 
        mpinfo$fixed.pos = fixed.pos
    class(mpinfo) <- c("mprior", class(mpinfo))
    return(mpinfo)
}
.constr.intmat <-
function (X, K) 
{
    intix = grep("#", colnames(X), fixed = TRUE)
    mPlus = diag(K)
    colnames(mPlus) <- colnames(X)
    for (jj in 1:length(intix)) {
        cix = intix[jj]
        mPlus[cix, unlist(strsplit(colnames(mPlus)[cix], "#", 
            fixed = TRUE))] = 1
    }
    return(mPlus)
}
.construct.arglist <-
function (funobj, envir = NULL) 
{
    namedlist = formals(funobj)
    argnames = names(namedlist)
    if (!is.environment(envir)) 
        envir = sys.frame(-1)
    for (argn in 1:length(namedlist)) {
        testval = as.logical(try(exists(argnames[argn], envir = envir), 
            silent = TRUE))
        if (is.na(testval)) 
            testval = FALSE
        if (testval) {
            namedlist[[argn]] = try(get(argnames[argn], envir = envir))
        }
    }
    return(namedlist)
}
.cor.topmod <-
function (tmo) 
{
    if (is.bma(tmo)) 
        tmo = tmo$topmod
    pmp.10 = pmp.bma(tmo, oldstyle = TRUE)
    if (nrow(pmp.10) == 1 | suppressWarnings(length(grep("error", 
        class(try(cor(pmp.10[, 1], pmp.10[, 2]), silent = TRUE)))))) {
        corr.pmp = NA
    }
    else {
        if (var(pmp.10[, 2]) == 0) 
            corr.pmp = NA
        else corr.pmp = cor(pmp.10[, 1], pmp.10[, 2])
    }
    return(corr.pmp)
}
.enum_fromindex <-
function (lindex) 
{
    lindex = lindex[[1]]
    if (lindex == 0) 
        return(FALSE)
    log2 = ceiling(log(lindex + 1, 2))
    return(as.logical((lindex + 2^((log2 - 1):0))%/%(2^(log2:1))%%2))
}
.enum_startend <-
function (iter = NA, start.value = 0, K = 1, maxk = K, fixed.pos = numeric(0)) 
{
    fixed.pos = {
        1:K
    }[fixed.pos]
    effk = K - length(fixed.pos)
    flexpos = {
        1:K
    }
    if (length(fixed.pos) > 0) 
        flexpos = {
            1:K
        }[-fixed.pos]
    start.value2 = 0
    if (length(start.value) == 1) {
        start.value2 = suppressWarnings(as.integer(start.value))
        if (any(is.na(start.value2)) | start.value2[[1]] < 0 | 
            start.value2[[1]] >= (2^effk - 1)) {
            start.value = 0
            start.value2 = 0
        }
    }
    else {
        start.value = 0
    }
    if (length(start.value) == 1) {
        start.value_cut = .enum_fromindex(start.value2)
        start.value = rep(1, K)
        start.value[flexpos] = c(numeric(effk - length(start.value_cut)), 
            start.value_cut)
    }
    if (K > maxk) {
        lastindex = 2^effk - 1 - sum(choose(effk, (maxk + 1):effk))
    }
    else lastindex = 2^effk - 1
    if (is.na(iter)) {
        iter = lastindex - start.value2
    }
    iter = min(iter, 2^effk - 1 - start.value2)
    return(list(start.value = start.value, iter = iter))
}
.f21_4hyperg <-
function (N, K, f21a, ltermbounds = c(200, 600, 1400, 3000)) 
{
    create.lterms = function(cc) lapply(as.list(ltermbounds), 
        function(x) (a + 0:x)/(cc + 0:x))
    a = (N - 1)/2
    cv = (f21a + 0:K)/2
    lterms = (lapply(cv, create.lterms))
    ltermbounds = c(0, ltermbounds[-length(ltermbounds)])
    return(list(calcit = function(z, k) {
        nbterms = sum(ceiling(abs({
            a - cv[k + 1]
        }/{
            1 - z
        }) * 1.5) >= ltermbounds)
        return(sum(cumprod(z * lterms[[k + 1]][[nbterms]])) + 
            1)
    }))
}
.f21simple <-
function (a, c, z) 
{
    f21o = .f21_4hyperg(2 * a + 1, 0, c * 2)
    f21o$calcit(z, 0)
}
.fixedset.mprior <-
function (mprior.function, fullK, fixed.pos = numeric(0), K = NA, 
    ...) 
{
    if (length(fixed.pos) == 0) 
        return(mprior.function(K = fullK, ...))
    fixed.pos = {
        1:fullK
    }[fixed.pos]
    flexpos = {
        1:fullK
    }[-fixed.pos]
    flexk = length(flexpos)
    mprior = mprior.function(K = flexk, ...)
    fixk = length(fixed.pos)
    mpl = list(mp.mode = mprior$mp.mode, mp.msize = mprior$mp.msize + 
        fixk, pmp = function(ki, mdraw, ...) {
        return(mprior$pmp(ki = ki - fixk, mdraw = mdraw[flexpos]))
    }, mp.Kdist = c(numeric(fixk), mprior$mp.Kdist))
    return(mpl)
}
.fixedset.sampler <-
function (sampler.function, fullK, fixed.pos = numeric(0), ...) 
{
    if (length(fixed.pos) == 0) 
        return(sampler.function)
    fixed.pos = {
        1:fullK
    }[fixed.pos]
    flexpos = {
        1:fullK
    }[-fixed.pos]
    flexk = length(flexpos)
    outdraw = rep(1, fullK)
    outfun = function(molddraw = molddraw, K = flexk, ...) {
        flexdraw = sampler.function(molddraw = molddraw[flexpos], 
            K = flexk, ...)
        outdraw[flexpos] = flexdraw[["mnewdraw"]]
        addi = flexdraw[["addi"]]
        dropi = flexdraw[["dropi"]]
        if (is.numeric(addi) || is.numeric(dropi)) {
            if (addi > 0) 
                addi = flexpos[addi]
            else addi = 0
            if (dropi > 0) 
                dropi = flexpos[dropi]
            else dropi = 0
        }
        return(list(mnewdraw = outdraw, positionnew = {
            1:fullK
        }[as.logical(outdraw)], addi = addi, dropi = dropi))
    }
    return(outfun)
}
.fls.samp <-
function (molddraw = molddraw, K = K, ..., maxk = Inf, oldk = 0) 
{
    indch <- ceiling(runif(1, 0, K))
    bdropit <- as.logical(molddraw[[indch]])
    if (oldk == maxk) 
        if (!bdropit) {
            indch = (1:K)[molddraw == 1][[ceiling(runif(1, 0, 
                sum(molddraw)))]]
            bdropit = molddraw[[indch]]
        }
    if (bdropit) {
        addvar <- 0
        dropvar <- indch
        molddraw[[indch]] <- 0
    }
    else {
        addvar <- indch
        dropvar <- 0
        molddraw[[indch]] <- 1
    }
    positionnew <- {
        1:K
    }[molddraw == 1]
    return(list(mnewdraw = molddraw, positionnew = positionnew, 
        addi = addvar, dropi = dropvar))
}
.fls.samp.int <-
function (molddraw = molddraw, K = K, mPlus = mPlus, maxk = Inf, 
    oldk = 0) 
{
    indch = ceiling(runif(1, 0, 1) * K)
    if (molddraw[indch] == 1) {
        mnewdraw = as.numeric(molddraw > mPlus[, indch])
        dropvar = (1:K)[xor(molddraw, mnewdraw)]
        addvar = 0
    }
    else {
        mnewdraw = as.numeric(molddraw | mPlus[indch, ])
        addvar = (1:K)[xor(molddraw, mnewdraw)]
        dropvar = 0
    }
    positionnew = which(mnewdraw == 1)
    if (length(positionnew) > maxk) {
        return(.fls.samp.int(molddraw = molddraw, K = K, mPlus = mPlus, 
            maxk, oldk))
    }
    else {
        return(list(mnewdraw = mnewdraw, positionnew = positionnew, 
            addi = addvar, dropi = dropvar))
    }
}
.getpolycoefs <-
function (polyroots) 
{
    if (length(polyroots) == 1) 
        return(c(1, polyroots))
    restterms = .getpolycoefs(polyroots[-1])
    c(restterms, 0) + c(0, polyroots[1] * restterms)
}
.gprior.constg.init <-
function (g = NA, return.g.stats = TRUE, N = N, K = K, yty = 1, 
    null.lik = NA, ...) 
{
    gg = NULL
    if (!(is.character(g) || is.numeric(g))) 
        g = "UIP"
    if (any(grep("BRIC", g, ignore.case = TRUE)) | any(grep("FLS", 
        g, ignore.case = TRUE))) {
        if (N <= (K^2)) {
            gg = (K^2)
        }
        else {
            gg = N
        }
        gtype = "BRIC"
    }
    if (any(grep("RIC", g, ignore.case = TRUE)) && (!any(grep("BRIC", 
        g, ignore.case = TRUE)))) {
        gg = (K^2)
        gtype = "RIC"
    }
    if (any(grep("HQ", g, ignore.case = TRUE)) | any(grep("Hannan", 
        g, ignore.case = TRUE))) {
        gg = (log(N))^3
        gtype = "Hannan-Quinn"
    }
    if (is.numeric(g)) {
        gg = g
        gtype = "numeric"
    }
    if (is.null(gg)) {
        if (!(any(grep("UIP", g, ignore.case = TRUE)) | any(grep("BIC", 
            g, ignore.case = TRUE)))) 
            warning("The provided g prior could not be identified. Therefore the default g prior (UIP) has been selected.")
        gg = N
        gtype = "UIP"
    }
    gprior.info = list(gtype = gtype, is.constant = TRUE, return.g.stats = return.g.stats, 
        shrinkage.moments = gg/(gg + 1), g = gg)
    g = gg
    if (!is.numeric(null.lik)) {
        null.lik = {
            1 - N
        }/2 * log(yty)
    }
    g2 = g/{
        g + 1
    }
    l1g = log(1 + g)
    g2sq = g2^2
    n1 = N - 1
    gprior.info$lprobcalc <- list(just.loglik = function(ymy, 
        k, ...) {
        return(0.5 * {
            {
                n1 - k
            } * l1g - n1 * log(g * ymy + yty)
        })
    }, lprob.all = function(ymy, k, bhat, diag.inverse, ...) {
        b1new = g2 * bhat
        return(list(lprob = 0.5 * {
            {
                n1 - k
            } * l1g - n1 * log(g * ymy + yty)
        }, b1new = b1new, b2new = {
            {
                yty/g + ymy
            } * g2sq/{
                N - 3
            }
        } * diag.inverse + b1new^2, otherstats = numeric(0)))
    })
    class(gprior.info) <- c("gprior", class(gprior.info))
    return(gprior.info)
}
.gprior.eblocal.init <-
function (g = NA, return.g.stats = TRUE, N = N, K = K, yty = 1, 
    null.lik = NA, ...) 
{
    gprior.info = list(gtype = "EBL", is.constant = FALSE, return.g.stats = return.g.stats, 
        shrinkage.moments = numeric(1), g = NA)
    if (!is.numeric(null.lik)) {
        null.lik = (1 - N)/2 * log(yty)
    }
    ymy.current = -1
    k.current = -1
    loglik = null.lik
    Fstat = numeric(0)
    g2 = 0
    if (return.g.stats) {
        otherstats = numeric(1)
    }
    else {
        otherstats = numeric(0)
    }
    gprior.info$lprobcalc <- list(just.loglik = function(ymy, 
        k, ...) {
        ymy.current <<- ymy
        k.current <<- k
        if (k == 0) {
            return(null.lik)
        }
        Fstat <<- (N - k - 1)/k * (yty - ymy)/ymy
        if (Fstat > 1) {
            g0 <- 1/(Fstat - 1)
            g2 <<- 1/(g0 + 1)
        } else {
            g2 <<- 0
            return(null.lik)
        }
        lFstat = log(Fstat)
        lg02 = -lFstat
        loglik <<- 0.5 * {
            k * lg02 - {
                N - 1
            } * {
                log(ymy + g0 * yty) + {
                  log(Fstat - 1) - lFstat
                }
            }
        }
        return(loglik)
    }, lprob.all = function(ymy, k, bhat, diag.inverse, ...) {
        if (k == 0) {
            return(list(lprob = null.lik, b1new = numeric(0), 
                b2new = numeric(0), otherstats = otherstats))
        }
        if ((ymy != ymy.current) | (k != k.current)) {
            Fstat <<- {
                N - k - 1
            }/k * (yty - ymy)/ymy
            if (Fstat > 1) {
                g0 = 1/{
                  Fstat - 1
                }
                g2 <<- 1/(g0 + 1)
                lFstat = log(Fstat)
                lg02 = -lFstat
                loglik <<- 0.5 * {
                  k * lg02 - {
                    N - 1
                  } * {
                    log(ymy + g0 * yty) + {
                      log(Fstat - 1) - lFstat
                    }
                  }
                }
            } else {
                g0 = 0
                g2 <<- 0
                loglik <<- null.lik
            }
        }
        if (return.g.stats) {
            otherstats = g2
        }
        b1new = g2 * bhat
        if (g2 > 0) {
            b2new = {
                {
                  (1/g2 - 1) * yty + ymy
                } * {
                  g2^2
                }/{
                  N - 3
                }
            } * diag.inverse + b1new^2
        } else {
            b2new = numeric(k)
        }
        return(list(lprob = loglik, b1new = b1new, b2new = b2new, 
            otherstats = otherstats))
    })
    class(gprior.info) <- c("gprior", class(gprior.info))
    return(gprior.info)
}
.gprior.hyperg.init <-
function (g = NA, return.g.stats = TRUE, N = N, K = K, yty = 1, 
    null.lik = NA, ...) 
{
    if (!is.character(g)) 
        g = "hyper"
    if (any(grep("=", g))) {
        f21a = suppressWarnings(as.numeric(unlist(strsplit(g, 
            "="))[2]))
        if (!is.numeric(f21a) | is.na(f21a)) {
            f21a.char = suppressWarnings(as.character(unlist(strsplit(g, 
                "="))[2]))
            if (any(grep("bric", f21a.char, ignore.case = TRUE))) {
                f21a = 2 + 2/max(N, K^2)
            }
            else if (any(grep("uip", f21a.char, ignore.case = TRUE))) {
                f21a = 2 + 2/N
            }
            else {
                warning("You did not supply a proper 'a' parameter for the hyper g prior (like e.g. the format g='hyperg=3.1' or g='hyper=UIP') - thus set to default value 'hyper=UIP' instead.")
                f21a = 2 + 2/N
            }
        }
        else {
            if (f21a <= 2 | f21a > 4) {
                f21a = 2 + 2/N
                warning("You provided an 'a' parameter for the hyper g prior that is not element of (2,4]. I chose the default value 'hyper=UIP' instead.")
            }
        }
    }
    else {
        f21a = 2 + 2/N
    }
    gprior.info = list(gtype = "hyper", is.constant = FALSE, 
        return.g.stats = return.g.stats, shrinkage.moments = numeric(2), 
        g = NA, hyper.parameter = f21a)
    if (!is.numeric(null.lik)) {
        null.lik = {
            1 - N
        }/2 * log(yty)
    }
    gmoments = numeric(2)
    N12 = {
        N - 1
    }/2
    la2 = log(f21a - 2)
    log.lik = null.lik
    ymy.current = -1
    k.current = -1
    intconstinv = f21a - 2
    f21o = .f21_4hyperg(N, K, f21a)
    gprior.info$lprobcalc <- list(just.loglik = function(ymy, 
        k, ...) {
        if (k == 0) {
            return(null.lik)
        }
        ymy.current <<- ymy
        k.current <<- k
        intconstinv <<- {
            k + f21a - 2
        }/f21o[["calcit"]](1 - ymy/yty, k)
        if (intconstinv < 0) {
            intconstinv <<- k + f21a - 2
        }
        log.lik <<- null.lik + la2 - log(intconstinv)
        return(log.lik)
    }, lprob.all = function(ymy, k, bhat, diag.inverse, ...) {
        if (k == 0) {
            return(list(lprob = null.lik, b1new = numeric(0), 
                b2new = numeric(0), otherstats = c(2/f21a, 8/f21a/(f21a + 
                  2))))
        }
        N3 = N - 3
        ka2 = k + f21a - 2
        R2 = 1 - ymy/yty
        if ((ymy != ymy.current) | (k != k.current)) {
            intconstinv <<- ka2/f21o[["calcit"]](R2, k)
            log.lik <<- null.lik + la2 - log(intconstinv)
        }
        g2hyper = {
            intconstinv - ka2 + N3 * R2
        }/{
            R2 * {
                N3 - ka2
            }
        }
        gbetavar = {
            {
                1 + 2/N3 * R2/{
                  1 - R2
                }
            } * intconstinv + {
                N3 - 2
            } * R2 - ka2
        } * N3 * {
            1 - R2
        }/{
            N3 - ka2
        }/{
            N3 - ka2 - 2
        }/R2 * yty/N3 * diag.inverse
        if (return.g.stats) {
            ka = ka2 + 2
            Eg22 = {
                {
                  {
                    N3 - 2
                  } * R2 - ka
                } * intconstinv + {
                  N3 * R2 - ka2
                }^2 - 2 * {
                  N3 * R2^2 - ka2
                }
            }/R2^2/{
                N3 - ka2
            }/{
                N3 - ka
            }
            gmoments = c(g2hyper, Eg22)
        }
        return(list(lprob = log.lik, b1new = g2hyper * bhat, 
            b2new = gbetavar + g2hyper^2 * bhat^2, otherstats = gmoments))
    })
    class(gprior.info) <- c("gprior", class(gprior.info))
    return(gprior.info)
}
.hexcode.binvec.convert <-
function (length.of.binvec) 
{
    if (length(length.of.binvec) > 1) 
        length.of.binvec = length(length.of.binvec)
    addpositions = 4 - length.of.binvec%%4
    positionsby4 = (length.of.binvec + addpositions)/4
    hexvec = c(0:9, "a", "b", "c", "d", "e", "f")
    hexcodelist = list(`0` = numeric(4), `1` = c(0, 0, 0, 1), 
        `2` = c(0, 0, 1, 0), `3` = c(0, 0, 1, 1), `4` = c(0, 
            1, 0, 0), `5` = c(0, 1, 0, 1), `6` = c(0, 1, 1, 0), 
        `7` = c(0, 1, 1, 1), `8` = c(1, 0, 0, 0), `9` = c(1, 
            0, 0, 1), a = c(1, 0, 1, 0), b = c(1, 0, 1, 1), c = c(1, 
            1, 0, 0), d = c(1, 1, 0, 1), e = c(1, 1, 1, 0), f = c(1, 
            1, 1, 1))
    return(list(as.hexcode = function(binvec) {
        incl = c(numeric(addpositions), binvec)
        dim(incl) = c(4, positionsby4)
        return(paste(hexvec[crossprod(incl, 2L^(3:0)) + 1], collapse = ""))
    }, as.binvec = function(hexcode) {
        return(unlist(hexcodelist[unlist(strsplit(hexcode, "", 
            fixed = TRUE), recursive = FALSE, use.names = FALSE)], 
            recursive = FALSE, use.names = FALSE)[-(1:addpositions)])
    }))
}
.index.bma <-
function (x, i, ...) 
{
    x$topmod <- x$topmod[i]
    return(x)
}
.index.topmod <-
function (x, i, ...) 
{
    tm = x
    idx = i
    if (any(is.na(suppressWarnings(as.integer(idx))))) 
        idx = 1:length(tm$lik())
    if (length(tm$betas_raw()) > 1) {
        bbeta = TRUE
        bet = as.vector(tm$betas()[, idx])
        bet = bet[bet != 0]
    }
    else {
        bbeta = FALSE
        bet = numeric(0)
    }
    if (length(tm$betas2_raw()) > 1) {
        bbeta2 = TRUE
        bet2 = as.vector(tm$betas2()[, idx])
        bet2 = bet2[bet2 != 0]
    }
    else {
        bbeta2 = FALSE
        bet2 = numeric(0)
    }
    fixvec = tm$fixed_vector()
    if (!length(as.vector(fixvec))) 
        fixvec = numeric(0)
    else fixvec = as.vector(t(fixvec[, idx]))
    .top10(nmaxregressors = tm$nregs, nbmodels = tm$nbmodels, 
        bbeta = bbeta, lengthfixedvec = nrow(tm$fixed_vector()), 
        bbeta2 = bbeta2, inivec_lik = tm$lik()[idx], inivec_bool = tm$bool()[idx], 
        inivec_count = tm$ncount()[idx], inivec_vbeta = bet, 
        inivec_vbeta2 = bet2, inivec_veck = tm$kvec_raw()[idx], 
        inivec_fixvec = fixvec)
}
.iterenum <-
function (molddraw = numeric(0), K = length(molddraw), ...) 
{
    even.lead1 = {
        1:K
    }[!{
        cumsum(molddraw)%%2
    }]
    i = even.lead1[length(even.lead1)]
    molddraw[i] = !molddraw[i]
    addi = molddraw[i] * i
    dropi = {
        !molddraw[i]
    } * i
    return(list(mnewdraw = molddraw, positionnew = {
        1:K
    }[as.logical(molddraw)], addi = addi, dropi = dropi))
}
.iterenum.bone <-
function (molddraw = numeric(0), maxk = Inf) 
{
    even.lead1 = ((1:length(molddraw))[!(cumsum(molddraw)%%2)])
    i = even.lead1[length(even.lead1)]
    molddraw[i] = !molddraw[i]
    if (sum(molddraw) > maxk) 
        return(.iterenum.bone(molddraw, maxk))
    else return(molddraw)
}
.iterenum.KgtN <-
function (molddraw = numeric(0), maxk = Inf, oldk = 0, ...) 
{
    mnewdraw = .iterenum.bone(molddraw = molddraw, maxk)
    addi = (1:length(mnewdraw))[molddraw < mnewdraw]
    if (length(addi) == 0) 
        addi = 0
    dropi = (1:length(mnewdraw))[molddraw > mnewdraw]
    if (length(dropi) == 0) 
        dropi = 0
    return(list(mnewdraw = mnewdraw, positionnew = (1:length(mnewdraw))[as.logical(mnewdraw)], 
        addi = addi, dropi = dropi))
}
.lprob.constg.init <-
function (...) 
{
    gpo = .gprior.constg.init(...)
    return(gpo$lprobcalc)
}
.lprob.eblocal.init <-
function (...) 
{
    gpo = .gprior.eblocal.init(...)
    return(gpo$lprobcalc)
}
.lprob.hyperg.init <-
function (...) 
{
    gpo = .gprior.hyperg.init(...)
    return(gpo$lprobcalc)
}
.mprior.customk.init <-
function (K, mpparam, ...) 
{
    if (any(is.na(mpparam))) 
        mpparam = rep(0.5, K)
    if (!is.numeric(mpparam)) 
        stop("For custom model size priors, you need to provide a K+1 vector with positive elements for argument 'mprior.size'.")
    mpparam = as.vector(mpparam)
    if (!((length(mpparam) == (K + 1)) & all(mpparam > 0))) {
        stop("For custom model size priors, you need to provide a K+1 vector with positive elements for argument 'mprior.size'.")
    }
    mpkvec = log(mpparam)
    return(list(mp.mode = "custom", mp.msize = sum(choose(K, 
        0:K) * mpparam * {
        0:K
    })/sum(choose(K, 0:K) * mpparam), pmp = function(ki, ...) {
        return(mpkvec[[ki + 1]])
    }, mp.Kdist = choose(K, 0:K) * mpparam/sum(choose(K, 0:K) * 
        mpparam)))
}
.mprior.fixedt.init <-
function (K, mpparam, ...) 
{
    if (is.na(mpparam[1])) 
        mpparam <- K/2
    if ((mpparam[[1]] >= K) & (length(mpparam) == 1)) {
        warning("Submitted prior model size is >= than the nr. of   regressors\n, used K/2 instead\n\n")
        mpparam <- K/2
    }
    m = mpparam[[1]]
    return(list(mp.mode = "fixed", mp.msize = m, pmp = function(ki, 
        ...) {
        post.odds1 = ki * log(m/K) + {
            K - ki
        } * log(1 - m/K)
        return(post.odds1)
    }, mp.Kdist = dbinom(x = 0:K, size = K, prob = m/K, log = FALSE)))
}
.mprior.pip.init <-
function (K, mpparam, ...) 
{
    if (any(is.na(mpparam))) 
        mpparam = rep(0.5, K)
    if (!is.numeric(mpparam)) 
        stop("For prior inclusion probabilites, you need to provide a K vector with elements between 0 and 1 for argument 'mprior.size'.")
    mpparam = as.vector(mpparam)
    if (!((length(mpparam) == K) & all(mpparam > 0) & all(mpparam <= 
        1))) 
        stop("For prior inclusion probabilites, you need to provide a K vector with elements between 0 and 1 for argument 'mprior.size'.")
    if (any(mpparam == 1L)) 
        warning("Prior Inclsuion Prob. = 1 are impractical. Try using the argument fixed.reg")
    inclfacts = log(mpparam/(1 - mpparam))
    return(list(mp.mode = "pip", mp.msize = sum(mpparam), pmp = function(mdraw, 
        ...) {
        return(sum(inclfacts[as.logical(mdraw)]))
    }, mp.Kdist = .getpolycoefs(mpparam/{
        1 - mpparam
    }) * prod(1 - mpparam)))
}
.mprior.randomt.init <-
function (K, mpparam, ...) 
{
    if (is.na(mpparam[1])) 
        mpparam <- K/2
    if ((mpparam[[1]] >= K) & (length(mpparam) == 1)) {
        warning("Submitted prior model size is >= than the nr. of   regressors\n, used K/2 instead\n\n")
        mpparam <- K/2
    }
    m = mpparam[[1]]
    vecofpriors = lgamma(1 + 0:K) + lgamma({
        K - m
    }/m + K - 0:K)
    beta.bin = function(a = 1, b = (K - m)/m, K = K, w = 0:K) {
        lgamma(a + b) - {
            lgamma(a) + lgamma(b) + lgamma(a + b + K)
        } + log(choose(K, w)) + lgamma(a + w) + lgamma(b + K - 
            w)
    }
    return(list(mp.mode = "random", mp.msize = m, pmp = function(ki, 
        ...) {
        return(vecofpriors[[ki + 1]])
    }, mp.Kdist = exp(beta.bin(a = 1, b = {
        K - m
    }/m, K = K, w = 0:K))))
}
.mprior.uniform.init <-
function (K, ...) 
{
    return(list(mp.mode = "uniform", mp.msize = K/2, pmp = function(...) return(0), 
        mp.Kdist = exp(lchoose(K, 0:K) - K * log(2))))
}
.ols.terms2 <-
function (positions, yty, k = NULL, N = N, K = K, XtX.big = XtX.big, 
    Xty.big = Xty.big, ...) 
{
    syminv <- function(symmat, ndim = ncol(symmat)) {
        if (!is.matrix(symmat)) {
            symmat = as.matrix(symmat)
        }
        return(chol2inv(chol(symmat), size = ndim))
    }
    if (is.null(k)) 
        k = length(positions)
    XtXinv.return = numeric(0)
    if (sum(k) == 0) {
        Xty = numeric(0)
        XtXinv = matrix(0, 0, 0)
        bhat = numeric(0)
        ymy = yty
        positions = 0
    }
    else {
        XtX <- XtX.big[positions, positions, drop = FALSE]
        Xty <- Xty.big[positions]
        XtXinv <- syminv(XtX, ndim = k)
        bhat <- crossprod(XtXinv, Xty)
        ymy <- yty - crossprod(Xty, bhat)[[1]]
    }
    return(list(full.results = function() {
        return(list(ymy = ymy, bhat = bhat, diag.inverse = XtXinv[1:k + 
            0:(k - 1) * k]))
    }, child.ymy = function(addix = 0, dropix = 0, ...) {
        if (!any(as.logical(c(addix, dropix)))) {
            return(ymy)
        }
        if (all(as.logical(c(addix, dropix)))) {
            jhere = {
                1:k
            }[positions == dropix]
            poshere = positions[-jhere]
            Xj = XtXinv[, jhere]
            Xtxi = XtX.big[poshere, addix]
            bxlessj = crossprod(XtXinv, XtX.big[positions, addix]) - 
                Xj * XtX.big[addix, dropix]
            bhatx = bxlessj[-jhere] - Xj[-jhere] * bxlessj[jhere]/Xj[jhere]
            child.ymy = ymy + bhat[jhere]^2/Xj[jhere] - {
                Xty.big[addix] - crossprod(Xty.big[poshere], 
                  bhatx)[[1]]
            }^2/{
                XtX.big[addix, addix] - crossprod(bhatx, Xtxi)[[1]]
            }
            return(child.ymy)
        } else {
            if (addix == 0) {
                jhere = {
                  1:k
                }[positions == dropix]
                child.ymy = ymy + bhat[jhere]^2/XtXinv[jhere, 
                  jhere]
                return(child.ymy)
            } else {
                Xtxi = XtX.big[positions, addix]
                bhatx = crossprod(XtXinv, Xtxi)[, 1]
                child.ymy = ymy - {
                  Xty.big[addix] - crossprod(bhatx, Xty)[[1]]
                }^2/{
                  XtX.big[addix, addix] - crossprod(bhatx, Xtxi)[[1]]
                }
                return(child.ymy)
            }
        }
    }, mutate = function(addix = 0, dropix = 0, newpos = numeric(0), 
        newk = 0, ...) {
        if (newk == 0) {
            XtXinv <<- matrix(0, 0, 0)
            Xty <<- numeric(0)
        } else {
            if (newk < 7 | addix[[1]] != 0 | length(c(dropix, 
                addix)) > 2) {
                Xty <<- Xty.big[newpos]
                XtXinv <<- syminv(XtX.big[newpos, newpos, drop = FALSE], 
                  ndim = newk)
            } else {
                if (dropix[1] > 0) {
                  jhere = sum(positions <= dropix)
                  Xty <<- Xty[-jhere]
                  Xj = XtXinv[, jhere]
                  XtXinv <<- {
                    XtXinv - tcrossprod(Xj/Xj[jhere], Xj)
                  }[-jhere, -jhere]
                } else {
                  jhere = sum(positions < addix) + 1
                  Xtxx = XtX.big[addix, newpos]
                  Xtx = Xtxx[-jhere]
                  Xty <<- Xty.big[newpos]
                  bhatx = crossprod(XtXinv, Xtx)[, 1]
                  bhatxadj = c(bhatx[0:(jhere - 1)], -1, bhatx[jhere:k])
                  if (jhere == newk) bhatxadj = bhatxadj[-(jhere + 
                    1:2)]
                  newinv = tcrossprod(bhatxadj, bhatxadj/(Xtxx[jhere] - 
                    crossprod(Xtx, bhatx)[[1]]))
                  newinv[-jhere, -jhere] = newinv[-jhere, -jhere] + 
                    XtXinv
                  XtXinv <<- newinv
                }
            }
        }
        positions <<- newpos
        k <<- newk
        bhat <<- crossprod(XtXinv, Xty)[, 1]
        ymy <<- yty - crossprod(Xty, bhat)[[1]]
        return(list(ymy = ymy, bhat = bhat, diag.inverse = XtXinv[1:k + 
            0:{
                k - 1
            } * k]))
    }, return.inverse = function() XtXinv, ymy = ymy, bhat = bhat, 
        diag.inverse = XtXinv[1:k + 0:{
            k - 1
        } * k]))
}
.post.beta.draws <-
function (topmods, reg.names, moment2 = FALSE) 
{
    if (moment2) 
        beta.draws = as.matrix(topmods$betas2())
    else beta.draws = as.matrix(topmods$betas())
    if (sum(beta.draws) == 0) {
        stop("The tompod object provided does not have saved betas. cf. bbeta argument in function topmod")
    }
    if (nrow(beta.draws) != length(reg.names)) {
        rownames(beta.draws) = c(reg.names, "W-Index")
    }
    else {
        rownames(beta.draws) = c(reg.names)
    }
    beta.names = topmods$bool()
    if (length(which(beta.names == "0")) > 0) {
        colnames(beta.draws) = beta.names[-c(which(beta.names == 
            "0"))]
    }
    else {
        colnames(beta.draws) = beta.names
    }
    return(beta.draws)
}
.post.calc <-
function (gprior.info, add.otherstats, k.vec, null.count, X.data, 
    topmods, b1mo, b2mo, iter, burn, inccount, models.visited, 
    K, N, msize, timed, cumsumweights = NA, mcmc = "bd", possign = NA) 
{
    postad.k.vec <- function(k.vec, null.count) c(null.count, 
        k.vec)
    postad.gprior.info <- function(gprior.info, add.otherstats = numeric(0), 
        cumsumweights = 1) {
        if (gprior.info$return.g.stats) {
            if (length(add.otherstats) > 0) {
                gprior.info$shrinkage.moments = add.otherstats/cumsumweights
            }
            else {
                gprior.info$shrinkage.moments = 1/(1 + 1/gprior.info$g)
            }
        }
        return(gprior.info)
    }
    postad.reg.names <- function(X.data) {
        if (is.null(colnames(X.data)[-1]) || colnames(X.data)[-1] == 
            "") {
            reg.names <- paste("beta", 1:K)
        }
        else {
            reg.names = colnames(X.data)[-1]
        }
        return(reg.names)
    }
    gprior.info = postad.gprior.info(gprior.info, add.otherstats, 
        cumsumweights)
    k.vec = postad.k.vec(k.vec, null.count)
    cons = .post.constant(X.data, b1mo/cumsumweights)
    pmp.10 = pmp.bma(topmods, oldstyle = TRUE)
    if (nrow(pmp.10) == 1 | suppressWarnings(length(grep("error", 
        class(try(cor(pmp.10[, 1], pmp.10[, 2]), silent = TRUE)))))) {
        corr.pmp = NA
    }
    else {
        if (var(pmp.10[, 2]) == 0) 
            corr.pmp = NA
        else corr.pmp = cor(pmp.10[, 1], pmp.10[, 2])
    }
    if (is.na(possign[[1]])) 
        possign = numeric(K)
    info.object = list(iter = iter, burn = burn, inccount = inccount, 
        models.visited = models.visited, b1mo = b1mo, b2mo = b2mo, 
        add.otherstats = add.otherstats, cumsumweights = cumsumweights, 
        K = K, N = N, corr.pmp = corr.pmp, msize = msize, timed = timed, 
        k.vec = k.vec, cons = cons, pos.sign = possign)
    reg.names = postad.reg.names(X.data)
    return(list(info = info.object, k.vec = k.vec, cons = cons, 
        gprior.info = gprior.info, pmp.10 = pmp.10, reg.names = reg.names))
}
.post.constant <-
function (X.data, Ebeta) 
{
    Xmeans = colMeans(X.data)
    cons = Xmeans[1] - crossprod(Ebeta, Xmeans[-1])
    return(as.vector(cons))
}
.post.estimates <-
function (b1mo = NULL, b2mo = NULL, cumsumweights = NULL, inccount = NULL, 
    topmods = NULL, X.data = NULL, reg.names = NULL, pos.sign = NULL, 
    exact = FALSE, order.by.pip = TRUE, include.constant = FALSE, 
    incl.possign = TRUE, std.coefs = FALSE, condi.coef = FALSE) 
{
    idx = 1:(length(b1mo))
    if (exact) {
        lt1 = topmods$lik() - max(topmods$lik())
        exact.pmp = as.vector(exp(lt1)/sum(exp(lt1)))
        pip = as.vector(topmods$bool_binary() %*% exact.pmp)
        idx = 1:(length(pip))
        betas = topmods$betas()
        betas2 = topmods$betas2()
        K = nrow(betas)
        Eb1 = tcrossprod(betas, t(exact.pmp))[, 1]
        Eb2 = tcrossprod(betas2, t(exact.pmp))[, 1]
        Ebsd = sqrt(Eb2 - Eb1^2)
        possign = round(tcrossprod((betas > 0), t(exact.pmp))[, 
            1]/pip, 8)
        possign[is.nan(possign)] = NA
    }
    else {
        pip = inccount/cumsumweights
        Eb1 = b1mo/cumsumweights
        Eb2 = b2mo/cumsumweights
        Ebsd = sqrt(Eb2 - Eb1^2)
        possign = round(pos.sign/inccount, 8)
        possign[is.nan(possign)] = NA
    }
    if (include.constant) 
        constterm = .post.constant(X.data, Eb1)
    if (condi.coef) {
        Eb1 = Eb1/pip
        Eb2 = Eb2/pip
        Ebsd = sqrt(Eb2 - Eb1^2)
        Eb1[is.nan(Eb1)] = 0
        Ebsd[is.nan(Ebsd)] = 0
    }
    if (std.coefs) {
        sddata = apply(as.matrix(X.data), 2, stats::sd)
        Eb1 = Eb1/sddata[1] * sddata[-1]
        Ebsd = Ebsd/sddata[1] * sddata[-1]
        if (include.constant) 
            constterm = constterm/sddata[1]
    }
    if (incl.possign) {
        post.mean <- cbind(pip, Eb1, Ebsd, possign, idx)
        rownames(post.mean) <- reg.names
        colnames(post.mean) <- c("PIP", "Post Mean", "Post SD", 
            "Cond.Pos.Sign", "Idx")
    }
    else {
        post.mean <- cbind(pip, Eb1, Ebsd, idx)
        rownames(post.mean) <- reg.names
        colnames(post.mean) <- c("PIP", "Post Mean", "Post SD", 
            "Idx")
    }
    if (order.by.pip) {
        post.mean <- post.mean[order(-post.mean[, 1]), ]
    }
    if (include.constant) {
        constrow = matrix(c(1, constterm, NA, rep(NA, incl.possign), 
            0), 1)
        rownames(constrow) = "(Intercept)"
        post.mean = rbind(post.mean, constrow)
    }
    return(post.mean)
}
.post.topmod.bma <-
function (topmods, reg.names = numeric(0)) 
{
    pmps = pmp.bma(topmods)
    if (is.bma(topmods)) {
        reg.names = topmods$reg.names
        topmods = topmods$topmod
    }
    rbind(.post.topmod.includes(topmods, reg.names), t(pmps))
}
.post.topmod.includes <-
function (topmods, reg.names) 
{
    topmod = topmods$bool_binary()
    colnames(topmod) <- topmods$bool()
    rownames(topmod) = reg.names
    return(topmod)
}
.quantile.density <-
function (x, probs = seq(0.25, 0.75, 0.25), names = TRUE, normalize = TRUE, 
    ...) 
{
    my.quantile.density = function(x, probs, names, normalize, 
        ...) {
        ycs = (cumsum(x$y) - (x$y - x$y[[1]])/2) * diff(x$x[1:2])
        if (normalize) 
            ycs = ycs/(ycs[[length(ycs)]])
        xin = x$x
        maxi = length(ycs)
        qqs = sapply(as.list(probs), function(qu) {
            iii = sum(ycs <= qu)
            if (iii == maxi) 
                return(Inf)
            else if (iii == 0L) 
                return(-Inf)
            else {
                return(xin[[iii + 1]] + ((ycs[[iii + 1]] - qu)/(ycs[[iii + 
                  1]] - ycs[[iii]])) * (xin[[iii]] - xin[[iii + 
                  1]]))
            }
        })
        if (as.logical(names)) 
            names(qqs) = paste(format(100 * probs, trim = TRUE, 
                digits = max(2L, getOption("digits"))), "%", 
                sep = "")
        return(qqs)
    }
    probs = as.vector(probs)
    if (is.element("density", class(x))) 
        return(my.quantile.density(x = x, probs = probs, names = names, 
            normalize = normalize))
    if (!all(sapply(x, function(dd) is.element("density", class(dd))))) 
        stop("x needs to be a density or list of densities")
    if (length(x) == 1L) 
        return(my.quantile.density(x = x[[1]], probs = probs, 
            names = names, normalize = normalize))
    qout = sapply(x, my.quantile.density, probs = probs, names = FALSE, 
        normalize = normalize)
    if (!is.matrix(qout)) {
        if (length(probs) > 1) 
            return(qout)
        qout = as.matrix(qout)
    }
    else qout = t(qout)
    if (as.logical(names)) 
        colnames(qout) = paste(format(100 * probs, trim = TRUE, 
            digits = max(2L, getOption("digits"))), "%", sep = "")
    return(qout)
}
.rev.jump <-
function (molddraw = molddraw, K = K, ..., maxk = Inf, oldk = 0) 
{
    rev.idx = ceiling(runif(1, 0, 2))
    if (rev.idx == 1) {
        birth.death = .fls.samp(molddraw = molddraw, K = K, maxk = maxk, 
            oldk = oldk)
        mnewdraw = birth.death[["mnewdraw"]]
        positionnew = birth.death[["positionnew"]]
        addvar = birth.death[["addi"]]
        dropvar = birth.death[["dropi"]]
    }
    if (rev.idx == 2) {
        var.in = (1:K)[as.logical(molddraw)]
        var.out = (1:K)[!as.logical(molddraw)]
        var.in.rand = ceiling(length(var.in) * runif(1, 0, 1))
        addvar = var.out[ceiling(length(var.out) * runif(1, 0, 
            1))]
        dropvar = var.in[var.in.rand]
        mnewdraw = molddraw
        mnewdraw[addvar] = 1
        mnewdraw[dropvar] = 0
        positionnew = (1:K)[as.logical(mnewdraw)]
        dropvar = max(dropvar, 0)
        addvar = max(addvar, 0)
    }
    return(list(mnewdraw = mnewdraw, positionnew = positionnew, 
        addi = addvar, dropi = dropvar))
}
.rev.jump.int <-
function (molddraw = molddraw, K = K, mPlus = mPlus, maxk = Inf, 
    oldk = 0) 
{
    rev.idx = floor(runif(1, 0, 1) * 2)
    if ((rev.idx) | oldk == 0) {
        birth.death = .fls.samp.int(molddraw = molddraw, K = K, 
            mPlus = mPlus, maxk, oldk)
        mnewdraw = birth.death$mnewdraw
        positionnew = birth.death$positionnew
        addvar = birth.death$addi
        dropvar = birth.death$dropi
    }
    else {
        var.in = (1:K)[as.logical(molddraw)]
        var.out = (1:K)[!as.logical(molddraw)]
        mnewdraw = (molddraw > mPlus[, var.in[ceiling(length(var.in) * 
            runif(1, 0, 1))]])
        mnewdraw = mnewdraw | mPlus[var.out[ceiling(length(var.out) * 
            runif(1, 0, 1))], ]
        positionnew = (1:K)[mnewdraw]
        addvar = (1:K)[molddraw < mnewdraw]
        dropvar = (1:K)[molddraw > mnewdraw]
        if (length(dropvar) == 0) 
            dropvar = 0
        if (length(addvar) == 0) 
            addvar = 0
    }
    if (length(positionnew) > maxk) {
        return(.rev.jump.int(molddraw = molddraw, K = K, mPlus = mPlus, 
            maxk, oldk))
    }
    else {
        return(list(mnewdraw = as.numeric(mnewdraw), positionnew = positionnew, 
            addi = addvar, dropi = dropvar))
    }
}
.starter <-
function (K, start.value, y, N = N, XtX.big = XtX.big, Xty.big = Xty.big, 
    X = X, fixed.pos = numeric(0)) 
{
    if (is.na(start.value[1])) {
        start.value = min((N - 3), K)
    }
    if (any(start.value < 0) | !(is.numeric(start.value) | is.logical(start.value))) {
        start.value = min((N - 3), K)
        warning("Argument 'start.value' did not conform to required format. start.value has been changed to default - min(N-3,K)")
    }
    if (length(start.value) == 0) {
        start.value = numeric(K)
    }
    if (length(start.value) == 1) {
        if (start.value == 0) {
            start.value = numeric(K)
        }
    }
    if (length(start.value) > 1 && any(start.value > 1)) {
        sv = numeric(K)
        sv[start.value] = 1
        start.value = sv
        rm(sv)
    }
    if (length(start.value) == 1) {
        if (start.value > min((N - 3), K)) {
            cat("Submitted Start value is too large, used\n min(N-3,K) as starting model size instead\n\n")
            start.value = min((N - 3), K)
        }
        sorter = runif(K)
        start.position = order(sorter, seq(1:K))[1:start.value]
        XtX.start <- XtX.big[start.position, start.position]
        XtXinv.start <- chol2inv(chol(XtX.start))
        bhat = XtXinv.start %*% Xty.big[start.position]
        e = y - X[, start.position] %*% bhat
        sse = crossprod(e)
        s2 = as.numeric(sse/(N - length(start.position)))
        bcov = s2 * XtXinv.start
        bt = bhat/sqrt(diag(bcov))
        molddraw = rep(0, K)
        goodguy = as.numeric(abs(bt) > 0.2)
        molddraw[start.position] = goodguy
        start.position = (1:K)[as.logical(molddraw)]
        outstart = list(molddraw = molddraw, bhat = bhat, start.position = start.position)
    }
    if (length(start.value) > 1 && sum(start.value) == 0) {
        outstart = list(molddraw = rep(0, K), bhat = rep(0, K), 
            start.position = integer(0))
    }
    if (length(start.value) > 1 && sum(start.value) > 0) {
        if (length(start.value) != K) {
            stop("Starting Model contains unequal to K regressors,please respecify")
        }
        start.position = which(as.logical(start.value))
        XtX.start <- XtX.big[start.position, start.position]
        XtXinv.start <- chol2inv(chol(XtX.start))
        bhat = XtXinv.start %*% Xty.big[start.position]
        molddraw = rep(0, K)
        molddraw[start.position] = 1
        outstart = list(molddraw = molddraw, bhat = bhat, start.position = start.position)
    }
    fixed.pos = (1:K)[fixed.pos]
    if (length(fixed.pos) > 0) {
        outstart$molddraw[fixed.pos] = 1
        outstart$start.position = (1:K)[as.logical(outstart$molddraw)]
    }
    return(outstart)
}
.top10 <-
function (nmaxregressors = 10, nbmodels = 10, bbeta = FALSE, 
    lengthfixedvec = 0, bbeta2 = FALSE, ..., inivec_lik = numeric(0), 
    inivec_bool = character(0), inivec_count = numeric(0), inivec_vbeta = numeric(0), 
    inivec_vbeta2 = numeric(0), inivec_veck = 0, inivec_fixvec = numeric(0)) 
{
    findex = function() {
        seq_incl = seq_len(nbmodel)
        if (nbmodel == nbmodels) {
            seq_incl[indices] = seq_incl
        }
        else {
            truncindex = indices
            truncindex[(nbmodel + 1):nbmodels] = 0L
            seq_incl[truncindex] = seq_incl
        }
        return(seq_incl)
    }
    betamat = function(top10_betavec) {
        bins = (sapply(as.list(top10_bool[findex()]), hexobject$as.binvec))
        betamatx = matrix(0, nmaxregressors, nbmodel)
        if (length(top10_betavec) > 0) {
            betamatx[which(bins == 1)] = top10_betavec
        }
        else betamatx = betamatx[, 1]
        return(betamatx)
    }
    hexobject <- .hexcode.binvec.convert(nmaxregressors)
    if (nbmodels < 0) {
        nbmodels = 0
    }
    indices = integer(nbmodels)
    top10_lik = rep(-Inf, nbmodels)
    top10_bool = character(nbmodels)
    top10_count = integer(nbmodels)
    top10_fixvec = numeric(lengthfixedvec * nbmodels)
    if (bbeta) 
        lbetas = vector("list", nbmodels)
    if (bbeta2) 
        lbetas2 = vector("list", nbmodels)
    seq_nbmodel = seq_len(nbmodels)
    ix_of_mybool = logical(nbmodels)
    nbmodel = length(inivec_lik)
    top10_lik[seq_len(nbmodel)] = inivec_lik
    top10_count[seq_len(nbmodel)] = inivec_count
    if (is.character(inivec_bool)) {
        top10_bool[seq_len(nbmodel)] = inivec_bool
    }
    else {
        if (is.vector(inivec_bool) & (length(inivec_bool) == 
            nmaxregressors)) {
            top10_bool[seq_len(nbmodel)] = hexobject$as.hexcode(inivec_bool)
        }
        else if (is.list(inivec_bool)) {
            top10_bool[seq_len(nbmodel)] = sapply(inivec_bool, 
                hexobject$as.hexcode)
        }
        else if (is.matrix(inivec_bool)) {
            top10_bool[seq_len(nbmodel)] = sapply(as.list(as.data.frame(inivec_bool)), 
                hexobject$as.hexcode)
        }
        else stop("inivec_bool is wrong format!")
    }
    top10_fixvec = inivec_fixvec
    if (is.na(inivec_veck[1])) {
        inivec_veck = 0
    }
    if (bbeta | bbeta2) {
        veck_ix = c(0, cumsum(inivec_veck))
        veckix_aux = as.list(seq_len(nbmodel))
        veckix_aux = lapply(veckix_aux, function(x) {
            if (veck_ix[[x]] == veck_ix[[x + 1]]) 
                c(0, 0)
            else c(veck_ix[[x]] + 1, veck_ix[[x + 1]])
        })
    }
    if (bbeta) {
        lbetas[seq_len(nbmodel)] = lapply(veckix_aux, function(x) inivec_vbeta[x[[1]]:x[[2]]])
    }
    else lbetas = list(numeric(0))
    if (bbeta2) {
        lbetas2[seq_len(nbmodel)] = lapply(veckix_aux, function(x) inivec_vbeta2[x[[1]]:x[[2]]])
    }
    else lbetas2 = list(numeric(0))
    lastvec01 = integer(nmaxregressors)
    modidx = length(top10_lik)
    indices[seq_len(nbmodel)] = order(inivec_lik, decreasing = TRUE)
    min.index = which.max(indices)
    if (length(min.index) > 0) {
        min.top10_lik = top10_lik[[min.index]]
    }
    else {
        if (nbmodels > 0) 
            min.top10_lik = -Inf
        else min.top10_lik = Inf
    }
    index.of.mybool = function(mybool) {
        ix_of_mybool <<- (mybool == top10_bool)
    }
    check4dupl = index.of.mybool
    dupl.possible = TRUE
    retlist = list(addmodel = function(mylik, vec01, vbeta = numeric(0), 
        vbeta2 = numeric(0), fixedvec = numeric(0)) {
        if (mylik >= min.top10_lik | nbmodel < nbmodels) {
            if (identical(lastvec01, vec01)) {
                top10_count[[modidx]] <<- top10_count[[modidx]] + 
                  1
            } else {
                lastvec01 <<- vec01
                mybool = hexobject$as.hexcode(vec01)
                check4dupl(mybool)
                if (!any(ix_of_mybool)) {
                  if (nbmodel < nbmodels) {
                    nbmodel <<- nbmodel + 1
                    modidx <<- nbmodel
                  } else {
                    modidx <<- min.index
                  }
                  ltmylik = (top10_lik <= mylik)
                  indices <<- indices + ltmylik
                  indices[[modidx]] <<- nbmodels - sum(ltmylik) + 
                    1
                  top10_lik[[modidx]] <<- mylik
                  top10_bool[[modidx]] <<- mybool
                  top10_count[[modidx]] <<- 1
                  min.index <<- which.max(indices)
                  min.top10_lik <<- top10_lik[[min.index]]
                  if (lengthfixedvec > 0) {
                    top10_fixvec[(modidx - 1) * lengthfixedvec + 
                      seq_len(lengthfixedvec)] <<- fixedvec
                  }
                  if (bbeta) {
                    lbetas[[modidx]] <<- vbeta
                  }
                  if (bbeta2) {
                    lbetas2[[modidx]] <<- vbeta2
                  }
                } else {
                  modidx <<- seq_nbmodel[ix_of_mybool]
                  top10_count[[modidx]] <<- top10_count[[modidx]] + 
                    1
                }
            }
        }
    }, lik = function() {
        return(top10_lik[findex()])
    }, bool = function() {
        return(top10_bool[findex()])
    }, ncount = function() {
        return(top10_count[findex()])
    }, nbmodels = nbmodels, nregs = nmaxregressors, betas_raw = function() {
        return(unlist(lbetas[findex()]))
    }, betas2_raw = function() {
        return(unlist(lbetas2[findex()]))
    }, kvec_raw = function() {
        return(sapply(lbetas, length)[findex()])
    }, bool_binary = function() {
        return(sapply(as.list(top10_bool[findex()]), hexobject$as.binvec))
    }, betas = function() {
        betamat(unlist(lbetas[findex()]))
    }, betas2 = function() {
        betamat(unlist(lbetas2[findex()]))
    }, fixed_vector = function() {
        if (lengthfixedvec <= 0) {
            return(matrix(0, 0, 0))
        } else {
            findex_base = (findex() - 1) * lengthfixedvec
            findex_fixvec = numeric(0)
            for (xx in 1:lengthfixedvec) findex_fixvec = rbind(findex_fixvec, 
                findex_base + xx)
            return(matrix(top10_fixvec[c(findex_fixvec)], lengthfixedvec))
        }
    }, duplicates_possible = function(possible = NULL) {
        if (!is.logical(possible)) return(dupl.possible)
        if (possible) {
            check4dupl <<- index.of.mybool
            dupl.possible <<- TRUE
            ix_of_mybool <<- logical(nbmodels)
        } else {
            check4dupl <<- function(mybool) {
            }
            dupl.possible <<- FALSE
            ix_of_mybool <<- FALSE
        }
    })
    class(retlist) = "topmod"
    return(retlist)
}
.topmod.as.bbetaT <-
function (tm, gprior.info = NULL, yXdata = NULL, addr2 = FALSE) 
{
    is.bmao = FALSE
    if (is.bma(tm)) {
        is.bmao = TRUE
        bmao = tm
        yXdata = bmao$X.data
        gprior.info = bmao$gprior.info
        tm = bmao$topmod
    }
    yXdata = as.matrix(yXdata)
    N = nrow(yXdata)
    K = ncol(yXdata) - 1
    yXdata = yXdata - matrix(colMeans(yXdata), N, K + 1, byrow = TRUE)
    if (length(tm$lik()) < 1) {
        if (is.bmao) 
            return(bmao)
        else return(tm)
    }
    if (!addr2) 
        if ((length(tm$betas_raw()) > 0) & (ncol(as.matrix(tm$betas())) == 
            length(tm$lik()))) {
            if (is.bmao) 
                return(bmao)
            else return(tm)
        }
    bools = (tm$bool_binary())
    yty = c(crossprod(yXdata[, 1]))
    positions = lapply(lapply(as.list(as.data.frame(bools)), 
        as.logical), which)
    olsmodels = lapply(lapply(positions, .ols.terms2, yty = yty, 
        N = N, K = K, XtX.big = crossprod(yXdata[, -1]), Xty.big = c(crossprod(yXdata[, 
            -1], yXdata[, 1]))), function(x) x$full.results())
    lprobo = gprior.info$lprobcalc
    lpl = lapply(olsmodels, function(x) lprobo$lprob(x$ymy, length(x$bhat), 
        x$bhat, x$diag.inverse))
    veck = as.vector(unlist(lapply(lapply(lpl, "[[", "b1new"), 
        length)))
    b1raw = as.vector(unlist(lapply(lpl, "[[", "b1new")))
    b2raw = as.vector(unlist(lapply(lpl, "[[", "b2new")))
    fixedvecmat = tm$fixed_vector()
    if (addr2) {
        r2 = 1 - sapply(olsmodels, function(x) x$ymy)/yty
        if (nrow(fixedvecmat) == 0) {
            fixedvecmat = matrix(0, 0, length(veck))
        }
        else if (mean(abs(r2 - fixedvecmat[1, ])) < 1e-17) {
            fixedvecmat = fixedvecmat[-1, , drop = FALSE]
        }
        fixedvecmat = rbind(r2, fixedvecmat)
    }
    lengthfixedvec = nrow(fixedvecmat)
    tm <- .top10(nmaxregressors = tm$nregs, nbmodels = tm$nbmodels, 
        bbeta = TRUE, lengthfixedvec = lengthfixedvec, bbeta2 = TRUE, 
        inivec_lik = tm$lik(), inivec_bool = tm$bool(), inivec_count = tm$ncount(), 
        inivec_vbeta = b1raw, inivec_vbeta2 = b2raw, inivec_veck = veck, 
        inivec_fixvec = c(fixedvecmat))
    if (is.bmao) {
        bmao$topmod <- tm
        return(bmao)
    }
    return(tm)
}

hex2bin <-
function (hexcode) 
{
    if (!is.character(hexcode)) 
        stop("please input a character like '0af34c'")
    hexcode <- paste("0", tolower(hexcode), sep = "")
    hexobj <- .hexcode.binvec.convert(length(hexcode) * 16L)
    return(hexobj$as.binvec(hexcode))
}

deviance.bma <-
function (object, exact = FALSE, ...) 
{
    if (is.bma(object)) {
        xx = as.matrix(object$X.data)
        ebeta = estimates.bma(object, order.by.pip = FALSE, exact = exact)[, 
            2, drop = TRUE]
    }
    else if (is(object, "lm")) {
        xx = as.matrix(object$model)
        ebeta = coef(object)
        if (length(ebeta) == ncol(xx)) 
            ebeta = ebeta[-1]
    }
    else stop("Required input is an object of class 'bma' or 'lm'/'zlm'.")
    xx = xx - matrix(colMeans(xx), nrow(xx), ncol(xx), byrow = TRUE)
    ess = as.vector(crossprod(ebeta, as.vector(crossprod(xx[, 
        -1, drop = FALSE], xx[, 1]))))
    return((as.vector(crossprod(xx[, 1, drop = TRUE])) - ess))
}

density.zlm <-
function (x, reg = NULL, addons = "lesz", std.coefs = FALSE, 
    n = 300, plot = TRUE, hnbsteps = 30, addons.lwd = 1.5, ...) 
{
    addons = gsub("E", "", addons, ignore.case = FALSE)
    addons = gsub("S", "", addons, ignore.case = FALSE)
    addons = gsub("b", "", addons, ignore.case = FALSE)
    addons = gsub("m", "", addons, ignore.case = FALSE)
    addons = gsub("p", "", addons, ignore.case = FALSE)
    N = length(x$residuals)
    K = length(x$coefficients) - 1
    tmo = topmod(1, nmaxregressors = K, bbeta = TRUE, liks = x$marg.lik, 
        ncounts = 1, modelbinaries = matrix(rep(1, K), K, 1), 
        betas = matrix(as.vector(x$coefficients[-1]), K), betas2 = matrix(as.vector(x$coef2moments[-1]), 
            K))
    tokenbma = list(info = list(K = K, N = N), arguments = list(), 
        topmod = tmo, start.pos = integer(0), gprior.info = x$gprior.info, 
        X.data = x$model, reg.names = names(x$coefficients)[-1], 
        bms.call = new("call"))
    class(tokenbma) = "bma"
    return(density.bma(tokenbma, reg = reg, addons = addons, 
        std.coefs = std.coefs, n = n, plot = plot, hnbsteps = hnbsteps, 
        addons.lwd = addons.lwd, ...))
}

estimates.bma <-
function (object, exact = FALSE, order.by.pip = TRUE, include.constant = FALSE, 
    incl.possign = TRUE, std.coefs = FALSE, condi.coef = FALSE) 
{
    bmao = object
    rm(object)
    if (!is.bma(bmao)) {
        stop("you need to provide a BMA object")
        return()
    }
    if (exact) {
        if (bmao$topmod$nbmodels == 0) 
            stop("exact=TRUE needs at least one 'top model': Run estimation again and set nmodel>0")
    }
    bmaest = .post.estimates(bmao$info$b1mo, bmao$info$b2mo, 
        bmao$info$cumsumweights, bmao$info$inccount, bmao$topmod, 
        bmao$X.data, bmao$reg.names, bmao$info$pos.sign, exact, 
        order.by.pip, include.constant, incl.possign, std.coefs, 
        condi.coef)
    return(bmaest)
}

quantile.density <-
function (x, probs = seq(0.25, 0.75, 0.25), names = TRUE, normalize = TRUE, 
    ...) 
{
    my.quantile.density = function(x, probs, names, normalize, 
        ...) {
        ycs = (cumsum(x$y) - (x$y - x$y[[1]])/2) * diff(x$x[1:2])
        if (normalize) 
            ycs = ycs/(ycs[[length(ycs)]])
        xin = x$x
        maxi = length(ycs)
        qqs = sapply(as.list(probs), function(qu) {
            iii = sum(ycs <= qu)
            if (iii == maxi) 
                return(Inf)
            else if (iii == 0L) 
                return(-Inf)
            else {
                return(xin[[iii + 1]] + ((ycs[[iii + 1]] - qu)/(ycs[[iii + 
                  1]] - ycs[[iii]])) * (xin[[iii]] - xin[[iii + 
                  1]]))
            }
        })
        if (as.logical(names)) 
            names(qqs) = paste(format(100 * probs, trim = TRUE, 
                digits = max(2L, getOption("digits"))), "%", 
                sep = "")
        return(qqs)
    }
    probs = as.vector(probs)
    if (is.element("density", class(x))) 
        return(my.quantile.density(x = x, probs = probs, names = names, 
            normalize = normalize))
    if (!all(sapply(x, function(dd) is.element("density", class(dd))))) 
        stop("x needs to be a density or list of densities")
    if (length(x) == 1L) 
        return(my.quantile.density(x = x[[1]], probs = probs, 
            names = names, normalize = normalize))
    qout = sapply(x, my.quantile.density, probs = probs, names = FALSE, 
        normalize = normalize)
    if (!is.matrix(qout)) {
        if (length(probs) > 1) 
            return(qout)
        qout = as.matrix(qout)
    }
    else qout = t(qout)
    if (as.logical(names)) 
        colnames(qout) = paste(format(100 * probs, trim = TRUE, 
            digits = max(2L, getOption("digits"))), "%", sep = "")
    return(qout)
}

pred.density <-
function (object, newdata = NULL, n = 300, hnbsteps = 30, ...) 
{
    dtcm = function(x, df, ncp, varp) {
        sqvarp = sqrt(varp)
        dt((x - ncp)/sqvarp, df = df)/sqvarp
    }
    dsgivenykernel <- function(sf, kpa, N, z) {
        (kpa - 2)/2 * (1 - sf)^((kpa - 4)/2) * (1 - sf * z)^(-(N - 
            1)/2)
    }
    nbsteps = max(hnbsteps, 2)
    n = max(ceiling(n), 1)
    is.hyper = (object$gprior.info$gtype == "hyper")
    if (is.hyper) 
        f21a = object$gprior.info$hyper.parameter
    if (is.bma(object)) {
        K = object$info$K
        N = object$info$N
        yXdata = as.matrix(object$X.data)
        tmo <- object$topmod
    }
    else if (is(object, "zlm")) {
        yXdata = as.matrix(object$model)
        K = ncol(yXdata) - 1
        N = nrow(yXdata)
        tmo <- topmod(1, nmaxregressors = K, bbeta = TRUE, liks = object$marg.lik, 
            ncounts = 1, modelbinaries = matrix(rep(1, K), K, 
                1), betas = matrix(as.vector(object$coefficients[-1]), 
                K), betas2 = matrix(as.vector(object$coef2moments[-1]), 
                K))
    }
    else stop("argument 'object' requires class 'bma' or 'zlm'")
    rm(object)
    if (missing(newdata)) {
        stop("You must provide the argument newdata")
    }
    else {
        newX = as.matrix(newdata)
        if (!is.numeric(newX)) 
            stop("newdata must be numeric!")
        if (is.vector(newdata)) 
            newX = matrix(newdata, 1)
        if (ncol(newX) != K) {
            if (ncol(newX) == K + 1) {
                newX = newX[, -1, drop = FALSE]
            }
            else {
                stop("newdata must be a matrix or data.frame with ", 
                  K, " columns.")
            }
        }
        orinames = colnames(yXdata[, -1, drop = FALSE])
        if (!is.null(colnames(newX)) && !is.null(orinames)) {
            if (all(orinames %in% colnames(newX)) && !all(orinames == 
                colnames(newX))) {
                warning("argument newdata had to be reordered according to its column names. Consider submitting the columns of newdata in the right order.")
                newX = newX[, orinames, drop = FALSE]
            }
        }
    }
    if (!is.null(rownames(newX))) {
        newXnames = rownames(newX)
    }
    else {
        newXnames = as.character(1:nrow(newX))
    }
    rnew = nrow(newX)
    y.mean = mean(yXdata[, 1])
    y <- yXdata[, 1] - matrix(y.mean, N, 1, byrow = TRUE)
    X <- yXdata[, -1, drop = FALSE] - matrix(colMeans(yXdata[, 
        -1, drop = FALSE]), N, K, byrow = TRUE)
    XtX.big = crossprod(X)
    Xty.big = as.vector(crossprod(X, y))
    yty = crossprod(y)[[1]]
    newXdm = newX - matrix(colMeans(yXdata[, -1, drop = FALSE]), 
        rnew, K, byrow = TRUE)
    hexobject <- .hexcode.binvec.convert(K)
    make_xfxxxf = function(hex) {
        syminv <- function(symmat, ndim = ncol(symmat)) {
            if (!is.matrix(symmat)) {
                symmat = as.matrix(symmat)
            }
            if (dim(symmat)[[1]] == 0) 
                return(matrix(numeric(0), 0, 0))
            return(chol2inv(chol(symmat), size = ndim))
        }
        boolvec = as.logical(hexobject$as.binvec(hex))
        if (!any(boolvec)) 
            return(c(numeric(rnew), numeric(rnew), Inf, Inf, 
                0))
        newXsub = newXdm[, boolvec, drop = FALSE]
        xtxinv = syminv(XtX.big[boolvec, boolvec, drop = FALSE])
        xty = Xty.big[boolvec]
        betas = as.vector(crossprod(xtxinv, xty), mode = "numeric")
        r2 = crossprod(xty, betas)[[1]]/yty
        xtxinv_xf = tcrossprod(xtxinv, newXsub)
        xf_xx_xf = unlist(lapply(1:nrow(newXsub), function(x) {
            crossprod(newXsub[x, ], xtxinv_xf[, x])[[1L]]
        }))
        xf_bhat = as.vector(newXsub %*% betas)
        return(c(xf_xx_xf, xf_bhat, xtxinv[[1L]], betas[[1L]], 
            r2))
    }
    pmps = pmp.bma(tmo, oldstyle = TRUE)[, 1, drop = TRUE]
    bools = tmo$bool()
    nmodel = length(bools)
    linvres = lapply(bools, make_xfxxxf)
    mat_xfxxxf = array(unlist(lapply(linvres, "[", 1:rnew)), 
        dim = c(rnew, nmodel))
    mat_xfbhat = array(unlist(lapply(linvres, "[", rnew + (1:rnew))), 
        dim = c(rnew, nmodel))
    xtxinv_elem1 = unlist(lapply(linvres, "[[", rnew * 2 + 1))
    betahat_elem1 = unlist(lapply(linvres, "[[", rnew * 2 + 2))
    r2 = unlist(lapply(linvres, "[[", rnew * 2 + 3))
    kvec = tmo$kvec_raw()
    kvec_cs = c(1, cumsum(kvec) + 1)
    kvec_cs = kvec_cs[-length(kvec_cs)]
    firstbetas = tmo$betas_raw()[kvec_cs]
    firstbetas2 = tmo$betas2_raw()[kvec_cs]
    Es = firstbetas/betahat_elem1
    varmult = (firstbetas2 - firstbetas^2)/xtxinv_elem1
    if (is.hyper) {
        first_factor = yty/(N - 3) * (N + 1)/N * (1 + 2/(N - 
            kvec - f21a - 1) - r2 * Es)
    }
    else {
        first_factor = yty/(N - 3) * (1 - Es * r2) * (N + 1)/N
    }
    Sigmas = (matrix((N - 3)/(N - 1) * first_factor, rnew, nmodel, 
        byrow = TRUE) + t(t(mat_xfxxxf) * ((N - 3)/(N - 1) * 
        varmult)))
    Evals_minusy = t(t(mat_xfbhat) * Es)
    Eyf = as.vector(Evals_minusy %*% pmps + y.mean)
    Varyf = as.vector(Sigmas %*% pmps) * (N - 1)/(N - 3)
    premultfactor = yty/(N - 1)
    interceptfactor = (N + 1)/N
    calcdensvec = function(xf_index, seqy, m_index) {
        sss = function(lbound, uboundp1, nbsteps, seqs, xf.index) {
            s.seq = seq(lbound, uboundp1, (uboundp1 - lbound)/nbsteps)[-nbsteps]
            tmat = array(unlist(lapply(as.list(s.seq), function(ss) {
                dtcm(seqs, N - 1, y.mean + ss * myev, premultfactor * 
                  (1 - ss * myr2) * (interceptfactor + ss * myxfxxxf))
            })), dim = c(length(seqs), nbsteps))
            smat = sapply(as.list(s.seq), dsgivenykernel, kpa = myk + 
                f21a, N = N, z = myr2)
            if (any(is.infinite(smat))) 
                smat[is.infinite(smat)] = 0
            intconst = (4 * sum(smat[c(FALSE, TRUE)]) + 2 * sum(smat[c(TRUE, 
                FALSE)]) - 3 * smat[nbsteps] - smat[1]) * (s.seq[nbsteps] - 
                s.seq[1])/nbsteps/3
            return(list(dv = c(4 * tmat[, c(FALSE, TRUE)] %*% 
                smat[c(FALSE, TRUE)] + 2 * tmat[, c(TRUE, FALSE)] %*% 
                smat[c(TRUE, FALSE)] - 3 * tmat[, nbsteps] * 
                smat[nbsteps] - tmat[, 1] * smat[1]) * (s.seq[nbsteps] - 
                s.seq[1])/nbsteps/3, ic = intconst))
        }
        if (any(is.na(newX[xf_index, ]))) {
            densvec = numeric(0)
        }
        if (is.hyper) {
            myev = mat_xfbhat[xf_index, m_index]
            myxfxxxf = mat_xfxxxf[xf_index, m_index]
            myk = kvec[[m_index]]
            myr2 = r2[[m_index]]
            midpoint = 1 - (1 - Es[[m_index]]) * 4
            if (midpoint < 0.5) {
                dvl = sss(1e-04, 0.9999999, nbsteps * 2, seqy, 
                  xf_index)
                densvec = dvl$dv/dvl$ic
            }
            else {
                dvl1 = sss(1e-04, midpoint, nbsteps, seqy, xf_index)
                dvl2 = sss(midpoint, 1, nbsteps, seqy, xf_index)
                densvec = (dvl1$dv + dvl2$dv)/(dvl1$ic + dvl2$ic)
            }
        }
        else {
            densvec = dtcm(seqy, N - 1, Evals_minusy[xf_index, 
                m_index] + y.mean, Sigmas[xf_index, m_index])
        }
        return(densvec)
    }
    dens_yf = function(yfr, xf_indices = NULL) {
        if (is.null(xf_indices)) 
            xf_indices = seq_len(rnew)
        yfdens = array(NA, dim = dim(yfr))
        for (myxf in 1:length(xf_indices)) {
            allm_dens = sapply(seq_len(nmodel), function(x) calcdensvec(xf_indices[[myxf]], 
                yfr[myxf, ], x))
            yfdens[myxf, ] = as.vector(allm_dens %*% pmps)
        }
        yfdens[!is.finite(yfdens)] = NA
        if (ncol(yfdens) == 1) 
            dim(yfdens) <- NULL
        return(yfdens)
    }
    emptydens = list(x = numeric(0), y = numeric(0), bw = NULL, 
        n = 0, has.na = TRUE)
    class(emptydens) = "density"
    dlist = lapply(vector("list", nrow(newX)), function(x) emptydens)
    densities_calculated <- FALSE
    calc_alldens = function() {
        if (densities_calculated) 
            return(NULL)
        for (xf.index in 1:rnew) {
            if (!any(is.na(newX[xf.index, ]))) {
                lbound = Eyf[[xf.index]] - sqrt(Varyf[[xf.index]]) * 
                  4
                ubound = Eyf[[xf.index]] + sqrt(Varyf[[xf.index]]) * 
                  4
                seqs = seq(lbound, ubound, (ubound - lbound)/(n - 
                  1))
                allm_dens = sapply(seq_len(nmodel), function(x) calcdensvec(xf.index, 
                  seqs, x))
                myy = as.vector(tcrossprod(t(as.matrix(pmps)), 
                  allm_dens))
                mydens = list(x = seqs, y = myy, bw = NULL, n = n, 
                  has.na = FALSE)
                class(mydens) = "density"
                dlist[[xf.index]] <<- mydens
            }
        }
        densities_calculated <<- TRUE
    }
    consistent.yf = function(yf, xf.indices = NULL) {
        xf_series = seq_len(rnew)
        wasnull = FALSE
        if (is.null(xf.indices)) {
            wasnull = TRUE
            xf.indices = xf_series
        }
        else {
            if (!all(xf.indices %in% xf_series)) 
                stop(paste("predict_index needs to be an integer between 1 and ", 
                  rnew, "!", sep = ""))
        }
        if (!is.numeric(yf)) 
            stop("realized.y must be a numeric matrix or vector!")
        if (!is.matrix(yf)) 
            yf <- as.matrix(yf)
        if ((length(xf.indices) == 1) & (nrow(yf) > 1) & (ncol(yf) == 
            1)) 
            yf <- t(yf)
        if (nrow(newX[xf.indices, , drop = FALSE]) != nrow(yf)) {
            if (wasnull) 
                stop(paste("realized.y must have", rnew, "elements/rows corresponding to newdata"))
            else stop("The number of rows/elements in realized.y must have the same length as predict_index!")
        }
        return(yf)
    }
    consistent_predict_index = function(pix) {
        if (is.character(pix)) {
            if (all(pix %in% newXnames)) {
                return(match(pix, newXnames))
            }
            else {
                stop("Forecast IDs provided in predict_index do not conform to rownames of predicted data")
            }
        }
        else return(pix)
    }
    plot.preddens = function(xf.index = 1, addons = "eslz", yf.addons = NULL, 
        predict_index = NULL, addons.lwd = 1.5, ...) {
        dotargs = match.call(expand.dots = FALSE)$...
        if (rnew > 1) {
            main_default <- paste("Predictive Density Obs ", 
                newXnames[[xf.index]], " (", nmodel, " Models)", 
                sep = "")
        }
        else {
            main_default <- paste("Predictive Density", " (", 
                nmodel, " Models)", sep = "")
        }
        dotargs = .adjustdots(dotargs, xlab = "Response variable", 
            main = main_default, col = 4, zero.line = FALSE)
        thingy = dlist[[xf.index]]
        eval(as.call(c(list(as.name("plot"), as.name("thingy")), 
            as.list(dotargs))))
        leg.col = numeric(0)
        leg.lty = numeric(0)
        leg.legend = character(0)
        if (any(grep("g", addons, ignore.case = TRUE))) {
            grid()
        }
        if (any(grep("e", addons, ignore.case = FALSE))) {
            abline(v = fit[[xf.index]], col = 2, lwd = addons.lwd)
            leg.col = c(leg.col, 2)
            leg.lty = c(leg.lty, 1)
            leg.legend = c(leg.legend, "Exp. Value")
        }
        if (any(grep("s", addons, ignore.case = FALSE))) {
            abline(v = fit[[xf.index]] - 2 * stderrs[[xf.index]], 
                col = 2, lty = 2, lwd = addons.lwd)
            abline(v = fit[[xf.index]] + 2 * stderrs[[xf.index]], 
                col = 2, lty = 2, lwd = addons.lwd)
            leg.col = c(leg.col, 2)
            leg.lty = c(leg.lty, 2)
            leg.legend = c(leg.legend, "2x Std.Errs")
        }
        if (any(grep("z", addons, ignore.case = TRUE))) {
            abline(h = 0, col = "gray", lwd = addons.lwd)
        }
        if (!is.null(yf.addons) && is.numeric(yf.addons)) {
            yfs = as.vector(yf.addons)
            if (!is.na(yfs[[xf.index]])) {
                abline(v = yfs[[xf.index]], col = 1, lwd = addons.lwd, 
                  lty = 2)
                leg.col = c(leg.col, 1)
                leg.lty = c(leg.lty, 2)
                leg.legend = c(leg.legend, "Realized y")
            }
            else warning("yf.addons must be a vector with the same number of elements as rows in newdata!")
        }
        if (any(grep("l", addons, ignore.case = TRUE)) & (length(leg.col) > 
            0)) {
            legend(x = "topright", lty = leg.lty, col = leg.col, 
                legend = leg.legend, box.lwd = 0, bty = "n", 
                lwd = addons.lwd)
        }
    }
    fit = Eyf
    names(fit) = newXnames
    stderrs = sqrt(Varyf)
    names(stderrs) = newXnames
    reslist = list()
    reslist$densities = function() {
        calc_alldens()
        return(dlist)
    }
    reslist$fit = fit
    reslist$std.err = stderrs
    reslist$dyf = function(realized.y, predict_index = NULL) {
        predict_index = consistent_predict_index(predict_index)
        if (missing(realized.y)) {
            stop("You must provide a realization of the dependent variable in realized.y")
        }
        return(dens_yf(consistent.yf(realized.y, predict_index), 
            predict_index))
    }
    reslist$lps = function(realized.y, predict_index = NULL) {
        predict_index = consistent_predict_index(predict_index)
        if (missing(realized.y)) {
            stop("You must provide a realization of the dependent variable in realized.y")
        }
        yf = consistent.yf(realized.y, predict_index)
        if (ncol(yf) != 1) 
            stop("realized.y must have only one column!")
        yf.dens = dens_yf(yf, predict_index)
        return(-sum(log(yf.dens[!is.na(yf.dens)]))/length(yf))
    }
    reslist$plot = function(predict_index = NULL, addons = "eslz", 
        realized.y = NULL, addons.lwd = 1.5, ...) {
        dotargs = match.call(expand.dots = FALSE)$...
        xf_series = seq_len(rnew)
        predict_index = consistent_predict_index(predict_index)
        if (is.null(predict_index)) {
            predict_index = xf_series
        }
        else if (!all(predict_index %in% xf_series)) 
            stop(paste("predict_index needs to be an integer between 1 and ", 
                rnew, "!", sep = ""))
        if (!(is.null(realized.y))) {
            if (length(realized.y) != length(predict_index)) {
                stop("realized.y must be a vector with the same number of elements as rows in newdata (or predict_index)!")
            }
        }
        if (!is.null(realized.y)) 
            realized.y <- consistent.yf(realized.y, predict_index)
        calc_alldens()
        oldask = par()$ask
        plotnb = 0
        for (xf_index in predict_index) {
            doplot = !dlist[[xf_index]]$has.na
            plotnb = plotnb + doplot
            if (plotnb == 2) 
                par(ask = TRUE)
            dotargs = .adjustdots(dotargs, main = NULL, col = "steelblue4", 
                xlab = "Response variable")
            if (doplot) {
                eval(as.call(c(list(as.name("plot.preddens"), 
                  as.name("xf_index"), addons = as.name("addons"), 
                  yf.addons = as.name("realized.y"), addons.lwd = as.name("addons.lwd")), 
                  as.list(dotargs))))
            }
        }
        par(ask = oldask)
    }
    reslist$n = n
    reslist$nmodel = nmodel
    reslist$call = sys.call(0)
    class(reslist) = "pred.density"
    rm(betahat_elem1, bools, emptydens, firstbetas, firstbetas2, 
        linvres)
    return(reslist)
}

coef.bma <-
function (object, exact = FALSE, order.by.pip = TRUE, include.constant = FALSE, 
    incl.possign = TRUE, std.coefs = FALSE, condi.coef = FALSE, 
    ...) 
{
    estimates.bma(object, exact = exact, order.by.pip = order.by.pip, 
        include.constant = include.constant, incl.possign = incl.possign, 
        std.coefs = std.coefs, condi.coef = condi.coef)
}

plotConv <-
function (bmao, include.legend = TRUE, add.grid = TRUE, ...) 
{
    if (!is.bma(bmao)) 
        stop("submit an object of class bma")
    mat = pmp.bma(bmao, oldstyle = TRUE)
    norm_const = sum(mat[, 1])/sum(mat[, 2])
    mat = cbind(mat[, 2] * norm_const, mat[, 1])
    if (length(bmao$topmod$lik()) == 0L) {
        stop("plotConv needs at least one model stored in topmod in order to produce a plot")
    }
    cor.pmp = format(round(.cor.topmod(bmao$topmod), 4), nsmall = 4)
    dotargs = match.call(plot, expand.dots = FALSE)$...
    dotargs = .adjustdots(dotargs, lwd = 2, main = paste("Posterior Model Probabilities\n(Corr: ", 
        cor.pmp, ")", sep = ""), lty = 1, col = c("steelblue3", 
        "tomato"), cex.main = 0.8, xlab = "Index of Models", 
        ylab = "", type = "l")
    eval(as.call(c(list(as.name("matplot"), as.name("mat")), 
        as.list(dotargs))))
    if (as.logical(add.grid)) 
        grid()
    if (as.logical(include.legend)) 
        legend("topright", lty = eval(dotargs$lty), legend = c("PMP (MCMC)", 
            "PMP (Exact)"), col = eval(dotargs$col), ncol = 2, 
            bty = "n", cex = 1, lwd = eval(dotargs$lwd))
}

pmpmodel <-
function (bmao, model = numeric(0), exact = TRUE) 
{
    if (!is.bma(bmao)) 
        stop("bmao needs to be a bma object")
    if (!is.vector(model)) 
        stop("model needs to be vector denoting a single model.")
    K = bmao$info$K
    was.enum = (bmao$arguments$mcmc == "enum")
    emptyindex = logical(K)
    modelhex = ""
    if (length(model) == 0L) 
        model = numeric(0)
    if ((is.character(model)) && (all(model %in% bmao$reg.names))) {
        mix = match(model, bmao$reg.names)
        if (any(is.na(mix))) 
            stop("Provided variable names do not conform to bma object")
        emptyindex[mix] = TRUE
        model = emptyindex
    }
    else if ((length(model) == 1L) && all(is.character(model))) {
        modelhex = model
        model = as.logical(hex2bin(model))
        if (length(model) > K) 
            model = model[-(1:(length(model) - K))]
    }
    else if (is.logical(model) || ((length(model) == K) && (is.numeric(model) && 
        max(model) < 2))) {
        if (length(model) > K) 
            model = model[-(1:(length(model) - K))]
        model = as.logical(model)
    }
    else if (is.numeric(model)) {
        emptyindex[model] = TRUE
        model = emptyindex
    }
    else stop("model needs to be an integer, logical or character model index representation (hexcode or variable names)")
    if (any(is.na(model))) 
        stop("Provided model index does not seem to exist.")
    if (modelhex == "") 
        modelhex = bin2hex(model)
    fixed.pos = bmao$mprior.info$fixed.pos
    if (is.null(fixed.pos)) 
        fixed.pos = numeric(0)
    if (any(model[fixed.pos] != TRUE)) 
        stop("Such a model was excluded by bmao's argument fixed.reg")
    bools = bmao$topmod$bool()
    liks = bmao$topmod$lik()
    ncounts = bmao$topmod$ncount()
    cumsumweights = bmao$info$cumsumweights
    yty = as.vector(crossprod(bmao$X.data[, 1, drop = TRUE] - 
        mean(bmao$X.data[, 1, drop = TRUE])))
    log.null.lik = bmao$gprior.info$lprobcalc$just.loglik(ymy = yty, 
        k = 0)
    mix = match(modelhex, bools)
    if ((!exact) && (!was.enum)) {
        if (!is.na(mix)) {
            return(ncounts[[mix]]/cumsumweights)
        }
        else if (!any(model) || all(model)) {
            return(bmao$info$k.vec[sum(model) + 1]/cumsumweights)
        }
        else {
            stop("Model MCMC-based PMP cannot be found. Try exact=TRUE .")
        }
    }
    if (!is.na(mix)) {
        loglik = liks[mix]
    }
    else if (was.enum && (!any(model) || all(model))) {
        loglik = log(bmao$info$k.vec[sum(model) + 1]) + log.null.lik
    }
    else {
        if (!was.enum && (length(liks) == 0L)) 
            stop("bmao needs to contain more than 0 top models to provide an estimate for your model index.")
        if (sum(model) == 0L) {
            loglik = log.null.lik + bmao$mprior.info$pmp(ki = 0, 
                mdraw = rep(0, K), ymy = yty)
        }
        else {
            zz = zlm(bmao$X.data[, c(TRUE, model), drop = FALSE], 
                g = bmao$gprior.info)
            loglik = zz$marg.lik + bmao$mprior.info$pmp(ki = sum(model), 
                mdraw = as.numeric(model), ymy = zz$olsres$ymy)
        }
    }
    if (was.enum) {
        return(exp(loglik - log.null.lik)/cumsumweights)
    }
    pmp_withintop = exp(loglik - log.null.lik)/sum(exp(liks - 
        log.null.lik))
    return(pmp_withintop * sum(ncounts)/cumsumweights)
}

print.bma <-
function (x, ...) 
{
    if (!is.bma(x)) {
        return(print(x))
    }
    print(estimates.bma(x), include.constant = TRUE, ...)
    cat("\n")
    print(info.bma(x), ...)
    cat("\n")
}

plot.bma <-
function (x, ...) 
{
    if (!is.bma(x)) 
        stop("Need to provide object of class 'bma'!")
    if (x$arguments$nmodel < 3) {
        try(plotModelsize(x, ...), silent = TRUE)
    }
    else {
        layout(matrix(1:2, 2, 1))
        try(plotModelsize(x, ...), silent = TRUE)
        try(plotConv(x, ...), silent = TRUE)
        layout(1)
    }
}

predict.bma <-
function (object, newdata = NULL, exact = FALSE, topmodels = NULL, 
    ...) 
{
    if (!is.bma(object)) {
        stop("you need to provide a BMA object")
        return()
    }
    if (!is.null(topmodels)) {
        if (!(is.numeric(topmodels) && is.vector(topmodels))) {
            stop("topmodels must denote the models to take into account, e.g. 1:5 for the best five.")
        }
        else if (object$topmod$nbmodels < max(topmodels)) {
            stop(paste("Only", object$topmod$nbmodels, "best models are available, but you asked to take the", 
                max(topmodels), "-best model into account."))
        }
        object = object[unique(topmodels)]
    }
    if ((!missing(topmodels)) && missing(exact)) 
        exact = TRUE
    betas = estimates.bma(object, exact = exact, order.by.pip = FALSE, 
        include.constant = FALSE, std.coefs = FALSE, condi.coef = FALSE)[, 
        2]
    if (is.null(newdata)) {
        newX <- as.matrix(object$X.data[, -1, drop = FALSE])
    }
    else {
        newX = as.matrix(newdata)
        if (!is.numeric(newX)) 
            stop("newdata must be numeric!")
        if (is.vector(newdata)) 
            newX = matrix(newdata, 1)
        if (ncol(newX) != length(betas)) {
            if (ncol(newX) == length(betas) + 1) {
                newX = newX[, -1, drop = FALSE]
            }
            else {
                stop("newdata must be a matrix or data.frame with", 
                  length(betas), "columns.")
            }
        }
        orinames = colnames(object$X.data[, -1, drop = FALSE])
        if (!is.null(colnames(newX)) && !is.null(orinames)) {
            if (all(orinames %in% colnames(newX)) && !all(orinames == 
                colnames(newX))) {
                warning("argument newdata had to be reordered according to its column names. Consider submitting the columns of newdata in the right order.")
                newX = newX[, orinames, drop = FALSE]
            }
        }
    }
    cons = .post.constant(object$X.data, betas)
    return(as.vector(newX %*% betas) + cons)
}

combine_chains <-
function (...) 
{
    combine_topmods <- function(topmodobj1, topmodobj2) {
        nregs1 = topmodobj1$nregs
        nregs2 = topmodobj2$nregs
        if (nregs1 != nregs2) {
            stop("The number of regressors in both BMA chains has to be the same!")
        }
        k1 = length(topmodobj1$ncount())
        k2 = length(topmodobj2$ncount())
        nbmodels1 = topmodobj1$nbmodels
        nbmodels2 = topmodobj2$nbmodels
        ncount1 = topmodobj1$ncount()
        ncount2 = topmodobj2$ncount()
        lik1 = topmodobj1$lik()
        lik2 = topmodobj2$lik()
        bool1 = topmodobj1$bool()
        bool2 = topmodobj2$bool()
        betas1 = topmodobj1$betas()
        betas2 = topmodobj2$betas()
        betas2_1 = topmodobj1$betas2()
        betas2_2 = topmodobj2$betas2()
        fv1 = topmodobj1$fixed_vector()
        fv2 = topmodobj2$fixed_vector()
        if (all(betas1 == 0) | all(betas2 == 0)) {
            dobetas = FALSE
        }
        else {
            dobetas = TRUE
        }
        if (all(betas2_1 == 0) | all(betas2_2 == 0)) {
            dobetas2 = FALSE
        }
        else {
            dobetas2 = TRUE
        }
        idxin2_boolof1in2 = match(bool1, bool2)
        idxin1_boolof1in2 = which(!is.na(idxin2_boolof1in2))
        idxin2_boolof1in2 = idxin2_boolof1in2[!is.na(idxin2_boolof1in2)]
        ncount2[idxin2_boolof1in2] = ncount2[idxin2_boolof1in2] + 
            ncount1[idxin1_boolof1in2]
        if (any(idxin1_boolof1in2)) {
            ncount1 = ncount1[-idxin1_boolof1in2]
            lik1 = lik1[-idxin1_boolof1in2]
            bool1 = bool1[-idxin1_boolof1in2]
        }
        lika = c(lik2, lik1)
        orderlika = order(lika, decreasing = TRUE)
        lika = lika[orderlika]
        ncounta = c(ncount2, ncount1)[orderlika]
        boola = c(bool2, bool1)[orderlika]
        if (dobetas) {
            if (any(idxin1_boolof1in2)) 
                betas1 = betas1[, -idxin1_boolof1in2]
            betasa = cbind(betas2, betas1)[, orderlika]
            betasa_not0 = betasa != 0
            vecka = colSums(betasa_not0)
            vbetaa = as.vector(betasa[as.vector(betasa_not0)])
        }
        else {
            vecka = 0
            vbetaa = numeric(0)
        }
        if (dobetas2) {
            if (any(idxin1_boolof1in2)) 
                betas2_1 = betas2_1[, -idxin1_boolof1in2]
            betasa2 = cbind(betas2_2, betas2_1)[, orderlika]
            vbetaa2 = as.vector(betasa2[as.vector(betasa_not0)])
        }
        else {
            vbetaa2 = numeric(0)
        }
        fva = numeric(0)
        lfv = 0
        if ((nrow(fv1) == nrow(fv2)) & ((nrow(fv1) > 0) & (nrow(fv2) > 
            0))) {
            if (any(idxin1_boolof1in2)) 
                fv1 = fv1[, -idxin1_boolof1in2]
            if (!is.matrix(fv1)) 
                fv1 = t(fv1)
            fva = as.vector(cbind(fv2, fv1)[, orderlika])
            lfv = nrow(fv1)
        }
        return(.top10(nmaxregressors = nregs1, nbmodels = length(lika), 
            bbeta = dobetas, lengthfixedvec = lfv, bbeta2 = dobetas2, 
            inivec_lik = lika, inivec_bool = boola, inivec_count = ncounta, 
            inivec_vbeta = vbetaa, inivec_vbeta2 = vbetaa2, inivec_veck = vecka, 
            inivec_fixvec = fva))
    }
    combine_2chains <- function(flso1, flso2) {
        topmod.combi = combine_topmods(flso1$topmod, flso2$topmod)
        gpi <- flso1$gprior.info
        gpi$shrinkage.moments = numeric(length(gpi$shrinkage.moments))
        io1 = flso1$info
        io2 = flso2$info
        obj.combi = .post.calc(gprior.info = gpi, add.otherstats = io1$add.otherstats + 
            io2$add.otherstats, k.vec = (io1$k.vec[-1] + io2$k.vec[-1]), 
            null.count = (io1$k.vec[1] + io2$k.vec[1]), flso1$X.data, 
            topmods = topmod.combi, b1mo = io1$b1mo + io2$b1mo, 
            b2mo = io1$b2mo + io2$b2mo, iter = io1$iter + io2$iter, 
            burn = io1$burn + io2$burn, inccount = io1$inccount + 
                io2$inccount, models.visited = io1$models.visited + 
                io2$models.visited, K = io1$K, N = io1$N, msize = io1$msize + 
                io2$msize, timed = io1$timed + io2$timed, cumsumweights = io1$cumsumweights + 
                io2$cumsumweights, mcmc = flso1$arguments$mcmc, 
            possign = io1$pos.sign + io2$pos.sign)
        stpos1 = as.matrix(flso1$start.pos)
        stpos2 = as.matrix(flso2$start.pos)
        startpos.combi = cbind(rbind(stpos1, matrix(0, max(0, 
            nrow(stpos2) - nrow(stpos1)), ncol(stpos1))), rbind(stpos2, 
            matrix(0, max(0, nrow(stpos1) - nrow(stpos2)), ncol(stpos2))))
        call.combi = c(flso1$bms.call, flso2$bms.call)
        args.combi = flso1$arguments
        args2 = flso2$arguments
        args.combi$burn = args.combi$burn + args2$burn
        args.combi$iter = args.combi$iter + args2$iter
        if ((length(args.combi$mprior.size) == 1) | (length(args.combi$mprior.size) == 
            1)) {
            args.combi$mprior.size = mean(c(args.combi$mprior.size, 
                args2$mprior.size))
        }
        args.combi$nmodel = topmod.combi$nbmodels
        args.combi$user.int = (args.combi$user.int & args2$user.int)
        args.combi$g.stats = (args.combi$g.stats & args2$g.stats)
        mp1 = flso1$mprior.info
        mp2 = flso2$mprior.info
        if (mp1$mp.mode != mp2$mp.mode) {
            mpall = list()
        }
        else {
            mpall = mp1
            mpall$mp.msize = 0.5 * mp1$mp.msize + 0.5 * mp2$mp.msize
            mpall$origargs$mpparam = 0.5 * mp1$origargs$mpparam + 
                0.5 * mp2$origargs$mpparam
            mpall$mp.Kdist = 0.5 * mp1$mp.Kdist + 0.5 * mp2$mp.Kdist
        }
        result = list(info = obj.combi$info, arguments = args.combi, 
            topmod = topmod.combi, start.pos = startpos.combi, 
            gprior.info = obj.combi$gprior.info, mprior.info = mpall, 
            X.data = flso1$arguments$X.data, reg.names = obj.combi$reg.names, 
            bms.call = call.combi)
        class(result) = "bma"
        return(result)
    }
    arglist = list(...)
    if (!all(unlist(lapply(arglist, is.bma)))) 
        stop("All of the input arguments must be BMA objects!")
    if (!all(lapply(arglist, function(xx) xx$info$K) == arglist[[1]]$info$K)) 
        stop("All of the input BMA objects must have an equal number of max regressors (i.e. equal (X.data))!")
    if (!all(lapply(arglist, function(xx) xx$info$N) == arglist[[1]]$info$N)) 
        stop("All of the input BMA objects must have equal X.data!")
    if (!all(lapply(arglist, function(xx) xx$gprior.info$gtype) == 
        arglist[[1]]$gprior.info$gtype)) 
        stop("All of the input BMA objects must have the same type of g-prior (bms-argument g)")
    if (length(arglist) == 1) 
        return(arglist[[1]])
    combined_output <- combine_2chains(arglist[[1]], arglist[[2]])
    if (nargs() > 2) {
        for (inarg in 3:nargs()) {
            combined_output <- combine_2chains(arglist[[inarg]], 
                combined_output)
        }
    }
    return(combined_output)
}

fullmodel.ssq <-
function (yX.data) 
{
    if (is.bma(yX.data)) {
        yX.data <- yX.data$X.data
    }
    y <- as.matrix(yX.data[, 1])
    X <- as.matrix(yX.data[, 2:ncol(yX.data)])
    N <- nrow(X)
    K = ncol(X)
    y.mean = mean(y)
    y <- y - matrix(y.mean, N, 1, byrow = TRUE)
    X.mean = colMeans(X)
    X <- X - matrix(X.mean, N, K, byrow = TRUE)
    Xqr <- qr(X)
    yty = as.numeric(crossprod(y))
    ymy = as.numeric(crossprod(qr.resid(Xqr, y)))
    ypy = as.numeric(crossprod(qr.fitted(Xqr, y)))
    R2 = ypy/yty
    return(list(R2 = R2, ymy = ymy, ypy = ypy, yty = yty, Fstat = (R2/(1 - 
        R2)) * (N - K - 1)/K))
}

`[.topmod` <-
function (x, i, ...) 
{
    tm = x
    idx = i
    if (any(is.na(suppressWarnings(as.integer(idx))))) 
        idx = 1:length(tm$lik())
    if (length(tm$betas_raw()) > 1) {
        bbeta = TRUE
        bet = as.vector(tm$betas()[, idx])
        bet = bet[bet != 0]
    }
    else {
        bbeta = FALSE
        bet = numeric(0)
    }
    if (length(tm$betas2_raw()) > 1) {
        bbeta2 = TRUE
        bet2 = as.vector(tm$betas2()[, idx])
        bet2 = bet2[bet2 != 0]
    }
    else {
        bbeta2 = FALSE
        bet2 = numeric(0)
    }
    fixvec = tm$fixed_vector()
    if (!length(as.vector(fixvec))) 
        fixvec = numeric(0)
    else fixvec = as.vector(t(fixvec[, idx]))
    .top10(nmaxregressors = tm$nregs, nbmodels = tm$nbmodels, 
        bbeta = bbeta, lengthfixedvec = nrow(tm$fixed_vector()), 
        bbeta2 = bbeta2, inivec_lik = tm$lik()[idx], inivec_bool = tm$bool()[idx], 
        inivec_count = tm$ncount()[idx], inivec_vbeta = bet, 
        inivec_vbeta2 = bet2, inivec_veck = tm$kvec_raw()[idx], 
        inivec_fixvec = fixvec)
}

f21hyper <-
function (a, b, c, z) 
{
    if ((length(a) != 1) | (length(b) != 1) | (length(c) != 1) | 
        (length(z) != 1)) 
        stop("All function arguments need to be scalars")
    if ((a < 0) | (b < 0) | (c < 0)) 
        stop("Arguments a, b, and c need to be non-negative")
    if ((z > 1) | (z <= (-1))) 
        stop("Argument z needs to be between -1 and 1")
    nmx = max(100, 3 * floor(((a + b) * z - c - 1)/(1 - z)))
    if (nmx > 10000) 
        warning("Power series probably does not converge")
    serie = 0:nmx
    return(1 + sum(cumprod((a + serie)/(c + serie) * (b + serie)/(1 + 
        serie) * z)))
}

predict.zlm <-
function (object, newdata = NULL, se.fit = FALSE, ...) 
{
    if (!is(object, "zlm")) {
        stop("you need to provide a zlm object")
        return()
    }
    betas = object$coefficients[-1, drop = FALSE]
    alpha = object$coefficients[[1]]
    if (is.null(newdata)) {
        newX <- as.matrix(object$model[, -1, drop = FALSE])
    }
    else {
        newX = as.matrix(newdata)
        if (!is.numeric(newX)) 
            stop("newdata must be numeric!")
        if (is.vector(newdata)) 
            newX = matrix(newdata, 1)
        if (ncol(newX) != length(betas)) {
            if (ncol(newX) == length(betas) + 1) {
                newX = newX[, -1, drop = FALSE]
            }
            else {
                stop("newdata must be a matrix or data.frame with ", 
                  length(betas), " columns.")
            }
        }
    }
    if (!se.fit) 
        return(as.vector(newX %*% betas) + alpha)
    yXdata <- as.matrix(object$model)
    oldXraw <- yXdata[, -1, drop = FALSE]
    if (!is.null(colnames(newX)) && !is.null(colnames(oldXraw))) {
        if (all(colnames(oldXraw) %in% colnames(newX)) && !all(colnames(oldXraw) == 
            colnames(newX))) {
            warning("argument newdata had to be reordered according to its column names. Consider submitting the columns of newdata in the right order.")
            newX = newX[, colnames(oldXraw), drop = FALSE]
        }
    }
    yraw <- yXdata[, 1, drop = TRUE]
    N <- length(yraw)
    k <- ncol(oldXraw)
    oldXmeans <- colMeans(oldXraw)
    oldXdm <- oldXraw - matrix(oldXmeans, N, k, byrow = TRUE)
    newXdm <- newX - matrix(oldXmeans, nrow(newX), k, byrow = TRUE)
    xtxinv = chol2inv(chol(crossprod(oldXdm)))
    xtxinv_xf = tcrossprod(xtxinv, newXdm)
    xf_xx_xf = unlist(lapply(1:nrow(newXdm), function(x) {
        crossprod(newXdm[x, ], xtxinv_xf[, x])[[1L]]
    }))
    bvar = object$coef2moments[-1] - object$coefficients[-1]^2
    bvar_factor = bvar[[1L]]/xtxinv[[1L]]
    yty = as.vector(crossprod(yraw) - N * mean(yraw)^2)
    r2 = 1 - object$olsres$ymy/yty
    if (object$gprior.info$gtype == "hyper") {
        f21a = object$gprior.info$hyper.parameter
        f21_recover = exp((object$marg.lik) + (N - 1)/2 * log(yty) + 
            log((k + f21a - 2)/(f21a - 2)))
        res_scale = yty/(N - 3)/(N - 1 - k - f21a) * ((N - 3) * 
            (1 - r2) - (k + f21a - 2)/f21_recover)
        svar_woscale = res_scale/N + bvar_factor * xf_xx_xf
    }
    else {
        sf = object$gprior.info$shrinkage.moments[[1]]
        res_scale = (1 - sf * r2) * yty/(N - 3)
        svar_woscale = res_scale/N + bvar_factor * xf_xx_xf
    }
    reslist = list()
    reslist$fit <- as.vector(newX %*% betas) + alpha
    reslist$std.err <- sqrt(svar_woscale + res_scale)
    reslist$se.fit <- sqrt(svar_woscale)
    reslist$residual.scale <- sqrt(res_scale)
    return(reslist)
}

topmod <-
function (nbmodels, nmaxregressors = NA, bbeta = FALSE, lengthfixedvec = 0, 
    liks = numeric(0), ncounts = numeric(0), modelbinaries = matrix(0, 
        0, 0), betas = matrix(0, 0, 0), betas2 = matrix(0, 0, 
        0), fixed_vector = matrix(0, 0, 0)) 
{
    if (!is.numeric(nbmodels)) 
        stop("argument 'nbmodels' needs to be an integer>0")
    nbmodels = floor(nbmodels[[1]])
    if (nbmodels[[1]] < 0) 
        stop("argument 'nbmodels' needs to be an integer>0")
    if (bbeta > 0) 
        bbeta2 = TRUE
    else bbeta2 = FALSE
    bbeta = as.logical(bbeta)
    if (!bbeta & (length(betas) > 0)) 
        bbeta = TRUE
    if (!bbeta2 & (length(betas2) > 0)) 
        bbeta2 = TRUE
    if (is.na(lengthfixedvec[1])) 
        lengthfixedvec = 0
    if ((lengthfixedvec == 0) & length(fixed_vector) > 0) {
        lengthfixedvec = nrow(fixed_vector)
    }
    if (length(liks) > nbmodels) 
        stop("liks longer than nbmodels allows")
    if (length(ncounts) > nbmodels) 
        stop("ncounts longer than nbmodels allows")
    if ((length(modelbinaries) == 0) & (length(betas) > 0)) {
        modelbinaries = as.logical(betas)
        dim(modelbinaries) = dim(betas)
    }
    if (ncol(modelbinaries) > nbmodels) 
        stop("modelbinaries has more columns than than nbmodels allows")
    bindim = dim(modelbinaries)
    modelbinaries = as.logical(modelbinaries)
    dim(modelbinaries) = bindim
    if ((is.na(nmaxregressors[1])) & (length(modelbinaries) > 
        0)) {
        nmaxregressors = nrow(modelbinaries)
    }
    if (is.na(nmaxregressors)) 
        stop("argument 'nmaxregressors' is missing")
    nmaxregressors = floor(nmaxregressors[[1]])
    if (nmaxregressors <= 0) 
        stop("argument 'nmaxregressors' needs to be a positive integer")
    if ((length(ncounts) == 0) & (length(liks) > 0)) {
        ncounts = rep(1, length(liks))
    }
    if (length(modelbinaries) > 0) 
        if (nmaxregressors != nrow(modelbinaries)) 
            stop("nrow() of modelbinaries does not match nmaxregressors")
    if (bbeta & (length(betas) > 0)) 
        if (nmaxregressors != nrow(betas)) 
            stop("nrow() of betas does not match nmaxregressors")
    if (bbeta2 & (length(betas2) > 0)) 
        if (nmaxregressors != nrow(betas2)) 
            stop("nrow() of betas2 does not match nmaxregressors")
    N = length(liks)
    if (length(ncounts) != length(liks)) 
        stop("lengths of arguments 'liks' and 'ncounts' do not conform")
    if (ncol(modelbinaries) != length(liks)) 
        stop("nrow of argument 'modelbinaries' does not conform to length of argument 'liks'")
    if (bbeta) 
        if (ncol(betas) != length(liks)) 
            stop("nrow of argument 'betas' does not conform to length of argument 'liks'")
    if (bbeta2) 
        if (ncol(betas2) != length(liks)) 
            stop("nrow of argument 'betas2' does not conform to length of argument 'liks'")
    if (lengthfixedvec) 
        if (ncol(fixed_vector) != length(liks)) 
            stop("nrow of argument 'fixed_vector' does not conform to length of argument 'liks'")
    morder = order(liks, decreasing = TRUE)
    liks = liks[morder]
    modelbinaries = modelbinaries[, morder, drop = FALSE]
    ncounts = ncounts[morder]
    if (bbeta) {
        betas = betas[, morder, drop = FALSE]
    }
    if (bbeta2) {
        betas2 = betas2[, morder, drop = FALSE]
    }
    if (lengthfixedvec) {
        fixed_vector = fixed_vector[, morder, drop = FALSE]
    }
    hexobj = .hexcode.binvec.convert(nmaxregressors)
    bools = as.vector(sapply(as.list(as.data.frame(modelbinaries)), 
        hexobj$as.hexcode))
    if (length(bools) == 0) {
        bools = character(0)
    }
    veck = numeric(0)
    betas_raw = numeric(0)
    betas2_raw = numeric(0)
    if (bbeta & (length(bbeta) > 0)) {
        veck = colSums(modelbinaries)
        betas_raw = as.vector(betas)[as.vector(modelbinaries)]
    }
    if (bbeta2 & (length(bbeta2) > 0)) {
        veck = colSums(modelbinaries)
        betas2_raw = as.vector(betas2)[as.vector(modelbinaries)]
    }
    fixedvec = as.vector(fixed_vector)
    .top10(nmaxregressors = nmaxregressors, nbmodels = nbmodels, 
        bbeta = bbeta, lengthfixedvec = lengthfixedvec, bbeta2 = bbeta2, 
        inivec_lik = liks, inivec_bool = bools, inivec_count = ncounts, 
        inivec_vbeta = betas_raw, inivec_vbeta2 = betas2_raw, 
        inivec_veck = veck, inivec_fixvec = fixedvec)
}

quantile.coef.density <-
function (x, probs = seq(0.25, 0.75, 0.25), names = TRUE, ...) 
{
    quout = .quantile.density(x, probs = probs, names = names, 
        normalize = TRUE)
    if (is.matrix(quout) && as.logical(names)) 
        rownames(quout) <- sapply(x, function(lx) lx[["data.name"]])
    return(quout)
}

beta.draws.bma <-
function (bmao, stdev = FALSE) 
{
    if (!is.bma(bmao)) {
        stop("you need to provide a BMA object")
        return()
    }
    resmat = .post.beta.draws(bmao$topmod, bmao$reg.names, FALSE)
    if (stdev) {
        mom2 = .post.beta.draws(bmao$topmod, bmao$reg.names, 
            TRUE)
        resmat = sqrt(mom2 - resmat^2)
    }
    return(resmat)
}

quantile.pred.density <-
function (x, probs = seq(0.25, 0.75, 0.25), names = TRUE, ...) 
{
    quout = .quantile.density(x$densities(), probs = probs, names = names, 
        normalize = FALSE)
    if (is.matrix(quout) && as.logical(names)) 
        rownames(quout) <- names(x$fit)
    return(quout)
}

bms <-
function (X.data = NULL, burn = 1000, iter = NA, nmodel = 500, 
    mcmc = "bd", g = "UIP", mprior = "random", mprior.size = NA, 
    user.int = TRUE, start.value = NA, g.stats = TRUE, logfile = FALSE, 
    logstep = 10000, force.full.ols = FALSE, fixed.reg = numeric(0), 
    data = NULL, randomizeTimer = TRUE) 
{
    if (missing(X.data) & !missing(data)) 
        X.data = data
    mf <- match.call(expand.dots = FALSE)
    if (!is.na(match("X.data", names(mf)))) 
        names(mf)[[match("X.data", names(mf))]] = "formula"
    m <- match(c("formula", "data"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    if (is.data.frame(X.data)) {
        mf <- X.data
    }
    else if (is.matrix(X.data)) {
        mf <- model.frame(as.data.frame(X.data, drop.unused.levels = TRUE))
    }
    else {
        mf <- eval(mf, parent.frame())
    }
    X.data = as.matrix(mf)
    if (any(is.na(X.data))) {
        X.data = na.omit(X.data)
        if (nrow(X.data) < 3) {
            stop("Too few data observations. Please provide at least three data rows without NA entries.")
        }
        warning("Argument 'X.data' contains NAs. The corresponding rows have not been taken into account.")
    }
    N <- nrow(X.data)
    K = ncol(X.data) - 1
    maxk = N - 3
    if (is.null(nmodel[1]) || is.na(nmodel[1]) || nmodel[1] <= 
        0) {
        dotop = FALSE
        nmodel = 0
    }
    else {
        dotop = TRUE
    }
    nameix = 1:K
    names(nameix) = colnames(X.data[, -1, drop = FALSE])
    fixed.pos = nameix[fixed.reg]
    rm(nameix)
    if (missing(mcmc) && ((K - length(fixed.pos)) < 15)) {
        mcmc = "enum"
    }
    int = FALSE
    is.enum = FALSE
    if (is.function(mcmc)) {
        samplingfun = mcmc
        mcmc = "custom"
    }
    else {
        if (length(grep("int", mcmc, ignore.case = TRUE))) {
            int = TRUE
        }
        if (length(grep("enum", mcmc, ignore.case = TRUE))) {
            is.enum = TRUE
            samplingfun = .iterenum
            if (K > maxk) 
                samplingfun = .iterenum.KgtN
        }
        else if (length(grep("bd", mcmc, ignore.case = TRUE))) {
            samplingfun = switch(int + 1, .fls.samp, .fls.samp.int)
        }
        else {
            samplingfun = switch(int + 1, .rev.jump, .rev.jump.int)
        }
    }
    if (int && (length(fixed.pos) > 0L)) {
        warning("interaction sampler does not allow for non-zero argument fixed.pos; consequently it was set fixed.pos=0")
        fixed.pos = numeric(0)
    }
    sampling = .fixedset.sampler(samplingfun, fullK = K, fixed.pos = fixed.pos, 
        X.data = X.data)
    if (is.enum) {
        burn = 0
        int = FALSE
        mcmc = "enum"
        is.enum = TRUE
        tmp = .enum_startend(iter = iter, start.value = start.value, 
            K = K, maxk = maxk, fixed.pos = fixed.pos)
        iter = tmp$iter
        start.value = tmp$start.value
    }
    else {
        if (is.na(iter)) {
            iter = 3000
        }
    }
    if (logfile != FALSE) {
        if (is.character(logfile)) {
            sfilename = logfile
        }
        else {
            sfilename = ""
        }
        if (nchar(sfilename) > 0) 
            file.create(sfilename)
        logfile = TRUE
        cat(as.character(Sys.time()), ": starting loop ... \n", 
            append = TRUE, file = sfilename)
        if (logstep != 10000) 
            fact = logstep
        else fact = max(floor((burn + iter)/100), logstep)
    }
    y <- as.matrix(X.data[, 1])
    X <- as.matrix(X.data[, 2:ncol(X.data)])
    y <- y - mean(y)
    X <- X - matrix(colMeans(X), N, K, byrow = TRUE)
    XtX.big = crossprod(X)
    Xty.big = crossprod(X, y)
    yty = as.vector(crossprod(y))
    coreig = eigen(cor(X), symmetric = TRUE, only.values = TRUE)$values
    if (!force.full.ols) {
        if (sum(coreig > 1e-07) < min(K, (N - 1))) {
            force.full.ols = TRUE
        }
    }
    if (any(coreig[1:min(K, (N - 1))] < 1e-16)) {
        warning(paste("data seems to be rank-deficient: its rank seems to be only ", 
            sum(coreig > 1e-13)))
    }
    if (int) {
        if (length(grep("#", colnames(X.data), fixed = TRUE)) == 
            0) 
            stop("Please separate column names of interaction terms by # (e.g. A#B)")
        mPlus = .constr.intmat(X, K)
    }
    else {
        mPlus <- NA
    }
    pmplist = .choose.mprior(mprior, mprior.size, K = K, X.data = X.data, 
        fixed.pos = fixed.pos)
    mprior = pmplist$mp.mode
    gprior.info = .choose.gprior(g, N, K, return.g.stats = g.stats, 
        yty = yty, X.data = X.data)
    lprobcalc = gprior.info$lprobcalc
    start.list = .starter(K, start.value, y, N = N, XtX.big = XtX.big, 
        Xty.big = Xty.big, X = X, fixed.pos = fixed.pos)
    molddraw = start.list$molddraw
    start.position = start.list$start.position
    kold = sum(molddraw)
    position = (1:K)[molddraw == 1]
    collect.otherstats = FALSE
    otherstats = numeric(0)
    add.otherstats = numeric(0)
    if (gprior.info$return.g.stats & !(gprior.info$is.constant)) {
        add.otherstats = gprior.info$shrinkage.moments
        collect.otherstats = TRUE
    }
    cumsumweights = iter
    null.lik = lprobcalc$just.loglik(ymy = yty, k = 0)
    if (collect.otherstats) {
        addup <- function() {
            inccount <<- inccount + molddraw
            msize <<- msize + kold
            if (kold != 0) {
                bm[c(position, K + position, 2 * K + kold, 3 * 
                  K + position)] = c(b1, b2, 1, b1 > 0)
                bmo <<- bmo + bm
            }
            else {
                null.count <<- null.count + 1
            }
            otherstats <<- lik.list[["otherstats"]]
            add.otherstats <<- add.otherstats + otherstats
        }
    }
    else {
        addup <- function() {
            inccount <<- inccount + molddraw
            msize <<- msize + kold
            if (kold != 0) {
                bm[c(position, K + position, 2 * K + kold, 3 * 
                  K + position)] = c(b1, b2, 1, b1 > 0)
                bmo <<- bmo + bm
            }
            else {
                null.count <<- null.count + 1
            }
        }
    }
    if (is.enum) {
        cumsumweights = 0
        if (collect.otherstats) {
            addup <- function() {
                weight = exp(pmpold + lprobold - null.lik)
                inccount <<- inccount + weight * molddraw
                msize <<- msize + weight * kold
                cumsumweights <<- cumsumweights + weight
                if (kold != 0) {
                  bm[c(position, K + position, 2 * K + kold, 
                    3 * K + position)] = weight * c(b1, b2, 1, 
                    b1 > 0)
                  bmo <<- bmo + bm
                }
                else {
                  null.count <<- null.count + weight
                }
                otherstats <<- lik.list[["otherstats"]]
                add.otherstats <<- add.otherstats + weight * 
                  otherstats
            }
        }
        else {
            addup <- function() {
                weight = exp(pmpold + lprobold - null.lik)
                inccount <<- inccount + weight * molddraw
                msize <<- msize + weight * kold
                cumsumweights <<- cumsumweights + weight
                if (kold != 0) {
                  bm[c(position, K + position, 2 * K + kold, 
                    3 * K + position)] = weight * c(b1, b2, 1, 
                    b1 > 0)
                  bmo <<- bmo + bm
                }
                else {
                  null.count <<- null.count + weight
                }
            }
        }
    }
    environment(addup) <- environment()
    ols.object = .ols.terms2(positions = (1:K)[molddraw == 1], 
        yty = yty, k = kold, N, K = K, XtX.big = XtX.big, Xty.big = Xty.big)
    lik.list = lprobcalc$lprob.all(ymy = ols.object$ymy, k = kold, 
        bhat = ols.object$bhat, diag.inverse = ols.object$diag.inverse)
    lprobold = lik.list$lprob
    b1 = lik.list$b1new
    b2 = lik.list$b2new
    pmpold = pmplist$pmp(ki = kold, mdraw = molddraw)
    topmods = topmod(nbmodels = nmodel, nmaxregressors = K, bbeta = FALSE, 
        lengthfixedvec = length(add.otherstats))
    if (mcmc == "enum") {
        try(topmods$duplicates_possible(FALSE), silent = TRUE)
    }
    if (dotop && (burn == 0L)) 
        topmods$addmodel(mylik = pmpold + lprobold, vec01 = molddraw, 
            fixedvec = lik.list$otherstats)
    null.count = 0
    models.visited = 0
    inccount = numeric(K)
    msize = 0
    k.vec = numeric(K)
    b1mo = numeric(K)
    ab = numeric(K)
    b2mo = numeric(K)
    bb = numeric(K)
    possign = inccount
    mnewdraw = numeric(K)
    if (force.full.ols) {
        candi.is.full.object = TRUE
    }
    else {
        candi.is.full.object = FALSE
    }
    bmo = numeric(4 * K)
    bm = bmo
    if (is.enum) {
        addup()
    }
    if (!is.finite(pmpold)) 
        pmpold = -1e+90
    if (randomizeTimer) 
        set.seed(as.numeric(Sys.time()))
    t1 <- Sys.time()
    nrep = burn + iter
    i = 0
    while (i < nrep) {
        i = i + 1
        if (logfile) {
            if (i%%fact == 0) {
                if (nchar(sfilename) == 0) 
                  message(as.character(Sys.time()), ":", i, "current draw")
                else cat(as.character(Sys.time()), ":", i, "current draw \n", 
                  append = TRUE, file = sfilename)
            }
        }
        a = sampling(molddraw = molddraw, K = K, mPlus = mPlus, 
            maxk = maxk, oldk = kold)
        mnewdraw = a[["mnewdraw"]]
        positionnew = a[["positionnew"]]
        knew = length(positionnew)
        pmpnew = pmplist[["pmp"]](ki = knew, mdraw = mnewdraw)
        if (!is.enum) {
            if (int) {
                if (length(c(a$dropi, a$addi)) > 2 | i < 3 | 
                  force.full.ols) {
                  candi.is.full.object = TRUE
                }
                else {
                  candi.is.full.object = FALSE
                }
            }
            if (candi.is.full.object) {
                ols.candidate = .ols.terms2(positions = positionnew, 
                  yty = yty, k = knew, N, K = K, XtX.big = XtX.big, 
                  Xty.big = Xty.big)
                ymy.candi = ols.candidate[["ymy"]]
            }
            else {
                ymy.candi = ols.object[["child.ymy"]](a$addi, 
                  a$dropi, k = knew)
            }
            if ((ymy.candi < 0) | is.na(ymy.candi)) 
                stop(paste("stumbled on rank-deficient model"))
            lprobnew = lprobcalc[["just.loglik"]](ymy = ymy.candi, 
                k = knew)
            accept.candi = as.logical(log(runif(1, 0, 1)) < lprobnew - 
                lprobold + pmpnew - pmpold)
        }
        else {
            accept.candi = TRUE
            candi.is.full.object = FALSE
        }
        if (accept.candi) {
            if (!candi.is.full.object) {
                ols.res = ols.object[["mutate"]](addix = a$addi, 
                  dropix = a$dropi, newpos = positionnew, newk = knew)
            }
            else {
                ols.object = ols.candidate
                ols.res = ols.candidate[["full.results"]]()
            }
            lik.list = lprobcalc[["lprob.all"]](max(0, ols.res$ymy), 
                knew, ols.res$bhat, ols.res$diag.inverse)
            lprobold = lik.list[["lprob"]]
            position = positionnew
            pmpold = pmpnew
            molddraw = mnewdraw
            kold = knew
            models.visited = models.visited + 1
        }
        if (i > burn) {
            b1 = lik.list[["b1new"]]
            b2 = lik.list[["b2new"]]
            addup()
            if (dotop) 
                topmods[["addmodel"]](mylik = pmpold + lprobold, 
                  vec01 = molddraw, fixedvec = otherstats)
        }
    }
    if (dotop) 
        topmods = .topmod.as.bbetaT(topmods, gprior.info, X.data)
    timed <- difftime(Sys.time(), t1)
    if (is.enum) {
        iter = iter + 1
        models.visited = models.visited + 1
    }
    bmo = matrix(bmo, 4, byrow = TRUE)
    b1mo = bmo[1, ]
    b2mo = bmo[2, ]
    k.vec = bmo[3, ]
    possign = bmo[4, ]
    rm(bmo)
    post.inf = .post.calc(gprior.info, add.otherstats, k.vec, 
        null.count, X.data, topmods, b1mo, b2mo, iter, burn, 
        inccount, models.visited, K, N, msize, timed, cumsumweights, 
        mcmc, possign)
    result = list(info = post.inf$info, arguments = .construct.arglist(bms, 
        environment()), topmod = topmods, start.pos = sort(start.position), 
        gprior.info = post.inf$gprior.info, mprior.info = pmplist, 
        X.data = X.data, reg.names = post.inf$reg.names, bms.call = try(match.call(bms, 
            sys.call(0)), silent = TRUE))
    class(result) = c("bma")
    if (user.int) {
        print(result)
        print(timed)
        plot.bma(result)
    }
    return(invisible(result))
}

zlm <-
function (formula, data = NULL, subset = NULL, g = "UIP") 
{
    thiscall = match.call()
    mf <- match.call(expand.dots = FALSE)
    m <- match(c("formula", "data", "subset"), names(mf), 0L)
    mf <- mf[c(1L, m)]
    mf$drop.unused.levels <- TRUE
    mf[[1L]] <- as.name("model.frame")
    if (is.matrix(formula)) {
        mf <- model.frame(as.data.frame(formula, drop.unused.levels = TRUE))
    }
    else {
        mf <- eval(mf, parent.frame())
    }
    yXdata = as.matrix(mf)
    N = nrow(yXdata)
    K = ncol(yXdata) - 1
    dmdata = yXdata - matrix(colMeans(yXdata), N, K + 1, byrow = TRUE)
    yty = c(crossprod(dmdata[, 1]))
    olsres = .ols.terms2(positions = rep(TRUE, K), yty = yty, 
        N = N, K = K, XtX.big = crossprod(dmdata[, -1, drop = FALSE]), 
        Xty.big = c(crossprod(dmdata[, -1, drop = FALSE], dmdata[, 
            1])))$full.results()
    if (is.list(g)) {
        if (any(is.element(names(g), "gtype"))) 
            gprior.info = g
        else stop("Please provide a proper g-prior. see help(zlm)")
    }
    gprior.info = .choose.gprior(g = g, N = N, K = K, return.g.stats = TRUE, 
        yty = yty)
    lprobcalc = gprior.info$lprobcalc
    zres = lprobcalc$lprob.all(ymy = olsres$ymy, k = K, bhat = olsres$bhat, 
        diag.inverse = olsres$diag.inverse)
    betas = c(zres$b1)
    betas2 = c(zres$b2)
    alpha = mean(yXdata[, 1]) - c(crossprod(betas, colMeans(yXdata)[-1]))
    fitval = c(yXdata[, -1, drop = FALSE] %*% betas) + alpha
    resids = yXdata[, 1] - fitval
    if (gprior.info$is.constant) {
        gprior.info$shrinkage.moments = 1 - 1/(1 + gprior.info$g)
    }
    else {
        gprior.info$shrinkage.moments = zres$otherstats
    }
    mt <- attr(mf, "terms")
    alphabeta = c(alpha, betas)
    names(alphabeta) <- c("(Intercept)", attr(mt, "term.labels"))
    res = list()
    res$coefficients <- alphabeta
    res$residuals <- resids
    res$rank <- K + 1
    res$fitted.values <- fitval
    res$df.residual <- N - K - 1
    res$xlevels <- .getXlevels(mt, mf)
    res$call <- thiscall
    res$terms <- mt
    res$model <- mf
    res$na.action <- attr(mf, "na.action")
    res$coef2moments <- c(NA, betas2)
    res$marg.lik <- zres$lprob
    res$gprior.info <- gprior.info
    res$olsres <- olsres
    res$zres <- zres
    class(res) = c("zlm", "lm")
    return(res)
}

logLik.zlm <-
function (object, ...) 
{
    if (!is(object, "zlm")) 
        stop("argument 'formula' needs to be zlm object")
    ret = object$marg.lik
    attr(ret, "df") = object$rank + 1
    attr(ret, "nbobs") = object$rank + object$df.residual
    class(ret) = "logLik"
    return(ret)
}

bin2hex <-
function (binvec) 
{
    if (!is.logical(binvec)) {
        if (is.numeric(binvec)) {
            binvec = as.logical(binvec)
        }
        else {
            stop("need to supply a logical vector like c(T,F) or c(1,0)")
        }
    }
    hexobj <- .hexcode.binvec.convert(length(binvec))
    hexcode = hexobj$as.hexcode(binvec)
    if (nchar(hexcode) > (floor((length(binvec) - 1)/4) + 1)) {
        hexcode = substring(hexcode, 2)
    }
    return(hexcode)
}

gdensity <-
function (x, n = 512, plot = TRUE, addons = "zles", addons.lwd = 1.5, 
    ...) 
{
    dsgivenykernel <- function(kpazvec, sf, N) {
        (kpazvec[[1]] - 2)/2 * (1 - sf)^((kpazvec[[1]] - 4)/2) * 
            (1 - sf * kpazvec[[2]])^(-(N - 1)/2)
    }
    if (!is.bma(x)) 
        stop("argument needs to an object of class 'bma'")
    if (!(x$gprior$gtype == "hyper")) 
        stop("g prior density makes only sense for hyper-g prior.")
    if (n < 2) 
        stop("n needs to be at least 2")
    n = floor(n)
    dotargs = match.call(expand.dots = FALSE)$...
    N = x$info$N
    K = x$info$K
    tm = x$topmod
    bools = tm$bool_binary()
    betas = tm$betas()
    betas2 = tm$betas2()
    smoments = tm$fixed_vector()
    yXdata = as.matrix(x$X.data)
    yXdata = yXdata - matrix(colMeans(yXdata), N, K + 1, byrow = TRUE)
    yty = c(crossprod(yXdata[, 1]))
    positions = lapply(lapply(as.list(as.data.frame(bools)), 
        as.logical), which)
    ymyvec = unlist(lapply(lapply(positions, .ols.terms2, yty = yty, 
        N = N, K = K, XtX.big = crossprod(yXdata[, -1]), Xty.big = c(crossprod(yXdata[, 
            -1], yXdata[, 1]))), function(x) x$full.results()$ymy))
    kvec = tm$kvec_raw()
    zvec = 1 - ymyvec/yty
    pmpexact = pmp.bma(x, oldstyle = TRUE)[, 1]
    f21a = x$gprior.info$hyper.parameter
    if (length(smoments) == 0) {
        lprob = x$gprior.info$lprobcalc
        smoments = sapply(lapply(as.list(as.data.frame(rbind(kvec, 
            ymyvec))), function(x) lprob$lprob.all(ymy = x[2], 
            k = x[1], bhat = numeric(x[1]), diag.inverse = rep(1, 
                x[1]))), "[[", "otherstats")
    }
    Es = c(crossprod(smoments[1, ], pmpexact))
    Es2 = c(crossprod(smoments[2, ], pmpexact))
    Esd = sqrt(Es2 - Es^2)
    nbsteps = n
    cutoff = max(0, Es - 5 * Esd)
    sdiff = (1 - cutoff)/(nbsteps + 1)
    s.seq = seq(sdiff + cutoff, cutoff + nbsteps * sdiff, sdiff)
    sdensl = lapply(as.list(as.data.frame(rbind(kvec + f21a, 
        zvec))), dsgivenykernel, sf = s.seq, N = N)
    intconsts = lapply(lapply(sdensl, sum), "*", sdiff)
    sdensvecs = mapply("/", sdensl, intconsts)
    sdens = sdensvecs %*% pmpexact
    reslist = list(x = s.seq, y = sdens, bw = NULL, n = n, call = sys.call(), 
        data.name = "Shrinkage", has.na = FALSE)
    class(reslist) = "density"
    if (!plot) {
        return(reslist)
    }
    dotargs = .adjustdots(dotargs, ylab = "Density", xlab = "Shrinkage factor", 
        main = "Posterior Density of the Shrinkage Factor", type = "l", 
        col = "steelblue4")
    eval(as.call(c(list(as.name("plot"), as.name("s.seq"), as.name("sdens")), 
        as.list(dotargs))))
    leg.col = numeric(0)
    leg.lty = numeric(0)
    leg.legend = character(0)
    if (any(grep("f", addons, ignore.case = TRUE))) {
        for (m in 1:length(pmpexact)) {
            Esm = smoments[1, m]
            if (as.logical(Esm)) {
                ixlower = max(sum(s.seq < Esm), 1)
                Esheight = (sdens[ixlower + 1] - sdens[ixlower]) * 
                  (Esm - s.seq[ixlower]) + sdens[ixlower]
                lines(x = rep(Esm, 2), y = c(0, Esheight), col = 8, 
                  lwd = addons.lwd)
            }
        }
        leg.col = c(leg.col, 8)
        leg.lty = c(leg.lty, 1)
        leg.legend = c(leg.legend, "EV Models")
    }
    if (any(grep("e", addons, ignore.case = FALSE))) {
        abline(v = Es, col = 2, lwd = addons.lwd)
        leg.col = c(leg.col, 2)
        leg.lty = c(leg.lty, 1)
        leg.legend = c(leg.legend, "EV")
    }
    if (any(grep("s", addons, ignore.case = FALSE))) {
        if (!(Es - 2 * Esd) < 0) 
            abline(v = Es - 2 * Esd, col = 2, lty = 2, lwd = addons.lwd)
        if (!(Es + 2 * Esd) > 1) 
            abline(v = Es + 2 * Esd, col = 2, lty = 2, lwd = addons.lwd)
        leg.col = c(leg.col, 2)
        leg.lty = c(leg.lty, 2)
        leg.legend = c(leg.legend, "2x SD")
    }
    if (any(grep("m", addons, ignore.case = TRUE))) {
        median_index = sum(cumsum(sdens) < sum(sdens)/2)
        abline(v = (s.seq[median_index] + s.seq[median_index + 
            1])/2, col = 3, lwd = addons.lwd)
        leg.col = c(leg.col, 3)
        leg.lty = c(leg.lty, 1)
        leg.legend = c(leg.legend, "Median")
    }
    if (any(grep("z", addons, ignore.case = TRUE))) {
        abline(h = 0, col = "gray", lwd = addons.lwd)
    }
    if (any(grep("E", addons, ignore.case = FALSE))) {
        if (all(x$gprior.info$shrinkage.moments == 0)) 
            warning("bma object needs to contain posterior g statistics - cf. argument 'g.stats' in 'help(bms)'")
        else {
            abline(v = x$gprior.info$shrinkage.moments[1], col = 4, 
                lwd = addons.lwd)
            leg.col = c(leg.col, 4)
            leg.lty = c(leg.lty, 1)
            leg.legend = c(leg.legend, "EV (MCMC)")
        }
    }
    if (any(grep("S", addons, ignore.case = FALSE))) {
        if (!all(x$gprior.info$shrinkage.moments == 0)) {
            ES = x$gprior.info$shrinkage.moments[1]
            SDs = sqrt(x$gprior.info$shrinkage.moments[2] - x$gprior.info$shrinkage.moments[1]^2)
            if (ES - 2 * SDs > 0) 
                abline(v = ES - 2 * SDs, col = 4, lty = 2, lwd = addons.lwd)
            if (ES + 2 * SDs < 1) 
                abline(v = ES + 2 * SDs, col = 4, lty = 2, lwd = addons.lwd)
            leg.col = c(leg.col, 4)
            leg.lty = c(leg.lty, 2)
            leg.legend = c(leg.legend, "2x SD (MCMC)")
        }
    }
    if (any(grep("l", addons, ignore.case = TRUE)) & (length(leg.col) > 
        0)) {
        leg.pos = "topleft"
        legend(x = leg.pos, lty = leg.lty, col = leg.col, legend = leg.legend, 
            box.lwd = 0, bty = "n")
    }
    return(invisible(reslist))
}

as.zlm <-
function (bmao, model = 1) 
{
    thiscall = match.call()
    if (!is.bma(bmao)) 
        stop("bmao needs to be a bma object")
    bools = bmao$topmod$bool()
    if (all(is.character(model)) && length(model) == 1) {
        model = (1:length(bools))[bools == model[[1]]]
        if (length(model) == 0) 
            stop("Provided model hex-index was not found in bmao object topmodels")
    }
    else if (all(is.character(model)) && (length(model) > 1)) {
        mix = match(model, bmao$reg.names)
        if (any(is.na(mix))) 
            stop("Provided variable names do not conform to bma object")
        ll = logical(bmao$info$K)
        ll[mix] = TRUE
        model = (1:length(bools))[bools == bin2hex(ll)]
        rm(ll, mix)
        if (length(model) == 0) 
            stop("Model conforming to provided variable names was not found in bmao object topmodels")
    }
    else if ((length(model) == bmao$info$K) && (is.numeric(model) || 
        is.logical(model))) {
        model = (1:length(bools))[bools == bin2hex(model)]
        if (length(model) == 0) 
            stop("Provided binary model index was not found in bmao object topmodels")
    }
    else if ((length(model) == 1) && (is.numeric(model) || is.logical(model))) {
        if (model < 1 | model > length(bools)) 
            stop("Provided numeric model index was not found in bmao object topmodels")
    }
    else stop("model needs to be an integer, logical or character model index representation (hexcode or variable names)")
    inclvbls = as.logical(bmao$topmod$bool_binary()[, model, 
        drop = TRUE])
    yXdf = as.data.frame(bmao$X.data)
    zlmres = zlm(as.formula(yXdf[, c(TRUE, inclvbls)]), data = yXdf, 
        g = bmao$gprior.info)
    zlmres$call <- thiscall
    return(zlmres)
}

vcov.zlm <-
function (object, include.const = FALSE, ...) 
{
    Xmat = as.matrix(model.frame(object)[, -1, drop = FALSE])
    if (ncol(Xmat) < 1) 
        stop("Needs at least one non-constant regressor")
    regnames = colnames(Xmat)
    Xmat = Xmat - matrix(colMeans(Xmat), nrow(Xmat), ncol(Xmat), 
        byrow = TRUE)
    xxinv = chol2inv(chol(crossprod(Xmat)))
    outmat = ((object$coef2moments[[2]] - object$coefficients[[2]]^2)/xxinv[[1]]) * 
        xxinv
    if (include.const) {
        outmat = rbind(rep(NA, nrow(outmat) + 1), cbind(rep(NA, 
            ncol(outmat)), outmat))
        regnames = c("(Intercept)", regnames)
    }
    colnames(outmat) <- rownames(outmat) <- regnames
    return(outmat)
}

\documentclass[a4paper]{article}
\title{Bayesian Model Averaging with BMS\\ 
\normalsize{for BMS Version 0.3.5}}
\author{Martin Feldkircher and Stefan Zeugner}
\usepackage{natbib}
\usepackage{vmargin}
\begin{document}
\setpapersize{A4}
% \VignetteIndexEntry{Bayesian Model Averaging with BMS}
\maketitle
\abstract{This manual is a brief introduction to applied Bayesian Model Averaging with the R package \verb+BMS+. The manual is structured as a hands-on tutorial for readers with few experience with BMA. Readers from a more technical background are advised to consult the table of contents for formal representations of the concepts used in BMS.\\
For other tutorials and more information, please refer to \mbox{ \texttt{http://bms.zeugner.eu}.} }
\tableofcontents
\clearpage

<<echo=FALSE>>=
options(width=75)
@

\section{A Brief Introduction to Bayesian Model Averaging}
This section reiterates some basic concepts, and introduces some notation for readers with limited knowledge of BMA. Readers with more experience in BMA should skip this chapter and directly go to section \ref{sec:example}. For a more thorough introduction to BMA, consult \citet{Hoetingetal99}. Note that a version this vignette has been published as \citet{fz15}. If needed, please cite from the latter reference.
\subsection{Bayesian Model Averaging}
Bayesian Model Averaging (BMA) addresses model uncertainty in a canonical regression problem. Suppose a linear model structure, with $y$ being the dependent variable, $\alpha_\gamma$ a constant, $\beta_\gamma$ the coefficients, and $\varepsilon$ a normal IID error term with variance $\sigma^2$:
\begin{equation}
	y= \alpha_\gamma + X_\gamma \beta_\gamma + \varepsilon \qquad \varepsilon \sim N(0, \sigma^2 I) 
	\label{eq:lm}
\end{equation} 

A problem arises when there are many potential explanatory variables in a matrix $X$: Which variables $X_\gamma \in \{X\}$ should be then included in the model? And how important are they? The direct approach to do inference on a single linear model that includes all variables is inefficient or even infeasible with a limited number of observations. 

BMA tackles the problem by estimating models for all possible combinations of $\{X\}$ and constructing a weighted average over all of them. If $X$ contains $K$ potential variables, this means estimating $2^K$ variable combinations and thus $2^K$ models. The model weights for this averaging stem from posterior model probabilities that arise from Bayes' theorem:
\begin{equation}
 p(M_\gamma | y, X) \; = \; \frac{p(y |M_\gamma, X) p(M_\gamma)}{p(y|X)} \;  = \frac{p(y |M_\gamma, X) p(M_\gamma)	}{\sum_{s=1}^{2^K} p(y| M_s, X) p(M_s)} 
	\label{eq:bf}
\end{equation}

Here, $p(y|X)$ denotes the \emph{integrated} likelihood which is constant over all models and is thus simply a multiplicative term. Therefore, the posterior model probability (PMP) $p(M_\gamma|y,X)$ is proportional to\footnote{ Proportionality is expressed with the sign $\propto$: i.e. $p(M_\gamma | y, X) \; \propto \; p(y |M_\gamma, X) p(M_\gamma)$ } the marginal likelihood of the model $p(y |M_\gamma, X) $ (the probability of the data given the model $M_\gamma$) times a prior model probability $p(M_\gamma)$ -- that is, how probable the researcher thinks model $M_\gamma$ before looking at the data. Renormalization then leads to the PMPs and thus the model weighted posterior distribution for any statistic $\theta$ (e.g. the coefficients $\beta$):
$$ p(\theta| y, X) = \sum_{\gamma=1}^{2^K } p(\theta | M_\gamma,  y, X) p(M_\gamma| X, y) $$
The model prior $p(M_\gamma)$ has to be elicited by the researcher and should reflect prior beliefs. A popular choice is to set a uniform prior probability for each model $p(M_\gamma) \propto 1$ to represent the lack of prior knowledge. Further model prior options will be explored in section \ref{sec:mpriors}.

\subsection{Bayesian Linear Models and Zellner's $g$ prior}\label{ssec:zg}
The specific expressions for marginal likelihoods $p(M_\gamma|y,X)$ and posterior distributions $p(\theta | M_\gamma, y, X)$ depend on the chosen estimation framework. The literature standard is to use a 'Bayesian regression' linear model with a specific prior structure called 'Zellner's g prior' as will be outlined in this section.\footnote{Note that the presented framework is very similar to the natural normal-gamma-conjugate model - which employs proper priors for $\alpha$ and $\sigma$. Nonetheless, the resulting posterior statistics are virtually identical.}\\
For each individual model $M_\gamma$ suppose a normal error structure as in (\ref{eq:lm}). The need to obtain posterior distributions requires to specify the priors on the model parameters. Here, we place 'improper' priors on the constant and error variance, which means they are evenly distributed over their domain: $p(\alpha_\gamma) \propto 1$, i.e. complete prior uncertainty where the prior is located. Similarly, set $p(\sigma) \propto \sigma^{-1}$.

The crucial prior is the one on regression coefficients $\beta_\gamma$: Before looking into the data $(y,X)$, the researcher formulates her prior beliefs on coefficients into a normal distribution with a specified mean and variance. It is common to assume a conservative prior mean of zero for the coefficients to reflect that not much is known about them. Their variance structure is defined according to Zellner's g: $\sigma^2 ( \frac{1}{g} X_\gamma'X_\gamma )^{-1}$:
$$ \beta_\gamma | g \sim N \left( 0,\sigma^2 \left( \frac{1}{g} X_\gamma'X_\gamma \right)^{-1}\right) $$
This means that the researcher thinks coefficients are zero, and that their variance-covariance structure is broadly in line with that of the data $X_\gamma$. The hyperparameter $g$ embodies how certain the researcher is that coefficients are indeed zero: A small $g$ means few prior coefficient variance and therefore implies the researcher is quite certain (or conservative) that the coefficients are indeed zero. In contrast, a large $g$ means that the researcher is very uncertain that coefficients are zero. 

The posterior distribution of coefficients reflects prior uncertainty: Given $g$, it follows a t-distribution with expected value $E(\beta_\gamma|y,X,g,M_\gamma) = \frac{g}{1+g} \hat{\beta}_\gamma$, where $\hat{\beta}_\gamma$ is the standard OLS estimator for model $\gamma$. The expected value of coefficients is thus a convex combination of OLS estimator and prior mean (zero). The more conservative (smaller) $g$, the more important is the prior, and the more the expected value of coefficients is shrunk toward the prior mean zero. As $g \rightarrow \infty$, the coefficient estimator approaches the OLS estimator. Similarly, the posterior variance of $\beta_\gamma$ is affected by the choice of $g$:\footnote{here, $N$ denotes sample size, and $\bar{y}$ the sample mean of the response variable}
$$ Cov(\beta_\gamma|y,X,g,M_\gamma) = \frac{(y-\bar{y})'(y-\bar{y})}{N-3} \frac{g}{1+g} \left(1-\frac{g}{1+g} R_\gamma^2\right) (X_\gamma'X_\gamma)^{-1} $$
I.e. the posterior covariance is similar to that of the OLS estimator, times a factor that includes $g$ and $R^2_\gamma$, the OLS R-squared for model $\gamma$. The appendix shows how to apply the function \texttt{zlm} in order to estimate such models out of the BMA context.

For BMA, this prior framwork results into a very simple marginal likelihood $p(y|M_\gamma,X,g)$, that is related to the R-squared and includes a size penalty factor adjusting for model size $k_\gamma$:
$$ p(y|M_\gamma,X,g) \propto (y-\bar{y})'(y-\bar{y})^{-\frac{N-1}{2} } (1+g)^{-\frac{k_\gamma}{2}} \left( 1- \frac{g}{1+g} \right)^{-\frac{N-1}{2}} $$
The crucial choice here concerns the form of the hyperparameter $g$. A popular 'default' approach is the 'unit information prior' (UIP), which sets $g=N$ commonly for all models and thus attributes about the same information to the prior as is contained in one observation. (Please refer to section \ref{sec:gprior} for a discussion of other $g$-priors.)\footnote{Note that BMS is, in principle not restricted to Zellner's $g$-priors, as quite different coefficient priors might be defined by R-savy users.}


\section{A BMA Example: Attitude Data}\label{sec:example}
This section shows how to perform basic BMA with a small data set and how to obtain posterior coefficient and model statistics. 
\subsection{Model Sampling}
Equipped with this basic framework, let us explore one of the data sets built into R: The 'attitude' dataset describes the overall satisfaction rating of a large organization's employees, as well as several specific factors such as \verb+complaints+, the way of handling complaints within the organization (for more information type \verb+help(attitude)+). The data includes 6 variables, which means $2^6=64$ model combinations. Let us stick with the UIP g-prior (in this case $g=N=30$). Moreover, assume uniform model priors (which means that our expected prior model parameter size is $K/2=3$).

First load the data set by typing 
<<>>=
data(attitude)
@
In order to perform BMA you have to load the BMS library first, via the command:
<<>>=
library(BMS)
@
Now perform Bayesian model sampling via the function \verb+bms+, and write results into the variable \verb+att+.
<<>>=
att = bms(attitude, mprior = "uniform", g="UIP", user.int=F)
@
\verb+mprior = "uniform"+ means to assign a uniform model prior, \verb+g="UIP"+, the unit information prior on Zellner's $g$. The option \verb+user.int=F+ is used to suppress user-interactive output for the moment.\footnote{Note that the argument \texttt{g="UIP"} is actually redundant, as this is the default option for \texttt{bms}. The default model prior is somewhat different but does not matter very much with this data. Therefore, the command \texttt{att = bms(attitude)} gives broadly similar results.} The first argument is the data frame \verb+attitude+, and \verb+bms+ assumes that its first column is the response variable.\footnote{The specification of data can be supplied in different manners, e.g. in 'formulas'. Type \texttt{help(lm)} for a comparable function.}

\subsection{Coefficient Results}
The coefficient results can be obtained via
<<>>=
coef(att)
@
The above matrix shows the variable names and corresponding statistics: The second column \verb+Post Mean+ displays the coefficients averaged over all models, including the models wherein the variable was not contained (implying that the coefficient is zero in this case). The covariate \verb+complaints+ has a comparatively large coefficient and seems to be most important. The importance of the variables in explaining the data is given in the first column \verb+PIP+ which represents posterior inclusion probabilities - i.e. the sum of PMPs for all models wherein a covariate was included. We see that with $99.9\%$, virtually all of posterior model mass rests on models that include \verb+complaints+. In contrast, \verb+learning+ has an intermediate PIP of $40.6\%$, while the other covariates do not seem to matter much. Consequently their (unconditional) coefficients\footnote{Unconditional coefficients are defined as $E(\beta|y,X)=\sum_{\gamma=1}^{2^K} p(\beta_\gamma|, y,X, M_\gamma) p(M_\gamma|y,X)$ i.e. a weighted average over all models, including those where this particular coeffiecnt was restricted to zero. A conditional coeffienct in contrast, is 'conditional on inclusion', i.e. a weighted average only over those models where its regressor was included. Conditional coefficients may be obtained with the command \mbox{\texttt{coef(att, condi.coef =TRUE)}}.} are quite low, since the results quite often include models where these coefficients are zero. 

The coefficients' posterior standard deviations (\verb+Post SD+) reflect further evidence: \verb+complaints+ is certainly positive, while \verb+advance+ is most likely negative. In fact, the coefficient sign can also be inferred from the fourth column \verb+Cond.Pos.Sign+, the 'posterior probability of a positive coefficient expected value conditional on inclusion', respectively 'sign certainty'. Here, we see that in all encountered models containing this variables, the (expected values of) coefficients for \verb+complaints+ and \verb+learning+ were positive. In contrast, the corresponding number for \verb+privileges+ is near to zero, i.e. in virtually all models that include \verb+privileges+, its coefficient sign is negative. Finally, the last column \verb+idx+ denotes the index of the variables' appearance in the original data set, as our results are obviously sorted by PIP. 

Further inferring about the importance of our variables, it might be really more interesting to look at their standardized coefficients.\footnote{Standardized coefficients arise if both the response $y$ and the regressors $X$ are normalized to mean zero and variance one -- thus effectively bringing the data down to same order of magnitude.} Type:
<<>>=
coef(att, std.coefs=T, order.by.pip=F, include.constant=T) 
@
The standardized coefficients reveal similar importance as discussed above, but one sees that \verb+learning+ actually does not matter much in terms of magnitude. Note that \verb+order.by.pip=F+ represents covariates in their original order. The argument \verb+include.constant=T+ also prints out a (standardized) constant.

\subsection{Other Results}
Other basic information about the sampling procedure can be obtained via \footnote{Note that the command \texttt{print(att)} is equivalent to \texttt{coef(att); summary(att)}}.
<<>>=
summary(att) 
@
It reiterates some of the facts we already know, but adds some additional information such as \verb+Mean no. regressors+, posterior expected model size (cf. section \ref{sec:mpriors}). 

Finally, let's look into which models actually perform best: The function \verb+topmodels.bma+ prints out binary representations for all models included, but for the sake of illustration let us focus on the best three:\footnote{\texttt{topmodel.bma} results in a matrix in which each row corresponds to a covariate and each column to a model (ordered left-to-right by their PMP). The best three models are therefore in the three leftmost columns resulting from \texttt{topmodel.bma}, which are extracted via index assignment \texttt{[, 1:3]}.}
<<>>=
topmodels.bma(att)[,1:3]
@
\label{topmodcall}We see that the output also includes the posterior model probability for each model.\footnote{To access the PMP for any model, consider the function \texttt{pmpmodel} -- cf. \texttt{help(pmpmodel)} . } The best model, with 29\% posterior model probability,\footnote{The differentiation between \texttt{PMP (Exact)} and \texttt{PMP (MCMC)} is of importance if an MCMC sampler was used -- cf. section \ref{ssec:anavsmcmc} } is the one that only includes \verb+complaints+. However the second best model includes \verb+learning+ in addition and has a PMP of 16\%. Use the command \verb+beta.draws.bma(att)+ to obtain the actual (expected values of) posterior coefficient estimates for each of these models.

In order to get a more comprehensive overview over the models, use the command
<<fig=TRUE>>=
image(att)
@

Here, blue color corresponds to a positive coefficient, red to a negative coefficient, and white to non-inclusion (a zero coefficient). On the horizontal axis it shows the best models, scaled by their PMPs. We see again that the best model with most mass only includes \verb+complaints+. Moreover we see that complaints is included in virtually all model mass, and unanimously with a positive coefficient. In contrast, \verb+raises+ is included very little, and its coefficient sign changes according to the model. (Use \verb+image(att,yprop2pip=T)+ for another illustrating variant of this chart.)


\section{Model Size and Model Priors}\label{sec:mpriors}
Invoking the command \verb+summary(att)+ yielded the important posterior statistic \verb+Mean no. regressors+, the posterior expected model size (i.e. the average number of included regressors), which in our case was $2.11$. Note that the posterior expected model size is equal to the sum of PIPs -- verify via
<<>>=
sum(coef(att)[,1])
@
This value contrasts with the prior expected model size implictely used in our model sampling: With $2^K$ possible variable combinations, a uniform model prior means a common prior model probability of $p(M_\gamma)=2^{-K}$. However, this implies a prior expected model size of $\sum_{k=0}^K{ {K \choose k} k 2^{-K}}=K/2$. Moreover, since there are more possible models of size 3 than e.g. of size 1 or 5, the uniform model prior puts more mass on intermediate model sizes -- e.g. expecting a model size of $k_\gamma=3$ with ${6 \choose 3} 2^{-K} = 31\%$ probability. In order to examine how far the posterior model size distribution matches up to this prior, type:

<<fig=TRUE>>=
plotModelsize(att)
@

We see that while the model prior implies a symmetric distribution around $K/2=3$, updating it with the data yields a posterior that puts more importance on parsimonious models.

In order to illustrate the impact of the uniform model prior assumption, we might consider other popular model priors that allow more freedom in choosing prior expected model size and other factors.
\subsection{Binomial Model Prior}
The binomial model prior constitutes a simple and popular alternative to the uniform prior we just employed. It starts from the covariates' viewpoint, placing a common and fixed inclusion probability $\theta$ on each regressor. The prior probability of a model of size $k$ is therefore the product of inclusion and exclusion probabilities: 
$$ p(M_\gamma) = \theta^{k_\gamma} (1-\theta)^{K-k_\gamma} $$
Since expected model size is $\bar{m}= K \theta$, the researcher's prior choice reduces to eliciting a prior expected model size $\bar{m}$ (which defines $\theta$ via the relation $\theta=\bar{m}/K$). Choosing a prior model size of $K/2$ yields $\theta=\frac{1}{2}$ and thus exactly the uniform model prior $p(M_\gamma)=2^{-K}$. Therefore, putting prior model size at a value $<\frac{1}{2}$ tilts the prior distribution toward smaller model sizes and vice versa.
For instance, let's impose this fixed inclusion probability prior such that prior model size equals $\bar{m}=2$: Here, the option \verb+user.int=T+ directly prints out the results as from \verb+coef+ and \verb+summary+.\footnote{The command \texttt{g="UIP"} was omitted here since \texttt{bms} sets this by default anyway.}
<<>>=
att_fixed = bms(attitude, mprior="fixed", mprior.size=2, user.int=T)
@
As seen in \verb+Mean no. regressors+, the posterior model size is now $1.61$ which is somewhat smaller than with uniform model priors. Since posterior model size equals the sum of PIPs, many of them have also become smaller than under \verb+att+ But interestingly, the PIP of \verb+complaints+ has remained at near 100\%.

\subsection{Custom Prior Inclusion Probabilities}
In view of the pervasive impact of \verb+complaints+, one might wonder whether its importance would also remain robust to a greatly unfair prior. For instance, one could define a prior inclusion probability of only $\theta=0.01$ for the \verb+complaints+ while setting a 'standard' prior inclusion probability of $\theta=0.5$ for all other variables. Such a prior might be submitted to \verb+bms+ by assigning a vector of prior inclusion probabilities via its \verb+mprior.size+ argument:\footnote{This implies a prior model size of $\bar{m}= 0.01 + 5 \times 0.5 = 2.51$}
<<>>=
att_pip = bms(attitude, mprior="pip", mprior.size=c(.01,.5,.5,.5,.5,.5), user.int=F)
@
But the results (obtained with \verb+coef(att_pip)+) show that \verb+complaints+ still retains its PIP of near 100\%. Instead, posterior model size decreases (as evidenced in a call to \newline\mbox{\texttt{plotModelsize(att\_pip)}}), and all other variables obtain a far smaller PIP. 

\subsection{Beta-Binomial Model Priors}
Like the uniform prior, the fixed common $\theta$ in the binomial prior centers the mass of of its distribution near the prior model size. A look on the prior model distribution with the following command shows that the prior model size distribution is quite concentrated around its mode. 
<<>>=
plotModelsize(att_fixed) 
@

This feature is sometimes criticized, in particular by \citet{ls08}: They note that to reflect prior uncertainty about model size, one should rather impose a prior that is less tight around prior expected model size. Therefore, \citet{ls08} propose to put a \emph{hyperprior} on the inclusion probability $\theta$, effectively drawing it from a Beta distribution. In terms of researcher input, this prior again only requires to choose the prior expected model size. However, the resulting prior distribution is considerably less tight and should thus reduce the risk of unintended consequences from imposing a particular prior model size.\footnote{Therefore, the beta-binomial model prior with random theta is implemented as the default choice in \texttt{bms}.} 

For example, take the beta-binomial prior with prior model size $K/2$\footnote{Note that the arguments here are actually the default values of \texttt{bms}, therefore this command is equivalent to \texttt{att\_random=bms(attitude)}.} -- and compare this to the results from \verb+att+ (which is equivalent to a fixed $\theta$ model prior of prior model size $K/2$.)
<<fig=TRUE>>=
att_random = bms(attitude, mprior="random", mprior.size=3, user.int=F)
plotModelsize(att_random)
@

With the beta-binomial specification and prior model size $\bar{m}=K/2$, the model prior is completely flat over model sizes, while the posterior model size turns out to be $1.73$. In terms of coefficient and posterior model size distribution, the results are very similar to those of \verb+att_fixed+, even though the latter approach involved a tighter model prior. Concluding, a decrease of prior importance by the use of the beta-binomial framework supports the results found in \verb+att_fixed+.

We can compare the PIPs from the four approaches presented so far with the following command:\footnote{This is equivalent to the command \texttt{plotComp(att, att\_fixed, att\_pip, att\_random)}}
<<eval=FALSE>>=
plotComp(Uniform=att, Fixed=att_fixed, PIP=att_pip, Random=att_random)
@

<<fig=TRUE, echo=FALSE>>=
plotComp(Uniform=att, Fixed=att_fixed, PIP=att_pip, Random=att_random, cex=2)
@

Here as well, \verb+att_fixed+ (Fixed) and \verb+att_random+ (Random) display similar results with PIPs plainly smaller than those of \verb+att+ (Uniform). 

Note that the appendix contains an overview of the built-in model priors available in BMS. Moreover, BMS allows the user to define any custom model prior herself and straightforwardly use it in \texttt{bms} - for examples, check \mbox{\texttt{http://bms.zeugner.eu/custompriors.php}}. Another concept relating to model priors is to keep fixed regressors to be included in every sampled model: Section \ref{ssec:fixreg} provides some examples.


\section{MCMC Samplers and More Variables}\label{sec:mcmc}

\subsection{MCMC Samplers}\label{ssec:mcmc}
With a small number of variables, it is straightforward to enumerate all potential variable combinations to obtain posterior results. For a larger number of covariates, this becomes more time intensive: enumerating all models for 25 covariates takes about 3 hours on a modern PC, and doing a bit more already becomes infeasible: With 50 covariates for instance, there are more than a quadrillion ($\approx 10^{15}$) potential models to consider. 
%
In such a case, MCMC samplers gather results on the most important part of the posterior model distribution and thus approximate it as closely as possible. BMA mostly relies on the Metropolis-Hastings algorithm, which 'walks' through the model space as follows: 

At step i, the sampler stands at a certain 'current' model $M_i$ with PMP $p(M_i|y,X)$. In step $i+1$ a candidate model $M_j$ is proposed. The sampler switches from the current model to model $M_j$ with probability $p_{i,j}$:
$$p_{i,j} = \min(1, p(M_j|y,X)/p(M_i|y,x) )$$
 In case model $M_j$ is rejected, the sampler moves to the next step and proposes a new model $M_k$ against $M_i$. In case model $M_j$ is accepted, it becomes the current model and has to survive against further candidate models in the next step. In this manner, the number of times each model is kept will converge to the distribution of posterior model probabilities $p(M_i|y,X)$.

In addition to enumerating all models, BMS implements two MCMC samplers that differ in the way they propose candidate models:
\begin{itemize}
	\item \emph{Birth-death sampler} (\verb+bd+): This is the standard model sampler used in most BMA routines. One of the $K$ potential covariates is randomly chosen; if the chosen covariate forms already part of the current model $M_i$, then the candidate model $M_j$ will have the same set of covariates as $M_i$ but for the chosen variable ('dropping' a variable). If the chosen covariate is not contained in $M_i$, then the candidate model will contain all the variables from $M_i$ plus the chosen covariate ('adding' a variable).
	\item \emph{Reversible-jump sampler} (\verb+rev.jump+): Adapted to BMA by \citet{mad95} this sampler either draws a candidate by the birth-death method with 50\% probability. In the other case (chosen with 50\% probability) a 'swap' is proposed, i.e. the candidate model $M_j$ randomly drops one covariate with respect to $M_i$ and randomly adds one chosen at random from the potential covariates that were not included in model $M_i$.
	\item \emph{Enumeration} (\verb+enum+): Up to fourteen covariates, complete enumeration of all models is the default option: This means that instead of an approximation by means of the aforementioned MCMC sampling schemes \textit{all} possible models are evaluated. As enumeration becomes quite time-consuming or infeasible for many variables, the default option is \verb+mcmc="bd"+ in case of $K>14$, though enumeration can still be invoked with the command \verb+mcmc="enumerate"+. 
\end{itemize}

The quality of an MCMC approximation to the actual posterior distribution depends on the number of draws the MCMC sampler runs through. In particular, the sampler has to start out from some model\footnote{\texttt{bms} has some simple algorithms implemented to choose 'good' starting models -- consult the option \texttt{start.value} under \texttt{help(bms)} for more information.} that might not be a 'good' one. Hence the first batch of iterations will typically not draw models with high PMPs as the sampler will only after a while converge to spheres of models with the largest marginal likelihoods. Therefore, this first set of iterations (the 'burn-ins') is to be omitted from the computation of results. In \verb+bms+, the argument \verb+burn+ specifies the number of burn-ins, and the argument \verb+iter+ the number of subsequent iterations to be retained. 


\subsection{An Example: Economic Growth}\label{ssec:fls}
In one of the most prominent applications of BMA, \citet{fls:ccg} analyze the importance of 41 explanatory variables on long-term term economic growth in 72 countries by the means of BMA. The data set is built into BMS, a short description is available via \verb+help(datafls)+. They employ a uniform model prior and the birth-death MCMC sampler. Their $g$ prior is set to $g=\max(N,K^2)$, a mechanism such that PMPs asymptotically either behave like the Bayesian information criterion (with $g=N$) or the risk inflation criterion ($g=K^2$) -- in \verb+bms+ this prior is assigned via the argument \verb+g="BRIC"+. 

Moreover \citet{fls:ccg} employ more than 200 million number of iterations after a substantial number of burn-ins. Since this would take quite a time, the following example reenacts their setting with only 50,000 burn-ins and 100,000 draws and will take about 30 seconds on a modern PC:
<<echo=TRUE,eval=FALSE>>=
data(datafls)
fls1 = bms(datafls, burn=50000, iter=100000, g="BRIC", mprior="uniform", nmodel=2000, mcmc="bd", user.int=F)
@
<<echo=FALSE>>=
fls1 = BMS:::.flsresultlist('fls1')
@
Before looking at the coefficients, check convergence by invoking the \verb+summary+ command:\label{sumfls1}\footnote{Since MCMC sampling chains are never completely equal, the results presented here might differ from what you get on your machine.}
<<>>=
summary(fls1)
@
Under \verb+Corr PMP+, we find the correlation between iteration counts and analytical PMPs for the 2000 best models (the number 2000 was specified with the \verb+nmodel=2000+ argument). At \Sexpr{summary(fls1)["Corr PMP"]}, this correlation is far from perfect but already indicates a good degree of convergence. For a closer look at convergence between analytical and MCMC PMPs, compare the actual distribution of both concepts:

<<fig=TRUE>>=
plotConv(fls1)
@

The chart presents the best 2,000 models encountered ordered by their analytical PMP (the red line), and plots their MCMC iteration counts (the blue line). For an even closer look, one might just check the corresponding image for just the best 100 models with the following command:\footnote{With \texttt{bma} objects such as \texttt{fls1}, the indexing parentheses \texttt{[]} are used to select subsets of the (ranked) best models retained in the object. For instance, while \texttt{fls1} contains 2,000 models, \texttt{fls1[1:100]} only contains the 100 best models among them. Correspondingly, \texttt{fls1[37]} would only contain the 37th-best model. Cf. \texttt{help('[.bma')}}
<<fig=TRUE>>=
plotConv(fls1[1:100])
@

\subsection{Analytical vs. MCMC likelihoods}\label{ssec:anavsmcmc}
The example above already achieved a decent level of correlation among analytical likelihoods and iteration counts with a comparatively small number of sampling draws. In general, the more complicated the distribution of marginal likelihoods, the more difficulties the sampler will meet before converging to a good approximation of PMPs. The quality of approximation may be inferred from the number of times a model got drawn vs. their actual marginal likelihoods. Partly for this reason, \verb+bms+ retains a pre-specified number of models with the highest PMPs encountered during MCMC sampling, for which PMPs and draw counts are stored. Their respective distributions and their correlation indicate how well the sampler has converged. 

However, due to RAM limits, the sampling chain can hardly retain more than a few 100,000 of these models. Instead, it computes aggregate statistics on-the-fly, taking iteration counts as model weights. For model convergence and some posterior statistics \verb+bms+ retains only the 'top' (highest PMP) \verb+nmodel+ models it encounters during iteration. 
Since the time for updating the iteration counts for the 'top' models grows in line with their number, the sampler becomes considerably slower the more 'top' models are to be kept. Still, if they are sufficiently numerous, those best models can already cover most of posterior model mass - in this case it is feasible to base posterior statistics on analytical likelihoods instead of MCMC frequencies, just as in the enumeration case from section \ref{sec:example}. From \verb+bms+ results, the PMPs of 'top' models may be displayed with the command \verb+pmp.bma+. For instance, one could display the PMPs of the best five models for \verb+fls1+ as follows:\footnote{\texttt{pmp.bma} returns a matrix with two columns and one row for each model. Consequently \texttt{pmp.bma(fls1)[1:5,]} extracts the first five rows and all columns of this matrix.}
<<>>=
pmp.bma(fls1)[1:5,]
@
The numbers in the left-hand column represent analytical PMPs (\texttt{PMP (Exact)}) while the right-hand side displays MCMC-based PMPs (\texttt{PMP (MCMC)}). Both decline in roughly the same fashion, however sometimes the values for analytical PMPs differ considerably from the MCMC-based ones. This comes from the fact that MCMC-based PMPs derive from the number of iteration counts, while the 'exact' PMPs are calculated from comparing the analytical likelihoods of the best models -- cf. equation (\ref{eq:bf}).\footnote{In the call to \texttt{topmodels.bma} on page \pageref{topmodcall}, the PMPs under 'MCMC' and analytical ('exact') concepts were equal since 1) enumeration bases both 'top' model calculation and aggregate on-the-fly results on analytical PMPs and 2) because all possible models were retained in the object \texttt{att}.} In order to see the importance of all 'top models' with respect to the full model space, we can thus sum up their MCMC-based PMPs as follows:
<<>>=
colSums(pmp.bma(fls1))
@
Both columns sum up to the same number and show that in total, the top 2,000 models account for ca. \Sexpr{round(colSums(pmp.bma(fls1))[[2]],2)*100}\% of posterior model mass.\footnote{Note that this share was already provided in column \texttt{\% Topmodels} resulting from the \texttt{summary} command on page \pageref{sumfls1}.} They should thus provide a rough approximation of posterior results that might or might not be better than the MCMC-based results. For this purpose, compare the best 5 covariates in terms of PIP by analytical and MCMC methods: \verb+coef(fls1)+ will display the results based on MCMC counts.
<<>>=
coef(fls1)[1:5,]
@
In contrast, the results based on analytical PMPs will be invoked with the \verb+exact+ argument:
<<>>=
coef(fls1,exact=TRUE)[1:5,]
@
The ordering of covariates in terms of PIP as well as the coefficients are roughly similar. However, the PIPs under \verb+exact = TRUE+ are somewhat larger than with MCMC results. Closer inspection will also show that the analytical results downgrade the PIPs of the worst variables with respect to MCMC-PIPs. This stems from the fact that analytical results do not take into account the many 'bad' models that include 'worse' covariates and are factored into MCMC results.

Whether to prefer analytical or MCMC results is a matter of taste -- however the literature prefers coefficients the analytical way: \citet{fls:ccg}, for instance, retain 5,000 models and report results based on them. 


\subsection{Combining Sampling Chains}
The MCMC samplers described in section \ref{ssec:mcmc} need to discard the first batch of draws (the burn-ins) since they start out from some peculiar start model and may reach the altitudes of 'high' PMPs only after many iterations. Here, choosing an appropriate start model may help to speed up convergence. By default \verb+bms+ selects its start model as follows: from the full model\footnote{actually, a model with randomly drawn $\min(K,N-3)$ variables}, all covariates with OLS t-statistics $>0.2$ are kept and included in the start model. Other start models may be assigned outright or chosen according to a similar mechanism (cf. argument \verb+start.value+ in \verb+help(bms)+). 

However, in order to improve the sampler's convergence to the PMP distribution, one might actually start from several different start models. This could by particularly helpful if the models with high PMPs are clustered in distant 'regions'. For instance, one could set up the \citet{fls:ccg} example above to get iteration chains from different starting values and combine them subsequently. Start e.g. a shorter chain from the null model (the model containing just an intercept), and use the 'reversible jump' MCMC sampler:
<<echo=FALSE>>=
fls2 = BMS:::.flsresultlist('fls2')
@
<<echo=TRUE,eval=FALSE>>=
fls2= bms(datafls, burn=20000, iter=50000, g="BRIC", mprior="uniform", mcmc="rev.jump", start.value=0, user.int=F)
@
<<echo=TRUE>>=
summary(fls2)
@

With \Sexpr{round(cor(pmp.bma(fls2))[2,1],2)}, the correlation between analytical and MCMC PMPs is a bit smaller than the \Sexpr{round(cor(pmp.bma(fls1))[2,1],2)} from the \verb+fls1+ example in section \ref{ssec:anavsmcmc}. However, the results of this sampling run may be combined to yield more iterations and thus a better representation of the PMP distribution.
<<>>=
fls_combi = c(fls1,fls2)
summary(fls_combi)
@
With \Sexpr{round(cor(pmp.bma(fls_combi))[2,1],2)}, the PMP correlation from the combined results is broadly better than either of its two constituent chains \verb+fls1+ and \verb+fls2+. Still, the PIPs and coefficients do not change much with respect to \verb+fls1+ -- as evidenced e.g. by \verb+plotComp(fls1, fls_combi, comp="Std Mean")+. 



\section{Alternative Formulations for Zellner's g Prior}\label{sec:gprior}
\subsection{Alternative Fixed g-Priors}
Virtually all BMA applications rely on the presented framework with Zellner's $g$ prior, and the bulk of them relies on specifying a fixed $g$. As mentioned in section \ref{ssec:zg}, the value of $g$ corresponds to the degree of prior uncertainty: A low $g$ renders the prior coefficient distribution tight around a zero mean, while a large $g$ implies large prior coefficient variance and thus decreases the importance of the coefficient prior. 

While some popular default elicitation mechanisms for the $g$ prior (we have seen UIP and BRIC) are quite popular, they are also subject to severe criticism. Some (e.g \citealt{fls:bmp}) advocate a comparatively large $g$ prior to minimize prior impact on the results, stay close to OLS coefficients, and represent the absolute lack of prior knowledge. Others (e.g. \citealt{cic08}) demonstrate that such a large $g$ may not be robust to noise innovations and risks over-fitting -- in particular if the the noise component plays a substantial role in the data. Again others \citep{eichi07} advocate intermediate fixed values for the $g$ priors or present alternative default specifications \citep{liang:mgp}.\footnote{Note however, that $g$ should in general be monotonously increasing in $N$: \citet{fls:bmp} prove that this sufficient for 'consistency', i.e. if there is one single linear model as in equation (\ref{eq:lm}), than its PMP asymptotically reaches 100\% as sample size $N \rightarrow \infty$.}

In BMS, any fixed $g$-prior may be specified directly by submitting its value to the \verb+bms+ function argument \verb+g+. For instance, compare the results for the \citet{fls:ccg} setting when a more conservative prior such as $g=5$ is employed (and far too few iterations are performed): 

<<echo=FALSE>>=
fls_g5 = BMS:::.flsresultlist('fls_g5')
@
<<echo=TRUE,eval=FALSE>>=
fls_g5 = bms(datafls, burn=20000, iter=50000, g=5, mprior="uniform", user.int=F)
@
<<echo=TRUE,eval=TRUE>>=
coef(fls_g5)[1:5,]
summary(fls_g5)
@
The PIPs and coefficients for the best five covariates are comparable to the results from section \ref{ssec:fls} but considerably smaller, due to a tight shrinkage factor of $\frac{g}{1+g}=\frac{5}{6}$ (cf. section \ref{ssec:zg}). More important, the posterior expected model size \Sexpr{round(sum(coef(fls_g5)[,1]),1)} exceeds that of \verb+fls_combi+ by a large amount. This stems from the less severe size penalty imposed by eliciting a small $g$. Finally, with \Sexpr{round(cor(pmp.bma(fls_g5))[2,1],2)}, the correlation between analytical and MCMC PMPs means that the MCMC sampler has not at all converged yet. \citet{fz:superM} show that the smaller the $g$ prior, the less concentrated is the PMP distribution, and therefore the harder it is for the MCMC sampler to provide a reasonable approximation to the actual PMP distribution. Hence the above command should actually be run with many more iterations in order to achieve meaningful results.

\subsection{Model-specific g-Priors}
Eliciting a fixed $g$-prior common to all models can be fraught with difficulties and unintended consequences. Several authors have therefore proposed to rely on model-specific priors (cf. \citealt{liang:mgp} for an overview), of which the following allow for closed-form solutions and are implemented in BMS:
\begin{itemize}
	\item Empirical Bayes $g$ -- local (\verb+EBL+): $g_\gamma=arg max_g \; p(y|M_\gamma,X,g)$. Authors such as \citet{george00} or \citet{hansen01} advocate an 'Empirical Bayes' approach by using information contained in the data $(y,X)$ to elicit $g$ via maximum likelihood. This amounts to setting $g_\gamma=\max(0,F^{OLS}_\gamma-1)$ where $F^{OLS}_\gamma$ is the standard OLS F-statistic for model $M_\gamma$. Apart from obvious advantages discussed below, the \verb+EBL+ prior is not so popular since it involves 'peeking' at the data in prior formulation. Moreover, asymptotic 'consistency' of BMA is not guaranteed in this case.
	\item Hyper-$g$ prior (\verb+hyper+): \citet{liang:mgp} propose putting a hyper-prior $g$; In order to arrive at closed-form solutions, they suggest a Beta prior on the shrinkage factor of the form $\frac{g}{1+g} \sim Beta \left(1, \frac{a}{2}-1 \right)$, where $a$ is a parameter in the range $2 < a \leq 4$. Then, the prior expected value of the shrinkage factor is $E(\frac{g}{1+g})=\frac{2}{a}$. Moreover, setting $a=4$ corresponds to uniform prior distribution of $\frac{g}{1+g}$ over the interval $[0,1]$, while $a \rightarrow 2$ concentrates prior mass very close to unity (thus corresponding to $g\rightarrow \infty$). (\verb+bms+ allows to set $a$ via the argument \verb+g="hyper=x"+, where \verb+x+ denotes the $a$ parameter.) 
	The virtue of the hyper-prior is that it allows for prior assumptions about $g$, but relies on Bayesian updating to adjust it. This limits the risk of unintended consequences on the posterior results, while retaining the theoretical advantages of a fixed $g$. Therefore \citet{fz:superM} prefer the use of hyper-$g$ over other available $g$-prior frameworks.
\end{itemize}

Both model-specific $g$ priors adapt to the data: The better the signal-to-noise ratio, the closer the (expected) posterior shrinkage factor will be to one, and vice versa. Therefore average statistics on the shrinkage factor offer the interpretation as a 'goodness-of-fit' indicator (\citet{fz:superM} show that both EBL and hyper-$g$ can be interpreted in terms of the OLS F-statistic).

Consider, for instance, the \citet{fls:ccg} example under an Empirical Bayes prior:
<<echo=FALSE>>=
fls_ebl = BMS:::.flsresultlist('fls_ebl')
@
<<echo=TRUE,eval=FALSE>>=
fls_ebl = bms(datafls, burn=20000, iter=50000, g="EBL", mprior="uniform", nmodel=1000, user.int=F)
@
<<>>=
summary(fls_ebl)
@
The result \verb+Shrinkage-Stats+ reports a posterior average EBL shrinkage factor of \Sexpr{round(as.numeric(strsplit(summary(fls_ebl)[13],"=")[[1]][[2]]),3)}, which corresponds to a shrinkage factor $\frac{g}{1+g}$ under  $g\approx $ \Sexpr{round(1/(1-as.numeric(strsplit(summary(fls_ebl)[13],"=")[[1]][[2]]))-1,-0)}. Consequently, posterior model size is considerably larger than under \verb+fls_combi+, and the sampler has had a harder time to converge, as evidenced in a quite low \verb+Corr PMP+. Both conclusions can also be drawn from performing the \verb+plot(fls_ebl)+ command that combines the \verb+plotModelsize+ and \verb+plotConv+ functions:
<<fig=TRUE>>=
plot(fls_ebl)
@

The upper chart shows that posterior model size distribution remains very close to the model prior; The lower part displays the discord between iteration count frequencies and analytical PMPs.


The above results show that using a flexible and model-specific prior on \citet{fls:ccg} data results in rather small posterior estimates of $\frac{g}{1+g}$, thus indicating that the \verb+g="BRIC"+ prior used in \verb+fls_combi+ may be set too far from zero. This interacts with the uniform model prior to concentrate posterior model mass on quite large models. However, imposing a uniform model prior means to expect a model size of $K/2=20.5$, which may seem overblown. Instead, try to impose smaller model size through a corresponding model prior -- e.g. impose a prior model size of 7 as in \citet{bace04}. This can be combined with a hyper-$g$ prior, where the argument \verb+g="hyper=UIP"+ imposes an $a$ parameter such that the prior expected value of $g$ corresponds to the unit information prior ($g=N$).\footnote{This is the default hyper-g prior and may therefore be as well obtained with \texttt{g=\textquotedbl hyper \textquotedbl}. }


<<echo=FALSE,eval=TRUE>>=
fls_hyper = BMS:::.flsresultlist('fls_hyper')
@
<<echo=TRUE,eval=FALSE>>=
fls_hyper = bms(datafls, burn=20000, iter=50000, g="hyper=UIP", mprior="random", mprior.size=7, nmodel=1000, user.int=F)
@
<<echo=TRUE,eval=TRUE>>=
summary(fls_hyper)
@

From \verb+Shrinkage-Stats+, posterior expected shrinkage is \Sexpr{round(fls_hyper$gprior.info$shrinkage.moments[[1]],3) }, with rather tight standard deviation bounds. Similar to the EBL case before, the data thus indicates that shrinkage should be rather small (corresponding to a fixed g of $g \approx$ \Sexpr{round(1/(1-as.numeric(fls_hyper$gprior.info$shrinkage.moments[[1]]))-1,-0)}) and not vary too much from its expected value. Since the hyper-g prior induces a proper posterior distribution for the shrinkage factor, it might be helpful to plot its density with the command below. The chart confirms that posterior shrinkage is tightly concentrated above \Sexpr{ round(fls_hyper$gprior.info$shrinkage.moments[[1]]-as.numeric(strsplit(summary(fls_hyper)[13],"=")[[1]][[3]]),2)}.
<<fig=TRUE>>=
gdensity(fls_hyper)
@

While the hyper-g prior had an effect similar to the EBL case \verb+fls_ebl+, the model prior now employed leaves the data more leeway to adjust posterior model size. The results depart from the expected prior model size and point to an intermediate size of ca. \Sexpr{round(sum(estimates.bma(fls_hyper)[,1]),0)}. The focus on smaller models is evidenced by charting the best 1,000 models with the \verb+image+ command:
<<fig=TRUE>>=
image(fls_hyper)
@

In a broad sense, the coefficient results correspond to those of \verb+fls_combi+, at least in expected values. However, the results from \verb+fls_hyper+ were obtained under more sophisticated priors that were specifically designed to avoid unintended influence from prior parameters: 
By construction, the large shrinkage factor under \verb+fls_combi+ induced a quite small posterior model size of \Sexpr{round(as.numeric(summary(fls_combi)[[1]]),1)} and concentrated posterior mass tightly on the best models encountered (they make up \Sexpr{round(as.numeric(summary(fls_combi)[[8]]),0)}\% of the entire model mass). In contrast, the hyper-g prior employed for \verb+fls_hyper+ indicated a rather low posterior shrinkage factor and consequently resulted in higher posterior model size (\Sexpr{round(as.numeric(summary(fls_hyper)[[1]]),1)}) and less model mass concentration (\Sexpr{round(as.numeric(summary(fls_hyper)[[8]]),0)}\%). 

\subsection{Posterior Coefficient Densities}
In order to compare more than just coefficient expected values, it is advisable to consult the entire posterior distribution of coefficients. For instance, consult the posterior density of the coefficient for \verb+Muslim+, a variable with a PIP of \Sexpr{round(estimates.bma(fls_combi)["Muslim",1],3)*100}\%: The \verb+density+ method produces marginal densities of posterior coefficient distributions and plots them, where the argument \verb+reg+ specifies the variable to be analyzed. 
<<fig=TRUE>>=
density(fls_combi,reg="Muslim")
@

We see that the coefficient is neatly above zero, but somewhat skewed. The integral of this density will add up to \Sexpr{round(estimates.bma(fls_combi, exact=T)["Muslim",1],3)}, conforming to the analytical PIP of \verb+Muslim+. The vertical bars correspond to the analytical coefficient conditional on inclusion from \verb+fls_combi+ as in
<<>>=
coef(fls_combi,exact=T,condi.coef=T)["Muslim",]
@

Note that the posterior marginal density is actually a model-weighted mixture of posterior densities for each model and can this be calculated only for the top models contained in \verb+fls_combi+ (here \Sexpr{length(fls_combi[["topmod"]][["lik"]]())}). 

Now let us compare this density with the results under the hyper-$g$ prior:\footnote{Since for the hyper-$g$ prior, the marginal posterior coefficient distribution derive from quite complicated expressions, executing this command could take a few seconds.}
<<fig=TRUE>>=
dmuslim=density(fls_hyper,reg="Muslim",addons="Eebl")
@

Here, the \verb+addons+ argument assigns the vertical bars to be drawn: the expected conditional coefficient from MCMC (\verb+E+) results should be indicated in contrast to the expected coefficient based on analytical PMPs (\verb+e+). In addition the expected coefficients under the individual models are plotted (\verb+b+) and a legend is included (\verb+l+). The density seems more symmetric than before and the analytical results seem to be only just smaller than what could be expected from MCMC results. 

Nonetheless, even though \verb+fls_hyper+ and \verb+fls_combi+ applied very different $g$ and model priors, the results for the \verb+Muslim+ covariate are broadly similar: It is unanimously positive, with a conditional expected value somewhat above $0.01$. In fact 95\% of the posterior coefficient mass seems to be concentrated between \Sexpr{round(BMS:::quantile.coef.density(dmuslim,.025),3)} and \Sexpr{round(BMS:::quantile.coef.density(dmuslim,.975),3)}:
<<>>=
quantile(dmuslim, c(0.025, 0.975))
@
\section{Predictive Densities}
Of course, BMA lends itself not only to inference, but also to prediction. The employed 'Bayesian Regression' models naturally give rise to predictive densities, whose mixture yields the BMA predictive density -- a procedure very similar to the coefficient densities explored in the previous section.

Let us, for instance, use the information from the first 70 countries contained in \texttt{datafls} to forecast economic growth for the latter two, namely Zambia (identifier \texttt{ZM}) and Zimbabwe (identifier \texttt{ZW}). We then can use the function \texttt{pred.density} on the BMA object \texttt{fcstbma} to form predictions based on the explanatory variables for Zambia and Zimbabwe (which are in \verb+datafls[71:72,]+). 

<<>>=
fcstbma= bms(datafls[1:70,], mprior="uniform", burn=20000, iter=50000, user.int=FALSE)

pdens = pred.density(fcstbma, newdata=datafls[71:72,])
@

The resulting object \texttt{pdens} holds the distribution of the forecast for the two countries, conditional on what we know from other countries, and the explanatory data from Zambia and Zimbabwe. The expected value of this growth forecast is very similar to the classical point forecast and can be accessed with \verb+pdens$fit+.\footnote{Note that this is equivalent to \texttt{predict(fcstbma, datafls[71:72, ])} .} Likewise the standard deviations of the predictive distribution correspond to classical standard errors and are returned by \verb+pdens$std.err+.
But the predictive density for the growth in e.g. Zimbabwe might be as well visualized with the following command:\footnote{Here, 2 means to plot for the second forecasted observation, in this case \texttt{ZW}, the 72-th row of \texttt{datafls}.}

<<fig=TRUE>>=
plot(pdens, 2)
@

Here, we see that conditional on Zimbabwe's explanatory data, we expect growth to be concentrated around \Sexpr{round(pdens[['fit']][2],2)}. And the actual value in \verb+datafls[72,1]+ with $0.0046$ is not too far off from that prediction. A closer look at both our densities with the function \texttt{quantile} shows that for Zimbabwe, any growth rate between 
\Sexpr{ round(quantile(pdens,c(.05,.95))[2,1], 2) } and \Sexpr{ round(quantile(pdens,c(.05,.95))[2,2], 2) } is quite likely.

<<>>=
quantile(pdens, c(0.05, 0.95))
@

For Zambia (\texttt{ZM}), though, the explanatory variables suggest that positive economic growth should be expected. But over our evaluation period, Zambian growth has been even worse than in Zimbabwe (with \Sexpr{ round(datafls[71,1], 2) } as from \verb+datafls["ZM",1]+).\footnote{Note that since \texttt{ZM} is the rowname of the 71-st row of \texttt{datafls}, this is equivalent to calling \texttt{datafls[71, ]}.} Under the predictive density for Zambia, this actual outcome seems quite unlikely. 

To compare the BMA prediction performs with actual outcomes, we could look e.g. at the forecast error \verb+pdens$fit - datafls[71:72,1]+. However it would be better to take standard errors into account, and even better follow the 'Bayesian way' and evaluate the predictive density of the outcomes as follows:
<<>>=
pdens$dyf(datafls[71:72,1])
@
The density for Zimbabwe is quite high (similar to the mode of predictive density as seen in the chart above), whereas the one for Zambia is quite low. In order to visualize how bad the forecast for Zambia was, compare a plot of predictive density to the actual outcome, which is situated far to the left.

<<fig=TRUE>>=
plot(pdens, "ZM", realized.y=datafls["ZM",1])
@

The results for Zambia suggest either that it is an outlier or that our forecast model might not perform that well. One could try out other prior settings or data, and compare the differing models in their joint predictions for Zambia and Zimbabwe (or even more countries). A standard approach to evaluate the goodness of forecasts would be to e.g. look at root mean squared errors. However Bayesians (as e.g \citealt{fls:bmp}) often prefer to look at densities of the outcome variables and combine them in a 'log-predictive score' (LPS). It is defined as follows, where $p(y^f_i | X, y, X^f_i)$ denotes predictive density for $y^f_i$ (Zambian growth) based on the model information $(y, X)$ (the first 70 countries) and the explanatory variables for the forecast observation (Zambian investment, schooling, etc.).
$$ - \sum_i \log(p(y^f_i | X, y, X^f_i)) $$

The log-predictive score can be accessed with \verb+lps.bma+. 
<<>>=
lps.bma(pdens, datafls[71:72,1])
@
Note however, that the LPS is only meaningful when comparing different forecast settings. 

\addcontentsline{toc}{section}{References}
\bibliography{bmasmall}

%\bibliography{C:/Programme/LatexLibs/bma}
\bibliographystyle{apalike}

\clearpage

\appendix
\section{Appendix}
\subsection{Available Model Priors -- Synopsis}\label{ssec:mpriorsyn}
The following provides an overview over the model priors available in \verb+bms+. Default is \verb+mprior="random"+. For details and examples on built-in priors, consult \verb+help(bms)+. For defining different, custom $g$-priors, consult \verb+help(gprior)+ or \texttt{http://bms.zeugner.eu/custompriors.php}.

\subsubsection*{Uniform Model Prior}
\begin{itemize}
	\item \emph{Argument}: \verb+mprior="uniform"+
	\item \emph{Parameter}: none
	\item \emph{Concept}: $p(M_\gamma) \propto 1$
	\item \emph{Reference}: none
\end{itemize}


\subsubsection*{Binomial Model Prior}
\begin{itemize}
	\item \emph{Argument}: \verb+mprior="fixed"+
	\item \emph{Parameter} (\verb+mprior.size+): prior model size $\bar{m}$ (scalar); Default is $\bar{m}=K/2$
	\item \emph{Concept}: $p(M_\gamma) \propto \left(\frac{\bar{m}}{K}\right)^{k_\gamma} \left(1-\frac{\bar{m}}{K}\right)^{K-k_\gamma}$
	\item \emph{Reference}: \citet{bace04} 
\end{itemize}

\subsubsection*{Beta-Binomial Model Prior}
\begin{itemize}
	\item \emph{Argument}: \verb+mprior="random"+
	\item \emph{Parameter} (\verb+mprior.size+): prior model size $\bar{m}$ (scalar)
	\item \emph{Concept}: $p(M_\gamma) \propto \Gamma(1+k_\gamma) \Gamma( \frac{K-m}{m} + K-k_\gamma)$; Default is $\bar{m}=K/2$
	\item \emph{Reference}: \citet{ls08}
\end{itemize}

\subsubsection*{Custom Prior Inclusion Probabilities}
\begin{itemize}
	\item \emph{Argument}: \verb+mprior="pip"+
	\item \emph{Parameter} (\verb+mprior.size+): A vector of size $K$, detailing $K$ prior inclusion probabilities $\pi_i$: $0<\pi<1 \; \forall i$
	\item \emph{Concept}: $p(M_\gamma) \propto \prod_{i \in \gamma} \pi_i \; \prod_{j \notin \gamma} (1-\pi_j) $
	\item \emph{Reference}: none
\end{itemize}

\subsubsection*{Custom Model Size Prior}
\begin{itemize}
	\item \emph{Argument}: \verb+mprior="customk"+
	\item \emph{Parameter} (\verb+mprior.size+): A vector of size $K+1$, detailing prior $\theta_j$ for 0 to K size models: any real >0 admissible 
	\item \emph{Concept}: $p(M_\gamma) \propto \theta_{k_\gamma}$
	\item \emph{Reference}: none
\end{itemize}



\subsection{Available g-Priors -- Synopsis}
The following provides an overview over the $g$-priors available in \verb+bms+. Default is \verb+g="UIP"+. For implementation details and examples, consult \verb+help(bms)+. For defining different, custom $g$-priors, consult \verb+help(gprior)+ or \texttt{http://bms.zeugner.eu/custompriors.php}.
\subsubsection*{Fixed $g$}
\begin{itemize}
	\item \emph{Argument}: \verb+g=x+ where \verb+x+ is a positive real scalar; 
	\item \emph{Concept}: Fixed $g$ common to all models
	\item \emph{Reference}: \citet{fls:bmp}
	\item \emph{Sub-options}: Unit information prior \verb+g="UIP"+ sets $g=N$; \verb+g="BRIC"+ sets $g=\max(N,K^2)$, a combination of BIC and RIC. (Note these two options guarantee asymptotic consistency.) Other options include \verb+g="RIC"+ for $g=K^2$ and \verb+g="HQ"'+ for the Hannan-Quinn setting $g=\log(N)^3$.
\end{itemize}

\subsubsection*{Empirical Bayes (Local) $g$}
\begin{itemize}
	\item \emph{Argument}: \verb+g="EBL"+ 
	\item \emph{Concept}: Model-specific $g_\gamma$ estimated via maximum likelihood: amounts to $g_\gamma=\max(0,F_\gamma-1)$, where $F_\gamma \equiv \frac{R^2_\gamma (N-1-k_\gamma)}{(1-R^2_\gamma) k_\gamma}$ and $R^2_\gamma$ is the OLS R-squared of model $M_\gamma$.
	\item \emph{Reference}: \citet{george00}; \citet{liang:mgp}
	\item \emph{Sub-options}: none
\end{itemize}

\subsubsection*{Hyper-$g$ prior}
\begin{itemize}
	\item \emph{Argument}: \verb+g="hyper"+ 
	\item \emph{Concept}: A Beta prior on the shrinkage factor with $p(\frac{g}{1+g}) = B(1,\frac{a}{2}-1)$. Parameter $a$ ($2 < a \leq 4$) represents prior beliefs: $a=4$ implies prior shrinkage to be uniformly distributed over $[0,1]$, $a \rightarrow 2$ concentrates mass close to unity. Note that prior expected value of the shrinkage facor is $E(\frac{g}{1+g}) = \frac{2}{a}$.
	\item \emph{Reference}: \citet{liang:mgp}; \citet{fz:superM}
	\item \emph{Sub-options}: \verb+g="hyper=x"+ with \verb+x+ defining the parameter $a$ (e.g. \verb+g="hyper=3"+ sets $a=3$). \verb+g="hyper"+ resp. \verb+g="hyper=UIP"+ sets the prior expected shrinkage factor equivalent to the UIP prior $E(\frac{g}{1+g})=\frac{N}{1+N}$; \verb+g="hyper=BRIC"+ sets the prior expected shrinkage factor equivalent to the BRIC prior. Note that the latter two options guarantee asymptotic consistency.
\end{itemize}


\subsection{'Bayesian Regression' with Zellner's $g$ -- Bayesian Model Selection}\label{ssec:zlm}
The linear model presented in section \ref{ssec:zg} using Zellner's $g$ prior is implemented under the function \verb+zlm+. For instance, we might consider the attitude data from section \ref{sec:example} and estimate just the full model containing all 6 variables. For this purpose, first load the built-in data set with the command
<<>>=
data(attitude)
@
The full model is obtained by applying the function \texttt{zlm} on the data set and storing the estimation into \texttt{att\_full}. Zellner's $g$ prior is estimated by the argument \texttt{g} just in the same way as in section \ref{sec:gprior}.\footnote{Likewise, most methods applicable to \texttt{bms}, such as \texttt{density}, \texttt{predict} or \texttt{coef}, work analogously for \texttt{zlm}.} 
<<>>=
att_full = zlm(attitude,g="UIP")
@
The results can then be displayed by using e.g. the \verb+summary+ method.
<<>>=
summary(att_full)
@
The results are very similar to those resulting from OLS (which can be obtained via \verb+summary(lm(attitude))+). The less conservative, i.e. the larger $g$ becomes, the closer the results get to OLS. But remember that the full model was not the best model from the BMA application in section \ref{sec:example}. In order to extract the best encountered model, use the function \verb+as.zlm+ to extract this single model for further analysis (with the argument \verb+model+ specifying the rank-order of the model to be extracted). The following command reads the best model from the BMA results in section into the variable \verb+att_best+.

<<>>=
att_best = as.zlm(att,model=1)
summary(att_best)
@

As suspected, the best model according to BMA is the on including only \verb+complaints+ and the intercept, as it has the highest log-marginal likelihood (\verb+logLik(att_best)+). In such a way, the command \texttt{as.zlm} can be combined with \texttt{bms} for 'Bayesian Model Selection', i.e. using the model prior and posterior framework to focus on teh model with highest posterior mass. Via the utility \texttt{model.frame}, this best model can be straightforwardly converted into a standard  OLS model:

<<>>=
att_bestlm = lm(model.frame(as.zlm(att)))
summary(att_bestlm)
@


\subsection{BMA when Keeping a Fixed Set of Regressors}\label{ssec:fixreg}
While BMA should usually compare as many models as possible, some considerations might dictate the restriction to a subspace of the $2^K$ models. For complicated settings one might employ a customly designed model prior (cf. section \ref{ssec:mpriorsyn}). The by far most common setting, though, is to keep some regressors fixed in the model setting, and apply Bayesian Model uncertainty only to a subset of regressors.

Suppose, for instance, that prior research tells us that any meaningful model for \texttt{attitude} (as in section \ref{sec:example}) must include the variables \texttt{complaints} and \texttt{learning}. The only question is whether the additional four variables matter (which reduces the potential model space to $2^4=16$). We thus sample over these models while keeping \texttt{complaints} and \texttt{learning} as fixed regressors: 

<<>>=
att_learn = bms(attitude,mprior="uniform", fixed.reg=c("complaints", "learning") )
@

The results show that the PIP and the coefficients for the remaining variables increase a bit compared to \texttt{att}. The higher PIPs are related to the fact that the posterior model size (as in \verb+sum(coef(att_learn)[,1])+) is quite larger as under \texttt{att}. This follows naturally from our model prior: putting a uniform prior on all models between parameter size $2$ (the base model) and $6$ (the full model) implies a prior expected model size of $4$ for \texttt{att\_learn} instead of the $3$ for \texttt{att}.\footnote{  The command
\texttt{ att\_learn2 = bms(attitude, mprior='fixed', mprior.size=3, fixed.reg=c('complaints', 'learning') ) } 
produces coefficients that are much more similar to \texttt{att}.} So to achieve comparable results, one needs to take the number of fixed regressors into account when setting the model prior parameter \texttt{mprior.size}. Consider another example: 

Suppose we would like to sample the importance and coefficients for the cultural dummies in the dataset \texttt{datafls}, conditional on information from the remaining 'hard' variables. This implies keeping 27 fixed regressors, while sampling over the 14 cultural dummies. Since model uncertainty thus applies only to $2^{14}=16,384$ models, we resort to full enumeration of the model space. 

<<>>=
fls_culture = bms(datafls,fixed.reg=c(1,8:16,24,26:41), mprior="random", mprior.size=28, mcmc="enumeration", user.int=F)
@
Here, the vector \texttt{c(1,8:16,24,26:41)} denotes the indices of the regressors in \texttt{datafls} to be kept fixed.\footnote{Here, indices start from the first regressor, i.e. they do not take the dependent variable into account. The fixed data used above therefore corresponds to \texttt{datafls[ ,c(1,8:16,24,26:41) $+$ 1]}.} Moreover, we use the beta-binomial ('random') model prior. The prior model size of $30$ embodies our prior expectation that on average $1$ out of the $14$ cultural dummies should be included in the true model. As we only care about those 14 variables, let us just display the results for the 14 variables with the least PIP: 

<<>>=
coef(fls_culture)[28:41, ]
@

As before, we find that \texttt{Confucian} (with positive sign) as well as \texttt{Hindu} and \texttt{SubSahara} (negative signs) have the most important impact conditional on 'hard' information. Moreover, the data seems to attribute more importance to cultural dummies as we expectd with our model prior: Comparing prior and posterior model size with the following command shows how much importance is attributed to the dummies.
<<fig=TRUE>>=
plotModelsize(fls_culture, ksubset=27:41)
@

Expected posterior model size is close to \Sexpr{ round(sum(coef(fls_culture)[,1]),0) }, which means that \Sexpr{ round(sum(coef(fls_culture)[,1]),0) -27 } of the cultural dummies should actually be included in a 'true' model. 


\end{document}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{topmod}
\alias{topmod}
\alias{is.topmod}
\title{Topmodel Object}
\usage{
topmod(
  nbmodels,
  nmaxregressors = NA,
  bbeta = FALSE,
  lengthfixedvec = 0,
  liks = numeric(0),
  ncounts = numeric(0),
  modelbinaries = matrix(0, 0, 0),
  betas = matrix(0, 0, 0),
  betas2 = matrix(0, 0, 0),
  fixed_vector = matrix(0, 0, 0)
)
}
\arguments{
\item{nbmodels}{The maximum number of models to be retained by the topmod
object}

\item{nmaxregressors}{The maximum number of covariates the models in the
topmod object are allowed to have}

\item{bbeta}{if \code{bbeta=TRUE}, then first and second moments of model
coefficients are stored in addition to basic model statistics (Note: if
\code{bbeta<0} then only the first moments are saved)}

\item{lengthfixedvec}{The length of an optional fixed vector adhering to
each model (for instance R-squared, etc). If \code{lengthfixedvec=0} then no
additonal fixed vector will be stored.}

\item{liks}{optional vector of log-likelihoods to initialize topmod object
with (length must be \code{<=nbmodels}) - see example below}

\item{ncounts}{optional vector of MCMC frequencies to initialize topmod
object with (same length as \code{liks}) - see example below}

\item{modelbinaries}{optional matrix whose columns detail model binaries to
initialize topmod object with (same nb columns as \code{liks}, nb rows as
\code{nmaxregressors}) - see example below}

\item{betas}{optional matrix whose columns are coefficients to initialize
topmod object with (same dimensions as \code{modelbinaries}) - see example
below}

\item{betas2}{optional matrix whose columns are coefficients' second moments
to initialize topmod object with (same dimensions as \code{modelbinaries}) -
see example below}

\item{fixed_vector}{optional matrix whose columns are a fixed vector
initialize topmod object with (same \code{ncol} as \code{modelbinaries}) -
see example below}
}
\value{
a call to \code{topmod} returns a list of class "topmod" with the
following elements:
\item{addmodel(mylik,vec01,vbeta=numeric(0),vbeta2=numeric(0),fixedvec=numeric(0))}{function
that adjusts the list of models in the 'topmod' object (see Details).
\code{mylik} is the basic selection criterion (usually log likelihood),
\code{vec01} is the model binary (logical or numeric) indicating which
regressors are included.\cr \code{vbeta} is a vector of length equal to
\code{sum(vec01)}, contianing only the non-zero coefficients (only accounted
for if \code{bbeta!=FALSE}). \code{vbeta2} is a similar vector of second
moments etc. (only accounted for if \code{bbeta=TRUE}); \code{fixedvec} is
an arbitrary vector of length \code{lengthfixedvec} (see above)}
\item{lik()}{A numeric vector of the best models (log) likelihoods, in
decreasing order} \item{bool()}{A character vector of hexmode expressions
for the model binaries (cf. \code{\link{bin2hex}}), sorted by \code{lik()} }
\item{ncount()}{A numeric vector of MCMC frequencies for the best models
(i.e. how often the respective model was introduced by \code{addmodel})}
\item{nbmodels}{Returns the argument \code{nbmodel}} \item{nregs}{Returns
the argument \code{nmaxregressors}} \item{bool_binary()}{Returns a matrix
whose columns present the models conforming to \code{lik()} in binary form}
\item{betas()}{a matrix whose columns are the coefficients conforming to
\code{bool_binary()} (Note that these include zero coefficients due to
non-inclusion of covariates); Note: if \code{bbeta=FALSE} this returns an
empty matrix} \item{betas2()}{similar to \code{betas} , for the second
moments of coefficients Note: if \code{bbeta<=0}, this returns an empty
matrix} \item{fixed_vector()}{The columns of this matrix return the
\code{fixed_vector} statistics conforming to \code{lik()} (see Details);
Note: if \code{lengthfixedvec=0} this returns an empty matrix}
}
\description{
Create or use an updateable list keeping the best x models it encounters
(for advanced users)
}
\details{
A 'topmod' object (as created by \code{topmod}) holds three basic vectors:
\code{lik} (for the (log) likelihood of models or similar), \code{bool()}
for a hexcode presentation of the model binaries (cf. \code{\link{bin2hex}})
and ncount() for the times the models have been drawn.\cr All these vectors
are sorted descendantly by \code{lik}, and are of the same length. The
maximum length is limited by the argument \code{nbmodels}.

If \code{tmo} is a topmod object, then a call to \code{tmo$addmodel} (e.g.
\code{tmo$addmodel(mylik=4,vec01=c(T,F,F,T))} updates the object \code{tmo}
by a model represented by \code{vec01} (here the one including the first and
fourth regressor) and the marginal (log) likelihood \code{lik} (here: 4).

If this model is already part of \code{tmo}, then its respective
\code{ncount} entry is incremented by one; else it is inserted into a
position according to the ranking of \code{lik}.

In addition, there is the possibility to save (the first moments of)
coefficients of a model (\code{betas}) and their second moments
(\code{betas2}), as well as an arbitrary vector of statistics per model
(\code{fixed_vector}).

\code{is.topmod} returns \code{TRUE} if the argument is of class 'topmod'
}
\note{
\code{topmod} is rather intended as a building block for programming;
it has no direct application for a user of the BMS package.
}
\examples{

#standard use
  tm= topmod(2,4,TRUE,0) #should keep a  maximum two models
  tm$addmodel(-2.3,c(1,1,1,1),1:4,5:8) #update with some model
  tm$addmodel(-2.2,c(0,1,1,1),1:3,5:7) #add another model
  tm$addmodel(-2.2,c(0,1,1,1),1:3,5:7) #add it again -> adjust ncount
  tm$addmodel(-2.5,c(1,0,0,1),1:2,5:6) #add another model
  
  #read out
  tm$lik()
  tm$ncount()
  tm$bool_binary()
  tm$betas()
  
  is.topmod(tm)
  
  #extract a topmod oobject only containing the second best model
  tm2=tm[2]
  
  
  
  #advanced: should return the same result as
  #initialize
  tm2= topmod(2,4,TRUE,0, liks = c(-2.2,-2.3), ncounts = c(2,1), 
          modelbinaries = cbind(c(0,1,1,1),c(1,1,1,1)), betas = cbind(0:3,1:4), 
          betas2 = cbind(c(0,5:7),5:8)) 

  #update 
  tm$addmodel(-2.5,c(1,0,0,1),1:2,5:6) #add another model
  
  #read out
  tm$lik()
  tm$ncount()
  tm$bool_binary()
  tm$betas()

}
\seealso{
the object resulting from \code{\link{bms}} includes an element of
class 'topmod'

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{hex2bin}
\alias{hex2bin}
\alias{bin2hex}
\title{Converting Binary Code to and from Hexadecimal Code}
\usage{
hex2bin(hexcode)

bin2hex(binvec)
}
\arguments{
\item{hexcode}{a single-element character denoting an integer in hexcode
(admissible character: 0 to 9, ato f)}

\item{binvec}{a logical vector (alternatively a vector coercible into
logical)}
}
\value{
\code{bin2hex} returns a single element character; \code{hex2bin}
returns a numeric vector equivalent to a logical vector
}
\description{
A simple-to-use function for converting a logical ('binary') vector into hex
code and reverse.
}
\details{
The argument is an integer in binary form (such as "101"), provided as a
logical (\code{c(T,F,T)}) or numeric vector (\code{c(1,0,1)}).\cr
\code{bin2hex} then returns a character denoting this number in hexcode (in
this case "5").

The function \code{hex2bin} does the reverse operation, e.g.
\code{hex2bin("5")} gives (\code{c(1,0,1)}).
}
\examples{

  bin2hex(c(TRUE,FALSE,TRUE,FALSE,TRUE,TRUE))
  bin2hex(c(1,0,1,0,1,1))
  hex2bin("b8a")
  bin2hex(hex2bin("b8a"))

}
\seealso{
\code{\link{hex2bin}} for converting hexcode into binary vectors,
\code{\link{format.hexmode}} for a related R function.

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{arith}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{image.bma}
\alias{image.bma}
\title{Plot Signs of Best Models}
\usage{
\method{image}{bma}(
  x,
  yprop2pip = FALSE,
  order.by.pip = TRUE,
  do.par = TRUE,
  do.grid = TRUE,
  do.axis = TRUE,
  cex.axis = 1,
  ...
)
}
\arguments{
\item{x}{a list of class bma (cf. \code{\link{bms}} for further details)}

\item{yprop2pip}{if \code{yprop2pip=TRUE} then the grid lines on the
vertical axis are scaled according to the coefficients' inclusion
probabilites.\cr If \code{yprop2pip=FALSE} (default) then the grid lines on
the vertical axis are equidistant.}

\item{order.by.pip}{with \code{order.by.pip=TRUE} (default), coefficients
are sorted according to their posterior inclusion probabilites along the
vertical axis. If \code{order.by.pip=FALSE} they are ordered as they were
provided to \code{\link{bms}}.}

\item{do.par}{Defaults to \code{do.par=TRUE}, which adjusts
\code{\link{par}()$mar} for optimal positioning. Set \code{do.par=FALSE} for
customizing \code{par} yourself.}

\item{do.grid}{\code{do.grid=TRUE} (default) plots grid lines among the
chart's boxes, akin to the low level command \code{\link{grid}}.
\code{do.grid=FALSE} omits the grid lines.}

\item{do.axis}{\code{do.axis=TRUE} (default) plots axis tick marks and
labels (cf. \code{\link{axis}}). \code{do.axis=FALSE} omits them.}

\item{cex.axis}{font size for the axes (cf. \code{\link{axis}}), defaults to
1}

\item{\dots}{Parameters to be passed on to \code{\link{image.default}}.}
}
\description{
Plots a grid with signs and inclusion of coefficients vs. posterior model
probabilities for the best models in a 'bma' object:
}
\details{
Under default settings, blue corresponds to positive sign, red to a negative
sign, white to non-inclusion.
}
\examples{

 data(datafls)
 
 model=bms(datafls,nmodel=200)
 
 #plot all models
 image(model,order.by.pip=FALSE)
 image(model,order.by.pip=TRUE,cex.axis=.8)
 
 #plot best 7 models, with other colors
 image(model[1:7],yprop2pip=TRUE,col=c("black","lightgrey"))
 
}
\seealso{
\link{coef.bma} for the coefficients in matrix form, \link{bms} for
creating 'bma' objects.

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{class}
\name{topmod-class}
\alias{topmod-class}
\title{Class "topmod"}
\description{
An updateable list keeping the best x models it encounters in any kind of
model iteration
}
\section{Objects from the Class}{
 Objects can be created by calls to
\code{\link{topmod}}, or indirectly by calls to \code{\link{bms}}.\cr

A 'topmod' object (as created by \code{topmod}) holds three basic vectors:
\code{lik} (for the (log) likelihood of models or similar), \code{bool()}
for a hexcode presentation of the model binaries (cf. \code{\link{bin2hex}})
and ncount() for the times the models have been drawn.\cr All these vectors
are sorted descendantly by \code{lik}, and are of the same length. The
maximum length is limited by the argument \code{nbmodels}.

If \code{tmo} is a topmod object, then a call to \code{tmo$addmodel} (e.g.
\code{tmo$addmodel(mylik=4,vec01=c(T,F,F,T))} updates the object \code{tmo}
by a model represented by \code{vec01} (here the one including the first and
fourth regressor) and the marginal (log) likelihood \code{lik} (here: 4).\cr
If this model is already part of \code{tmo}, then its respective
\code{ncount} entry is incremented by one; else it is inserted into a
position according to the ranking of \code{lik}.\cr In addition, there is
the possibility to save (the first moments of) coefficients of a model
(\code{betas}) and their second moments (\code{betas2}), as well as an
arbitrary vector of statistics per model (\code{fixed_vector}).\cr
}

\examples{

  tm= topmod(2,4,TRUE,0) #should keep a  maximum two models
  tm$addmodel(-2.3,c(1,1,1,1),1:4,5:8) #update with some model
  tm$addmodel(-2.2,c(0,1,1,1),1:3,5:7) #add another model
  tm$addmodel(-2.2,c(0,1,1,1),1:3,5:7) #add it again -> adjust ncount
  tm$addmodel(-2.5,c(1,0,0,1),1:2,5:6) #add another model
  
  #read out
  tm$lik()
  tm$ncount()
  tm$bool_binary()
  tm$betas()

}
\references{
\url{http://bms.zeugner.eu}
}
\seealso{
\code{\link{topmod}} to create \code{topmod} objects and a more
detailed description,
\code{\link{is.topmod}} to test for this class
}
\author{
Martin Feldkircher and Stefan Zeugner
}
\keyword{classes}

\name{[.bma}
\alias{[.bma}
\alias{[.topmod}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Extract Parts of a topmod Object }
\description{
  returns a bma (or topmod) object whose 
}
\usage{
\method{[}{bma}(x, i, ...)

\method{[}{topmod}(x, i, ...)

}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ an object of class 'bma' or of class \code{\link{topmod}} }
  \item{i}{ index of the best models to be retained }
  \item{\ldots}{ arguments such as 'drop' will not have an effect }
}
\details{
  Both functions are equivalent: they serve to select a subset of the best models in a bma object.
}
\value{
  If \code{x} is a topmod object (see \code{\link{topmod}}), then \code{x[i]} returns a topmod object that only retains the models specified by \code{i}
  
  if \code{x} is an object of class bma (see \code{\link{bms}}), \code{x[i]} returns a bma object whose topmod object (i.e. \code{x$topmod} has been adjusted accordingly)
}
\author{ Martin Feldkircher and Stefan Zeugner }
\note{ Note that the method parameter \code{drop} is of no importance here. Moreover the corresponding assignment operator \code{\link{[<-}} is of no use.}
\seealso{ \code{\link{[}} for the default method, \code{\link{bms}} for crating bma objects 

Check \url{http://bms.zeugner.eu} for additional help.}
\examples{
#sample some BMA
 data(datafls)
 mm=bms(datafls[,1:11])
 
#plotConv for all models 
 plotConv(mm)

#plotConv for the best 20 models 
 plotConv(mm[1:20])

#model binary for the second best model 
topmodels.bma(mm[2])

#similar:
as.vector(mm$topmod[2]$bool_binary())

#exactly the same
as.vector(mm[2]$topmod$bool_binary())


#print stats for second best model
print(mm$topmod[2])




}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{fullmodel.ssq}
\alias{fullmodel.ssq}
\title{OLS Statistics for the Full Model Including All Potential Covariates}
\usage{
fullmodel.ssq(yX.data)
}
\arguments{
\item{yX.data}{a bma object (cf. \code{\link{bms}}) - alternatively a
\link{data.frame} or \link{matrix} whose first column is the dependent
variable}
}
\value{
Returns a list with some basic OLS statistics \item{R2}{The
R-squared of the full model} \item{ymy}{The sum of squares of residuals of
the full model} \item{ypy}{The explained sum of squares of the full model}
\item{yty}{The sum of squares of the (demeaned) dependent variable}
\item{Fstat}{The F-statistic of the full model}
}
\description{
A utility function for reference: Returns a list with R2 and sum of squares
for the OLS model encompassing all potential covariates that are included in
a bma object.
}
\note{
This function is just for quick comparison; for proper OLS estimation
consider \code{\link{lm}}
}
\examples{

data(datafls)
mm=bms(datafls)

fullmodel.ssq(mm)

#equivalent:
fullmodel.ssq(datafls)


}
\seealso{
\code{\link{bms}} for creating bma objects, \code{\link{lm}} for
OLS estimation

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{topmodels.bma}
\alias{topmodels.bma}
\title{Model Binaries and their Posterior model Probabilities}
\usage{
topmodels.bma(bmao)
}
\arguments{
\item{bmao}{an object of class 'bma' - see \code{\link{bma-class}}}
}
\value{
Each column in the resulting matrix corresponds to one of the 'best' models in bmao: the first column for the best model, the second for the second-best model, etc.
The model binaries have elements 1 if the regressor given by the row name was included in the respective models, and 0 otherwise.
The second-last row shows the model's posterior model probability based on marginal likelihoods (i.e. its marginal likelihood over the sum of likelihoods of all best models)
The last row shows the model's posterior model probability based on MCMC frequencies (i.e. how often the model was accepted vs sum of acceptance of all models) 
Note that the column names are hexcode representations of the model binaries (e.g. "03" for c(0,0,0,1,0,0))
}
\description{
Returns a matrix whose columns show which covariates were included in the best models in a 'bma' object. The last two columns detail posterior model probabilities.
}
\details{
Each bma class (the result of bms) contains 'top models', the x models with tthe best 
analytical likelihood  that bms had encountered while sampling

See \code{\link{pmp.bma}} for an explanation of likelihood vs. MCMC
frequency concepts
}
\examples{

data(datafls)
#sample with a limited data set for demonstration
mm=bms(datafls[,1:12],nmodel=20)

#show binaries for all
topmodels.bma(mm)

#show binaries for 2nd and 3rd best model, without the model probs
topmodels.bma(mm[2:3])[1:11,]

#access model binaries directly
mm$topmod$bool_binary()
 
}
\seealso{
\code{\link{topmod}} for creating topmod objects, \code{\link{bms}}
for their typical use, \code{\link{pmp.bma}} for comparing posterior model
probabilities

Check \url{http://bms.zeugner.eu} for additional help.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{estimates.bma}
\alias{estimates.bma}
\alias{coef.bma}
\title{Posterior Inclusion Probabilities and Coefficients from a 'bma' Object}
\usage{
estimates.bma(
  bmao,
  exact = FALSE,
  order.by.pip = TRUE,
  include.constant = FALSE,
  incl.possign = TRUE,
  std.coefs = FALSE,
  condi.coef = FALSE
)

\method{coef}{bma}(
  object,
  exact = FALSE,
  order.by.pip = TRUE,
  include.constant = FALSE,
  incl.possign = TRUE,
  std.coefs = FALSE,
  condi.coef = FALSE,
  ...
)
}
\arguments{
\item{exact}{if \code{exact=FALSE}, then PIPs, coefficients, etc. will be
based on aggregate information from the sampling chain with posterior model
distributions based on MCMC frequencies (except in case of enumeration - cf.
'Details');\cr if \code{exact=TRUE}, estimates will be based on the
\code{\link[=bms]{nmodel}} best models encountered by the sampling chain,
with the posterior model distribution based on their \emph{exact} marginal
likelihoods - cf. 'Details' below.}

\item{order.by.pip}{\code{order.by.pip=TRUE} orders the resulting matrix
according to posterior inclusion probabilites, \code{order.by.pip=FALSE}
ranks them according to the original data (order of the covariates as in
provided in \code{X.data} to \code{\link{bms}}), default \code{TRUE}}

\item{include.constant}{If \code{include.constant=TRUE} then the resulting
matrix includes the expected value of the constant in its last row. Default
\code{FALSE}}

\item{incl.possign}{If \code{incl.possign=FALSE}, then the sign probabilites
column (cf. 'Values' below) is omitted from the result. Default \code{TRUE}}

\item{std.coefs}{If \code{std.coefs=TRUE} then the expected values and
standard deviations are returned in standardized form, i.e. as if the
original data all had mean zero and variance 1. If \code{std.coefs=FALSE}
(default) then both expected values and standard deviations are returned 'as
is'.}

\item{condi.coef}{If \code{condi.coef=FALSE} (default) then coefficients
\eqn{\beta_i} and standard deviations are unconditional posterior expected
values, as in standard model averaging; if \code{condi.coef=FALSE} then they
are given as conditional on inclusion (equivalent to \eqn{\beta_i / PIP_i}).}

\item{object, bmao}{a 'bma' object (cf. \code{\link{bms}})}

\item{...}{further arguments for other \code{\link{coef}} methods}
}
\value{
A matrix with five columns (or four if \code{incl.possign=FALSE})
\item{Column 'PIP'}{Posterior inclusion probabilities \eqn{\sum p(\gamma|i
\in \gamma, Y) / sum p(\gamma|Y) }} \item{Column 'Post Mean'}{posterior
expected value of coefficients, unconditional \eqn{E(\beta|Y)=\sum
p(\gamma|Y) E(\beta|\gamma,Y)}, where \eqn{E(\beta_i|\gamma,i \notin \gamma,
Y)=0} if \code{condi.coef=FALSE}, or conditional on inclusion
(\eqn{E(\beta|Y) / \sum p(\gamma|Y, i \in \gamma) } ) if
\code{condi.coef=TRUE}} \item{Column 'Post SD'}{posterior standard deviation
of coefficients, unconditional or conditional on inclusion, depending on
\code{condi.coef}} \item{Column 'Cond.Pos.Sign'}{The ratio of how often the
coefficients' expected values were positive conditional on inclusion. (over
all visited models in case \code{exact=FALSE}, over the best models in case
\code{exact=TRUE})} \item{Column 'Idx'}{the original order of covariates as
the were used for sampling. (if included, the constant has index 0)}
}
\description{
Returns a matrix with aggregate covariate-specific Bayesian model Averaging:
posterior inclusion probabilites (PIP), post. expected values and standard
deviations of coefficients, as well as sign probabilites
}
\details{
More on the argument \code{exact}: \cr In case the argument
\code{exact=TRUE}, the PIPs, coefficient statistics and conditional sign
probabilities are computed on the basis of the (500) best models the
sampling chain encountered (cf. argument \code{nmodel} in
\code{\link{bms}}). Here, the weights for Bayesian model averaging (BMA) are
the posterior marginal likelihoods of these best models. \cr In case
\code{exact=FALSE}, then these statistics are based on all accepted models
(except burn-ins): If \code{mcmc="enumerate"} then this are simply all
models of the traversed model space, with their marginal likelihoods
providing the weights for BMA.\cr If, however, the bma object \code{bmao}
was based on an MCMC sampler (e.g. when \code{\link{bms}} argument
\code{mcmc="bd"}), then BMA statistics are computed differently: In contrast
to above, the weights for BMA are MCMC frequencies, i.e. how often the
respective models were encountered by the MCMC sampler. (cf. a comparison of
MCMC frequencies and marginal likelihoods for the best models via the
function \code{\link{pmp.bma}}).
}
\examples{

#sample, with keeping the best 200 models:
data(datafls)
mm=bms(datafls,burn=1000,iter=5000,nmodel=200)

#standard BMA PIPs and coefficients from the MCMC sampling chain, based on 
#  ...how frequently the models were drawn
coef(mm)

#standardized coefficients, ordered by index
coef(mm,std.coefs=TRUE,order.by.pip=FALSE)

#coefficients conditional on inclusion:
coef(mm,condi.coef=TRUE)

#same as
ests=coef(mm,condi.coef=FALSE)
ests[,2]/ests[,1]

#PIPs, coefficients, and signs based on the best 200 models
estimates.bma(mm,exact=TRUE)

#... and based on the 50 best models
coef(mm[1:50],exact=TRUE)


}
\seealso{
\code{\link{bms}} for creating bma objects, \code{\link{pmp.bma}}
for comparing MCMC frequencies and marginal likelihoods.

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{summary.zlm}
\alias{summary.zlm}
\title{Summarizing Linear Models under Zellner's g}
\usage{
\method{summary}{zlm}(object, printout = TRUE, ...)
}
\arguments{
\item{object}{an object of class \code{zlm}: see "Examples" below}

\item{printout}{If \code{TRUE} (default, then information is printed to
console in a neat form}

\item{\dots}{further arguments passed to or from other methods}
}
\value{
A \code{\link{list}} with the following elements \item{residuals}{
The expected value of residuals from the model} \item{coefficients}{The
posterior expected values of coefficients (including the intercept) }
\item{coef.sd}{Posterior standard deviations of the coefficients (the
intercept SD is \code{NA}, since an improper prior was used)}
\item{gprior}{The g prior as it has been submitted to \code{object}}
\item{E.shrinkage}{the shrinkage factor \eqn{g/(1+g)}, respectively its
posterior expected value in case of a hyper-g prior}
\item{SD.shrinkage}{(Optionally) the shrinkage factor's posterior standard
deviation (in case of a hyper-g prior)} \item{log.lik}{The log marginal
likelihood of the model}
}
\description{
summary method for class "\code{zlm}"
}
\details{
\code{summary.zlm} prints out coefficients expected values and their
standard deviations, as well as information on the gprior and the log
marginal likelihood. However, it invisibly returns a list with elements as
described below:
}
\examples{

data(datafls)

#simple example
foo = zlm(datafls)
summary(foo)

sfoo = summary(foo,printout=FALSE)
print(sfoo$E.shrinkage)

}
\seealso{
\code{\link{zlm}} for creating \code{zlm} objects,
\code{link{summary.lm}} for a similar function on OLS models

See also \url{http://bms.zeugner.eu} for additional help.
}
\author{
Stefan Zeugner
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{print.topmod}
\alias{print.topmod}
\title{Printing topmod Objects}
\usage{
\method{print}{topmod}(x, ...)
}
\arguments{
\item{x}{an object of class 'topmod' - see \code{\link{topmod}}}

\item{\dots}{additional arguments passed to \code{link{print}}}
}
\value{
if \code{x} contains more than one model, then the function returns
a 2-column matrix: \item{Row Names}{show the model binaries in hexcode 
} \item{Column 'Marg.Log.Lik'}{shows the
marginal log-likelihoods of the models in \code{x}} \item{Column 'MCMC
Freq'}{shows the MCMC frequencies of the models in \code{x}}

if \code{x} contains only one model, then more detailed information is shown
for this model: \item{first line}{'Model Index' provides the model binary in
hexcode, 'Marg.Log.Lik' its marginal log likelhood, 'Sampled Freq.' how
often it was accepted (function \code{ncount()} in \code{\link{topmod}})}
\item{Estimates}{first column: covariate indices included in the model,
second column: posterior expected value of the coefficients, third column:
their posterior standard deviations (excluded if no coefficients were stored
in the topmod object - cf. argument \code{bbeta} in \code{\link{topmod}}) }
\item{Included Covariates}{the model binary} \item{Additional
Statistics}{any custom additional statistics saved with the model}
}
\description{
Print method for objects of class 'topmod', typically the best models stored
in a 'bma' object
}
\details{
See \code{\link{pmp.bma}} for an explanation of likelihood vs. MCMC
frequency concepts
}
\examples{

# do some small-scale BMA for demonstration
data(datafls)
mm=bms(datafls[,1:10],nmodel=20)

#print info on the best 20 models
print(mm$topmod)
print(mm$topmod,digits=10)

#equivalent:
cbind(mm$topmod$lik(),mm$topmod$ncount())



#now print info only for the second-best model:
print(mm$topmod[2])

#compare 'Included Covariates' to:
topmodels.bma(mm[2])

#and to
as.vector(mm$topmod[2]$bool_binary())


}
\seealso{
\code{\link{topmod}} for creating topmod objects, \code{\link{bms}}
for their typical use, \code{\link{pmp.bma}} for comparing posterior model
probabilities

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{as.zlm}
\alias{as.zlm}
\title{Extract a Model from a bma Object}
\usage{
as.zlm(bmao, model = 1)
}
\arguments{
\item{bmao}{A \code{bma} object, e.g. resulting from a call to
\code{\link{bms}}}

\item{model}{The model index, in one of the following forms:\cr An integer,
denoting the rank of the model (1 for best, 2 for second-best, ...)\cr A
numeric or logical vector of length K describing which covariates are
contained in the model\cr A hexcode character describing which covariates
are contained in the model}
}
\value{
a list of class \code{\link{zlm}}
}
\description{
Extracts a model out of a \code{bma} object's saved models and converts it
to a \code{\link{zlm}} linear model
}
\details{
A bma object stores several 'best' models it encounters (cf. argument
\code{nmodel} in \code{\link{bms}}). \code{as.zlm} extracts a single model
and converts it to an object of class \code{\link{zlm}}, which represents a
linear model estimated under Zellner's g prior.\cr The utility
\code{\link{model.frame}} allows to transfrom a \code{zlm} model into an OLS
model of class \code{\link{lm}}.
}
\examples{

data(datafls)

mm=bms(datafls[,1:6],mcmc="enumeration") # do a small BMA chain
topmodels.bma(mm)[,1:5] #display the best 5 models

m2a=as.zlm(mm,4) #extract the fourth best model
summary(m2a)

# Bayesian Model Selection:
# transform the best model into an OLS model:
lm(model.frame(as.zlm(mm)))

# extract the model only containing the 5th regressor
m2b=as.zlm(mm,c(0,0,0,0,1)) 

# extract the model only containing the 5th regressor in hexcode
print(bin2hex(c(0,0,0,0,1)))
m2c=as.zlm(mm,"01")




}
\seealso{
\code{\link{bms}} for creating \code{bma} objects,
\code{\link{zlm}} for creating \code{zlm} objects,
\code{\link{pmp.bma}} for displaying the
topmodels in a \code{bma} object

Check \url{http://bms.zeugner.eu} for additional help.
}
\author{
Stefan Zeugner
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{class}
\name{gprior-class}
\alias{gprior-class}
\title{Class "gprior"}
\description{
An object pertaining to a coefficient prior
}
\section{Objects from the Class}{
 A \code{gprior} object holds descriptions
and subfunctions pertaining to coefficient priors. Functions such as
\code{\link{bms}} or \code{\link{zlm}} rely on this class to 'convert' the
output of OLS results into posterior expressions for a Bayesian Linear
Model. Post-processing functions such as \code{\link{density.bma}} also
resort to gprior objects.\cr There are currently three coefficient prior
structures built into the BMS package, generated by the following functions
(cf. Feldkircher and Zeugner, 2009) : \cr \code{gprior.constg.init}: creates
a Zellner's g-prior object with constant \code{g}.\cr
\code{gprior.eblocal.init}: creates an Empricial Bayes Zellner's g-prior.\cr
\code{gprior.hyperg.init}: creates a hyper g-prior with a Beta-prior on the
shrinkage parameter.\cr The following describes the necessary slots
}

\examples{


data(datafls)
mm1=bms(datafls[,1:10], g="EBL")
gg=mm1$gprior.info # is the g-prior object, augmented with some posterior statistics

mm2=bms(datafls[,1:10], g=gg) #produces the same result

mm3=bms(datafls[,1:10], g=BMS:::.gprior.eblocal.init) 
#this passes BMS's internal Empirical Bayes g-prior object as the coefficient prior 
# - any other obejct might be used as well



}
\references{
Feldkircher, M. and S. Zeugner (2009): Benchmark Priors
Revisited: On Adaptive Shrinkage and the Supermodel Effect in Bayesian Model
Averaging, IMF Working Paper 09/202.
}
\seealso{
\code{\link{bms}} and \code{\link{zlm}} for creating \code{bma} or
\code{zlm} objects. \cr Check the appendix of \code{vignette(BMS)} for a
more detailed description of built-in priors.\cr Check
\url{http://bms.zeugner.eu/custompriors.php} for examples.
}
\author{
Martin Feldkircher and Stefan Zeugner
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{quantile.density}
\alias{quantile.density}
\alias{quantile.coef.density}
\alias{quantile.pred.density}
\title{Extract Quantiles from 'density' Objects}
\usage{
\method{quantile}{density}(x, probs = seq(0.25, 0.75, 0.25), names = TRUE, normalize = TRUE, ...)

\method{quantile}{coef.density}(x, probs = seq(0.25, 0.75, 0.25), names = TRUE, ...)

\method{quantile}{pred.density}(x, probs = seq(0.25, 0.75, 0.25), names = TRUE, ...)
}
\arguments{
\item{x}{a object of class \code{\link{pred.density}}, \code{coef.density},
\code{\link{density}}, or a list of densities.}

\item{probs}{numeric vector of probabilities with values in [0,1] - elements
very close to the boundaries return \code{Inf} or \code{-Inf}}

\item{names}{logical; if \code{TRUE}, the result has a \code{names}
attribute, resp. a \code{rownames} and \code{colnames} attributes. Set to
\code{FALSE} for speedup with many probs.}

\item{normalize}{logical; if \code{TRUE} then the values in \code{x$y} are
multiplied with a factor such that their integral is equal to one.}

\item{\dots}{further arguments passed to or from other methods.}
}
\value{
If \code{x} is of class \code{density} (or a list with exactly one
element), a vector with quantiles.\cr If \code{x} is a \code{\link{list}} of
densities with more than one element (e.g. as resulting from
\code{pred.density} or \code{coef.density}), then the output is a matrix of
quantiles, with each matrix row corresponding to the respective density.
}
\description{
Quantiles for objects of class "density", "pred.density" or "coef.density"
}
\details{
The methods \code{quantile.coef.density} and \code{quantile.pred.density}
both apply \code{quantile.density} to densities nested with object of class
\code{coef.density} or \code{pred.density}.\cr The function
\code{quantile.density} applies generically to the built-in class
\code{\link{density}} (as least for versions where there is no such method
in the pre-configured packages).\cr Note that \code{quantile.density} relies
on trapezoidal integration in order to compute the cumulative densities
necessary for the calculation of quantiles.
}
\examples{

 data(datafls)
 mm = bms(datafls[1:70,], user.int=FALSE)
 
 #predict last two observations with preceding 70 obs:
 pmm = pred.density(mm, newdata=datafls[71:72,], plot=FALSE) 
 #'standard error' quantiles
 quantile(pmm, c(.05, .95))
 
 #Posterior density for Coefficient of "GDP60"
 cmm = density(mm, reg="GDP60", plot=FALSE) 
 quantile(cmm, probs=c(.05, .95))
 
 
 #application to generic density:
 dd1 = density(rnorm(1000))
 quantile(dd1)
 
\dontrun{
 #application to list of densities:
 quantile.density( list(density(rnorm(1000)), density(rnorm(1000))) )
}

}
\seealso{
\code{\link{quantile.default}} for a comparable function,
\code{\link{pred.density}} and \code{\link{density.bma}} for the
BMA-specific objects.

Check \url{http://bms.zeugner.eu} for additional help.
}
\author{
Stefan Zeugner
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{plot.bma}
\alias{plot.bma}
\title{Plot Posterior Model Size and Model Probabilities}
\usage{
\method{plot}{bma}(x, ...)
}
\arguments{
\item{x}{an object of class 'bma'}

\item{\dots}{additional arguments for \code{\link{matplot}}}
}
\value{
combines the plotting functions \code{\link{plotModelsize}} and
\code{\link{plotConv}}
}
\description{
Produces a combined plot: upper row shows prior and posterior model size
distribution, lower row shows posterior model probabilities for the best
models
}
\note{
The upper plot shows the prior and posterior distribution of model
sizes (\code{\link{plotModelsize}}).\cr The lower plot is an indicator of
how well the bma object has converged (\code{\link{plotConv}}).
 and Paul
}
\examples{

data(datafls)
mm=bms(datafls,user.int=FALSE)

plot(mm)
}
\seealso{
\code{\link{plotModelsize}} and \code{\link{plotConv}}

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{class}
\name{zlm-class}
\alias{zlm-class}
\title{Class "zlm"}
\description{
A list holding output from the Bayesian Linar Model under Zellner's g prior
akin to class 'lm'
}
\section{Objects from the Class}{
 Objects can be created via calls to
\code{\link{zlm}}, but indirectly also via \code{\link{as.zlm}}.\cr
\code{\link{zlm}} estimates a Bayesian Linear Model under Zellner's g prior
- its output is very similar to objects of class \code{\link{lm}} (cf.
section 'Value')
}

\references{
\url{http://bms.zeugner.eu}
}
\seealso{
\code{\link{zlm}} and \code{\link{as.zlm}} for creating \code{zlm}
objects,\cr \code{\link{density.zlm}}, \code{\link{predict.zlm}} and
\code{\link{summary.zlm}} for other posterior results
}
\author{
Martin Feldkircher and Stefan Zeugner
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/BMS-package.R
\docType{package}
\name{BMS-package}
\alias{BMS}
\alias{BMS-package}
\title{BMS: Bayesian Model Averaging Library}
\description{
Bayesian Model Averaging for linear models with a wide choice of (customizable) priors. Built-in priors include coefficient priors (fixed, hyper-g and empirical priors), 5 kinds of model priors, moreover model sampling by enumeration or various MCMC approaches. Post-processing functions allow for inferring posterior inclusion and model probabilities, various moments, coefficient and predictive densities. Plotting functions available for posterior model size, MCMC convergence, predictive and coefficient densities, best models representation, BMA comparison. Also includes Bayesian normal-conjugate linear model with Zellner's g prior, and assorted methods.
}
\details{
The key function you need is  \code{\link{bms}}.
}
\references{
\url{http://bms.zeugner.eu}: BMS package homepage with help and tutorials

Feldkircher, M. and S. Zeugner (2015): Bayesian Model Averaging Employing 
Fixed and Flexible Priors: The BMS Package for R, Journal of Statistical Software 68(4).

Feldkircher, M. and S. Zeugner (2009): Benchmark Priors
Revisited: On Adaptive Shrinkage and the Supermodel Effect in Bayesian Model
Averaging, IMF Working Paper 09/202.
}
\seealso{
\code{\link{coef.bma}}, \code{\link{plotModelsize}} and
\code{\link{density.bma}} for some operations on the resulting 'bma' object, 
as well as 
\code{\link{predict.bma}} or \code{\link{gdensity}}, or
\code{\link{zlm}} for individual Zellner regression models.

Check \url{http://bms.zeugner.eu} for additional help.
}
\author{
Martin Feldkircher, Paul Hofmarcher, and Stefan Zeugner
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{beta.draws.bma}
\alias{beta.draws.bma}
\title{Coefficients of the Best Models}
\usage{
beta.draws.bma(bmao, stdev = FALSE)
}
\arguments{
\item{bmao}{a 'bma' object (as e.g. resulting from \code{\link{bms}})}

\item{stdev}{if \code{stdev=FALSE} then \code{beta.draws.bma} returns the
(conditional) posterior expected values of the coefficients (i.e. 'Bayesian
coefficients'). If \code{stdev=TRUE} it returns their posterior standard
deviations.}
}
\value{
Each column presents the coefficients for the model indicated by its
column name. The zero coefficients are the excluded covariates per model.
Note that the coefficients returned are only those of the best (100) models
encountered by the \code{bma} object (cf. argument \code{nmodels} of
\code{\link{bms}}).

For aggregate coefficients please refer to \code{\link{coef.bma}}.
}
\description{
Returns a matrix whose columns are the (expected value or standard
deviations of) coefficients for the best models in a bma object.
}
\note{
Note that the elements of \code{beta.draws.bma(bmao)} correspond to
\code{bmao$topmod$betas()}
}
\examples{


  #sample a bma object:
  data(datafls)
  mm=bms(datafls,burn=500,iter=5000,nmodel=20)
  
  #coefficients for all
  beta.draws.bma(mm) 
  
  #standard deviations for the fourth- to eight best models
  beta.draws.bma(mm[4:8],TRUE); 

}
\seealso{
\code{\link{bms}} for creating bms objects, \code{\link{coef.bma}}
for aggregate coefficients

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{class}
\name{mprior-class}
\alias{mprior-class}
\title{Class "mprior"}
\description{
An object pertaining to a BMA model prior
}
\section{Objects from the Class}{
 An \code{mprior} object holds descriptions
and subfunctions pertaining to model priors. The BMA functions
\code{\link{bms}} and post-processing functions rely on this class. \cr
There are currently five model prior structures built into the BMS package,
generated by the following functions (cf. the appendix of
\code{vignette(BMS)}): \cr \code{mprior.uniform.init}: creates a uniform
model prior object.\cr \code{mprior.fixedt.init}: creates the popular
binomial model prior object with common inclusion probabilities.\cr
\code{mprior.randomt.init}: creates a beta-binomial model prior object.\cr
\code{mprior.pip.init}: creates a binomial model prior object that allows
for defining individual prior inclusion probabilities.\cr
\code{mprior.customk.init}: creates a model prior object that allows for
defining a custom prior for each model parameter size.\cr The following
describes the necessary slots:
}

\seealso{
\code{\link{bms}} for creating \code{bma} objects. \cr Check the
appendix of \code{vignette(BMS)} for a more detailed description of built-in
priors.\cr Check \url{http://bms.zeugner.eu/custompriors.php} for examples.
}
\author{
Martin Feldkircher and Stefan Zeugner
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data.R
\docType{data}
\name{datafls}
\alias{datafls}
\title{FLS (2001) growth data}
\format{
A data frame with 53940 rows and 10 variables:
A data frame with 72 observations on the following 42 variables.
 \describe{
   \item{\code{y}}{numeric: Economic growth 1960-1992 as from the Penn World Tables Rev 6.0}
   \item{\code{Abslat}}{numeric: Absolute latitude}
   \item{\code{Spanish}}{numeric: Spanish colony dummy}
   \item{\code{French}}{numeric: French colony dummy}
   \item{\code{Brit}}{numeric: British colony dummy}
   \item{\code{WarDummy}}{numeric: War dummy}
   \item{\code{LatAmerica}}{numeric: Latin America dummy}
   \item{\code{SubSahara}}{numeric; Sub-Sahara dummy}
   \item{\code{OutwarOr}}{numeric: Outward Orientation}
   \item{\code{Area}}{numeric: Area surface}
   \item{\code{PrScEnroll}}{numeric: Primary school enrolment}
   \item{\code{LifeExp}}{numeric: Life expectancy}
   \item{\code{GDP60}}{numeric: Initial GDP in 1960}
   \item{\code{Mining}}{numeric: Fraction of GDP in mining}
   \item{\code{EcoOrg}}{numeric: Degree of capitalism}
   \item{\code{YrsOpen}}{numeric: Number of years having an open economy}
   \item{\code{Age}}{numeric: Age}
   \item{\code{Buddha}}{numeric: Fraction Buddhist}
   \item{\code{Catholic}}{numeric: Fraction Catholic}
   \item{\code{Confucian}}{numeric: Fraction Confucian}
   \item{\code{EthnoL}}{numeric: Ethnolinguistic fractionalization}
   \item{\code{Hindu}}{numeric: Fraction Hindu}
   \item{\code{Jewish}}{numeric: Fraction Jewish}
   \item{\code{Muslim}}{numeric: Fraction Muslim}
   \item{\code{PrExports}}{numeric: Primary exports 1970}
   \item{\code{Protestants}}{numeric: Fraction Protestants}
   \item{\code{RuleofLaw}}{numeric: Rule of law}
   \item{\code{Popg}}{numeric: Population growth}
   \item{\code{WorkPop}}{numeric: workers per inhabitant}
   \item{\code{LabForce}}{numeric: Size of labor force}
   \item{\code{HighEnroll}}{numeric: Higher education enrolment}
   \item{\code{PublEdupct}}{numeric: Public education share}
   \item{\code{RevnCoup}}{numeric: Revolutions and coups}
   \item{\code{PolRights}}{numeric: Political rights}
   \item{\code{CivlLib}}{numeric: Civil liberties}
   \item{\code{English}}{numeric: Fraction speaking English}
   \item{\code{Foreign}}{numeric: Fraction speaking foreign language}
   \item{\code{RFEXDist}}{numeric: Exchange rate distortions}
   \item{\code{EquipInv}}{numeric: Equipment investment}
   \item{\code{NequipInv}}{numeric: Non-equipment investment}
   \item{\code{stdBMP}}{numeric: stand. dev. of black market premium}
   \item{\code{BlMktPm}}{numeric: black market premium}
}
}
\source{
Fernandez, C., Ley, E., and Steel, M. F. (2001b). Model Uncertainty in Cross-Country Growth Regressions. Journal of Applied Econometrics, 16:563-576.
 Data set from \url{https://warwick.ac.uk/fac/sci/statistics/staff/academic-research/steel/steel_homepage/software}.

 A working paper version of Fernandez, Ley and Steel (2001) is available via \url{https://econpapers.repec.org/article/jaejapmet/v_3a16_3ay_3a2001_3ai_3a5_3ap_3a563-576.htm}.
}
\usage{
datafls
}
\description{
The economic growth data set from Fernandez, Ley and Steel, Journal of Applied Econometrics 2001
}
\keyword{datasets}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{gdensity}
\alias{gdensity}
\title{Posterior Density of the Shrinkage Factor}
\usage{
gdensity(x, n = 512, plot = TRUE, addons = "zles", addons.lwd = 1.5, ...)
}
\arguments{
\item{x}{A bma object (see \code{\link{bms}}).}

\item{n}{The integer number of equally spaced points at which the density is
to be estimated. see 'Details' below}

\item{plot}{logical.  If \code{TRUE} (default), the density is plotted; if
\code{FALSE} then \code{gdensity} only returns the estimated posterior
density without plotting.}

\item{addons}{character, defaulting to \code{"zles"}. Specifies which
additional information should be added to the plot via low-level commands
(see 'Details' below).}

\item{addons.lwd}{scalar, default 1.5. Line width to be used for the
low-level plotting commands specified by \code{addons}. Cf. argument
\code{lwd} in \code{\link{par}}}

\item{\dots}{Additional arguments for \code{\link{plot.default}} with
sensible defaults}
}
\value{
\code{gdensity} returns an object of the class \code{\link{density}}
detailing the posterior mixture density of the shrinkage factor.
}
\description{
Calculates the mixture marginal posterior density for the shrinkage factor
(g/(1+g)) from a BMA object under the hyper-g prior and plots it
}
\details{
The function \code{gdensity} estimates and plots the posterior density for
the shrinkage factor \eqn{g/(1+g)}\cr This is evidently only possible if the
shrinkage factor if not fixed, i.e. if the bma object \code{x} was estimated
with a hyper-g prior - cf. argument \code{g} in \code{\link{bms}}\cr The
density is based only on the best models retained in the bma object
\code{x}, cf. argument \code{nmodel} in \code{\link{bms}}\cr A note on
argument \code{n}: The points at which the density is estimated start at
\eqn{max(0,E-5*SD)}, where \eqn{E} and \eqn{SD} are the expected value and
standard deviation of the shrinkage factor, respectively. For plotting the
entire domain \eqn{(0,1)} use \code{xlim=c(0,1)} as an argument for
\code{gdensity}.

The argument \code{addons} specifies what additional information should be
added to the plot(s) via the low-level commands \code{\link{lines}} and
\code{\link{legend}}:\cr \code{"e"} for the posterior expected value (EV) of
the shrinkage factor,\cr \code{"s"} for 2 times posterior standard deviation
(SD) bounds,\cr \code{"m"} for the posterior median,\cr \code{"f"} for
posterior expected values of the individual models whom the density is
averaged over,\cr \code{"z"} for a zero line, \code{"l"} for including a
\code{\link{legend}}\cr The following two are only possible if the bma
object collected statistics on shrinkage, cf. argument \code{g.stats} in
\code{\link{bms}} \code{"E"} for posterior expected value under MCMC
frequencies (see argument \code{exact} in \code{\link{coef.bma}}),\cr
\code{"S"} for the corresponding 2 times standard deviation bounds
(MCMC),\cr

Any combination of these letters will give the desired result. Use
\code{addons=""} for not using any of these.
}
\note{
The computed marginal posterior density is a Bayesian Model Averaging
mixture of the marginal posterior densities of the shrinkage factor under
individual models.  The accuracy of the result therefore depends on the
number of 'best' models contained in \code{x} (cf. argument \code{nmodel} in
\code{\link{bms}}).

Correspondingly, the posterior EV and SD specified by \code{addons="es"} are
based on 'best' model likelihoods ('exact') and are conditional on
inclusion.

The low-level commands enacted by the argument \code{addons} rely on colors
of the \code{\link{palette}}: color 2 for \code{"e"} and \code{"s"}, color 3
for \code{"m"}, color 8 for \code{"f"}, color 4 for \code{"E"} and
\code{"S"}. The default colors may be changed by a call to
\code{\link{palette}}.
}
\examples{


 data(datafls)
 mm=bms(datafls,g="hyper=UIP")

 gdensity(mm) # default plotting
 
 # the grey bars represent expected shrinkage factors of the individual models
 gdensity(mm,addons="lzfes") 
 
 # #plotting the median 'm' and the posterior mean and bounds based on MCMC results:
 gdensity(mm,addons="zSEm",addons.lwd=2)

# plot the posterior shrinkage density only for the very best model
 gdensity(mm[1],addons="esz")


#using the calculated density for other purposes...
 dd=gdensity(mm,plot=FALSE)
 plot(dd) 

}
\seealso{
\code{\link{density.bma}} for computing coefficient densities,
\code{\link{bms}} for creating bma objects, \code{\link{density}} for the
general method

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{aplot}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{zlm}
\alias{zlm}
\title{Bayesian Linear Model with Zellner's g}
\usage{
zlm(formula, data = NULL, subset = NULL, g = "UIP")
}
\arguments{
\item{formula}{an object of class "formula" (or one that can be coerced to
that class), such as a data.frame - cf. \code{\link{lm}}}

\item{data}{an optional \code{\link{data.frame}} (or one that can be coerced
to that class): cf. \code{\link{lm}}}

\item{subset}{an optional vector specifying a subset of observations to be
used in the fitting process.}

\item{g}{specifies the hyperparameter on Zellner's g-prior for the
regression coefficients.\cr \code{g="UIP"} corresponds to \eqn{g=N}, the
number of observations (default); \code{g="BRIC"} corresponds to the
benchmark prior suggested by Fernandez, Ley and Steel (2001), i.e
\eqn{g=max(N, K^2)}, where K is the total number of covariates;\cr
\code{g="EBL"} estimates a local empirical Bayes g-parameter (as in Liang et
al. (2008));\cr \code{g="hyper"} takes the 'hyper-g' prior distribution (as
in Liang et al., 2008) with the default hyper-parameter \eqn{a=3}; This
hyperparameter can be adjusted (between \eqn{2<a<=4}) by setting
\code{g="hyper=2.9"}, for instance.\cr Alternatively, \code{g="hyper=UIP"}
sets the prior expected value of the shrinkage factor equal to that of UIP
(above), \code{g="hyper=BRIC"} sets it according to BRIC}
}
\value{
Returns a list of class \code{zlm} that contains at least the
following elements (cf. \code{\link{lm}}):

\item{coefficients}{a named vector of posterior coefficient expected values}
\item{residuals}{the residuals, that is response minus fitted values}
\item{fitted.values}{the fitted mean values} \item{rank}{the numeric rank of
the fitted linear model} \item{df.residual}{the residual degrees of freedom}
\item{call}{the matched call} \item{terms}{the \code{\link{terms}} object
used} \item{model}{the model frame used} \item{coef2moments}{a named vector
of coefficient posterior second moments} \item{marg.lik}{the log marginal
likelihood of the model} \item{gprior.info}{a list detailing information on
the g-prior, cf. output value \code{gprior.info} in \code{\link{bms}}}
}
\description{
Used to fit the Bayesian normal-conjugate linear model with Zellner's g
prior and mean zero coefficient priors. Provides an object similar to the
\code{\link{lm}} class.
}
\details{
\code{zlm} estimates the coefficients of the following model \eqn{y = \alpha
+ X \beta + \epsilon} where \eqn{\epsilon} ~ \eqn{N(0,\sigma^2)} and \eqn{X}
is the design matrix\cr The priors on the intercept \eqn{\alpha} and the
variance \eqn{\sigma} are improper: \eqn{alpha \propto 1}, \eqn{sigma
\propto \sigma^{-1}} \cr Zellner's g affects the prior on coefficients:
\eqn{beta} ~ \eqn{N(0, \sigma^2 g (X'X)^{-1})}. \cr Note that the prior mean
of coefficients is set to zero by default and cannot be adjusted. Note
moreover that \code{zlm} always includes an intercept.
}
\examples{


data(datafls)

#simple example
foo = zlm(datafls)
summary(foo)

#example with formula and subset
foo2 = zlm(y~GDP60+LifeExp, data=datafls, subset=2:70) #basic model, omitting three countries
summary(foo2)


}
\references{
The representation follows Fernandez, C. E. Ley and M. Steel
(2001): Benchmark priors for Bayesian model averaging. Journal of
Econometrics 100(2), 381--427

See also \url{http://bms.zeugner.eu} for additional help.
}
\seealso{
The methods \code{\link{summary.zlm}} and \code{\link{predict.lm}}
provide additional insights into \code{zlm} output.\cr The function
\code{\link{as.zlm}} extracts a single out model of a \code{bma} object (as
e.g. created through\code{\link{bms}}).\cr Moreover, \code{\link{lm}} for
the standard OLS object, \code{\link{bms}} for the application of \code{zlm}
in Bayesian model averaging.

Check \url{http://bms.zeugner.eu} for additional help.
}
\author{
Stefan Zeugner
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{predict.bma}
\alias{predict.bma}
\title{Predict Method for bma Objects}
\usage{
\method{predict}{bma}(object, newdata = NULL, exact = FALSE, topmodels = NULL, ...)
}
\arguments{
\item{object}{a bma object - see \code{\link{bms}}}

\item{newdata}{An optional data.frame, matrix or vector containing variables
with which to predict. If omitted, then (the expected values of) the fitted
values are returned.}

\item{exact}{If \code{FALSE} (default), then prediction is based on all
models (i.e. on their MCMC frequencies in case the \code{\link{bms}}
parameter \code{mcmc} was set to an mcmc sampler.\cr If \code{TRUE}, then
prediction is based on analytical likelihoods of the best models retained in
\code{object} - cf. \code{\link{bms}} parameter \code{nmodel}.}

\item{topmodels}{index of the models with whom to predict: for instance,
\code{topmodels=1} predicts based solely on the best model, whereas
\code{topmodels=1:5} predicts based on a combination of the five best
models.\cr Note that setting \code{topmodels} triggers \code{exact=TRUE}.}

\item{\dots}{further arguments passed to or from other methods.}
}
\value{
A vector with (expected values of) fitted values.
}
\description{
Expected value of prediction based on 'bma' object
}
\examples{

 data(datafls)
 mm=bms(datafls,user.int=FALSE)
 
 predict(mm) #fitted values based on MCM frequencies
 predict(mm, exact=TRUE) #fitted values based on best models
 
 predict(mm, newdata=1:41) #prediction based on MCMC frequencies 
 
 predict(mm, newdata=datafls[1,], exact=TRUE) #prediction based on a data.frame
 
 # the following two are equivalent:
 predict(mm, topmodels=1:10)
 predict(mm[1:10], exact=TRUE)
 
 
}
\seealso{
\code{\link{coef.bma}} for obtaining coefficients,
\code{\link{bms}} for creating bma objects, \code{\link{predict.lm}} for a
comparable function

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

\name{print.bma}
\alias{print.bma}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Printing bma Objects }
\description{
  Print method for objects of class 'bma'
}
\usage{
 \method{print}{bma}(x, ...)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ a bma object - see \code{\link{bms}} }
  \item{\dots}{ additional arguments to \code{\link{print}} }
}
\value{
  prints calls to \code{coef.bma{x}} and \code{summary.bma(x)}
}
%\references{ ~put references to the literature/web site here ~ }
\author{ Martin Feldkircher and Stefan Zeugner }
\seealso{ \code{\link{coef.bma}} and \code{\link{summary.bma}} for the underlying functions, \code{\link{bms}} for creating bma objects, \code{\link{print}} 

Check \url{http://bms.zeugner.eu} for additional help.}
\examples{
 data(datafls)
 mm=bms(datafls,user.int=FALSE)
 print(mm)
 
 ## similar:
 coef(mm)
 summary(mm)
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{print}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{is.bma}
\alias{is.bma}
\title{Tests for a 'bma' Object}
\usage{
is.bma(bmao)
}
\arguments{
\item{bmao}{a 'bma' object: see 'value'}
}
\value{
Returns \code{TRUE} if bmao is of class 'bma', \code{FALSE}
otherwise.
}
\description{
tests for objects of class "bma"
}
\examples{

 data(datafls)
 mm=bms(datafls,burn=1000, iter=4000)
 is.bma(mm)
}
\seealso{
'Output' in \code{\link{bms}} for the structure of a 'bma' object

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{plotConv}
\alias{plotConv}
\title{Plot Convergence of BMA Sampler}
\usage{
plotConv(bmao, include.legend = TRUE, add.grid = TRUE, ...)
}
\arguments{
\item{bmao}{an object of class 'bma' - see \code{\link{bms}}}

\item{include.legend}{whether to include a \code{\link{legend}} in the plot}

\item{add.grid}{whether to include a \code{\link{grid}} in the plot}

\item{\dots}{other parameters for \code{\link{matplot}}}
}
\description{
Plots the posterior model probabilites based on 1) marginal likelihoods and
2) MCMC frequencies for the best models in a 'bma' object and details the
sampler's convergence by their correlation
}
\details{
A call to bms with a MCMC sampler (e.g.
\code{bms(datafls,mcmc="bd",nmodel=100)} uses a Metropolis-Hastings
algorithm to sample through the model space: the frequency of how often
models are drawn converges to the distribution of their posterior marginal
likelihoods.\cr While sampling, each 'bma' object stores the best models
encountered by its sampling chain with their marginal likelihood and their
MCMC frequencies.\cr \code{plotConv} compares the MCMC frequencies to
marginal likelihoods, and thus visualizes how well the sampler has
converged.
}
\note{
\code{plotConv} is also used by \code{\link{plot.bma}}
}
\examples{


data(datafls)
mm=bms(datafls[,1:12],user.int=FALSE)

plotConv(mm)

#is similar to
matplot(pmp.bma(mm),type="l")

}
\seealso{
\code{\link{pmp.bma}} for posterior model probabilites based on the
two concepts, \code{\link{bms}} for creating objects of class 'bma'

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{aplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{c.bma}
\alias{c.bma}
\alias{combine_chains}
\title{Concatenate bma objects}
\usage{
\method{c}{bma}(..., recursive = FALSE)
}
\arguments{
\item{\dots}{At least two 'bma' objects (cf. \code{\link{bms}})}

\item{recursive}{retained for compatibility with \code{\link{c}} method}
}
\description{
Combines bma objects (resulting from \code{\link{bms}}). Can be used to
split estimation over several machines, or combine the MCMC results obtained
from different starting points.
}
\details{
Aggregates the information obtained from several chains. The result is a
'bma' object (cf. 'Values' in \code{\link{bms}}) that can be used just as a
standard 'bma' object.\cr Note that \code{combine_chains} helps in
particular to paralllelize the enumeration of the total model space: A model
with \eqn{K} regressors has \eqn{2^K} potential covariate combinations: With
\eqn{K} large (more than 25), this can be pretty time intensive.  With the
\code{\link{bms}} arguments \code{start.value} and \code{iter}, sampling can
be done in steps: cf. example 'enumeration' below.
}
\examples{

 data(datafls)
  
 #MCMC case ############################
 model1=bms(datafls,burn=1000,iter=4000,mcmc="bd",start.value=c(20,30,35))
 model2=bms(datafls,burn=1500,iter=7000,mcmc="bd",start.value=c(1,10,15))
 
 model_all=c(model1,model2)
 coef(model_all)
 plot(model_all)
 
 
 
 #splitting enumeration ########################
 
 #standard case with 12 covariates (4096 differnt combinations):
 enum0=bms(datafls[,1:13],mcmc="enumerate")
 
 # now split the task:
 # enum1 does everything from model zero (the first model) to model 1999
 enum1=bms(datafls[,1:13],mcmc="enumerate",start.value=0,iter=1999)
 
 # enum2 does models from index 2000 to the index 3000 (in total 1001 models)
 enum2=bms(datafls[,1:13],mcmc="enumerate",start.value=2000,iter=1000)
 
 # enum3 does models from index 3001 to the end
 enum3=bms(datafls[,1:13],mcmc="enumerate",start.value=3001)
 
 enum_combi=c(enum1,enum2,enum3)
 coef(enum_combi)
 coef(enum0)
 #both enum_combi and enum0 have exactly the same results 
 #(one difference: enum_combi has more 'top models' (1500 instead of 500))

}
\seealso{
\code{\link{bms}} for creating bma objects

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{models}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{predict.zlm}
\alias{predict.zlm}
\title{Predict Method for zlm Linear Model}
\usage{
\method{predict}{zlm}(object, newdata = NULL, se.fit = FALSE, ...)
}
\arguments{
\item{object}{a zlm linear model object - see \code{\link{zlm}}}

\item{newdata}{An optional data.frame, matrix or vector containing variables
with which to predict. If omitted, then (the expected values of) the fitted
values are returned.}

\item{se.fit}{A switch indicating if the standard deviations for the
predicted varaibles are required.}

\item{\dots}{further arguments passed to or from other methods.}
}
\value{
A vector with (expected values of) fitted values.\cr If
\code{se.fit} is \code{TRUE}, then the output is a list with the following
elements: \item{fit}{ a vector with the expected values of fitted values}
\item{std.err}{ a vector with the standard deviations of fitted values}
\item{se.fit}{ a vector with the standard errors without the residual scale
akin to \code{se.fit} in \code{\link{predict.lm}} } \item{residual.scale}{
The part from the standard deviations that involves the identity matrix.
Note that \code{sqrt(se.fit^2+residual.scale^2)} yields \code{std.err}. }
}
\description{
Expected value (And standard errors) of predictions based on 'zlm' linear
Bayesian model under Zellner's g prior
}
\examples{

 data(datafls)
 mm=zlm(datafls,g="EBL")
 
 predict(mm) #fitted values 
 predict(mm, newdata=1:41) #prediction based on a 'new data point' 
 
 #prediction based on a 'new data point', with 'standard errors'
 predict(mm, newdata=datafls[1,], se.fit=TRUE) 
 
}
\seealso{
\code{\link{bms}} for creating zlm objects,
\code{\link{predict.lm}} for a comparable function,
\code{\link{predict.bma}} for predicting with bma objects

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{pmpmodel}
\alias{pmpmodel}
\title{Posterior Model Probability for any Model}
\usage{
pmpmodel(bmao, model = numeric(0), exact = TRUE)
}
\arguments{
\item{bmao}{A bma object as created by \code{\link{bms}}.}

\item{model}{A model index - either variable names, or a logical with model
binaries, or the model hexcode (cf. \code{\link{hex2bin}}, or a numeric with
positions of the variables to be included.}

\item{exact}{If \code{TRUE}, then the resulting PMP is based on analytical
model likelihoods (works for any model). \cr If \code{FALSE}, the the
resulting PMP is derived from MCMC frequencies (works only for the null and
fullmodel, as well as for models contained in \code{bmao}'s topmod
object.\cr If \code{bmao} is based on enumeration (cf. argument \code{mcmc}
in \code{\link{bms}}, then \code{exact} does not matter.}
}
\value{
A scalar with (an estimate of) the posterior model probability for
\code{model}
}
\description{
Returns the posterior model probability for any model based on bma results
}
\details{
If the model as provided in \code{model} is the null or the full model, or
is contained in \code{bmao}'s topmod object (cf. argument \code{nmodel} in
\code{\link{bms}}), \cr then the result is the same as in
\code{\link{pmp.bma}}.\cr If not and \code{exact=TRUE}, then \code{pmpmodel}
estimates the model based on comparing its marginal likelihood (times model
prior) to the likelihoods in the \code{topmod} object and multiplying by
their sum of PMP according to MCMC frequencies,
}
\examples{


## sample BMA for growth dataset, enumeration sampler
data(datafls)
mm=bms(datafls[,1:10],nmodel=5)

#show the best 5 models:
pmp.bma(mm)
#first column: posterior model prob based on model likelihoods,
#second column: posterior model prob based MCMC frequencies,

### Different ways to get the same result: #########

#PMP of 2nd-best model (hex-code representation)
pmpmodel(mm,"00c")

#PMP of 2nd-best model (binary representation)
incls=as.logical(beta.draws.bma(mm)[,2])
pmpmodel(mm,incls)

#PMP of 2nd-best model (via variable names)
#names of regressors in model "00c": 
names(datafls[,2:10])[incls]
pmpmodel(mm,c("SubSahara", "LatAmerica"))

#PMP of 2nd-best model (via positions)
pmpmodel(mm,c(6,7))

####PMP of another model #########
pmpmodel(mm,1:5)


}
\seealso{
\code{\link{pmp.bma}} for similar
functions

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

\name{BMS-internal}
\title{Internal BMS objects}
\alias{.addup.enum}
\alias{.addup.enum.wotherstats}
\alias{.addup.mcmc}
\alias{.addup.mcmc.wotherstats}
\alias{.adjustdots}
\alias{.arglist}
\alias{.arglist.matchenvir}
\alias{.choose.gprior}
\alias{.choose.mprior}
\alias{.constr.intmat}
\alias{.construct.arglist}
\alias{.cor.topmod}
\alias{.enum_fromindex}
\alias{.f21_4hyperg}
\alias{.f21simple}
\alias{.fixedset.mprior}
\alias{.fixedset.sampler}
\alias{.fls.samp}
\alias{.getpolycoefs}
\alias{.hexcode.binvec.convert}
\alias{.iterenum}
\alias{.iterenum.bone}
\alias{.iterenum.KgtN}
\alias{.gprior.constg.init}
\alias{.gprior.eblocal.init}
\alias{.gprior.hyperg.init}
\alias{.mprior.customk.init}
\alias{.mprior.fixedt.init}
\alias{.mprior.pip.init}
\alias{.mprior.randomt.init}
\alias{.mprior.uniform.init}
\alias{.ols.terms2}
\alias{.post.beta.draws}
\alias{.post.calc}
\alias{.post.constant}
\alias{.post.estimates}
\alias{.post.topmod.bma}
\alias{.post.topmod.includes}
\alias{.quantile.density}
\alias{.rev.jump}
\alias{.starter}
\alias{.top10}
\alias{.topmod.as.bbetaT}
\description{Internal BMS objects.}
\details{These are not to be called by the user, but may come handy for programming tasks. 
The \code{BMS.RData} corresponding to the package (available at \url{http://bms.zeugner.eu} contains some comments inside the functions to show how they work

The functions \code{.mprior.fixedt.init} and \code{.gprior.constantg.init} are templates of functions to create objects of class \code{\link{mprior-class}} and \code{\link{gprior-class}}, which may be used to customize \code{\link{bms}} and \code{\link{zlm}}.

\code{.topmod.as.bbetaT} could be of use to a programmer interested in the \code{\link{topmod}} object.

\code{.ols.terms2} provides the basic OLS results on which this BMA routine is based.

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{f21hyper}
\alias{f21hyper}
\title{Gaussian Hypergeometric Function F(a,b,c,z)}
\usage{
f21hyper(a, b, c, z)
}
\arguments{
\item{a}{The parameter \code{a} of the Gaussian hypergeometric function,
must be a positive scalar here}

\item{b}{The parameter \code{b} of the Gaussian hypergeometric function,
must be a positive scalar here}

\item{c}{The parameter \code{c} of the Gaussian hypergeometric function,
must be a positive scalar here}

\item{z}{The parameter \code{z} of the Gaussian hypergeometric function,
must be between -1 and 1 here}
}
\value{
The value of the Gaussian hypergeometric function \eqn{ F(a,b,c,z) }
}
\description{
Computes the value of a Gaussian hypergeometric function \eqn{ F(a,b,c,z) }
for \eqn{-1 \leq z \leq 1} and \eqn{a,b,c \geq 0}
}
\details{
The function \code{f21hyper} complements the analysis of the 'hyper-g prior'
introduced by Liang et al. (2008).\cr For parameter values, compare cf.
\url{https://en.wikipedia.org/wiki/Hypergeometric_function}.
}
\note{
This function is a simple wrapper function of sped-up code that is
intended for sporadic application by the user; it is neither efficient nor
general; for a more general version cf. the package '\code{hypergeo}'
}
\examples{

  
  f21hyper(30,1,20,.8) #returns about 165.8197
  
  f21hyper(30,10,20,0) #returns one
  
  f21hyper(10,15,20,-0.1) # returns about 0.4872972
}
\references{
Liang F., Paulo R., Molina G., Clyde M., Berger J.(2008):
Mixtures of g-priors for Bayesian variable selection. J. Am. Statist. Assoc.
103, p. 410-423

\url{https://en.wikipedia.org/wiki/Hypergeometric_function}
}
\seealso{
package \code{hypergeo} for a more proficient implementation.

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{post.var}
\alias{post.var}
\alias{post.pr2}
\alias{deviance.bma}
\alias{deviance.zlm}
\title{Posterior Variance and Deviance}
\usage{
post.var(object, exact = FALSE, ...)
}
\arguments{
\item{object}{A \code{bma} object (as produced by \code{\link{bms}}) or a
\code{\link{zlm}} object.}

\item{exact}{When \code{exact=FALSE}, then \code{deviance} will be based on
MCMC frequencies, if \code{exact=TRUE} then it will be based on\cr
analytical posterior model probabilities - cf. argument \code{exact} in
\code{\link{coef.bma}}.}

\item{...}{further arguments passed to or from other methods}
}
\description{
Returns posterior residual variance, deviance, or pseudo R-squared,
according to the chosen prior structure
}
\details{
\code{post.var}: Posterior residual variance as according to the prior
definitions contained in \code{object} \cr \code{post.pr2}: A
pseudo-R-squared corresponding to unity minus posterior variance over
dependent variance. \cr \code{deviance.bma}: returns the
\code{\link{deviance}} of a \code{bma} model as returned from
\code{\link{bms}}. \cr \code{deviance.zlm}: returns the
\code{\link{deviance}} of a \code{\link{zlm}} model.
}
\examples{

 data(datafls)
  
 mm=bms(datafls[,1:10])
 deviance(mm)/nrow(datafls) # is equivalent to
 post.var(mm)
 
 post.pr2(mm) # is equivalent to
 1 - post.var(mm) / ( var(datafls[,1])*(1-1/nrow(datafls)) )
 
}
\seealso{
\code{\link{bms}} for creating \code{bma} objects and priors,
\code{\link{zlm}} object.

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{info.bma}
\alias{info.bma}
\alias{summary.bma}
\title{Summary Statistics for a 'bma' Object}
\usage{
info.bma(bmao)

\method{summary}{bma}(object, ...)
}
\arguments{
\item{bmao}{same as \code{object}}

\item{object}{a list/object of class 'bma' that typically results from the
function \code{bms} (see \code{\link{bms}} for details)}

\item{\dots}{further arguments passed to or from other methods}
}
\value{
A character vector summarizing the results of a call to \code{bms}
\item{Mean no. of Regressors}{ the posterior mean of model size}
\item{Draws}{the number of iterations (ex burn-ins)} \item{Burnins}{the
number of burn-in iterations} \item{Time}{the time spent on iterating
through the model space} \item{No. of models visited}{the number of times a
model was accepted (including burn-ins)} \item{Modelspace }{the total model
space \eqn{2^K}}\item{list(list("2^K"))}{the total model space \eqn{2^K}}
\item{Percentage visited}{\code{No. of models visited/Modelspace*100}}
\item{Percentage Topmodels}{number of times the best models were drawn in
percent of \code{Draws}} \item{Corr. PMP}{the correlation between the MCMC
frequencies of the best models (the number of times they were drawn) and
their marginal likelihoods.} \item{No. Obs.}{Number of observations}
\item{Model Prior}{a character conforming to the argument \code{mprior} of
\code{bms}, and the expected prior model size} \item{g-prior}{a character
corresponding to argument \code{g} of function \code{bms}}
\item{Shrinkage-Stats}{Posterior expected value und standard deviation (if
applicable) of the shrinkage factor. Only included if argument
\code{g.stats} of function \code{bms} was set to TRUE}
}
\description{
Returns a vector with summary statistics for a 'bma' object
}
\details{
\code{info.bma} is equivalent to \code{summary.bma}, its argument
\code{bmao} conforms to the argument \code{object}
}
\note{
All of the above statistics can also be directly extracted from the
bma object (\code{bmao}). Therefore \code{summary.bma} only returns a
character vector.
}
\examples{

  data(datafls)

  m_fixed=bms(datafls,burn=1000,iter=3000,user.int=FALSE, )
  summary(m_fixed)
   
  m_ebl=bms(datafls,burn=1000,iter=3000,user.int=FALSE, g="EBL",g.stats=TRUE)
  info.bma(m_ebl)

}
\seealso{
\code{\link{bms}} and \code{\link{c.bma}} for functions creating
bma objects, \code{print.bma} makes use of \code{summary.bma}.

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{pmp.bma}
\alias{pmp.bma}
\title{Posterior Model Probabilities}
\usage{
pmp.bma(bmao, oldstyle = FALSE)
}
\arguments{
\item{bmao}{A bma object (see argument \code{nmodel} in \code{\link{bms}}),
alternatively an object of class \code{\link{topmod}}}

\item{oldstyle}{For normal use, leave this at \code{FALSE}. It is an
argument for compatibility with older BMS versions - see section 'Notes'}
}
\value{
the result is a matrix, its row names describe the model binaries\cr
There are two columns in the matrix: \item{PMP (Exact)}{posterior model
probabilities based on the posterior likelihoods of the best models in
\code{bmao} } \item{PMP (MCMC)}{posterior model probabilities of the best
models in \code{bmao} based on their MCMC frequencies, relative to all
models encountered by \code{bmao} - see 'Details' }
}
\description{
Returns the posterior model probabilites for the best models encountered by
a 'bma' object
}
\details{
A call to bms with an MCMC sampler (e.g.
\code{bms(datafls,mcmc="bd",nmodel=100)} uses a Metropolis-Hastings
algorithm to sample through the model space - and the frequency of how often
models are drawn converges to the distribution of their posterior marginal
likelihoods.  While sampling, each 'bma' object stores the best models
encountered by its sampling chain with their marginal likelihood and their
MCMC frequencies.

\code{pmp.bma} then allows for comparing the posterior model probabilities
(PMPs) for the two different methods, similar to \code{\link{plotConv}}.  It
calculates the PMPs based on marginal likelihoods (first column) and the
PMPs based on MCMC frequencies (second column) for the best x models stored
in the bma object.

The correlation of the two columns is an indicator of how well the MCMC
sampler has converged to the actual PMP distribution - it is therefore also
given in the output of \code{\link{summary.bma}}.

The second column is slightly different in case the \code{\link{bms}}
argument \code{mcmc} was set to \code{mcmc="enumeration"}: In this case, the
second column is also based on marginal likelihoods. The correlation between
the two columns is therefore one.
}
\note{
The second column thus shows the PMPs of the best models relative to
all models the call to \code{\link{bms}} has sampled through (therefore
typically the second column adds up to less than one).  The first column
relates to the likelihoods of the best models, therefore it would add up to
1.  In order estimate for their marginal likelihoods with respect to the
other models (the ones not retained in the best models), these PMP aadding
up to one are multiplied with the sum of PMP of the best models accroding to
MCMC frequencies.  Therefore, the two columns have the same column sum.

CAUTION: In package versions up to \code{BMS 0.2.5}, the first column was
indeed set always equal to one. This behaviour can still be mimicked by
setting \code{oldstyle=TRUE}.
}
\examples{


## sample BMA for growth dataset, MCMC sampler
data(datafls)
mm=bms(datafls[,1:10],nmodel=20, mcmc="bd")

## mmodel likelihoods and MCMC frequencies of best 20 models
print(mm$topmod)

pmp.bma(mm)
#first column: posterior model prob based on model likelihoods,
#  relative to best models in 'mm'
#second column: posterior model prob based MCMC frequencies,
#  relative to all models encountered by 'mm'

#consequently, first column adds up to one
#second column shows how much of the sampled model space is
# contained in the best models
colSums(pmp.bma(mm))


#correlation betwwen the two shows how well the sampler converged
cor(pmp.bma(mm)[,1],pmp.bma(mm)[,2])

#is the same as given in summary.bma
summary(mm)["Corr PMP"]

#plot the two model probabilites
plotConv(mm)

#equivalent to the following chart
plot(pmp.bma(mm)[,2], type="s")
lines(pmp.bma(mm)[,1],col=2)


#moreover, note how the first column is constructed
liks=exp(mm$top$lik())
liks/sum(liks)
pmp.bma(mm)[,1] #these two are equivalent



#the example above does not converge well,
#too few iterations and best models
# this is already better, but also not good
mm=bms(datafls[,1:10],burn=2000,iter=5000,nmodel=200)


# in case the sampler has been 'enumeration' instead of MCMC,
# then both matrix columns are of course equivalent
mm=bms(datafls[,1:10],nmodel=512,mcmc="enumeration")
cor(pmp.bma(mm)[,1],pmp.bma(mm)[,2])
colSums(pmp.bma(mm))


}
\seealso{
\code{\link{plotConv}} for plotting \code{pmp.bma},
\code{\link{pmpmodel}} to obtain the PMP for any individual model,
\code{\link{bms}} for sampling bma objects

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{density.bma}
\alias{density.bma}
\alias{density.zlm}
\title{Coefficient Marginal Posterior Densities}
\usage{
\method{density}{bma}(
  x,
  reg = NULL,
  addons = "lemsz",
  std.coefs = FALSE,
  n = 300,
  plot = TRUE,
  hnbsteps = 30,
  addons.lwd = 1.5,
  ...
)
}
\arguments{
\item{x}{A bma object (see \code{\link{bms}}) or a \code{\link{zlm}} object.}

\item{reg}{A scalar integer or character detailing which covariate's
coefficient should be plotted. If \code{reg=NULL} (default), then all
regressors are plotted one after the other, waiting for user interaction.}

\item{addons}{character. Specifies which additional information should be
added to the plot via low-level commands (see 'Details' below).}

\item{std.coefs}{logical. If \code{TRUE} then the posterior density is
estimated for standardized coefficients (representing the case where all
variables have mean zero and standard deviation 1) - default is
\code{FALSE}.}

\item{n}{numeric. the number of equally spaced points at which the density
is to be estimated.}

\item{plot}{logical.  If \code{TRUE} (default), the density is plotted; if
\code{FALSE} then \code{density.bma} only returns the estimated posterior
densities without plotting.}

\item{hnbsteps}{even integer, default 30. The number of numerical
integration steps to be used in case of a hyper-g prior (cf. argument
\code{g} in \code{\link{bms}}). Increase this number to increase accuracy.}

\item{addons.lwd}{scalar, default 1.5. Line width to be used for the
low-level plotting commands specified by \code{addons}. Cf. argument
\code{lwd} in \code{\link{par}}}

\item{\dots}{Additional arguments for \code{\link{plot.default}} with
sensible defaults}
}
\value{
The function returns a list containing objects of the class
\code{\link{density}} detailing the marginal posterior densities for each
coefficient provided in \code{reg}.\cr In case of \code{density.zlm}, simple
marginal posterior coefficient densities are computed, while
\code{density.bma} calculates there mixtures over models according to
posterior model probabilities.\cr These densities contain only the density
points apart from the origin. (see 'Note' below)

As long as \code{plot=TRUE}, the densities are plotted too.  Note that (for
\code{density.bma}) if the posterior inclusion probability of a covariate is
zero, then it will not be plotted, and the returned density will be
\code{list(x=numeric(n),y=numeric(n))}.
}
\description{
Calculates the mixture marginal posterior densities for the coefficients
from a BMA object and plots them
}
\details{
The argument \code{addons} specifies what additional information should be
added to the plot(s) via the low-level commands \code{\link{lines}} and
\code{\link{legend}}:\cr \code{"e"} for the posterior expected value (EV) of
coefficients conditional on inclusion (see argument \code{exact=TRUE} in
\code{\link{coef.bma}}),\cr \code{"s"} for 2 times posterior standard
deviation (SD) bounds,\cr \code{"m"} for the posterior median,\cr \code{"b"}
for posterior expected values of the individual models whom the density is
averaged over,\cr \code{"E"} for posterior EV under MCMC frequencies (see
argument \code{exact=FALSE} in \code{\link{coef.bma}}),\cr \code{"S"} for
the corresponding SD bounds (MCMC),\cr \code{"p"} for plotting the Posterior
Inclusion Probability above the density plot,\cr \code{"l"} for including a
\code{\link{legend}}, \code{"z"} for a zero line, \code{"g"} for adding a
\code{\link{grid}}

Any combination of these letters will give the desired result. Use
\code{addons=""} for not using any of these.\cr In case of
\code{density.zlm}, only the letters \code{e}, \code{s}, \code{l}, \code{z},
and \code{g} will have an effect.
}
\note{
The computed marginal posterior densities from \code{density.bma} are
a Bayesian Model Averaging mixture of the marginal posterior densities of
the individual models.  The accuracy of the result therefore depends on the
number of 'best' models contained in \code{x} (cf. argument \code{nmodel} in
\code{\link{bms}}).

The marginal posterior density can be interpreted as 'conditional on
inclusion': If the posterior inclusion probability of a variable is smaller
than one, then some of its posterior density is Dirac at zero.  Therefore
the integral of the returned density vector adds up to the posterior
inclusion probability, i.e. the probability that the coefficient is not
zero.

Correspondingly, the posterior EV and SD specified by \code{addons="es"} are
based on 'best' model likelihoods ('exact') and are conditional on
inclusion.  They correspond to the results from command
\code{coef.bma(x,exact=TRUE,condi.coef=TRUE,order.by.pip=FALSE)} (cf. the
example below).

The low-level commands enacted by the argument \code{addons} rely on colors
of the \code{\link{palette}}: color 2 for \code{"e"} and \code{"s"}, color 3
for \code{"m"}, color 8 for \code{"b"}, color 4 for \code{"E"} and
\code{"S"}. The default colors may be changed by a call to
\code{\link{palette}}.

Up to BMS version 0.3.0, \code{density.bma} may only cope with built-in
\code{gprior}s, not with any user-defined priors.
}
\examples{


 data(datafls)
 mm=bms(datafls)

 density(mm,reg="SubSahara")
 density(mm,reg=7,addons="lbz") 
 density(mm,1:9)
 density(mm,reg=2,addons="zgSE",addons.lwd=2,std.coefs=TRUE)

# plot the posterior density only for the very best model
 density(mm[1],reg=1,addons="esz")


#using the calculated density for other purposes...
 dd=density(mm,reg="SubSahara")
 plot(dd) 

 dd_list=density(mm,reg=1:3,plot=FALSE,n=400)
 plot(dd_list[[1]])


#Note that the shown density is only the part that is not zero
 dd=density(mm,reg="Abslat",addons="esl")
 pip_Abslat=sum(dd$y)*diff(dd$x)[1]

 #this pip and the EV conform to what is done by the follwing command
 coef(mm,exact=TRUE,condi.coef=TRUE)["Abslat",]

}
\seealso{
\code{\link{quantile.coef.density}} for extracting quantiles,
\code{\link{coef.bma}} for similar concepts, \code{\link{bms}} for creating
bma objects

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{aplot}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{variable.names.bma}
\alias{variable.names.bma}
\alias{model.frame.bma}
\title{Variable names and design matrix}
\usage{
\method{variable.names}{bma}(object, ...)
}
\arguments{
\item{object}{A \code{bma} object (as produced by \code{\link{bms}})}

\item{...}{further arguments passed to or from other methods}
}
\description{
Simple utilities retrieving variable names and design matrix from a bma
object
}
\details{
All functions are \code{bma}-functions for the generic methods
\code{\link{variable.names}}, \code{\link{deviance}}, and
\code{\link{model.frame}}.
}
\examples{

 data(datafls)
 bma_enum=bms(datafls[1:20,1:10])
 
 model.frame(bma_enum) # similar to 
 bma_enum$arguments$X.data
 
 variable.names(bma_enum)[-1] # is equivalent to
 bma_enum$reg.names
 
}
\seealso{
\code{\link{bms}} for creating bma objects

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/bma.R
\name{bms}
\alias{bms}
\title{Bayesian Model Sampling and Averaging}
\usage{
bms(
  X.data,
  burn = 1000,
  iter = NA,
  nmodel = 500,
  mcmc = "bd",
  g = "UIP",
  mprior = "random",
  mprior.size = NA,
  user.int = TRUE,
  start.value = NA,
  g.stats = TRUE,
  logfile = FALSE,
  logstep = 10000,
  force.full.ols = FALSE,
  fixed.reg = numeric(0)
)
}
\arguments{
\item{X.data}{a data frame or a matrix, with the dependent variable in the
first column, followed by the covariates (alternatively, \code{X.data} can
also be provided as a \code{\link{formula}}).  Note that \code{bms}
automatically estimates a constant, therefore including constant terms is
not necessary.}

\item{burn}{The (positive integer) number of burn-in draws for the MC3
sampler, defaults to 1000. (Not taken into account if mcmc="enumerate")}

\item{iter}{If mcmc is set to an MC3 sampler, then this is the number of
iteration draws to be sampled (ex burn-ins), default 3000 draws. \cr If
\code{mcmc="enumerate"}, then iter is the number of models to be sampled,
starting from 0 (defaults to \eqn{2^K-1}) - cf. \code{start.value}.}

\item{nmodel}{the number of best models for which information is stored
(default 500). Best models are used for convergence analysis between
likelihoods and MCMC frequencies, as well as likelihood-based inference.\cr
Note that a very high value for \code{nmodel} slows down the sampler
significantly. Set nmodel=0 to speed up sampling (if best model information
is not needed).}

\item{mcmc}{a character denoting the model sampler to be used.\cr The MC3
sampler \code{mcmc="bd"} corresponds to a birth/death MCMC algogrithm.
\code{mcmc="rev.jump"} enacts a reversible jump algorithm adding a "swap"
step to the birth / death steps from "bd".\cr Alternatively, the entire
model space may be fully enumerated by setting \code{mcmc="enumerate"} which
will iterate all possible regressor combinations (Note: consider that this
means \eqn{2^K} iterations, where K is the number of covariates.)\cr Default
is full enumeration (\code{mcmc="enumerate"}) with less then 15 covariates,
and the birth-death MC3 sampler (\code{mcmc="bd"}) with 15 covariates or
more. Cf. section 'Details' for more options.}

\item{g}{the hyperparameter on Zellner's g-prior for the regression
coefficients.\cr \code{g="UIP"} corresponds to \eqn{g=N}, the number of
observations (default);\cr \code{g="BRIC"} corresponds to the benchmark
prior suggested by Fernandez, Ley and Steel (2001), i.e \eqn{g=max(N, K^2)},
where K is the total number of covariates;\cr \code{g="RIC"} sets
\eqn{g=K^2} and conforms to the risk inflation criterion by George and
Foster (1994)\cr \code{g="HQ"} sets \eqn{g=log(N)^3} and asymptotically
mimics the Hannan-Quinn criterion with \eqn{C_{HQ}=3} (cf. Fernandez, Ley
and Steel, 2001, p.395)\cr \code{g="EBL"} estimates a local empirical Bayes
g-parameter (as in Liang et al. (2008));\cr \code{g="hyper"} takes the
'hyper-g' prior distribution (as in Liang et al., 2008) with the default
hyper-parameter \eqn{a} set such that the prior expected shrinkage factor
conforms to 'UIP';\cr This hyperparameter \eqn{a} can be adjusted (between
\eqn{2<a<=4}) by setting \code{g="hyper=2.9"}, for instance.\cr
Alternatively, \code{g="hyper=UIP"} sets the prior expected value of the
shrinkage factor equal to that of UIP (default), \code{g="hyper=BRIC"} sets
it according to BRIC \cr cf section 'Details' fro more on the hyper-g prior}

\item{mprior}{a character denoting the model prior choice, defaulting to
"random":\cr \code{mprior="fixed"} denotes fixed common prior inclusion
probabilities for each regressor as e.g. in Sala-i-Martin, Doppelhofer, and
Miller(2004) - for their fine-tuning, cf. \code{mprior.size}. Preferable to
\code{mcmc="random"} if strong prior information on model size exists;\cr
\code{mprior="random"} (default) triggers the 'random theta' prior by Ley
and Steel (2008), who suggest a binomial-beta hyperprior on the a priori
inclusion probability;\cr \code{mprior="uniform"} employs the uniform model
prior;\cr \code{mprior="customk"} allows for custom model size priors (cf.
\code{mprior.size});\cr \code{mprior="pip"} allows for custom prior
inclusion probabilities (cf. \code{mprior.size});\cr Note that the prior on
models with more than N-3 regressors is automatically zero: these models
will not be sampled.}

\item{mprior.size}{if \code{mprior} is "fixed" or "random",
\code{mprior.size} is a scalar that denotes the prior expected value of the
model size prior (default K/2).\cr If \code{mprior="customk"} then a custom
model size prior can be provided as a K+1 vector detailing the priors from
model size 0 to K (e.g. rep(1,K+1) for the uniform model prior);\cr if
\code{mprior="pip"}, then custom prior inclusion probabilities can be
provided as a vector of size K, with elements in the interval (0,1)}

\item{user.int}{'interactive mode': print out results to console after
ending the routine and plots a chart (default TRUE).}

\item{start.value}{specifies the starting model of the iteration chain. For
instance a specific model by the corresponding column indices (e.g.
starting.model=numeric(K) starts from the null model including solely a
constant term) or \code{start.value=c(3,6)} for a starting model only
including covariates 3 and 6.\cr If \code{start.model} is set to an integer
(e.g. \code{start.model=15}) then that number of covariates (here: 15
covariates) is randomly chosen and the starting model is identified by those
regressors with an OLS t-statistic>0.2.\cr The default value
\code{start.value=NA} corresponds to
\code{start.value=min(ncol(X.data),nrow(X.data)-3)}. Note that
\code{start.value=0} or \code{start.value=NULL} starts from the null
model.\cr If \code{mcmc="enumerate"} then \code{start.value} is the index to
start the iteration (default: 0, the null model) . Any number between 0 and
\eqn{K^2-1} is admissible.}

\item{g.stats}{\code{TRUE} if statistics on the shrinkage factor g/(1+g)
should be collected, defaulting to TRUE (Note: set \code{g.stats=FALSE} for
faster iteration.)}

\item{logfile}{setting \code{logfile=TRUE} produces a logfile named
\code{"test.log"} in your current working directory, in order to keep track
of the sampling procedure. \code{logfile} equal to some filepath (like
\code{logfile="subfolder/log.txt"}) puts the logfile into that specified
position. (default: \code{logfile=FALSE}). Note that \code{logfile=""}
implies log printouts on the console.}

\item{logstep}{specifies at which number of posterior draws information is
written to the log file; default: 10 000 iterations}

\item{force.full.ols}{default FALSE. If \code{force.full.ols=TRUE}, the OLS
estimation part of the sampling procedure relies on slower matrix inversion,
instead of streamlined routines. \code{force.full.ols=TRUE} can slow down
sampling but may deal better with highly collinear data}

\item{fixed.reg}{indices or variable names of \code{X.data} that are fixed
regressors to be always included in every sampled model. Note: the parameter
\code{mprior.size} refers to prior model size including these fixed
regressors.}
}
\value{
A list of class \code{bma}, that may be displayed using e.g.
\code{\link{summary.bma}} or \code{\link{coef.bma}}. The list contains the
following elements: \item{info}{a list of aggregate statistics: \code{iter}
is the number of iterations, \code{burn} the number of burn-ins.\cr The
following have to be divided by \code{cumsumweights} to get posterior
expected values: \code{inccount} are the posterior inclusion probabilities,
\code{b1mo} and \code{b2mo} the first and second moment of coefficients,
\code{add.otherstats} other statistics of interest (typically the moments of
the shrinkage factor), \code{msize} is the post. expected model size,
\code{k.vec} the posterior model size distribution, \code{pos.sign} the
unconditional post. probability of positive coefficients, \code{corr.pmp} is
the correlation between the best models' MCMC frequencies and their marg.
likelihoods.\cr \code{timed} is the time that was needed for MCMC sampling,
\code{cons} is the posterior expected value of the constant. \code{K} and
\code{N} are the maximum number of covariates and the sample size,
respectively.} \item{arguments}{a list of the evaluated function arguments
provided to \code{bms} (see above)} \item{topmod}{a 'topmod' object
containing the best drawn models. see \code{\link{topmod}} for more details}
\item{start.pos}{the positions of the starting model. If bmao is a'bma'
object this corresponds to covariates bmao$reg.names[bmao$start.pos]. If
bmao is a chain that resulted from several starting models (cf.
\code{\link{c.bma}}, then \code{start.pos} is a list detailing all of them.}
\item{gprior.info}{a list of class \code{\link{gprior-class}}, detailing
information on the g-prior: \code{gtype} corresponds to argument \code{g}
above, \code{is.constant} is FALSE if \code{gtype} is either "hyper" or
"EBL", \code{return.g.stats} corresponds to argument \code{g.stats} above,
\code{shrinkage.moments} contains the first and second moments of the
shrinkage factor (only if \code{return.g.stats==TRUE}), \code{g} details the
fixed g (if \code{is.constant==TRUE}), \code{hyper.parameter} corresponds to
the hyper-g parameter \eqn{a} as in Liang et al. (2008) }
\item{mprior.info}{a list of class \code{\link{mprior-class}}, detailing
information on the model prior: \code{origargs} lists the original arguments
to \code{mprior} and \code{mprior.size} above; \code{mp.msize} denotes the
prior mode size; \code{mp.Kdist} is a (K+1) vector with the prior model size
distribution from 0 to K} \item{X.data}{data.frame or matrix: corresponds to
argument \code{X.data} above, possibly cleaned for NAs}
\item{reg.names}{character vector: the covariate names to be used for X.data
(corresponds to \code{\link{variable.names.bma}} } \item{bms.call}{the
original call to the \code{bms} function}
}
\description{
Given data and prior information, this function samples all possible model
combinations via MC3 or enumeration and returns aggregate results.
}
\details{
Ad \code{mcmc}: \cr Interaction sampler: adding an ".int" to an MC3 sampler
(e.g. "mcmc="bd.int") provides for special treatment of interaction terms.
Interaction terms will only be sampled along with their component variables:
In the colnumn names of X.data, interaction terms need to be denominated by
names consisting of the base terms separated by \code{#} (e.g. an
interaction term of base variables \code{"A"}, \code{"B"} and \code{"C"}
needs column name \code{"A#B#C"}). Then variable \code{"A#B#C"} will only be
included in a model if all of the component variables ("A", "B", and "C")
are included.

The MC3 samplers "\code{bd}", "\code{rev.jump}", "\code{bd.int}" and
"\code{rev.jump.int}", iterate away from a starting model by adding,
dropping or swapping (only in the case of rev.jump) covariates.

In an MCMC fashion, they thus randomly draw a candidate model and then move
to it in case its marginal likelihood (marg.lik.) is superior to the
marg.lik. of the current model.

In case the candidate's marg.lik is inferior, it is randomly accepted or
rejected according to a probability formed by the ratio of candidate
marg.lik over current marg.lik.  Over time, the sampler should thus converge
to a sensible distribution. For aggregate results based on these MC3
frequencies, the first few iterations are typically disregarded (the
'burn-ins').

Ad \code{g} and the hyper-g prior: The hyper-g prior introduced by Liang et
al. (2008) puts a prior distribution on the shrinkage factor \eqn{g/(1+g)},
namely a Beta distribution \eqn{ Beta(1, 1/2-1)} that is governed by the
parameter \eqn{a}. \eqn{a=4} means a uniform prior distribution of the
shrinkage factor, while \eqn{a>2} close to 2 concentrates the prior
shrinkage factor close to one. \cr The prior expected value is
\eqn{E(g/1+g)) = 2/a}. In this sense \code{g="hyper=UIP"} and
\code{g="hyper=BRIC"} set the prior expected shrinkage such that it conforms
to a fixed UIP-g (eqng=N) or BRIC-g (\eqn{g=max(K^2,N)} ).
}
\note{
There are several ways to speed-up sampling: \code{nmodel=10} saves
only the ten best models, at most a marginal improvement. \code{nmodels=0}
does not save the best (500) models, however then posterior convergence and
likelihood-based inference are not possible.  %\code{beta.save=FALSE} saves
the best models, but not their coefficients, which renders the use of
\code{image.bma} and the paramer \code{exact=TRUE} in functions such as
\code{coef.bma} infeasible.  \code{g.stats=FALSE} saves some time by not
retaining the shrinkage factors for the MC3 chain (and the best models).
\code{force.fullobject=TRUE} in contrast, slows sampling down significantly
if \code{mcmc="enumerate"}.
}
\section{Theoretical background}{
 The models analyzed are Bayesian
normal-gamma conjugate models with improper constant and variance priors
akin to Fernandez, Ley and Steel (2001): A model \eqn{M} can be described as
follows, with \eqn{\epsilon} ~ \eqn{N(0,\sigma^2 I)}: \deqn{latex}{ y=
\alpha + X \beta + \epsilon} \deqn{f(\beta | \sigma, M, g) ~ N(0, g \sigma^2
(X'X)^-1) }

Moreover, the (improper) prior on the constant \eqn{f(\alpha)} is put
proportional to 1. Similarly, the variance prior \eqn{f(\sigma)} is
proportional to \eqn{1/\sigma}.
}

\examples{

  data(datafls)
  #estimating a standard MC3 chain with 1000 burn-ins and 2000 iterations and uniform model priors
  bma1 = bms(datafls,burn=1000, iter=2000, mprior="uniform")

  ##standard coefficients based on exact likelihoods of the 100 best models:
  coef(bma1,exact=TRUE, std.coefs=TRUE) 
  
  #suppressing user-interactive output, using a customized starting value, and not saving the best 
  #  ...models for only 19 observations (but 41 covariates)
  bma2 = bms(datafls[20:39,],burn=1000, iter=2000, nmodel=0, start.value=c(1,4,7,30),
     user.int=FALSE)
  coef(bma2)
  
  #MC3 chain with a hyper-g prior (custom coefficient a=2.1), saving only the 20 best models, 
  # ...and an alternative sampling procedure; putting a log entry to console every 1000th step
  bma3 = bms(datafls,burn=1000, iter=5000, nmodel=20, g="hyper=2.1", mcmc="rev.jump",
      logfile="",logstep=1000)
  image(bma3) #showing the coefficient signs of the 20 best models
  
  #enumerating with 10 covariates (= 1024 models), keeping the shrinkage factors 
  #  ...of the best 200 models
  bma4 = bms(datafls[,1:11],mcmc="enumerate",nmodel=200,g.stats=TRUE)

  #using an interaction sampler for two interaction terms
  dataint=datafls
  dataint=cbind(datafls,datafls$LifeExp*datafls$Abslat/1000,
        datafls$Protestants*datafls$Brit-datafls$Muslim)
  names(dataint)[ncol(dataint)-1]="LifeExp#Abslat"
  names(dataint)[ncol(dataint)]="Protestants#Brit#Muslim"
  bma5 = bms(X.data=dataint,burn=1000,iter=9000,start.value=0,mcmc="bd.int") 
  
  density(bma5,reg="English") # plot posterior density for covariate "English"
  
  # a matrix as X.data argument
  bms(matrix(rnorm(1000),100,10))
  
  # keeping a set of fixed regressors:
  bms(datafls, mprior.size=7, fixed.reg = c("PrScEnroll", "LifeExp", "GDP60"))
  # Note that mprior.size=7 means prior model size of 3 fixed to 4 'uncertain' regressors
  
}
\references{
\url{http://bms.zeugner.eu}: BMS package homepage with help and tutorials

Feldkircher, M. and S. Zeugner (2015): Bayesian Model Averaging Employing 
Fixed and Flexible Priors: The BMS Package for R, Journal of Statistical Software 68(4).

Feldkircher, M. and S. Zeugner (2009): Benchmark Priors
Revisited: On Adaptive Shrinkage and the Supermodel Effect in Bayesian Model
Averaging, IMF Working Paper 09/202.

Fernandez, C. E. Ley and M. Steel (2001): Benchmark priors for Bayesian
model averaging. Journal of Econometrics 100(2), 381--427

Ley, E. and M. Steel (2008): On the Effect of Prior Assumptions in Bayesian
Model Averaging with Applications to Growth Regressions. working paper

Liang, F., Paulo, R., Molina, G., Clyde, M. A., and Berger, J. O. (2008).
Mixtures of g Priors for Bayesian Variable Selection. Journal of the
American Statistical Association 103, 410-423.

Sala-i-Martin, X. and G. Doppelhofer and R.I. Miller (2004): Determinants of
long-term growth: a Bayesian averaging of classical estimates (BACE)
approach. American Economic Review 94(4), 813--835
}
\seealso{
\code{\link{coef.bma}}, \code{\link{plotModelsize}} and
\code{\link{density.bma}} for some operations on the resulting 'bma' object,
\code{\link{c.bma}} for integrating separate MC3 chains and splitting of
sampling over several runs.

Check \url{http://bms.zeugner.eu} for additional help.
}
\author{
Martin Feldkircher, Paul Hofmarcher, and Stefan Zeugner
}
\keyword{models}

\name{plot.pred.density}
\alias{plot.pred.density}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{ Predictive Densities for bma Objects }
\description{
  Plots predictive densities for conditional forecasts of class 'pred.density' 
}
\usage{
## S3 method for class 'pred.density':
 \method{plot}{pred.density}(x, predict_index = NULL, addons = "eslz", realized.y = NULL, 
    addons.lwd = 1.5, ...) 
 }
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{x}{ an object of class \code{\link{pred.density}} }
  \item{predict_index}{An integer vector detailing which forecasted observations (corresponding to the argument \code{newdata} in \code{\link{pred.density}}) should be plotted.\cr
     Or the observation names to be plotted (as in \code{rownames(newdata)}). }
  \item{addons}{  character, defaulting to \code{"eslz"}. Specifies which additional information should be added to the plot via low-level commands (see 'Details' below).}
  \item{realized.y}{A vector with realized values of the dependent variables to be plotted in addition to the predictive density, must have its length conforming to \code{predict_index} (or \code{newdata})}
  \item{addons.lwd}{Line width to be used for the low-level plotting commands specified by \code{addons}. Cf. argument \code{lwd} in \code{\link{par}}.}
  \item{\dots}{arguments to be passed on to \code{\link{plot.density}}. }
}
\details{
The argument \code{addons} specifies what additional information should be added to the plot(s) via the low-level commands \code{\link{lines}} and \code{\link{legend}}:\cr
  "e" for the posterior expected value (EV) of the prediction,\cr
  "s" for 2 times its posterior standard deviation ('standard errors'),\cr
  "z" for a zero line, 
  "l" for including a \code{\link{legend}}\cr
  
  Any combination of these letters will give the desired result. Use \code{addons=""} for not using any of these.
  

}
%\references{ ~put references to the literature/web site here ~ }
\author{ Martin Feldkircher and Stefan Zeugner }
\seealso{ \code{\link{pred.density}} for constructing predictive densities, \code{\link{bms}} for creating \code{bma} objects, \code{\link{density.bma}} for plotting coefficient densities

Check \url{http://bms.zeugner.eu} for additional help.}
\examples{
 data(datafls)
 mm=bms(datafls,user.int=FALSE)
 
 #predictive density for two 'new' data points
 pd=pred.density(mm,newdata=datafls[1:2,]) 
 
 
 #plot the density for the second forecast observation
 plot(pd,2)  
 
 #plot the density with realized dep. variable, and no standard errors
 plot(pd, 1, realized.y=0,addons="lzeg") 
 
 
 
}
% Add one or more standard keywords, see file 'KEYWORDS' in the
% R documentation directory.
\keyword{aplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/zzz.R
\docType{class}
\name{bma-class}
\alias{bma-class}
\title{Class "bma"}
\description{
A list holding results from a BMA iteration chain
}
\section{Objects from the Class}{
 Objects can be created via calls to
\code{\link{bms}}, but indirectly also via \code{\link{c.bma}}\cr A
\code{bma} object is a list whose elements hold information on input and
output for a Bayesian Model Averaging iteration chain, such as from a call
to \code{\link{bms}}:
}

\examples{

 data(datafls)
 mm=bms(datafls)
 #show posterior model size
 print(mm$info$msize/mm$info$cumsumweights)
 #is the same number as in
 summary(mm)
 

}
\references{
\url{http://bms.zeugner.eu}
}
\seealso{
\code{\link{bms}} for creating \code{bma} objects,\cr or
\code{\linkS4class{topmod}} for the topmod object
}
\author{
Martin Feldkircher and Stefan Zeugner
}
\keyword{classes}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{plotModelsize}
\alias{plotModelsize}
\title{Plot Model Size Distribution}
\usage{
plotModelsize(
  bmao,
  exact = FALSE,
  ksubset = NULL,
  include.legend = TRUE,
  do.grid = TRUE,
  ...
)
}
\arguments{
\item{bmao}{a 'bma' object (cf. \code{\link{bms}})}

\item{exact}{if \code{TRUE}, then the posterior model distribution is based
on the best models of \code{bmao} and their marginal likelihoods;\cr if
\code{FALSE} (default) then the distribution is based on all encountered
models and their MCMC frequencies (cf. 'Details' in \code{\link{coef.bma}})}

\item{ksubset}{integer vector detailing for which model sizes the plot
should be done}

\item{include.legend}{if \code{TRUE}, a small legend is included via the
low-level command \code{\link{legend}}}

\item{do.grid}{if \code{TRUE}, a \code{\link{grid}} is added to the plot
(with a simple \code{grid()}).}

\item{\dots}{parameters passed on to \code{\link{matplot}} with sensible
defaults}
}
\value{
As a default, \code{plotModelsize} plots the posterior model size
distribution as a blue line, and the prior model distribution as a dashed
red line.\cr In addition, it returns a list with the following elements:
\item{mean}{The posterior expected value of model size} \item{var}{The
variance of the posterior model size distribution} \item{dens}{A vector
detailing the posterior model size distribution from model size \eqn{0} (the
first element) to \eqn{K} (the last element)}
}
\description{
Plots posterior and prior model size distribution
}
\examples{

data(datafls)
mm=bms(datafls,burn=1500, iter=5000, nmodel=200,mprior="fixed",mprior.size=6)

#plot Nb.1 based on aggregate results
postdist= plotModelsize(mm)

#plot based only on 30 best models
plotModelsize(mm[1:30],exact=TRUE,include.legend=FALSE)

#plot based on all best models, but showing distribution only for model sizes 1 to 20
plotModelsize(mm,exact=TRUE,ksubset=1:20)

# create a plot similar to plot Nb. 1
plot(postdist$dens,type="l") 
lines(mm$mprior.info$mp.Kdist)


}
\seealso{
See also \code{\link{bms}}, \code{\link{image.bma}},
\code{\link{density.bma}}, \code{\link{plotConv}}

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{plotComp}
\alias{plotComp}
\title{Compare Two or More bma Objects}
\usage{
plotComp(
  ...,
  varNr = NULL,
  comp = "PIP",
  exact = FALSE,
  include.legend = TRUE,
  add.grid = TRUE,
  do.par = TRUE,
  cex.xaxis = 0.8
)
}
\arguments{
\item{\dots}{one or more objects of class 'bma' to be compared.
\code{plotComp} passes on any other parameters in \code{\dots{}} to
\code{\link{matplot}}.}

\item{varNr}{optionally, covariate indices to be included in the plot, can
be either integer vector or character vector - see examples}

\item{comp}{a character denoting what should be compared: \code{comp="PIP"}
(default) for posterior inclusion probabilities, \code{comp="Post Mean"} for
coefficients, \code{comp="Post SD"} for their standard deviations,
\code{comp="Std Mean"} or standardized coefficients, or \code{comp="Std SD"}
for standardized standard deviations}

\item{exact}{if \code{FALSE}, the statistics to be compared are based on
aggregate bma statistics, if \code{TRUE}, they are based solely on the best
models retained in the bma objects}

\item{include.legend}{whether to include a default legend in the plot
(custom legends can be added with the command \code{\link{legend}})}

\item{add.grid}{whether to add a \code{\link{grid}} to the plot}

\item{do.par}{whether to adjust \code{par("mar")} in order to fit in the
tick labels on the x-axis}

\item{cex.xaxis}{font size scaling parameter for the x-axis - cf. argument
\code{cex.axis} in \code{\link{par}}}
}
\description{
Plots a comparison of posterior inclusion probabilites, coefficients or
their standard deviation between various bma objects
}
\examples{

## sample two simple bma objects
data(datafls)
mm1=bms(datafls[,1:15])
mm2=bms(datafls[,1:15])

#compare PIPs
plotComp(mm1,mm2)

#compare standardized coefficeitns
plotComp(mm1,mm2,comp="Std Mean")

#...based on the lieklihoods of best models 
plotComp(mm1,mm2,comp="Std Mean",exact=TRUE)

#plot only PIPs for first four covariates
plotComp(mm1,mm2,varNr=1:4, col=c("black","red"))

#plot only coefficients for covariates 'GDP60 ' and 'LifeExp'
plotComp(mm1,mm2,varNr=c("GDP60", "LifeExp"),comp="Post Mean")



}
\seealso{
\code{\link{coef.bma}} for the underlying function

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{hplot}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{lps.bma}
\alias{lps.bma}
\title{Log Predictive Score}
\usage{
lps.bma(object, realized.y, newdata = NULL)
}
\arguments{
\item{object}{an object of class \code{\link{pred.density}}, or class
\code{bma} (cf. \code{\link{bms}}), or class \code{\link{zlm}}}

\item{realized.y}{a vector with realized values of the dependent variables
to be plotted in addition to the predictive density, must have its length
conforming to \code{newdata}}

\item{newdata}{Needs to be provided if \code{object} is not of class
\code{\link{pred.density}}: a data.frame, matrix or vector containing
variables with which to predict.}
}
\value{
A scalar denoting the log predictive score
}
\description{
Computes the Log Predictive Score to evaluate a forecast based on a bma
object
}
\details{
The log predictive score is an indicator for the likelihood of several
forecasts.\cr It is defined as minus the arithmethic mean of the logarithms
of the point densities for \code{realized.y} given \code{newdata}.\cr Note
that in most cases is more efficient to first compute the predictive density
object via a call to \code{\link{pred.density}} and only then pass the
result on to \code{lps.bma}.
}
\examples{

 data(datafls)
 mm=bms(datafls,user.int=FALSE,nmodel=100)
 
 #LPS for actual values under the used data (static forecast)
 lps.bma(mm, realized.y=datafls[,1] , newdata=datafls[,-1])
 
 #the same result via predicitve.density
 pd=pred.density(mm, newdata=datafls[,-1])
 lps.bma(pd,realized.y=datafls[,1])
 
 # similarly for a linear model (not BMA)
 zz = zlm(datafls)
 lps.bma(zz, realized.y=datafls[,1] , newdata=datafls[,-1])
 
}
\seealso{
\code{\link{pred.density}} for constructing predictive densities,
\code{\link{bms}} for creating \code{bma} objects, \code{\link{density.bma}}
for plotting coefficient densities

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{variable.names.zlm}
\alias{variable.names.zlm}
\alias{vcov.zlm}
\alias{logLik.zlm}
\title{Variable names and design matrix}
\usage{
\method{variable.names}{zlm}(object, ...)
}
\arguments{
\item{object}{A \code{bma} object (as produced by \code{\link{bms}})}

\item{...}{further arguments passed to or from other methods}
}
\description{
Simple utilities retrieving variable names and design matrix from a bma
object
}
\details{
\code{variable.names.zlm}: method \code{\link{variable.names}} for a
\code{\link{zlm}} model. \cr \code{vcov.zlm}: the posterior
variance-covariance matrix of the coefficients of a \code{\link{zlm}} model
- cf. \code{\link{vcov}} \cr \code{logLik.zlm}: a \code{\link{zlm}} model's
log-likelihood \code{p(y|M)} according to the implementation of the
respective coefficent prior \cr
}
\examples{

 data(datafls)
  
 zz=zlm(datafls)
 variable.names(zz)
 vcov(zz)
 logLik(zz)
 
}
\seealso{
\code{\link{zlm}} for creating \code{zlm} objects

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/aux_outer.R
\name{pred.density}
\alias{pred.density}
\alias{pred.density-class}
\alias{print.pred.density}
\title{Predictive Densities for bma Objects}
\usage{
pred.density(object, newdata = NULL, n = 300, hnbsteps = 30, ...)
}
\arguments{
\item{object}{a bma object - see \code{\link{bms}}, alternativel a
\code{\link{zlm}} object}

\item{newdata}{A data.frame, matrix or vector containing variables with
which to predict.}

\item{n}{The integer number of equally spaced points at which the density is
to be estimated.}

\item{hnbsteps}{The number of numerical integration steps to be used in case
of a hyper-g prior (cf. argument \code{g} in \code{\link{bms}}). Increase
this number to increase accuracy. Must be an even integer.}

\item{\dots}{arguments to be passed on to \code{\link{plot.density}}.}
}
\value{
\code{pred.density} returns a list of class \code{pred.density} with
the following elements \item{densities()}{a list whose elements each contain
the estimated density for each forecasted observation} \item{fit}{a vector
with the expected values of the predictions (the 'point forecasts')}
\item{std.err}{a vector with the standard deviations of the predictions (the
'standard errors')} \item{dyf(realized.y, predict_index=NULL)}{Returns the
densities of realized response variables provided in \code{realized.y}. \cr
If \code{realized.y} is a matrix, then each row corresponds to a forecast
observation in \code{newdata}\cr if not left empty, \code{predict.index}
specifies to which observations in newdata the realized.y should apply}
\item{lps(realized.y, predict_index=NULL)}{Computes the log predictive score
for the response varaible provided in \code{realized.y} (cf.
\code{\link{lps.bma}}) -\cr Note that the LPS equals minus the mean of the
logarithmized results from \code{dyf}) } \item{plot((x, predict_index =
NULL, addons = "eslz", realized.y = NULL, addons.lwd = 1.5, ...)}{the same
as \code{plot.pred.density}} \item{n}{The number of equally spaced
points for which the density (under \code{densities()} was computed.}
\item{nmodel}{The number of best models predictive densities are based
upon.} \item{call}{the call that created this \code{pred.density} object}
}
\description{
Predictive densities for conditional forecasts
}
\details{
The predictive density is a mixture density based on the \code{nmodels} best
models in a \code{bma} object (cf. \code{nmodel} in \code{\link{bms}}).\cr
The number of 'best models' to retain is therefore vital and should be set
quite high for accuracy.
}
\note{
In BMS version 0.3.0, \code{pred.density} may only cope with built-in
\code{gprior}s, not with any user-defined priors.
}
\examples{

 data(datafls)
 mm=bms(datafls,user.int=FALSE)
 
 #predictive densityfor two 'new' data points
 pd=pred.density(mm,newdata=datafls[1:2,]) 
 
 
 #fitted values based on best models, same as predict(mm, exact=TRUE)
 pd$fit
 
 #plot the density for the first forecast observation
 plot(pd,1)  
 
 # the same plot ' naked'
 plot(pd$densities()[[1]])
 
 
 #predict density for the first forecast observation if the dep. variable is 0
 pd$dyf(0,1) 
 
 #predict densities for both forecasts for the realizations 0 and 0.5
 pd$dyf(rbind(c(0,.5),c(0,.5)))
 
 # calc. Log Predictive Score if both forecasts are realized at 0:
 lps.bma(pd,c(0,0))
 
 
}
\seealso{
\code{\link{predict.bma}} for simple point forecasts,
\code{plot.pred.density} for plotting predictive densities,
\code{\link{lps.bma}} for calculating the log predictive score
independently, \code{\link{quantile.pred.density}} for extracting quantiles

Check \url{http://bms.zeugner.eu} for additional help.
}
\keyword{utilities}
